/*******************************************************************************

NAME            tlb.c
VERSION         %version: 13 %
UPDATE DATE     %date_modified: Thu Oct 17 11:34:51 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/*
 * (C) Copyright 2007
 * Stefan Roese, DENX Software Engineering, sr@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>

#if defined(CONFIG_440)

#include <asm/ppc440.h>
#include <asm/cache.h>
#include <asm/io.h>
#include <asm/mmu.h>

typedef struct region {
    u64 base;
    u32 size;
    u32 tlb_word2_i_value;
} region_t;

void remove_tlb(u32 vaddr, u32 size)
{
    int i,j;
    u32 tlb_word0_value;
    u32 tlb_word1_value;
    u32 tlb_word2_value;
    u32 tlb_vaddr;
    u32 tlb_size = 0;
    u32 tlb_index;

    for (i=0; i<PPC4XX_TLB_SIZE; i++)
    {
        for(j=0;j<4;j++) /* 4 ways in each TLB entry */
        {
            tlb_index = ((i<<16) | (j<<29));
            tlb_word0_value = mftlb1(tlb_index);
            tlb_word1_value = mftlb2(tlb_index);
            tlb_word2_value = mftlb3(tlb_index);
            tlb_vaddr = TLB_WORD0_EPN_DECODE(tlb_word0_value);
            if (((tlb_word0_value & TLB_WORD0_V_MASK) == TLB_WORD0_V_ENABLE) &&
                (tlb_vaddr >= vaddr))
            {
                /*
                 * TLB is enabled and start address is lower or equal
                 * than the area we are looking for. Now we only have
                 * to check the size/end address for a match.
                 */
                switch (tlb_word0_value & TLB_WORD0_SIZE_MASK)
                {
                case TLB_WORD0_SIZE_4KB:
                    tlb_size = 4 << 10;
                    break;
                case TLB_WORD0_SIZE_16KB:
                    tlb_size = 16 << 10;
                    break;
                case TLB_WORD0_SIZE_64KB:
                    tlb_size = 64 << 10;
                    break;
                case TLB_WORD0_SIZE_1MB:
                    tlb_size = 1 << 20;
                    break;
                case TLB_WORD0_SIZE_16MB:
                    tlb_size = 16 << 20;
                    break;
                case TLB_WORD0_SIZE_256MB:
                    tlb_size = 256 << 20;
                    break;
                case TLB_WORD0_SIZE_1GB:
                    tlb_size = 1024 << 20;
                }

                /*
                 * Now check the end-address if it's in the range
                 */
                if ((tlb_vaddr + tlb_size - 1) <= (vaddr + size - 1))
                    {
                    /*
                     * Found a TLB in the range.
                     * Disable it by writing 0 to tlb0 word.
                     */
                    tlb_word0_value &= (~(TLB_WORD0_V_ENABLE));
                    mttlb1(tlb_index|0x80000000, tlb_word0_value);
                    mttlb2(tlb_index|0x80000000, tlb_word1_value);
                    mttlb3(tlb_index|0x80000000, tlb_word2_value);
                }
            }
        }
    }

    /* Execute an ISYNC instruction so that the new TLB entry takes effect */
    asm("isync");
}

/*
 * Change the I attribute (cache inhibited) of a TLB or multiple TLB's.
 * This function is used to either turn cache on or off in a specific
 * memory area.
 */
void change_tlb(u32 vaddr, u32 size, u32 tlb_word2_i_value)
{
    int i,j;
    u32 tlb_word0_value;
    u32 tlb_word1_value;
    u32 tlb_word2_value;
    u32 tlb_vaddr;
    u32 tlb_size = 0;
    u32 tlb_index = 0x00;

    for (i=0; i<PPC4XX_TLB_SIZE; i++)
    {
        for(j = 0; j<4;j++)
        {
            tlb_index = ((i<<16) | (j<<29));
            tlb_word0_value = mftlb1(tlb_index);
            tlb_word1_value = mftlb2(tlb_index);
            tlb_word2_value = mftlb3(tlb_index);
            tlb_vaddr = TLB_WORD0_EPN_DECODE(tlb_word0_value);
            if (((tlb_word0_value & TLB_WORD0_V_MASK) == TLB_WORD0_V_ENABLE) &&
                (tlb_vaddr >= vaddr))
            {
                /*
                 * TLB is enabled and start address is lower or equal
                 * than the area we are looking for. Now we only have
                 * to check the size/end address for a match.
                 */
                switch (tlb_word0_value & TLB_WORD0_SIZE_MASK) {
                case TLB_WORD0_SIZE_4KB:
                    tlb_size = 4 << 10;
                    break;
                case TLB_WORD0_SIZE_16KB:
                    tlb_size = 16 << 10;
                    break;
                case TLB_WORD0_SIZE_64KB:
                    tlb_size = 64 << 10;
                    break;
                case TLB_WORD0_SIZE_1MB:
                    tlb_size = 1 << 20;
                    break;
                case TLB_WORD0_SIZE_16MB:
                    tlb_size = 16 << 20;
                    break;
                case TLB_WORD0_SIZE_256MB:
                    tlb_size = 256 << 20;
                    break;
                case TLB_WORD0_SIZE_1GB:
                    tlb_size = 1024 << 20;
                    break;
                }

                /*
                 * Now check the end-address if it's in the range
                 */
                if (((tlb_vaddr + tlb_size - 1) <= (vaddr + size - 1)) ||
                    ((tlb_vaddr < (vaddr + size - 1)) &&
                     ((tlb_vaddr + tlb_size - 1) > (vaddr + size - 1)))) {
                    /*
                     * Found a TLB in the range.
                     * Change cache attribute in tlb2 word.
                     */
                    tlb_word2_value =
                        TLB_WORD2_U0_DISABLE | TLB_WORD2_U1_DISABLE |
                        TLB_WORD2_U2_DISABLE | TLB_WORD2_U3_DISABLE |
                        TLB_WORD2_W_DISABLE | tlb_word2_i_value |
                        TLB_WORD2_M_DISABLE | TLB_WORD2_G_DISABLE |
                        TLB_WORD2_E_DISABLE | TLB_WORD2_UX_ENABLE |
                        TLB_WORD2_UW_ENABLE | TLB_WORD2_UR_ENABLE |
                        TLB_WORD2_SX_ENABLE | TLB_WORD2_SW_ENABLE |
                        TLB_WORD2_SR_ENABLE;

                    /*
                     * Now either flush or invalidate the dcache
                     */
                    if (tlb_word2_i_value)
                        flush_dcache();
                    else
                        invalidate_dcache();

                    mttlb1(tlb_index|0x80000000, tlb_word0_value);
                    mttlb2(tlb_index|0x80000000, tlb_word1_value);
                    mttlb3(tlb_index|0x80000000, tlb_word2_value);
                    asm("iccci 0,0");
                    asm("isync");
                    return;
                }
            }
        }
    }

    /* Execute an ISYNC instruction so that the new TLB entry takes effect */
}

static int add_tlb_entry(u64 phys_addr,
             u32 virt_addr,
             u32 tlb_word0_size_value,
             u32 tlb_word2_i_value)
{
    int i,j;
    unsigned int flag;
    unsigned long tlb_word0_value;
    unsigned long tlb_word1_value;
    unsigned long tlb_word2_value;

    flag = 0;
    /* First, find the index of a TLB entry not being used */
    for (i=0; i<PPC4XX_TLB_SIZE; i++)
    {
        for(j=0;j<4;j++)
        {
            tlb_word0_value = mftlb1(i<<16|j<<29|0x80000000);
            if ((tlb_word0_value & TLB_WORD0_V_MASK) == TLB_WORD0_V_DISABLE)
            {
                flag = 1;
                break;
            }
        }
        if(flag == 1)
        {
            break;
        }
    }
    if (i >= PPC4XX_TLB_SIZE)
        return -1;

    /* Second, create the TLB entry */
    tlb_word0_value = TLB_WORD0_EPN_ENCODE(virt_addr) | TLB_WORD0_V_ENABLE |
        TLB_WORD0_TS_0 | tlb_word0_size_value;
    tlb_word1_value = TLB_WORD1_RPN_ENCODE((u32)phys_addr) |
        TLB_WORD1_ERPN_ENCODE(phys_addr >> 32);
    tlb_word2_value = TLB_WORD2_U0_DISABLE | TLB_WORD2_U1_DISABLE |
        TLB_WORD2_U2_DISABLE | TLB_WORD2_U3_DISABLE |
        TLB_WORD2_W_DISABLE | tlb_word2_i_value |
        TLB_WORD2_M_DISABLE | TLB_WORD2_G_DISABLE |
        TLB_WORD2_E_DISABLE | TLB_WORD2_UX_ENABLE |
        TLB_WORD2_UW_ENABLE | TLB_WORD2_UR_ENABLE |
        TLB_WORD2_SX_ENABLE | TLB_WORD2_SW_ENABLE |
        TLB_WORD2_SR_ENABLE;

    /* Wait for all memory accesses to complete */
    sync();

    /* Third, add the TLB entries */
    mttlb1(i, tlb_word0_value);
    mttlb2(i, tlb_word1_value);
    mttlb3(i, tlb_word2_value);

    /* Execute an ISYNC instruction so that the new TLB entry takes effect */
    asm("isync");
    asm("msync");

    return 0;
}

static void program_tlb_addr(u64 phys_addr,
                 u32 virt_addr,
                 u32 mem_size,
                 u32 tlb_word2_i_value)
{
    int rc;
    u32 tlb_i;

    tlb_i = tlb_word2_i_value;
    while (mem_size != 0) {
        rc = 0;

        /* Add the TLB entries in to map the region. */
        if (((phys_addr & TLB_1GB_ALIGN_MASK) == phys_addr) &&
            (mem_size >= TLB_1GB_SIZE)) {
            /* Add a 1GB TLB entry */
            if ((rc = add_tlb_entry(phys_addr, virt_addr,
                        TLB_WORD0_SIZE_1GB, tlb_i)) == 0) {
                mem_size -= TLB_1GB_SIZE;
                phys_addr += TLB_1GB_SIZE;
                virt_addr += TLB_1GB_SIZE;
            }
        } else if (((phys_addr & TLB_256MB_ALIGN_MASK) == phys_addr) &&
            (mem_size >= TLB_256MB_SIZE)) {
            /* Add a 256MB TLB entry */
            if ((rc = add_tlb_entry(phys_addr, virt_addr,
                        TLB_WORD0_SIZE_256MB, tlb_i)) == 0) {
                mem_size -= TLB_256MB_SIZE;
                phys_addr += TLB_256MB_SIZE;
                virt_addr += TLB_256MB_SIZE;
            }
        } else if (((phys_addr & TLB_16MB_ALIGN_MASK) == phys_addr) &&
               (mem_size >= TLB_16MB_SIZE)) {
            /* Add a 16MB TLB entry */
            if ((rc = add_tlb_entry(phys_addr, virt_addr,
                        TLB_WORD0_SIZE_16MB, tlb_i)) == 0) {
                mem_size -= TLB_16MB_SIZE;
                phys_addr += TLB_16MB_SIZE;
                virt_addr += TLB_16MB_SIZE;
            }
        } else if (((phys_addr & TLB_1MB_ALIGN_MASK) == phys_addr) &&
               (mem_size >= TLB_1MB_SIZE)) {
            /* Add a 1MB TLB entry */
            if ((rc = add_tlb_entry(phys_addr, virt_addr,
                        TLB_WORD0_SIZE_1MB, tlb_i)) == 0) {
                mem_size -= TLB_1MB_SIZE;
                phys_addr += TLB_1MB_SIZE;
                virt_addr += TLB_1MB_SIZE;
            }
        } else if (((phys_addr & TLB_64KB_ALIGN_MASK) == phys_addr) &&
               (mem_size >= TLB_64KB_SIZE)) {
            /* Add a 64KB TLB entry */
            if ((rc = add_tlb_entry(phys_addr, virt_addr,
                        TLB_WORD0_SIZE_64KB, tlb_i)) == 0) {
                mem_size -= TLB_64KB_SIZE;
                phys_addr += TLB_64KB_SIZE;
                virt_addr += TLB_64KB_SIZE;
            }
        } else if (((phys_addr & TLB_16KB_ALIGN_MASK) == phys_addr) &&
               (mem_size >= TLB_16KB_SIZE)) {
            /* Add a 16KB TLB entry */
            if ((rc = add_tlb_entry(phys_addr, virt_addr,
                        TLB_WORD0_SIZE_16KB, tlb_i)) == 0) {
                mem_size -= TLB_16KB_SIZE;
                phys_addr += TLB_16KB_SIZE;
                virt_addr += TLB_16KB_SIZE;
            }
        } else if (((phys_addr & TLB_4KB_ALIGN_MASK) == phys_addr) &&
               (mem_size >= TLB_4KB_SIZE)) {
            /* Add a 4KB TLB entry */
            if ((rc = add_tlb_entry(phys_addr, virt_addr,
                        TLB_WORD0_SIZE_4KB, tlb_i)) == 0) {
                mem_size -= TLB_4KB_SIZE;
                phys_addr += TLB_4KB_SIZE;
                virt_addr += TLB_4KB_SIZE;
            }
        } else {
            printf("ERROR: no TLB size(0x%x) exists for the base address 0x%llx, 0x%llx.\n",mem_size,
                phys_addr,(phys_addr & TLB_4KB_ALIGN_MASK));
        }

        if (rc != 0)
            printf("ERROR: no TLB entries available for the base addr 0x%llx.\n",
                phys_addr);
    }

    return;
}

/*
 * Program one (or multiple) TLB entries for one memory region
 *
 * Common usage for boards with SDRAM DIMM modules to dynamically
 * configure the TLB's for the SDRAM
 */
void program_tlb(u64 phys_addr, u32 virt_addr, u32 size, u32 tlb_word2_i_value)
{
    region_t region_array;

    region_array.base = phys_addr;
    region_array.size = size;
    region_array.tlb_word2_i_value = tlb_word2_i_value; /* en-/disable cache */

    /* Call the routine to add in the tlb entries for the memory regions */
    program_tlb_addr(region_array.base, virt_addr, region_array.size,
             region_array.tlb_word2_i_value);

    /* Execute an ISYNC instruction so that the new TLB entry takes effect */
    asm("isync");
    asm("msync");

    return;
}

/*
 * Modify the TLB entries to boot little-endian VxWorks
 *
 * This re-maps the DDR entries to use little-endian mode.
 * It re-maps the flash entries to LE mode as well, so we don't flip
 * data around by copying from BE flash to LE DDR. The ELF header on
 * the kernel is LE, so flash needs to be LE in order to parse it.
 * 
 * This is called externally right before looking through flash for a valid
 * ELF header. It will prohibit BE kernels from being found and booted.
 */
void fixup_tlbs(void)
{
    static int fixed=0;

    if (!fixed)
    {
        printf("Fixing TLBs for VxWorks\n");
    }
    else
    {
        printf("TLBs already set up\n");
        return;
    }

    /* Change TLBs to be LE
     * Both VxWorks and Linux will boot out of a 256MB LE TLB entry. Set that
     * region to LE now.
     * If booting Linux, everything has already been unpacked into this BE
     * region from BE source regions (either NOR flash or a DDR buffer). */
    change_tlb(CONFIG_OS_BASE, 0x10000000, TLB_WORD2_E_ENABLE | TLB_WORD2_M_ENABLE);

    /* If booting VxWorks, the ELF file has not yet been copied from its source
     * location (either NOR flash or a DDR buffer) to the now-LE destination.
     * The source locations are still BE, so change both to LE so we will
     * perform a LE-to-LE copy. */

    /* 32MB DDR buffer space */
    change_tlb(CONFIG_OS_BUF_BASE1, 0x01000000,  TLB_WORD2_E_ENABLE | TLB_WORD2_M_ENABLE);
    change_tlb(CONFIG_OS_BUF_BASE2, 0x01000000,  TLB_WORD2_E_ENABLE | TLB_WORD2_M_ENABLE);
    /* PBAM Decode 4 (Flash) */
    change_tlb(0xFE000000, 0x01000000, TLB_WORD2_I_ENABLE | TLB_WORD2_E_ENABLE | TLB_WORD2_G_ENABLE);
    change_tlb(0xFF000000, 0x01000000, TLB_WORD2_I_ENABLE | TLB_WORD2_E_ENABLE | TLB_WORD2_G_ENABLE);

    fixed=1;
}

/*
 * TLB setup for USB support
 * Remap the base of DRAM as uncached LE to use for buffers.
 * Create a LE-mapped window to the USB registers at 0x92000000.
 * USB LE register base is CONFIG_SYS_PPC4XX_USB_ADDR.
 */
void usb_tlb_setup (void)
{
    program_tlb(0x60C2000000ULL, 0x92000000, 0x00100000, TLB_WORD2_I_ENABLE | TLB_WORD2_E_ENABLE | TLB_WORD2_G_ENABLE);
}

/*
 * Dumps contents of valid TLBs (for debugging) 
 * The TLB index is from 0x00-0xFF and each index has 4 ways, for a total of 
 * 1024 possible entries.
 */
void TlbDebugDump(void) {

    unsigned long  tlbIndex, tlbWord0, tlbWord1, tlbWord2, way;
    char sizeStr[7];

    printf ("[TLB Dump]\n");
    for (tlbIndex=0; tlbIndex < PPC4XX_TLB_SIZE; tlbIndex++) 
    {

        for(way=0;way<4;way++)
        {
            tlbWord0 = mftlb1(tlbIndex<<16 | way << 29);
            tlbWord1 = mftlb2(tlbIndex<<16 | way << 29);
            tlbWord2 = mftlb3(tlbIndex<<16 | way << 29);

            if ((tlbWord0 & TLB_WORD0_V_MASK)==0)
                continue;

            switch (tlbWord0 & TLB_WORD0_SIZE_MASK)
            {
            case TLB_WORD0_SIZE_4KB:
                sprintf(sizeStr, "4 KB  ");
                break;
            case TLB_WORD0_SIZE_16KB:
                sprintf(sizeStr, "16 KB ");
                break;
            case TLB_WORD0_SIZE_64KB:
                sprintf(sizeStr, "64 KB ");
                break;
            case TLB_WORD0_SIZE_1MB:
                sprintf(sizeStr, "1 MB  ");
                break;
            case TLB_WORD0_SIZE_16MB:
                sprintf(sizeStr, "16 MB ");
                break;
            case TLB_WORD0_SIZE_256MB:
                sprintf(sizeStr, "256 MB");
                break;
            case TLB_WORD0_SIZE_1GB:
                sprintf(sizeStr, "1 GB  ");
                break;
            default:
                sprintf(sizeStr, "? x%03lx", tlbWord0 & TLB_WORD0_SIZE_MASK);
                break;
            }
            printf ("TLB #%02lx [%lx]: Virt=%08lx, Phys=%02lx_%08lx, V=%d, E=%c, C=%d, WT=%d, G=%d, Size=%s\n", 
                tlbIndex, way,                                   /* TLB index & way  */
                tlbWord0 & TLB_WORD0_EPN_MASK,                   /* virtual address  */
                tlbWord1 & 0xFF, tlbWord1 & TLB_WORD1_RPN_MASK,  /* physical address */
                tlbWord0 & TLB_WORD0_V_MASK ? 1 : 0,             /* valid            */
                tlbWord2 & TLB_WORD2_E_ENABLE ? 'L' : 'B',       /* endian           */
                tlbWord2 & TLB_WORD2_I_ENABLE ? 0 : 1,           /* caching enabled  */
                tlbWord2 & TLB_WORD2_W_ENABLE ? 1 : 0,           /* write through    */
                tlbWord2 & TLB_WORD2_G_ENABLE ? 1 : 0,           /* guarded          */
                sizeStr);                                        /* size             */
        }
    }
    printf ("Legend: V=Valid, E=Endian, C=Cacheable, WT=Write-Through, G=Guarded\n");
}

/*
 * Dumps raw contents of all TLBs (for debugging) 
 * The TLB index is from 0x00-0xFF and each index has 4 ways, for a total of 
 * 1024 possible entries.
 */
void TlbDebugDumpRaw(void) {

    unsigned long  tlbIndex, tlbWord0, tlbWord1, tlbWord2, way;

    printf ("[TLB Raw Dump]\n");
    for (tlbIndex=0; tlbIndex < PPC4XX_TLB_SIZE; tlbIndex++) 
    {
        for(way=0;way<4;way++)
        {
            tlbWord0 = mftlb1(tlbIndex<<16 | way << 29);
            tlbWord1 = mftlb2(tlbIndex<<16 | way << 29);
            tlbWord2 = mftlb3(tlbIndex<<16 | way << 29);
            printf("TLB #%02lx [%lx]: %08lx %08lx %08lx\n", tlbIndex, way, tlbWord0, tlbWord1, tlbWord2);
        }
    }
}

/* This is just a general purpose shell command that can be modified and used
 * for whatever purposes a developer sees necessary.
 */
int do_shell_debug ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    fixup_tlbs();

    return 0;
}

/* Wrapper function called from the shell */
int do_tlbdump ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    TlbDebugDump();
    return 0;
}

/* Wrapper function called from the shell */
int do_tlbdumpraw ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    TlbDebugDumpRaw();
    return 0;
}

/* u-boot macros to add TLB commands to the shell */
U_BOOT_CMD(
    tlbvxw, 1, 1, do_shell_debug,
    "Debug: Execute shell debug function",
    ""
);

U_BOOT_CMD(
    tlbdump, 1, 1, do_tlbdump,
    "Dump TLB info",
    ""
);

U_BOOT_CMD(
    tlbdumpraw, 1, 1, do_tlbdumpraw,
    "Dump raw TLB info",
    ""
);

#endif /* CONFIG_440 */
