/*******************************************************************************

NAME            boardDisplayCode.h
SUMMARY         Header file for posting display codes.
VERSION         %version: 2 %
UPDATE DATE     %date_modified: Tue Nov  5 11:44:02 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2012-2013 NetApp, Inc. All rights reserved.

DESCRIPTION:
    This file provides an API and constant definitions to the functionality to
post display codes to a display as implemented in the source file.

*******************************************************************************/


#ifndef __INCboardDisplayCode
#define __INCboardDisplayCode


/***  INCLUDES  ***/  


/***  CONSTANT DEFINITIONS  ***/

/***************************************** 
 * DEFINITIONS FOR SEVEN SEGMENT DISPLAY 
 *****************************************/
#define DISPLAY_0           0x3F /* 0 */
#define DISPLAY_1           0x06 /* 1 */
#define DISPLAY_2           0x5B /* 2 */
#define DISPLAY_3           0x4F /* 3 */
#define DISPLAY_4           0x66 /* 4 */
#define DISPLAY_5           0x6D /* 5 */
#define DISPLAY_6           0x7D /* 6 */
#define DISPLAY_7           0x07 /* 7 */
#define DISPLAY_8           0x7F /* 8 */
#define DISPLAY_9           0x67 /* 9 */
#define DISPLAY_A           0x77 /* A */
#define DISPLAY_B           0x7C /* b */
#define DISPLAY_C           0x39 /* C */
#define DISPLAY_D           0x5E /* d */
#define DISPLAY_E           0x79 /* E */
#define DISPLAY_F           0x71 /* F */
#define DISPLAY_H           0x76 /* H */
#define DISPLAY_J           0x1E /* J */
#define DISPLAY_L           0x38 /* L */
#define DISPLAY_N           0x54 /* n */
#define DISPLAY_O           0x5C /* o */
#define DISPLAY_P           0x73 /* P */
#define DISPLAY_R           0x50 /* r */
#define DISPLAY_S           DISPLAY_5 /* S, also 5 */
#define DISPLAY_T           0x78 /* t */
#define DISPLAY_U           0x3E /* U */
#define DISPLAY_Y           0x6E /* y */
#define DISPLAY_DP          0x80 /* decimal point */
#define DISPLAY_DASH        0x40 /* dash */
#define DISPLAY_BLANK       0x00


/***************************************** 
 * Checkpoint are error 7-segment display codes.
 *****************************************/
#define UBOOT_DDR3_SPD_READ_START           ((DISPLAY_E << 8) | DISPLAY_7)
#define UBOOT_DDR3_NO_DIMM_FOUND            ((DISPLAY_E << 8) | DISPLAY_8)
#define UBOOT_DDR3_TRAINING_START           ((DISPLAY_E << 8) | DISPLAY_A)
#define UBOOT_DDR3_TRAINING_COMPLETE        ((DISPLAY_E << 8) | DISPLAY_B)
#define UBOOT_DDR3_CLEARING_MEMORY          ((DISPLAY_E << 8) | DISPLAY_C)
#define UBOOT_DDR3_SPD_READ_FAILURE         ((DISPLAY_5 << 8) | DISPLAY_1)
#define UBOOT_PCI_INITIALIZATION            ((DISPLAY_9 << 8) | DISPLAY_4)
#define UBOOT_PCI_INIT_DONE                 ((DISPLAY_9 << 8) | DISPLAY_F)
#define UBOOT_USING_ALT_BOOT_PATH           ((DISPLAY_A << 8) | DISPLAY_B)
#define UBOOT_STARTING_DEFAULT_OS           ((DISPLAY_A << 8) | DISPLAY_E)
#define UBOOT_OS_STARTUP_FAILED             ((DISPLAY_A << 8) | DISPLAY_F)

#define UBOOT_NO_BOOT_IMAGE_FOUND           ((DISPLAY_2 << 8) | DISPLAY_2)
#define UBOOT_DEFAULT_BOOT_DEVICE_NOT_FOUND ((DISPLAY_2 << 8) | DISPLAY_3)
#define UBOOT_UNSUPPORTED_DIMMS             ((DISPLAY_4 << 8) | DISPLAY_0)
#define UBOOT_MEM_TEST_FAILED               ((DISPLAY_4 << 8) | DISPLAY_2)
#define UBOOT_I2C_STUCK_BUS                 ((DISPLAY_2 << 8) | DISPLAY_A)


/***  TYPE DEFINITIONS  ***/


/***  GLOBALS  ***/ 

void boardDisplayCode (int value);

#endif        /* End of __INCboardDisplayCode */
