/*******************************************************************************

NAME            lsisas3108_common.h
VERSION         %version: 6 %
UPDATE DATE     %date_modified: Thu Oct 17 11:31:03 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright(C) 2010-2011 LSI Corporation.		                   *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  lsisas3108_common.h
 *          Title:  Common header file for LSISAS3108 board
 *     Programmer:  Anup Behare
 *     Modified  :  Naresh Madhusudana
 *  Creation Date:  Apr 10, 2010
 *
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  04/10/10  AB   Initial version.
 *  05/17/10  AB   Review comments Implemented.
 *  06/14/10  AB   Modified for DDR3 integration.
 *  06/16/10  AB   Modified for miniDMA integration.
 *  16/08/11  NM   Porting this to LSISAS3108
 *
 * Description
 * ------------
 *
 *    File containing the common definitions for LSISAS3108 board
 *
 *--------------------------------------------------------------------------
 */

#ifndef _LSISAS3108_COMMON_H
#define _LSISAS3108_COMMON_H

#include <asm/processor.h>
#include <common.h>

/* Below Changes are for using original blade.h file received from LSI */
typedef u32 U32;
typedef u8 U8;
typedef u16 U16;

typedef struct _S64
{
    u32 Low;
    s32 High;
} S64;

typedef struct _U64
{
    u32 Low;
    u32 High;
} U64;

/* Below Changes are for using original minidma.h file received from LSI */
#define LSISAS3108 1
#define INLINE static __inline__

/* DDR3 Min Size */
#define DDR_MINIMUM_SIZE	256
#define SPD_SIZE    256
#define TLB_PAGE_SIZE 256

/** 
 ** The following 2 sets of DDR3 constants must remain in sync with those
 ** of the ddr3Init.h header file in ubootLib.
 **/
#define DDR3_PHY_ADDR_ALIAS_BASE                0x400000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_NON_CACHED     0x404000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_1GB            0x47bfff000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_1GB_NON_CACHED 0x47c000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_2GB            0x4fbfff000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_2GB_NON_CACHED 0x4fc000000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_4GB            0x5fbfff000ULL
#define DDR3_PHY_ADDR_ALIAS_BASE_4GB_NON_CACHED 0x5fc000000ULL

#define DDR3_VIRT_ADDR_1ST_RANK_CACHED            0x00000000
#define DDR3_VIRT_ADDR_1ST_RANK_NON_CACHED        0x04000000
#define DDR3_VIRT_ADDR_1GB_2ND_RANK_CACHED        0x20000000
#define DDR3_VIRT_ADDR_1GB_2ND_RANK_NON_CACHED    0x30000000
#define DDR3_VIRT_ADDR_2GB_2ND_RANK_CACHED        0x40000000
#define DDR3_VIRT_ADDR_2GB_2ND_RANK_NON_CACHED    0x50000000
#define DDR3_VIRT_ADDR_4GB_2ND_RANK_CACHED        0x60000000
#define DDR3_VIRT_ADDR_4GB_2ND_RANK_NON_CACHED    0x70000000

#define DDR_SIZE_16MB                    (16*1024*1024)
#define DDR_SIZE_256MB                   (256*1024*1024)
#define DDR_SIZE_512MB                   (512*1024*1024)
#define DDR_SIZE_1GB                     (1024*1024*1024)

#define L3_MEM0_REGION_REG          0x30
#define L3_MEM1_REGION_REG          0x34
#define L3_MEM2_REGION_REG          0x38
#define L3_MEM3_REGION_REG          0x3c

#define L3_MEM_REGION_INVALID       0x00000000
#define L3_MEM_REGION_SIZE_64MB     0x00000001
#define L3_MEM_REGION_SIZE_128MB    0x00000002
#define L3_MEM_REGION_SIZE_256MB    0x00000003
#define L3_MEM_REGION_SIZE_512MB    0x00000004
#define L3_MEM_REGION_SIZE_1GB      0x00000005
#define L3_MEM_REGION_SIZE_2GB      0x00000006
#define L3_MEM_REGION_SIZE_4GB      0x00000007
#define L3_MEM_REGION_SIZE_8GB      0x00000008
#define L3_MEM_REGION_SIZE_16GB     0x00000009
#define L3_MEM_REGION_SIZE_32GB     0x0000000A
#define L3_MEM_REGION_SIZE_64GB     0x0000000B
#define L3_MEM_REGION_SIZE_128GB    0x0000000C

#define L3_MEM_REGION_STARTADDRESS_DDR3_LOWER_PLB    0x00000000
#define L3_MEM_REGION_STARTADDRESS_DDR3_UPPER_PLB    0x400000000ULL

#define L3_INDEX_REG                0x11000
#define L3_DATA_REG                 0x11004


#define PPC476_TBLR_SPR  0x10C
#define PPC476_GET_TICKS()      mfspr(PPC476_TBLR_SPR)
#define PPC476_CHECK_TICKS(NextStamp) \
       ( ~(PPC476_GET_TICKS() - (NextStamp)) >> 31 )

/* Macro definition for Root Complex library */
#define RC_LIB                        (5)

/* Root Complex functions */
#define RC_IS_ENABLED                 (1)
#define RC_IS_LINK_UP                 (2)
#define RC_PERFORM_LINK_WIDTH_TRNG    (3)
#define RC_PERFORM_LINK_SPEED_TRNG    (4)
#define RC_CONFIGURE_POM              (5)
#define RC_CONFIGURE_PIM              (6)
#define RC_ENABLE_MSIX                (7)
#define RC_CONFIGURE_MSIX             (8)
#define RC_FIND_CAPABILITY            (9)
#define RC_CFG_READ_DWORD             (10)
#define RC_CFG_READ_WORD              (11)
#define RC_CFG_READ_BYTE              (12)
#define RC_CFG_WRITE_DWORD            (13)
#define RC_CFG_WRITE_WORD             (14)
#define RC_CFG_WRITE_BYTE             (15)
#define RC_MEM_READ_DWORD             (16)
#define RC_MEM_WRITE_DWORD            (17)
#define RC_READ_BACKDOOR              (18)
#define RC_WRITE_BACKDOOR             (19)
#define RC_GET_LINK_SPEED             (20)
#define RC_GET_LINK_WIDTH             (21)
#define RC_GET_PHY_ADDR               (22)
#define RC_GET_VIRT_ADDR              (23)
#define RC_PCI_BOARD_INIT             (24)

#define RC_NUM_FUNCTIONS              (24)

#endif
