/*******************************************************************************

NAME            vxBootString.h
SUMMARY         Definition of Boot String passed to the OS
VERSION         %version: 23 %
UPDATE DATE     %date_modified: Wed Jan 22 10:06:59 2014 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013-2014 NetApp, Inc. All rights reserved.

DESCRIPTION:
    Definition of the boot string structure passed to the OS.

*******************************************************************************/


#ifndef __INCvxBootString
#define __INCvxBootString

/* Cannot increment UBOOT_LIB_BIN_VERSION w/o incrementing UBOOT_BIN_VERSION
 * because both values are stored in uboot.bin. */
#define UBOOT_BIN_VERSION       37
#define UBOOT_LIB_BIN_VERSION   14

/* Once values are added here, they MUST NOT CHANGE in order to prevent
 * compatibility and upgrade problems between bootloader and OS. */
typedef enum {
    UBOOT_NORMAL = 0,
    UBOOT_NOR_FLASH = 1,
    UBOOT_EXTERNAL_USB = 2,
    UBOOT_MSATA_RECOVERY = 3,
    UBOOT_TFTP = 4,
    UBOOT_DFW_NOR_FLASH = 5,
    UBOOT_DFW_TFTP = 6,
    UBOOT_DFW_USB = 7,
    UBOOT_DFW_MSATA = 8,
    UBOOT_UNKNOWN = 0xFF /* Catch-all for when manual boot commands are run */
} VX_BOOTTYPE;


/* Size and order of these fields MUST NOT CHANGE in order to 
 * prevent compatibility and upgrade problems (newer BIOS
 * should maintain boot String compatibility for "older"
 * Operating Systems that may use it).  Make sure this stays
 * "packed" well so the compiler doesn't add buffer space
 * to align any members.
 */
typedef struct {
    unsigned long DimmSizeMB;       // Offset 0x00, Size 4B
    unsigned long bootStrLen;       // Offset 0x04, Size 4B      
    unsigned long uboot_ver;        // Offset 0x08, Size 4B; CM version of uboot.bin
    unsigned long ubootLib_ver;     // Offset 0x0C, Size 4B; CM version of ubootLib.bin
    unsigned long bootType;         // Offset 0x10, Size 4B;
} VX_BOOTSTRING;


#endif        /* End of __INCvxBootString */
