/*******************************************************************************

NAME            pci.c
VERSION         %version: 4 %
UPDATE DATE     %date_modified: Thu Oct 17 11:33:08 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pci.c
 *          Title:  PCIe root complex initialization for LSI SAS3108
 *     Programmer:  Sandeep Darisala
 *  Creation Date:  March 13, 2012
 *
 * Description
 * ------------
 *
 *    This file is a sample application that uses the PCIe root complex
 *    library to initialize and operate the LSI SAS3108 chip in root complex
 *    mode.
 *-------------------------------------------------------------------------
 */

#include <common.h>
#include <asm/mmu.h>
#include <pci.h>
#include "basetype.h"
#include "rootComplexApi.h"
#include "lsisas3108_pci.h"

static struct pci_controller ppc476_hose = { 0 };

#define LSISAS3108_PCIE_OP(rw, size, type) \
static int pcie##rw##Config##size(struct pci_controller *hose, pci_dev_t dev, int where, type value) \
{ \
    U8 b, d, f; \
    b = PCI_BUS(dev); \
    d = PCI_DEV(dev); \
    f = PCI_FUNC(dev); \
    return (pcieRciCfg##rw##size(b, d, f, where, value)); \
}

LSISAS3108_PCIE_OP(Read, Byte, U8 *);
LSISAS3108_PCIE_OP(Read, Word, U16 *);
LSISAS3108_PCIE_OP(Read, Dword, U32 *);
LSISAS3108_PCIE_OP(Write, Byte, U8);
LSISAS3108_PCIE_OP(Write, Word, U16);
LSISAS3108_PCIE_OP(Write, Dword, U32);

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pci_init_board
* SUMMARY:  U-Boot entry point for board specific PCI initialization.
* SCOPE:
*
*
* DESCRIPTION: This function is called by u-boot to perform board specific
*              PCI initialization. This function calls several other routines
*              to setup and initialize lsisas3108 chip for root complex
*              operation.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:    In the lsisas3108 configuration file, CONFIG_PCI
*                   must be defined for u-boot to include PCI support.
----------------------------------------------------------------------------*/
void pci_init_board(void)
{
    int status;
    unsigned long long int physAddr;
    unsigned int virtAddr;
    struct pci_controller *hose;

    pcieRciIsRootComplexEnabled();
    printf("Starting PCIe init for LSISAS3108 Root Complex mode\n");

    pcieRciGetPhyAddr(&physAddr);
    pcieRciGetVirtAddr(&virtAddr);
    
    program_tlb(physAddr, virtAddr, 1024*1024, TLB_WORD2_I_ENABLE|TLB_WORD2_E_ENABLE);

    status = pcieRciInitBoard();
    if (status <=0)
        return;

    /*
     * Register to u-boot
     */
    hose = &ppc476_hose;

    hose->first_busno = 0;
    hose->last_busno = status;

    pci_set_region(hose->regions + 0,
               PCIE_NON_PREFETCHABLE_MEMORY_BASE_ADDR_LOW,
               PCIE_NON_PREFETCHABLE_MEMORY_BASE_ADDR_LOW,
               PCIE_NON_PREFETCHABLE_MEMORY_LIMIT - PCIE_NON_PREFETCHABLE_MEMORY_BASE_ADDR_LOW + 1,
               PCI_REGION_MEM);

    pci_set_region(hose->regions + 1,
               PCIE_IO_BASE_ADDR_LOW,
               PCIE_IO_BASE_ADDR_LOW,
               PCIE_IO_LIMIT_LOW - PCIE_IO_BASE_ADDR_LOW + 1,
               PCI_REGION_IO);

    hose->region_count = 2;

    pci_set_ops(hose,
            pcieReadConfigByte,
            pcieReadConfigWord,
            pcieReadConfigDword,
            pcieWriteConfigByte,
            pcieWriteConfigWord,
            pcieWriteConfigDword);

    pci_register_hose(hose);

    return;
}


