/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  commonInterface.h
 *          Title:  Engine Driver - Common Interface Header File.
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  July 21, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  07/21/11  NKB   Initial version - OSS Compliance.
 *  08/18/11  NKB   Modified as per the review comments.
 *  09/23/11  NKB   Consolidated and made common for following interfaces:
 *                  1. RAID Accelerator Interface
 *                  2. PCI-e API  / IOC Interface
 *                  3. PL Library / IOP Interface
 *                  4. Ethernet         Interface
 *  10/13/11  NKB   Macro added "CHECK_ENG_DRV_PTR"
 *
 *
 * Description
 * ------------
 * This file contains the definitions used for the engine driver as
 * an interface.
 *
 * FUNCTION MACRO Details:
 * ================================================================
 *        NAME              No of Arguments     Return Type
 * ================================================================
 * 1.  FUNC_1ARG_VST               1               void*
 * 2.  FUNC_1ARG_VOID              1               void
 * 3.  FUNC_1ARG_U32               1               U32
 * 4.  FUNC_1ARG_ISTATUS           1               ISTATUS/ U32
 * 5.  FUNC_2ARG_VOID              2               void
 * 6.  FUNC_2ARG_VST               2               void*
 * 7.  FUNC_2ARG_ISTATUS           2               ISTATUS/ U32
 * 8.  FUNC_3ARG_VOID              3               void
 * 9.  FUNC_3ARG_ISTATUS           3               ISTATUS/ U32
 * 10. FUNC_3ARG_VOID              3               void
 * 11. FUNC_3ARG_VOIDST            3               void*
 * 12. FUNC_4ARG_ISTATUS           4               ISTATUS/ U32
 * 13. FUNC_4ARG_VOID              4               void
 * 14. FUNC_5ARG_ISTATUS           5               ISTATUS/ U32
 * 15. FUNC_5ARG_VOID              5               void
 * 16. FUNC_5ARG_S32               5               S32
 * 17. FUNC_6ARG_VOID              6               void
 * 18. FUNC_7ARG_ISTATUS           7               ISTATUS/ U32
 * 19. FUNC_8ARG_ISTATUS           8               ISTATUS/ U32
 * 20. FUNC_12ARG_ISTATUS         12               ISTATUS/ U32
 * 21. FUNC_0ARG_VOID             None             void
 * 22. CHECK_ENG_DRV_PTR          None             0:Pass / 1: Fail
 * 23. FUNC_13ARG_ISTATUS         13               ISTATUS
 *
 *-------------------------------------------------------------------------
 */

#ifndef COMMON_INTERFACE_H
#define COMMON_INTERFACE_H

#include "basetype.h"

/****************** MACRO : Engine Driver Pointer Check ******************/
/* This Macro checks Engine Driver Pointer. Pointer should not be NULL */
#define CHECK_ENG_DRV_PTR ((gEngineDriver.PtrEngDrv == NULL)?0:1)

/******************** MACRO : 01 ARGUMENT ********************/
#define FUNC_1ARG_VST(fname, strtypcast, rettype, arg1type, arg1var) \
  rettype fname ( arg1type arg1var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                    fname(arg1var)); \
        } \
        else \
        { \
            return NULL; \
        } \
    }

/******************** MACRO : 01 ARGUMENT ********************/
#define FUNC_1ARG_VOID(fname, strtypcast, rettype, arg1type, arg1var) \
  rettype fname ( arg1type arg1var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))->fname(arg1var); \
        } \
    }

/******************** MACRO : 01 ARGUMENT ********************/
#define FUNC_1ARG_U32(fname, strtypcast, rettype, arg1type, arg1var) \
  rettype fname ( arg1type arg1var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                    fname(arg1var)); \
        } \
        else \
        { \
            return 0; \
        } \
    }

/******************** MACRO : 02 ARGUMENT ********************/
#define FUNC_2ARG_U32(fname, strtypcast, rettype, arg1type, arg1var, arg2type, arg2var) \
  rettype fname ( arg1type arg1var, arg2type arg2var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                    fname(arg1var, arg2var)); \
        } \
        else \
        { \
            return 0; \
        } \
    }


/******************** MACRO : NO ARGUMENT ********************/
#define FUNC_0ARG_U32(fname, strtypcast, rettype) \
  rettype fname (void) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))->fname()); \
        } \
        else \
        { \
            if (!CHECK_ENG_DRV_PTR) \
            { \
                PRINTK("Symbol not registered."); \
            } \
            return 0; \
        } \
    }


/******************** MACRO : 02 ARGUMENTS ********************/
#define FUNC_2ARG_VOID(fname, strtypcast, rettype, arg1type, arg1var, \
                       arg2type, arg2var) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                    fname(arg1var, arg2var); \
        } \
    }

/******************** MACRO : 02 ARGUMENTS ********************/
#define FUNC_2ARG_VST(fname, strtypcast, rettype, arg1type, arg1var, \
                      arg2type, arg2var) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
           (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                    fname(arg1var, arg2var)); \
        } \
        else \
        { \
            return NULL; \
        } \
    }

/******************** MACRO : 03 ARGUMENTS ********************/
#define FUNC_3ARG_VOID(fname, strtypcast, rettype, arg1type, arg1var, \
                       arg2type, arg2var, arg3type, arg3var) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                    fname(arg1var, arg2var, arg3var); \
        } \
    }

/******************** MACRO : 01 ARGUMENTS ********************/
#define FUNC_1ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var) \
  rettype fname ( arg1type arg1var )                     \
    {           \
        if ( CHECK_ENG_DRV_PTR && \
           (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                    fname(arg1var)); \
        } \
        else  \
        {  \
            return ISTATUS_FAILURE;   \
        }  \
    }

/******************** MACRO : 02 ARGUMENTS ********************/
#define FUNC_2ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var)); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }

/******************** MACRO : 02 ARGUMENTS ********************/
#define FUNC_2ARG_VOIDST(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var)); \
        } \
        else \
        { \
            return NULL; \
        } \
    }


/******************** MACRO : 03 ARGUMENTS ********************/
#define FUNC_3ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
           (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var)); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }

/******************** MACRO : 03 ARGUMENTS ********************/
#define FUNC_3ARG_VOID(fname, strtypcast, rettype, arg1type, arg1var, \
                       arg2type, arg2var, arg3type, arg3var) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
           (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var); \
        } \
    }

/******************** MACRO : 03 ARGUMENTS ********************/
#define FUNC_3ARG_VOIDST(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var)); \
        } \
        else \
        { \
            return NULL; \
        } \
    }

/******************** MACRO : 04 ARGUMENTS ********************/
#define FUNC_4ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var, \
                  arg4type, arg4var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var)); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }

/******************** MACRO : 04 ARGUMENTS ********************/
#define FUNC_4ARG_VOID(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var, \
                  arg4type, arg4var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var); \
        } \
    }

/******************** MACRO : 05 ARGUMENTS ********************/
#define FUNC_5ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var, \
                  arg4type, arg4var, arg5type, arg5var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var, arg5type arg5var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, arg5var)); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }

/******************** MACRO : 05 ARGUMENTS ********************/
#define FUNC_5ARG_VOID(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var, \
                  arg4type, arg4var, arg5type, arg5var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var, arg5type arg5var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, arg5var); \
        } \
    }

/******************** MACRO : 06 ARGUMENTS ********************/
#define FUNC_6ARG_VOID(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var, \
                  arg4type, arg4var, arg5type, arg5var, \
                  arg6type, arg6var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var, arg5type arg5var, arg6type arg6var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
           (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, \
                      arg5var, arg6var); \
        } \
    }

/******************** MACRO : 07 ARGUMENTS ********************/
#define FUNC_7ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var, \
                  arg4type, arg4var, arg5type, arg5var, \
                  arg6type, arg6var, arg7type, arg7var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var, arg5type arg5var, arg6type arg6var, \
      arg7type arg7var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
           (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, \
                      arg5var, arg6var, arg7var)); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }

/******************** MACRO : 08 ARGUMENTS ********************/
#define FUNC_8ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                  arg2type, arg2var, arg3type, arg3var, \
                  arg4type, arg4var, arg5type, arg5var, \
                  arg6type, arg6var, arg7type, arg7var, \
                  arg8type, arg8var ) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var, arg5type arg5var, arg6type arg6var, \
      arg7type arg7var, arg8type arg8var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, \
                      arg5var, arg6var, arg7var, arg8var)); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }

/******************** MACRO : 12 ARGUMENTS ********************/
#define FUNC_12ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                   arg2type, arg2var, arg3type, arg3var, \
                   arg4type, arg4var, arg5type, arg5var, \
                   arg6type, arg6var, arg7type, arg7var, \
                   arg8type, arg8var, arg9type, arg9var, \
                   arg10type, arg10var, arg11type, arg11var, \
                   arg12type, arg12var ) \
    rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var, arg5type arg5var, arg6type arg6var, \
      arg7type arg7var, arg8type arg8var, arg9type arg9var, \
      arg10type arg10var, arg11type arg11var, arg12type arg12var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, arg5var, \
                      arg6var, arg7var, arg8var, arg9var, arg10var, \
                      arg11var, arg12var)); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }
/******************** MACRO : 13 ARGUMENTS ********************/
#define FUNC_13ARG_ISTATUS(fname, strtypcast, rettype, arg1type, arg1var, \
                   arg2type, arg2var, arg3type, arg3var, \
                   arg4type, arg4var, arg5type, arg5var, \
                   arg6type, arg6var, arg7type, arg7var, \
                   arg8type, arg8var, arg9type, arg9var, \
                   arg10type, arg10var, arg11type, arg11var, \
                   arg12type, arg12var, arg13type, arg13var ) \
    rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, \
      arg4type arg4var, arg5type arg5var, arg6type arg6var, \
      arg7type arg7var, arg8type arg8var, arg9type arg9var, \
      arg10type arg10var, arg11type arg11var, arg12type arg12var, \
      arg13type arg13var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, arg5var, \
                      arg6var, arg7var, arg8var, arg9var, arg10var, \
                      arg11var, arg12var, arg13var )); \
        } \
        else \
        { \
            return ISTATUS_FAILURE; \
        } \
    }

/******************** MACRO : NO ARGUMENT ********************/
#define FUNC_0ARG_U16(fname, strtypcast, rettype) \
  rettype fname (void) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))->fname()); \
        } \
        else \
        { \
            return 0; \
        } \
    }

/******************** MACRO : NO ARGUMENT ********************/
#define FUNC_0ARG_VOID(fname, strtypcast, rettype) \
  rettype fname (void) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            ((strtypcast)(gEngineDriver.PtrEngDrv))->fname(); \
        } \
    }

/******************** MACRO : 05 ARGUMENTS ********************/
#define FUNC_5ARG_S32(fname, strtypcast, rettype, arg1type, arg1var, \
                      arg2type, arg2var, arg3type, arg3var, arg4type, \
                      arg4var, arg5type, arg5var) \
  rettype fname \
    ( arg1type arg1var, arg2type arg2var, arg3type arg3var, arg4type arg4var, \
      arg5type arg5var ) \
    { \
        if ( CHECK_ENG_DRV_PTR && \
            (((strtypcast)(gEngineDriver.PtrEngDrv))->fname != NULL) ) \
        { \
            return (((strtypcast)(gEngineDriver.PtrEngDrv))-> \
                fname(arg1var, arg2var, arg3var, arg4var, arg5var)); \
        } \
        else \
        { \
            return 0; \
        } \
    }

/* Common interface symbols to be registered. */
typedef struct _ENGINE_DRIVER
{
    U32  MajorNumber;
    PU8  Name;
    void *PtrEngDrv; /* For access of Engine Driver Functions */

} ENGINE_DRIVER, *PTR_ENGINE_DRIVER;

extern ENGINE_DRIVER gEngineDriver;

#endif /* COMMON_INTERFACE_H */
