/***************************************************************************
 *                                                                         *
 *                                                                         *
 *  Copyright 2006-2010 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  Racc.h
 *          Title:  Definitions for DMA Engine and RAID Accelerator KATANA
 *  Creation Date:  May 12, 2010
 *
 *
 * Description
 * ------------
 *
 *    This software module will prepare the KATANA RAID Accelerator for
 *    normal operation.
 *
 *-------------------------------------------------------------------------
 */

#ifndef __RACC_H
#define __RACC_H


#if  !defined(__LANGUAGE_ASM__)
typedef struct _COMPLETION_QUEUE_ENTRY
{
  #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
          U32 LowerCP                        :27;     /* bits  5:31  */
          U32 ValidEntry                     : 1;     /* bit   4 */
          U32 NoInterrupt                    : 1;     /* bit   3 */
          U32 LinkedList                     : 1;     /* bit   2 */
          U32 Status                         : 2;     /* bits  0:1 */

          U32 UpperCP;                                /* bits 32:63 */
  #else     /* __LITTLE_ENDIAN__ */
          U32 Status                         : 2;     /* bits  0:1 */
          U32 LinkedList                     : 1;     /* bit   2 */
          U32 NoInterrupt                    : 1;     /* bit   3 */
          U32 ValidEntry                     : 1;     /* bit   4 */
          U32 LowerCP                        :27;     /* bits  5:31  */

          U32 UpperCP;                                /* bits 32:63 */
  #endif
} COMPLETION_QUEUE_ENTRY, *PTR_COMPLETION_QUEUE_ENTRY;

#define COMPLETION_QUEUE_ENTRY_LOWER_CP_SHIFT           (5)
/* Completion Queue Status */
#define RACC_CQ_STATUS_NORMAL            (0)
#define RACC_CQ_STATUS_SOURCE_ZERO       (1)
#define RACC_CQ_STATUS_MISCOMPARE        (2)


/* Raid Accelerator PLB Address Map */
#define RACC_TOP_PLB_CONFIG_OFFSET               (0x00000000)
#define RACC_DMA_ENGINE0_PLB_CONFIG_OFFSET       (0x00000400)
#define RACC_DMA_ENGINE1_PLB_CONFIG_OFFSET       (0x00000800)

#define RACC_DMA_ARB_MODE_1_TO_1       (0)     // 1 from A, 1 from B
#define RACC_DMA_ARB_MODE_PRIORITY     (1)     // Keep fetch from A until it runs out, then fetch from B.
#define RACC_DMA_ARB_MODE_2_TO_1       (2)     // 2 from A, 1 from B

#define RACC_COMPLETION_Q_SIZE_32      (0)
#define RACC_COMPLETION_Q_SIZE_128     (1)
#define RACC_COMPLETION_Q_SIZE_1024    (2)
#define RACC_COMPLETION_Q_SIZE_2048    (3)

#if  !defined(__LANGUAGE_ASM__)
/* Raid Accelerator Interrupt Status */
typedef struct _RACC_PLB_INT_STATUS_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_16_31            :12; /* 16:31 */
            U32    DMA1Idle                  :1;  /* 19 */
            U32    DMA0Idle                  :1;  /* 18 */
            U32    DMA1CompletionQNotEmpty   :1;  /* 17 */
            U32    DMA0CompletionQNotEmpty   :1;  /* 16 */
            U32    ConfigAccessErr           :1;  /* 15 */
            U32    Reserved_14               :1;  /* 14 */
            U32    PLBSlave0Err              :1;  /* 13 */
            U32    Reserved_10_12            :3;  /* 10:12 */
            U32    DMA1CompIntStatusDPF      :1;  /* 9 */
            U32    DMA0CompIntStatusDPF      :1;  /* 8 */
            U32    DMA1ErrStat               :1;  /* 7 */
            U32    DMA0ErrStat               :1;  /* 6 */
            U32    Reserved_5                :1;  /* 5 */
            U32    DMA1CompletionQFull       :1;  /* 4 */
            U32    DMA0CompletionQFull       :1;  /* 3 */
            U32    Reserved_0_2              :3;  /* 0:2 */
#else     /* __LITTLE_ENDIAN__ */
            U32    Reserved_0_2              :3;  /* 0:2 */
            U32    DMA0CompletionQFull       :1;  /* 3 */
            U32    DMA1CompletionQFull       :1;  /* 4 */
            U32    Reserved_5                :1;  /* 5 */
            U32    DMA0ErrStat               :1;  /* 6 */
            U32    DMA1ErrStat               :1;  /* 7 */
            U32    DMA0CompIntStatusDPF      :1;  /* 8 */
            U32    DMA1CompIntStatusDPF      :1;  /* 9 */
            U32    Reserved_10_12            :3;  /* 10:12 */
            U32    PLBSlave0Err              :1;  /* 13 */
            U32    Reserved_14               :1;  /* 14 */
            U32    ConfigAccessErr           :1;  /* 15 */
            U32    DMA0CompletionQNotEmpty   :1;  /* 16 */
            U32    DMA1CompletionQNotEmpty   :1;  /* 17 */
            U32    DMA0Idle                  :1;  /* 18 */
            U32    DMA1Idle                  :1;  /* 19 */
            U32    Reserved_16_31            :12; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_PLB_INT_STATUS_REG, * PTR_RACC_PLB_INT_STATUS_REG;

/* Raid Accelerator Interrupt Mask */
typedef struct _RACC_PLB_INT_MASK_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_16_31            :16; /* 16:31 */
            U32    ConfigAccessErrMask       :1;  /* 15 */
            U32    Reserved_14               :1;  /* 14 */
            U32    PLBSlave0ErrMask          :1;  /* 13 */
            U32    Reserved_10_12            :3;  /* 10:12 */
            U32    DMA1CompIntMaskDPF        :1;  /* 9 */
            U32    DMA0CompIntMaskDPF        :1;  /* 8 */
            U32    DMA1ErrStatMask           :1;  /* 7 */
            U32    DMA0ErrStatMask           :1;  /* 6 */
            U32    Reserved_0_5              :6;  /* 0:5 */
#else     /* __LITTLE_ENDIAN__ */
            U32    Reserved_0_5              :6;  /* 0:5 */
            U32    DMA0ErrStatMask           :1;  /* 6 */
            U32    DMA1ErrStatMask           :1;  /* 7 */
            U32    DMA0CompIntMaskDPF        :1;  /* 8 */
            U32    DMA1CompIntMaskDPF        :1;  /* 9 */
            U32    Reserved_10_12            :3;  /* 10:12 */
            U32    PLBSlave0ErrMask          :1;  /* 13 */
            U32    Reserved_14               :1;  /* 14 */
            U32    ConfigAccessErrMask       :1;  /* 15 */
            U32    Reserved_16_31            :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_PLB_INT_MASK_REG, * PTR_RACC_PLB_INT_MASK_REG;

/* Raid Accelerator Control */
typedef struct _RACC_PLB_CONTROL_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_18_31            :14; /* 18:31 */
            U32    KsmfifoTgtSoftrst         :1;  /* 17 */
            U32    KsmfifoMstSoftrst         :1;  /* 16 */
            U32    Reserved_13_15            :3;  /* 13:15 */
            U32    ParFlipLoc                :4;  /* 9:12 */
            U32    GenBadRdPar               :1;  /* 8 */
            U32    Reserved_1_7              :7;  /* 1:7 */
            U32    PolynomialSel             :1;  /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    PolynomialSel             :1;  /* 0 */
            U32    Reserved_1_7              :7;  /* 1:7 */
            U32    GenBadRdPar               :1;  /* 8 */
            U32    ParFlipLoc                :4;  /* 9:12 */
            U32    Reserved_13_15            :3;  /* 13:15 */
            U32    KsmfifoMstSoftrst         :1;  /* 16 */
            U32    KsmfifoTgtSoftrst         :1;  /* 17 */
            U32    Reserved_18_31            :14; /* 18:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_PLB_CONTROL_REG, * PTR_RACC_PLB_CONTROL_REG;

/* Raid Accelerator Configuration AI Error Log */
typedef struct _RACC_PLB_CONFIG_AI_ERROR_LOG_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_4_31             :28; /* 4:31 */
            U32    AISlaveRWErrLog           :1;  /* 3 */
            U32    AISlaveErrLog             :3;  /* 0:2 */
#else     /* __LITTLE_ENDIAN__ */
            U32    AISlaveErrLog             :3;  /* 0:2 */
            U32    AISlaveRWErrLog           :1;  /* 3 */
            U32    Reserved_4_31             :28; /* 4:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_PLB_CONFIG_AI_ERROR_LOG_REG, * PTR_RACC_PLB_CONFIG_AI_ERROR_LOG_REG;


typedef volatile struct _RACC_TOP_PLB_CONFIG
{
    RACC_PLB_INT_STATUS_REG          IntStatus;             /* 0x00 */
    RACC_PLB_INT_MASK_REG            IntMask;               /* 0x04 */
    U32                              Scratch1;              /* 0x08 */
    RACC_PLB_CONTROL_REG             Control;               /* 0x0C */
    U32                              Scratch2;              /* 0x10 */
    U32                              ConfigAIAddrErrLog;    /* 0x14 */
    RACC_PLB_CONFIG_AI_ERROR_LOG_REG ConfigAIErrLog;        /* 0x18 */
} RACC_TOP_PLB_CONFIG, *PTR_RACC_TOP_PLB_CONFIG;
#endif


/* Raid Accelerator Top PLB Config Offsets */
#define RACC_PLB_INT_STAT_OFFSET                 (0X00000000)
#define RACC_PLB_INT_MASK_OFFSET                 (0X00000004)
#define RACC_PLB_SCRATCHPAD1_OFFSET              (0X00000008)
#define RACC_PLB_CONTROL_OFFSET                  (0X0000000C)
#define RACC_PLB_SCRATCHPAD2_OFFSET              (0X00000010)
#define RACC_PLB_CONFIG_AI_ADDR_ERR_LOG_OFFSET   (0X00000014)
#define RACC_PLB_CONFIG_AI_ERR_LOG_OFFSET        (0X00000018)


#if  !defined(__LANGUAGE_ASM__)
/* Register for DMAn CDB Root Pointer A nd B */
typedef struct _RACC_DMA_CDB_ROOT_POINTER_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    CDBRootPointer            :27; /* 5:31 */
            U32    Reserved_1_4              :4;  /* 1:4 */
            U32    NewCDBRPFlag              :1;  /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    NewCDBRPFlag              :1;  /* 0 */
            U32    Reserved_1_4              :4;  /* 1:4 */
            U32    CDBRootPointer            :27; /* 5:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_CDB_ROOT_POINTER_REG, * PTR_RACC_DMA_CDB_ROOT_POINTER_REG;

/* DMAn Linked List Flags and Alternate CRC Reg */
typedef struct _RACC_DMA_LINKED_LIST_FLAGS_ALT_CRC_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    AlternateCRCSeed          :16; /* 16:31 */
            U32    Reserved_9_15             :7;  /* 9:15 */
            U32    NewCDBPntrFlagB           :1;  /* 8 */
            U32    Reserved_1_7              :7;  /* 1:7 */
            U32    NewCDBPntrFlagA           :1;  /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    NewCDBPntrFlagA           :1;  /* 0 */
            U32    Reserved_1_7              :7;  /* 1:7 */
            U32    NewCDBPntrFlagB           :1;  /* 8 */
            U32    Reserved_9_15             :7;  /* 9:15 */
            U32    AlternateCRCSeed          :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_LINKED_LIST_FLAGS_ALT_CRC_REG, * PTR_RACC_DMA_LINKED_LIST_FLAGS_ALT_CRC_REG;

/* DMAn Completion Queue Tail Pointer */
typedef struct _RACC_DMA_COMP_QUEUE_TAIL_PTR_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_12_31            :20; /* 12:31 */
            U32    CompQTailPtr              :12; /* 0:11 */
#else     /* __LITTLE_ENDIAN__ */
            U32    CompQTailPtr              :12; /* 0:11 */
            U32    Reserved_12_31            :20; /* 12:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_COMP_QUEUE_TAIL_PTR_REG, * PTR_RACC_DMA_COMP_QUEUE_TAIL_PTR_REG;

/* DMAn Completion Queue Interrupt Control */
typedef struct _RACC_DMA_COMPQ_INT_CTL_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_16_31            :16; /* 16:31 */
            U32    CompQIntTimer             :8;  /* 8:15 */
            U32    CompQIntCounter           :8;  /* 0:7 */
#else     /* __LITTLE_ENDIAN__ */
            U32    CompQIntCounter           :8;  /* 0:7 */
            U32    CompQIntTimer             :8;  /* 8:15 */
            U32    Reserved_16_31            :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_COMPQ_INT_CTL_REG, * PTR_RACC_DMA_COMPQ_INT_CTL_REG;

/* DMAn Engine Control and Status */
typedef struct _RACC_DMA_ENGINE_CTL_STAT_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_16_31            :16; /* 16:31 */
            U32    DestinationIPCBByteSwap   :1;  /* 15 */
            U32    SourceIPCBByteSwap        :1;  /* 14 */
            U32    ResetDPFFilter            :1;  /* 13 */
            U32    CplQNPWrEn                :1;  /* 12 */
            U32    DMAHaltedStat             :1;  /* 11 */
            U32    SGPrefetchEn              :1;  /* 10 */
            U32    DMAEnable                 :1;  /* 9 */
            U32    ParCheckEn                :1;  /* 8 */
            U32    CompQIntCoalescing        :1;  /* 7 */
            U32    CompQFullDetEn            :1;  /* 6 */
            U32    CompQSize                 :2;  /* 4:5 */
            U32    LinkedListArbMode         :2;  /* 2:3 */
            U32    InvertGuard               :1;  /* 1 */
            U32    DataReqThresh             :1;  /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    DataReqThresh             :1;  /* 0 */
            U32    InvertGuard               :1;  /* 1 */
            U32    LinkedListArbMode         :2;  /* 2:3 */
            U32    CompQSize                 :2;  /* 4:5 */
            U32    CompQFullDetEn            :1;  /* 6 */
            U32    CompQIntCoalescing        :1;  /* 7 */
            U32    ParCheckEn                :1;  /* 8 */
            U32    DMAEnable                 :1;  /* 9 */
            U32    SGPrefetchEn              :1;  /* 10 */
            U32    DMAHaltedStat             :1;  /* 11 */
            U32    CplQNPWrEn                :1;  /* 12 */
            U32    ResetDPFFilter            :1;  /* 13 */
            U32    SourceIPCBByteSwap        :1;  /* 14 */
            U32    DestinationIPCBByteSwap   :1;  /* 15 */
            U32    Reserved_16_31            :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_ENGINE_CTL_STAT_REG, * PTR_RACC_DMA_ENGINE_CTL_STAT_REG;

/* DMAn Halt Error Attributes */
typedef struct _RACC_DMA_HALT_ERR_ATTR_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    EEDPErrSourceNum          :8;  /* 24:31 */
            U32    Reserved_21_23            :3;  /* 21:23 */
            U32    ErrStatBitLog             :5;  /* 16:20 */
            U32    Reserved_15               :1;  /* 15 */
            U32    DPFAccessErrAgent         :4;  /* 11:14 */
            U32    EEDPRefTagMiscomp         :1;  /* 10 */
            U32    EEDPAppTagMiscomp         :1;  /* 9 */
            U32    EEDPGuardMiscomp          :1;  /* 8 */
            U32    IllegalCDBFailType        :7;  /* 1:7 */
            U32    ErrorType                 :1;  /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    ErrorType                 :1;  /* 0 */
            U32    IllegalCDBFailType        :7;  /* 1:7 */
            U32    EEDPGuardMiscomp          :1;  /* 8 */
            U32    EEDPAppTagMiscomp         :1;  /* 9 */
            U32    EEDPRefTagMiscomp         :1;  /* 10 */
            U32    DPFAccessErrAgent         :4;  /* 11:14 */
            U32    Reserved_15               :1;  /* 15 */
            U32    ErrStatBitLog             :5;  /* 16:20 */
            U32    Reserved_21_23            :3;  /* 21:23 */
            U32    EEDPErrSourceNum          :8;  /* 24:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_HALT_ERR_ATTR_REG, * PTR_RACC_DMA_HALT_ERR_ATTR_REG;

/* DMAn Halt Error Status */
typedef struct _RACC_DMA_HALT_ERR_STAT_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_29_31            :3;  /* 29:31 */
            U32    CheckPatternFail          :1;  /* 28 */
            U32    CheckLFSRFail             :1;  /* 27 */
            U32    XORZeroCheckFail          :1;  /* 26 */
            U32    DIFDrainRAMParErr         :1;  /* 25 */
            U32    DIFFillRAMParErr          :1;  /* 24 */
            U32    DIFDestSGFetchErr         :1;  /* 23 */
            U32    DIFSourceSGFetchErr       :1;  /* 22 */
            U32    DIFDrainCompErr           :1;  /* 21 */
            U32    DIFFillCompErr            :1;  /* 20 */
            U32    DPFAccessErr              :1;  /* 19 */
            U32    EEDPMiscompHalt           :1;  /* 18 */
            U32    SCFillCtxRAMParErr        :1;  /* 17 */
            U32    SGDrainCtxRAMParErr       :1;  /* 16 */
            U32    Reserved_15               :1;  /* 15 */
            U32    Modify3DatRAMParErr       :1;  /* 14 */
            U32    Modify2DatRAMParErr       :1;  /* 13 */
            U32    Modify1DatRAMParErr       :1;  /* 12 */
            U32    DatRAMParErr              :1;  /* 11 */
            U32    KeyRAMParErr              :1;  /* 10 */
            U32    ContentRAMParErr          :1;  /* 9 */
            U32    CompQCompErr              :1;  /* 8 */
            U32    DrainCompErr              :1;  /* 7 */
            U32    FillCompErr               :1;  /* 6 */
            U32    CDBEncryKeyFetchCompErr   :1;  /* 5 */
            U32    DestSGFetchErr            :1;  /* 4 */
            U32    SourceSGFetchErr          :1;  /* 3 */
            U32    NextCDBPntrFetchErr       :1;  /* 2 */
            U32    CDBFetchErr               :1;  /* 1 */
            U32    IllegalCDB                :1;  /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    IllegalCDB                :1;  /* 0 */
            U32    CDBFetchErr               :1;  /* 1 */
            U32    NextCDBPntrFetchErr       :1;  /* 2 */
            U32    SourceSGFetchErr          :1;  /* 3 */
            U32    DestSGFetchErr            :1;  /* 4 */
            U32    CDBEncryKeyFetchCompErr   :1;  /* 5 */
            U32    FillCompErr               :1;  /* 6 */
            U32    DrainCompErr              :1;  /* 7 */
            U32    CompQCompErr              :1;  /* 8 */
            U32    ContentRAMParErr          :1;  /* 9 */
            U32    KeyRAMParErr              :1;  /* 10 */
            U32    DatRAMParErr              :1;  /* 11 */
            U32    Modify1DatRAMParErr       :1;  /* 12 */
            U32    Modify2DatRAMParErr       :1;  /* 13 */
            U32    Modify3DatRAMParErr       :1;  /* 14 */
            U32    Reserved_15               :1;  /* 15 */
            U32    SGDrainCtxRAMParErr       :1;  /* 16 */
            U32    SCFillCtxRAMParErr        :1;  /* 17 */
            U32    EEDPMiscompHalt           :1;  /* 18 */
            U32    DPFAccessErr              :1;  /* 19 */
            U32    DIFFillCompErr            :1;  /* 20 */
            U32    DIFDrainCompErr           :1;  /* 21 */
            U32    DIFSourceSGFetchErr       :1;  /* 22 */
            U32    DIFDestSGFetchErr         :1;  /* 23 */
            U32    DIFFillRAMParErr          :1;  /* 24 */
            U32    DIFDrainRAMParErr         :1;  /* 25 */
            U32    XORZeroCheckFail          :1;  /* 26 */
            U32    CheckLFSRFail             :1;  /* 27 */
            U32    CheckPatternFail          :1;  /* 28 */
            U32    Reserved_29_31            :3;  /* 29:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_HALT_ERR_STAT_REG, * PTR_RACC_DMA_HALT_ERR_STAT_REG;

/* DMAn Halt Parity Error Generation */
typedef struct _RACC_DMA_HALT_PAR_ERR_GEN_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_18_31             :14; /* 18:31 */
            U32    GenDIFDrainPLBSHELLParErr  :1;  /* 17 */
            U32    GenDIFDrainDDRSHELLPerErr  :1;  /* 16 */
            U32    GenDIFDrainPCIESHELLParErr :1;  /* 15 */
            U32    GenDIFDrainBufParErr       :1;  /* 14 */
            U32    GenDrainPLBSHELLParErr     :1;  /* 13 */
            U32    GenDrainDDRSHELLPerErr     :1;  /* 12 */
            U32    GenDrainPCIESHELLParErr    :1;  /* 11 */
            U32    GenMod3BufParErr           :1;  /* 10 */
            U32    GenMod2BufParErr           :1;  /* 9 */
            U32    GenMod1BufParErr           :1;  /* 8 */
            U32    Reserved_7                 :1;  /* 7 */
            U32    GenSGDrainCtxRAMPar        :1;  /* 6 */
            U32    GenSGFillCtxRAMPar         :1;  /* 5 */
            U32    GenBadCompWrPar            :1;  /* 4 */
            U32    ParBitFlipLoc              :4;  /* 0:3 */
#else     /* __LITTLE_ENDIAN__ */
            U32    ParBitFlipLoc              :4;  /* 0:3 */
            U32    GenBadCompWrPar            :1;  /* 4 */
            U32    GenSGFillCtxRAMPar         :1;  /* 5 */
            U32    GenSGDrainCtxRAMPar        :1;  /* 6 */
            U32    Reserved_7                 :1;  /* 7 */
            U32    GenMod1BufParErr           :1;  /* 8 */
            U32    GenMod2BufParErr           :1;  /* 9 */
            U32    GenMod3BufParErr           :1;  /* 10 */
            U32    GenDrainPCIESHELLParErr    :1;  /* 11 */
            U32    GenDrainDDRSHELLPerErr     :1;  /* 12 */
            U32    GenDrainPLBSHELLParErr     :1;  /* 13 */
            U32    GenDIFDrainBufParErr       :1;  /* 14 */
            U32    GenDIFDrainPCIESHELLParErr :1;  /* 15 */
            U32    GenDIFDrainDDRSHELLPerErr  :1;  /* 16 */
            U32    GenDIFDrainPLBSHELLParErr  :1;  /* 17 */
            U32    Reserved_18_31             :14; /* 18:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_HALT_PAR_ERR_GEN_REG, * PTR_RACC_DMA_HALT_PAR_ERR_GEN_REG;

/* DMAn EEDP Halt Condition Error Expected/Calculated Application Tag */
typedef struct _RACC_DMA_EEDP_HALT_CON_ERR_TAG_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    ExpectedAppTag            :16; /* 16:31 */
            U32    CalculatedAppTag          :16; /* 0:15 */
#else     /* __LITTLE_ENDIAN__ */
            U32    CalculatedAppTag          :16; /* 0:15 */
            U32    ExpectedAppTag            :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_EEDP_HALT_CON_ERR_TAG_REG, * PTR_RACC_DMA_EEDP_HALT_CON_ERR_TAG_REG;

/* DMAn EEDP Halt Condition Error Expected/Calculated Application GUARD */
typedef struct _RACC_DMA_EEDP_HALT_CON_ERR_GUARD_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    ExpectedAppGuard          :16; /* 16:31 */
            U32    CalculatedAppGuard        :16; /* 0:15 */
#else     /* __LITTLE_ENDIAN__ */
            U32    CalculatedAppGuard        :16; /* 0:15 */
            U32    ExpectedAppGuard          :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_EEDP_HALT_CON_ERR_GUARD_REG, * PTR_RACC_DMA_EEDP_HALT_CON_ERR_GUARD_REG;

/* DMAn Completion Queue Head Pointer */
typedef struct _RACC_DMA_COMP_QUEUE_HEAD_PTR_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_12_31            :20; /* 12:31 */
            U32    CompQHeadPtr              :12; /* 0:11 */
#else     /* __LITTLE_ENDIAN__ */
            U32    CompQHeadPtr              :12; /* 0:11 */
            U32    Reserved_12_31            :20; /* 12:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_COMP_QUEUE_HEAD_PTR_REG, * PTR_RACC_DMA_COMP_QUEUE_HEAD_PTR_REG;

/* DMAn NonHalt Error Status and Attributes */
typedef struct _RACC_DMA_NON_HALT_ERR_STAT_ATTR_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    EEDPErrSourceNumber      :8;   /* 24:31 */
            U32    Reserved_7_23            :17;  /* 7:23 */
            U32    CheckPatternFail         :1;   /* 6 */
            U32    CheckLFSRFail            :1;   /* 5 */
            U32    XorZeroCheckFailed       :1;   /* 4 */
            U32    EEDPNonHaltRefTagMiscomp :1;   /* 3 */
            U32    EEDPNonHaltAppTagMiscomp :1;   /* 2 */
            U32    EEDPNonHaltGuardMiscomp  :1;   /* 1 */
            U32    NonHaltErrorDet          :1;   /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    NonHaltErrorDet          :1;   /* 0 */
            U32    EEDPNonHaltGuardMiscomp  :1;   /* 1 */
            U32    EEDPNonHaltAppTagMiscomp :1;   /* 2 */
            U32    EEDPNonHaltRefTagMiscomp :1;   /* 3 */
            U32    XorZeroCheckFailed       :1;   /* 4 */
            U32    CheckLFSRFail            :1;   /* 5 */
            U32    CheckPatternFail         :1;   /* 6 */
            U32    Reserved_7_23            :17;  /* 7:23 */
            U32    EEDPErrSourceNumber      :8;   /* 24:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_NON_HALT_ERR_STAT_ATTR_REG, * PTR_RACC_DMA_NON_HALT_ERR_STAT_ATTR_REG;

/* DMAn EEDP NonHalt Condition Error Expected/Calculated Application Tag */
typedef struct _RACC_DMA_EEDP_NONHALT_CON_ERR_TAG_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    ExpectedAppTag            :16; /* 16:31 */
            U32    CalculatedAppTag          :16; /* 0:15 */
#else     /* __LITTLE_ENDIAN__ */
            U32    CalculatedAppTag          :16; /* 0:15 */
            U32    ExpectedAppTag            :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_EEDP_NONHALT_CON_ERR_TAG_REG, * PTR_RACC_DMA_EEDP_NONHALT_CON_ERR_TAG_REG;

/* DMAn EEDP NonHalt Condition Error Expected/Calculated Application Guard */
typedef struct _RACC_DMA_EEDP_NONHALT_CON_ERR_GUARD_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    ExpectedAppGuard          :16; /* 16:31 */
            U32    CalculatedAppGuard        :16; /* 0:15 */
#else     /* __LITTLE_ENDIAN__ */
            U32    CalculatedAppGuard        :16; /* 0:15 */
            U32    ExpectedAppGuard          :16; /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_EEDP_NONHALT_CON_ERR_GUARD_REG, * PTR_RACC_DMA_EEDP_NONHALT_CON_ERR_GUARD_REG;

/* DMAn Prefetcher S3, S2, S1 and S0 CDB Pinter */
typedef struct _RACC_DMA_PREFETCHER_CDB_PTR_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    DMAPrefetcherCDBPointer   :27; /* 5:31 */
            U32    Reserved_2_4              :3;  /* 2:4 */
            U32    DMAPreFLnkLst             :1;  /* 1 */
            U32    DMAPreFPend               :1;  /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    DMAPreFPend               :1;  /* 0 */
            U32    DMAPreFLnkLst             :1;  /* 1 */
            U32    Reserved_2_4              :3;  /* 2:4 */
            U32    DMAPrefetcherCDBPointer   :27; /* 5:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DMA_PREFETCHER_CDB_PTR_REG, * PTR_RACC_DMA_PREFETCHER_CDB_PTR_REG;


typedef volatile struct _RACC_DMA_ENGINE_PLB_CONFIG
{
    U32                                        LinkedListAUpperBase;    /* 0x00 */
    RACC_DMA_CDB_ROOT_POINTER_REG              CDBRootPtrA;             /* 0x04 */
    U32                                        LinkedListBUpperBase;    /* 0x08 */
    RACC_DMA_CDB_ROOT_POINTER_REG              CDBRootPtrB;             /* 0x0C */
    RACC_DMA_LINKED_LIST_FLAGS_ALT_CRC_REG     LinkedListFlagsAltCrc;   /* 0x10 */
    U32                                        IntScratch;              /* 0x14 */
    RACC_DMA_COMP_QUEUE_TAIL_PTR_REG           CompQTail;               /* 0x18 */
    RACC_DMA_COMPQ_INT_CTL_REG                 CompQIntCtl;             /* 0x1C */
    RACC_DMA_ENGINE_CTL_STAT_REG               EngineCtlStat;           /* 0x20 */
    U32                                        HaltErrCDBUAPtr;         /* 0x24 */
    U32                                        HaltErrCDBLAPtr;         /* 0x28 */
    RACC_DMA_HALT_ERR_ATTR_REG                 HaltErrAttr;             /* 0x2C */
    RACC_DMA_HALT_ERR_STAT_REG                 HaltErrStat;             /* 0x30 */
    RACC_DMA_HALT_PAR_ERR_GEN_REG              HaltParErrGen;           /* 0x34 */
    U32                                        HaltConErrExpRefTag;     /* 0x38 */
    U32                                        HaltConErrCalcRefTag;    /* 0x3C */
    RACC_DMA_EEDP_HALT_CON_ERR_TAG_REG         HaltExpCalcAppTag;       /* 0x40 */
    RACC_DMA_EEDP_HALT_CON_ERR_GUARD_REG       HaltExpCalcGuard;        /* 0x44 */
    U32                                        HaltErrByteCount;        /* 0x48 */
    RACC_DMA_COMP_QUEUE_HEAD_PTR_REG           CompQHeadPtr;            /* 0x4C */
    RACC_DMA_NON_HALT_ERR_STAT_ATTR_REG        NonHaltErrStatAttr;      /* 0x50 */
    U32                                        NonHaltExpRefTag;        /* 0x54 */
    U32                                        NonHaltCalcRefTag;       /* 0x58 */
    RACC_DMA_EEDP_NONHALT_CON_ERR_TAG_REG      NonHaltExpCalcAppTag;    /* 0x5C */
    RACC_DMA_EEDP_NONHALT_CON_ERR_GUARD_REG    NonHaltExpCalcGuard;     /* 0x60 */
    RACC_DMA_PREFETCHER_CDB_PTR_REG            S2CDBPtr;                /* 0x64 */
    RACC_DMA_PREFETCHER_CDB_PTR_REG            S1CDBPtr;                /* 0x68 */
    RACC_DMA_PREFETCHER_CDB_PTR_REG            S0CDBPtr;                /* 0x6C */
    RACC_DMA_PREFETCHER_CDB_PTR_REG            S3CDBPtr;                /* 0x70 */
    U32                                        NonHaltErrByteCount;     /* 0x74 */
    U32                                        NonHaltErrCDBUAPtr;      /* 0x78 */
    U32                                        NonHaltErrCDBLAPtr;      /* 0x7C */
    U32                                        CompQBaseAddrLo;         /* 0x80 */
    U32                                        CompQBaseAddrHi;         /* 0x84 */
} RACC_DMA_ENGINE_PLB_CONFIG, *PTR_RACC_DMA_ENGINE_PLB_CONFIG;

typedef struct _RACC_DMA_REGS
{
    PTR_RACC_TOP_PLB_CONFIG              PtrRaidAccConfigRegs;
    PTR_RACC_DMA_ENGINE_PLB_CONFIG       PtrDMAConfigRegs[2];
} RACC_DMA_REGS, *PTR_RACC_DMA_REGS;

/* Raid Accelerator (Katana) Registers */
extern RACC_DMA_REGS gRaidAccRegs;

#endif


/* Raid Accelerator DMA Engine Config Offsets */
#define RACC_DMA_LINKEDLIST_A_UPPER_BASE_OFFSET                 (0X00000000)
#define RACC_DMA_CDB_ROOT_PTR_A_OFFSET                          (0X00000004)
#define RACC_DMA_LINKEDLIST_B_UPPER_BASE_OFFSET                 (0X00000008)
#define RACC_DMA_CDB_ROOT_PTR_B_OFFSET                          (0X0000000C)
#define RACC_DMA_LINKEDLIST_FLAGS_ALT_CRC_OFFSET                (0X00000010)
#define RACC_DMA_INTERNAL_SCRATCH_PAD_OFFSET                    (0X00000014)
#define RACC_DMA_COMP_Q_TAIL_PTR_OFFSET                         (0X00000018)
#define RACC_DMA_COMP_Q_INT_CTL_OFFSET                          (0X0000001C)
#define RACC_DMA_ENGINE_CTL_STAT_OFFSET                         (0X00000020)
#define RACC_DMA_HALT_ERR_CDB_UA_PTR_OFFSET                     (0X00000024)
#define RACC_DMA_HALT_ERR_CDB_LA_PTR_OFFSET                     (0X00000028)
#define RACC_DMA_HALT_ERR_ATTR_OFFSET                           (0X0000002C)
#define RACC_DMA_HALT_ERR_STAT_OFFSET                           (0X00000030)
#define RACC_DMA_HALT_PAR_ERR_GEN_OFFSET                        (0X00000034)
#define RACC_DMA_EEDP_HALTCON_ERR_EXPREFTAG_OFFSET              (0X00000038)
#define RACC_DMA_EEDP_HALTCON_ERR_CALCREFTAG _OFFSET            (0X0000003C)
#define RACC_DMA_EEDP_HALTCON_ERR_EXPCALC_APPTAG_OFFSET         (0X00000040)
#define RACC_DMA_EEDP_HALTCON_ERR_EXPCALC_GUARD_OFFSET          (0X00000044)
#define RACC_DMA_HALT_ERR_BYTE_COUNT_OFFSET                     (0X00000048)
#define RACC_DMA_COMP_Q_HEAD_PTR_OFFSET                         (0X0000004C)
#define RACC_DMA_NONHALT_ERR_STAT_ATTR_OFFSET                   (0X00000050)
#define RACC_DMA_EEDP_NONHALTCON_ERR_EXPREFTAG_OFFSET           (0X00000054)
#define RACC_DMA_EEDP_NONHALTCON_ERR_CALCREFTAG_OFFSET          (0X00000058)
#define RACC_DMA_EEDP_NONHALTCON_ERR_EXPCALC_APPTAG_OFFSET      (0X0000005C)
#define RACC_DMA_EEDP_NONHALTCON_ERR_EXPCALC_GUARD_OFFSET       (0X00000060)
#define RACC_DMA_PREFETCHER_S2_CDB_PTR_OFFSET                   (0X00000064)
#define RACC_DMA_PREFETCHER_S1_CDB_PTR_OFFSET                   (0X00000068)
#define RACC_DMA_PREFETCHER_S0_CDB_PTR_OFFSET                   (0X0000006C)
#define RACC_DMA_PREFETCHER_S3_CDB_PTR_OFFSET                   (0X00000070)
#define RACC_DMA_NONHALT_ERR_BYTE_COUNT_OFFSET                  (0X00000074)
#define RACC_DMA_NONHALT_ERR_CDB_UA_PTR_OFFSET                  (0X00000078)
#define RACC_DMA_NONHALT_ERR_CDB_LA_PTR_OFFSET                  (0X0000007C)
#define RACC_DMA_COMPQ_BASE_ADDR_LO_OFFSET                      (0X00000080)
#define RACC_DMA_COMPQ_BASE_ADDR_HI_OFFSET                      (0X00000084)


/****************************************************************************
 *                        D C R SPACE
 ****************************************************************************/

/* DCR Config Base Addresses */
#define RACC_DCR_PLBS_CONFIG_BASE_OFFSET          (0X00000000)
#define RACC_DCR_PLBM0_CONFIG_BASE_OFFSET         (0X00000040)
#define RACC_DCR_PLBM1_CONFIG_BASE_OFFSET         (0X00000080)
#define RACC_DCR_PMU_PLBM0_CONFIG_BASE_OFFSET     (0X000000C0)
#define RACC_DCR_PMU_PLBM1_CONFIG_BASE_OFFSET     (0X00000100)
#define RACC_DCR_PMU_PCIE_CONFIG_BASE_OFFSET      (0X00000140)
#define RACC_DCR_PMU_DDR0_CONFIG_BASE_OFFSET      (0X00000180)
#define RACC_DCR_PMU_DDR1_CONFIG_BASE_OFFSET      (0X000001C0)
#define RACC_DCR_MISC_CONFIG_BASE_OFFSET          (0X00000200)
#define RACC_DCR_TMUX_CONFIG_BASE_OFFSET          (0X00000240)


/********************************************************************
 *                     PLB SLAVE CONFIG                             *
 ********************************************************************/

#if  !defined(__LANGUAGE_ASM__)
/* Decode Configuration 3-0 */
typedef struct _RACC_DCR_DECODE_CONFIG_3_0_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_19_31              :13;    /* 19:31  */
            U32    DMA1Addr2OrderMode          :1;     /* 18 */
            U32    Reserved_17                 :1;     /* 17 */
            U32    DMA1AddrRegion2Disable      :1;     /* 16 */
            U32    Reserved_11_15              :5;     /* 11:15 */
            U32    DMA0Addr1OrderMode          :1;     /* 10 */
            U32    Reserved_9                  :1;     /* 9 */
            U32    DMA0AddrRegion1Disable      :1;     /* 8 */
            U32    Reserved_3_7                :5;     /* 3:7 */
            U32    Addr01OrderMode             :1;     /* 2 */
            U32    Reserved_1                  :1;     /* 1 */
            U32    AddrRegion0Disable          :1;     /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    AddrRegion0Disable          :1;     /* 0 */
            U32    Reserved_1                  :1;     /* 1 */
            U32    Addr01OrderMode             :1;     /* 2 */
            U32    Reserved_3_7                :5;     /* 3:7 */
            U32    DMA0AddrRegion1Disable      :1;     /* 8 */
            U32    Reserved_9                  :1;     /* 9 */
            U32    DMA0Addr1OrderMode          :1;     /* 10 */
            U32    Reserved_11_15              :5;     /* 11:15 */
            U32    DMA1AddrRegion2Disable      :1;     /* 16 */
            U32    Reserved_17                 :1;     /* 17 */
            U32    DMA1Addr2OrderMode          :1;     /* 18 */
            U32    Reserved_19_31              :13;    /* 19:31  */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_DECODE_CONFIG_3_0_REG, * PTR_RACC_DCR_DECODE_CONFIG_3_0_REG;

/* Slave General Configuration */
typedef struct _RACC_DCR_SLAVE_GENERAL_CONFIG_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_23_31          :9;    /* 23:31 */
            U32    DisableReadParity       :1;    /* 22 */
            U32    DataOrderMode           :4;    /* 18:21 */
            U32    AIRdFlowThroughWM       :3;    /* 15:17 */
            U32    Reserved_13_14          :2;    /* 13:14 */
            U32    ConfigParChkDis         :1;    /* 12 */
            U32    Reserved_10_11          :2;    /* 10:11 */
            U32    ParErrOn                :1;    /* 9 */
            U32    PLBErrDis               :1;    /* 8 */
            U32    Reserved_6_7            :2;    /* 6:7 */
            U32    AIRdFlowThroug          :1;    /* 5 */
            U32    WrSingleThread          :1;    /* 4 */
            U32    RdSingleThread          :1;    /* 3 */
            U32    WrQDisable              :1;    /* 2 */
            U32    RdQDisable              :1;    /* 1 */
            U32    Reserved_0              :1;    /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    Reserved_0              :1;    /* 0 */
            U32    RdQDisable              :1;    /* 1 */
            U32    WrQDisable              :1;    /* 2 */
            U32    RdSingleThread          :1;    /* 3 */
            U32    WrSingleThread          :1;    /* 4 */
            U32    AIRdFlowThroug          :1;    /* 5 */
            U32    Reserved_6_7            :2;    /* 6:7 */
            U32    PLBErrDis               :1;    /* 8 */
            U32    ParErrOn                :1;    /* 9 */
            U32    Reserved_10_11          :2;    /* 10:11 */
            U32    ConfigParChkDis         :1;    /* 12 */
            U32    Reserved_13_14          :2;    /* 13:14 */
            U32    AIRdFlowThroughWM       :3;    /* 15:17 */
            U32    DataOrderMode           :4;    /* 18:21 */
            U32    DisableReadParity       :1;    /* 22 */
            U32    Reserved_23_31          :9;    /* 23:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_SLAVE_GENERAL_CONFIG_REG, * PTR_RACC_DCR_SLAVE_GENERAL_CONFIG_REG;

/* PLB Slave Error Log */
typedef struct _RACC_DCR_PLB_SLAVE_ERR_LOG_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_11_31          :21;    /* 11:31 */
            U32    PlbSlaveErrorSource     :4;     /* 7:10 */
            U32    PLBSlaveErrStat         :1;     /* 6 */
            U32    PLBSlaveRWErrLog        :1;     /* 5 */
            U32    PLBSlaveErrorMasterID   :5;     /* 0:4 */
#else     /* __LITTLE_ENDIAN__ */
            U32    PLBSlaveErrorMasterID   :5;     /* 0:4 */
            U32    PLBSlaveRWErrLog        :1;     /* 5 */
            U32    PLBSlaveErrStat         :1;     /* 6 */
            U32    PlbSlaveErrorSource     :4;     /* 7:10 */
            U32    Reserved_11_31          :21;    /* 11:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PLB_SLAVE_ERR_LOG_REG, * PTR_RACC_DCR_PLB_SLAVE_ERR_LOG_REG;

/* PLB Slave Status */
typedef struct _RACC_DCR_PLB_SLAVE_STAT_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_4_31        :28;    /* 4:31 */
            U32    AIWrBuffEmpty        :1;     /* 3 */
            U32    AIRdBuffEmpty        :1;     /* 2 */
            U32    WrQEmpty             :1;     /* 1 */
            U32    RdQEmpty             :1;     /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    RdQEmpty             :1;     /* 0 */
            U32    WrQEmpty             :1;     /* 1 */
            U32    AIRdBuffEmpty        :1;     /* 2 */
            U32    AIWrBuffEmpty        :1;     /* 3 */
            U32    Reserved_4_31        :28;    /* 4:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PLB_SLAVE_STAT_REG, * PTR_RACC_DCR_PLB_SLAVE_STAT_REG;
#endif

/* PLB Slave Config Offsets */
#define RACC_DCR_LOWER_BASE_ADDR0_OFFSET           (0X00000000)
#define RACC_DCR_UPPER_BASE_ADDR1_OFFSET           (0X00000001)
/* 0x02 to 0x17 are reserved */
#define RACC_DCR_DECODE_CONFIG_3_0_OFFSET          (0X00000018)
/* 0X19 is reserved */
#define RACC_DCR_SLAVE_GENERAL_CONFIG_OFFSET       (0X0000001A)
#define RACC_DCR_PLB_SLAVE_UABUS_ERR_LOG_OFFSET    (0X0000001B)
#define RACC_DCR_PLB_SLAVE_ABUS_ERR_LOG_OFFSET     (0X0000001C)
#define RACC_DCR_PLB_SLAVE_ERR_LOG_OFFSET          (0X0000001D)
#define RACC_DCR_PLB_SLAVE_STATUS_OFFSET           (0X0000001E)
/* 0x1F to 0x3F are reserved */



/********************************************************************
 *                     PLB MASTER CONFIG                             *
 ********************************************************************/

#if  !defined(__LANGUAGE_ASM__)
/* PLB Master N Configuration Register 0 */
typedef struct _RACC_DCR_PLB_MASTER_CONFIG0_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_28_31        :4;    /* 28:31 */
            U32    DisableSIL            :1;    /* 27 */
            U32    Reserved_26           :1;    /* 26 */
            U32    SingleThread          :1;    /* 25 */
            U32    Reserved_16_24        :9;    /* 16:24 */
            U32    ForceParErr           :1;    /* 15 */
            U32    ParDisable            :1;    /* 14 */
            U32    Reserved_6_13         :8;    /* 6:13 */
            U32    DisableNTA            :1;    /* 5 */
            U32    Reserved_4            :1;    /* 4 */
            U32    DisRelaxedOrder       :1;    /* 3 */
            U32    DisNoSnoop            :1;    /* 2 */
            U32    Guarded               :1;    /* 1 */
            U32    TargetDisable         :1;    /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    TargetDisable         :1;    /* 0 */
            U32    Guarded               :1;    /* 1 */
            U32    DisNoSnoop            :1;    /* 2 */
            U32    DisRelaxedOrder       :1;    /* 3 */
            U32    Reserved_4            :1;    /* 4 */
            U32    DisableNTA            :1;    /* 5 */
            U32    Reserved_6_13         :8;    /* 6:13 */
            U32    ParDisable            :1;    /* 14 */
            U32    ForceParErr           :1;    /* 15 */
            U32    Reserved_16_24        :9;    /* 16:24 */
            U32    SingleThread          :1;    /* 25 */
            U32    Reserved_26           :1;    /* 26 */
            U32    DisableSIL            :1;    /* 27 */
            U32    Reserved_28_31        :4;    /* 28:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PLB_MASTER_CONFIG0_REG, * PTR_RACC_DCR_PLB_MASTER_CONFIG0_REG;

/* PLB Master N Configuration Register 1 */
typedef struct _RACC_DCR_PLB_MASTER_CONFIG1_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    TargetQuiesce          :1;     /* 31 */
            U32    Reserved_12_30         :19;    /* 12:30 */
            U32    RetryDivide            :4;     /* 8:11 */
            U32    EnableWrNonposted      :1;     /* 7 */
            U32    DisableWeComb          :1;     /* 6 */
            U32    Enable128bCross        :1;     /* 5 */
            U32    CacheInhibited         :1;     /* 4 */
            U32    MemoryCoherent         :1;     /* 3 */
            U32    RelaxedOrder           :1;     /* 2 */
            U32    OrderedTransfer        :1;     /* 1 */
            U32    NoSnoop                :1;     /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    NoSnoop                :1;     /* 0 */
            U32    OrderedTransfer        :1;     /* 1 */
            U32    RelaxedOrder           :1;     /* 2 */
            U32    MemoryCoherent         :1;     /* 3 */
            U32    CacheInhibited         :1;     /* 4 */
            U32    Enable128bCross        :1;     /* 5 */
            U32    DisableWeComb          :1;     /* 6 */
            U32    EnableWrNonposted      :1;     /* 7 */
            U32    RetryDivide            :4;     /* 8:11 */
            U32    Reserved_12_30         :19;    /* 12:30 */
            U32    TargetQuiesce          :1;     /* 31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PLB_MASTER_CONFIG1_REG, * PTR_RACC_DCR_PLB_MASTER_CONFIG1_REG;

/* Register for PLB Master N Snoo Region 0-3 Base or Size Part 1 */
typedef struct _RACC_DCR_PLB_MASTER_SNOOP_REG_PART1_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_16_31         :16;    /* 16:31 */
            U32    SnoopRegionPart1       :16;    /* 0:15  */
#else     /* __LITTLE_ENDIAN__ */
            U32    SnoopRegionPart1       :16;    /* 0:15  */
            U32    Reserved_16_31         :16;    /* 16:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PLB_MASTER_SNOOP_REG_PART1_REG, * PTR_RACC_DCR_PLB_MASTER_SNOOP_REG_PART1_REG;
#endif

/* PLB Master Config Offsets */
#define RACC_DCR_PLBM_CONFIG0_OFFSET                  (0X00000000)
#define RACC_DCR_PLBM_CONFIG1_OFFSET                  (0X00000001)
#define RACC_DCR_PLBM_SNOOP_REG0_BASE_PART2_OFFSET    (0x00000002)
#define RACC_DCR_PLBM_SNOOP_REG0_BASE_PART1_OFFSET    (0x00000003)
#define RACC_DCR_PLBM_SNOOP_REG0_SIZE_PART2_OFFSET    (0x00000004)
#define RACC_DCR_PLBM_SNOOP_REG0_SIZE_PART1_OFFSET    (0x00000005)
#define RACC_DCR_PLBM_SNOOP_REG1_BASE_PART2_OFFSET    (0x00000006)
#define RACC_DCR_PLBM_SNOOP_REG1_BASE_PART1_OFFSET    (0x00000007)
#define RACC_DCR_PLBM_SNOOP_REG1_SIZE_PART2_OFFSET    (0x00000008)
#define RACC_DCR_PLBM_SNOOP_REG1_SIZE_PART1_OFFSET    (0x00000009)
#define RACC_DCR_PLBM_SNOOP_REG2_BASE_PART2_OFFSET    (0x0000000A)
#define RACC_DCR_PLBM_SNOOP_REG2_BASE_PART1_OFFSET    (0x0000000B)
#define RACC_DCR_PLBM_SNOOP_REG2_SIZE_PART2_OFFSET    (0x0000000C)
#define RACC_DCR_PLBM_SNOOP_REG2_SIZE_PART1_OFFSET    (0x0000000D)
#define RACC_DCR_PLBM_SNOOP_REG3_BASE_PART2_OFFSET    (0x0000000E)
#define RACC_DCR_PLBM_SNOOP_REG3_BASE_PART1_OFFSET    (0x0000000F)
#define RACC_DCR_PLBM_SNOOP_REG3_SIZE_PART2_OFFSET    (0x00000010)
#define RACC_DCR_PLBM_SNOOP_REG3_SIZE_PART1_OFFSET    (0x00000011)
/* 0x12 to 0x3F are reserved */


/***************************************************************************
 *                     SHELL PMU CONFIG                                    *
 **************************************************************************/

/* SHELL Performance Monitor Unit Config Offsets */
#define RACC_DCR_PMU_INT_CLEAR_OFFSET                               (0x00000000)
#define RACC_DCR_PMU_INT_EN_OFFSET                                  (0x00000001)
#define RACC_DCR_PMU_TIMEOUT_OFFSET                                 (0x00000002)
#define RACC_DCR_PMU_INT_STAT_OFFSET                                (0x00000003)
#define RACC_DCR_PMU_STAT_OFFSET                                    (0x00000004)
#define RACC_DCR_PMU_TIMEOUT_VAL_OFFSET                             (0x00000005)
#define RACC_DCR_PMU_TIME_OFFSET                                    (0x00000006)
#define RACC_DCR_PMU_CLK_COUNT_OFFSET                               (0x00000007)
/* 0x08 to 0x0A are reserved */
#define RACC_DCR_PMU_READ_CMD_COUNT_OFFSET                          (0x0000000B)
#define RACC_DCR_PMU_TOTAL_READ_LEN_OFFSET                          (0x0000000C)
#define RACC_DCR_PMU_WRITE_CMD_COUNT_OFFSET                         (0x0000000D)
#define RACC_DCR_PMU_TOTAL_WRITE_LEN_OFFSET                         (0x0000000E)
#define RACC_DCR_PMU_XFER_INBOUND_DATA_AVAIL_COUNT_OFFSET           (0x0000000F)
#define RACC_DCR_PMU_XFER_OUTBOUND_DATA_AVAIL_COUNT_OFFSET          (0x00000010)
#define RACC_DCR_PMU_XFER_OUTBOUND_DATA_NOTAVAIL_COUNT_OFFSET       (0x00000011)
#define RACC_DCR_PMU_RD_TRANSACTION_DUR_MIN_OFFSET                  (0x00000012)
#define RACC_DCR_PMU_RD_TRANSACTION_DUR_AVG_OFFSET                  (0x00000013)
#define RACC_DCR_PMU_RD_TRANSACTION_DUR_MAX_OFFSET                  (0x00000014)
#define RACC_DCR_PMU_RD_TRANSACTION_COUNT_OFFSET                    (0x00000015)
#define RACC_DCR_PMU_WR_TRANSACTION_DUR_MIN_OFFSET                  (0x00000016)
#define RACC_DCR_PMU_WR_TRANSACTION_DUR_AVG_OFFSET                  (0x00000017)
#define RACC_DCR_PMU_WR_TRANSACTION_DUR_MAX_OFFSET                  (0x00000018)
#define RACC_DCR_PMU_WR_TRANSACTION_COUNT_OFFSET                    (0x00000019)
#define RACC_DCR_PMU_CMD_DELAY_DUR_MIN_OFFSET                       (0x0000001A)
#define RACC_DCR_PMU_CMD_DELAY_DUR_MAX_OFFSET                       (0x0000001B)
#define RACC_DCR_PMU_CMD_DELAY_COUNT_OFFSET                         (0x0000001C)
/* 0x1D to 0x3F are reserved */


/********************************************************************
 *                     MISCELLANEOUS CONFIG                         *
 ********************************************************************/

#if  !defined(__LANGUAGE_ASM__)
/* Raid Accelerator PMU Combined Status */
typedef struct _RACC_DCR_PMU_COMB_STAT_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_2_31          :30;    /* 2:31 */
            U32    PMUEndTriggerOut       :1;     /* 1 */
            U32    PMUInt                 :1;     /* 0 */
#else     /* __LITTLE_ENDIAN__ */
            U32    PMUInt                 :1;     /* 0 */
            U32    PMUEndTriggerOut       :1;     /* 1 */
            U32    Reserved_2_31          :30;    /* 2:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PMU_COMB_STAT_REG, * PTR_RACC_DCR_PMU_COMB_STAT_REG;
#endif

/* Miscellaneous Config Offset */
#define RACC_DCR_PMU_COMBINED_STAT_OFFSET            (0x00000000)
/*0x01 to 0x3F are reserved */

/*************************************************************************
 *                     TESTMUX CONFIG                                    *
 *************************************************************************/

#if  !defined(__LANGUAGE_ASM__)
/* Register for SHELL MasterArb PLB0, PLB1,PCIE, MEM Test Mux Control */
typedef struct _RACC_DCR_SHELL_MASTER_ARB_TMUX_CTL_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_5_31         :27;    /* 5:31 */
            U32    TMuxSelSMA            :5;     /* 0:4 */
#else     /* __LITTLE_ENDIAN__ */
            U32    TMuxSelSMA            :5;     /* 0:4 */
            U32    Reserved_5_31         :27;    /* 5:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_SHELL_MASTER_ARB_TMUX_CTL_REG, * PTR_RACC_DCR_SHELL_MASTER_ARB_TMUX_CTL_REG;

/* Register for DMA 0,1 Test Mux Control High */
typedef struct _RACC_DCR_DMA_TMUX_CTL_HI_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    dma0_TMuxSelDMAB7     :8;    /* 24:31 */
            U32    dma0_TMuxSelDMAB6     :8;    /* 16:23 */
            U32    dma0_TMuxSelDMAB5     :8;    /* 8:15 */
            U32    dma0_TMuxSelDMAB4     :8;    /* 0:7 */
#else     /* __LITTLE_ENDIAN__ */
            U32    dma0_TMuxSelDMAB4     :8;    /* 0:7 */
            U32    dma0_TMuxSelDMAB5     :8;    /* 8:15 */
            U32    dma0_TMuxSelDMAB6     :8;    /* 16:23 */
            U32    dma0_TMuxSelDMAB7     :8;    /* 24:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_DMA_TMUX_CTL_HI_REG, * PTR_RACC_DCR_DMA_TMUX_CTL_HI_REG;

/* Register for DMA 0,1 Test Mux Control Low */
typedef struct _RACC_DCR_DMA_TMUX_CTL_LO_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    TMuxSelDMAB3     :8;    /* 24:31 */
            U32    TMuxSelDMAB2     :8;    /* 16:23 */
            U32    TMuxSelDMAB1     :8;    /* 8:15 */
            U32    TMuxSelDMAB0     :8;    /* 0:7 */
#else     /* __LITTLE_ENDIAN__ */
            U32    TMuxSelDMAB0     :8;    /* 0:7 */
            U32    TMuxSelDMAB1     :8;    /* 8:15 */
            U32    TMuxSelDMAB2     :8;    /* 16:23 */
            U32    TMuxSelDMAB3     :8;    /* 24:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_DMA_TMUX_CTL_LO_REG, * PTR_RACC_DCR_DMA_TMUX_CTL_LO_REG;

/* Raid Accelerator Test Mux Control */
typedef struct _RACC_DCR_TMUX_CTL_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_21_31        :11;    /* 21:31 */
            U32    TMuxSelRAID_1         :5;     /* 16:20 */
            U32    Reserved_5_15         :11;    /* 5:15 */
            U32    TMuxSelRAID_0         :5;     /* 0:4 */
#else     /* __LITTLE_ENDIAN__ */
            U32    TMuxSelRAID_0         :5;     /* 0:4 */
            U32    Reserved_5_15         :11;    /* 5:15 */
            U32    TMuxSelRAID_1         :5;     /* 16:20 */
            U32    Reserved_21_31        :11;    /* 21:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_TMUX_CTL_REG, * PTR_RACC_DCR_TMUX_CTL_REG;

/* TMuxSelSYSSMFifo */
typedef struct _RACC_DCR_TMUX_SEL_SYS_SMFIFO_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_8_31         :24;    /* 8:31 */
            U32    TMuxSelSYSSMFifo      :8;     /* 0:7 */
#else     /* __LITTLE_ENDIAN__ */
            U32    TMuxSelSYSSMFifo      :8;     /* 0:7 */
            U32    Reserved_8_31         :24;    /* 8:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_TMUX_SEL_SYS_SMFIFO_REG, * PTR_RACC_DCR_TMUX_SEL_SYS_SMFIFO_REG;

/* PlbSlave0 Test Mux Control High (bits 39:32) */
typedef struct _RACC_DCR_PLBSLAVE_TMUX_CTL_HI_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_8_31               :24;    /* 8:31 */
            U32    Plbs0TMuxSelPlbSlaveHigh    :8;     /* 0:7 */
#else     /* __LITTLE_ENDIAN__ */
            U32    Plbs0TMuxSelPlbSlaveHigh    :8;     /* 0:7 */
            U32    Reserved_8_31               :24;    /* 8:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PLBSLAVE_TMUX_CTL_HI_REG, * PTR_RACC_DCR_PLBSLAVE_TMUX_CTL_HI_REG;

/* PlbMaster 0, 1 Test Mux Control */
typedef struct _RACC_DCR_PLB_MASTER_TMUX_CTL_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32    Reserved_5_31         :27;    /* 5:31 */
            U32    TMuxSelPlbMaster      :5;     /* 0:4 */
#else     /* __LITTLE_ENDIAN__ */
            U32    TMuxSelPlbMaster      :5;     /* 0:4 */
            U32    Reserved_5_31         :27;    /* 5:31 */
#endif
        } Bits;
        U32 Word;
    } u;
} RACC_DCR_PLB_MASTER_TMUX_CTL_REG, * PTR_RACC_DCR_PLB_MASTER_TMUX_CTL_REG;
#endif

/* Test Mux Config Offsets */
/* 0x00 is reserved */
#define RACC_DCR_SHELL_MASTERARB_PLB0_TMUX_CTL_OFFSET            (0x00000001)
#define RACC_DCR_SHELL_MASTERARB_PLB1_TMUX_CTL_OFFSET            (0x00000002)
/* 0x03 is reserved */
#define RACC_DCR_SHELL_MASTERARB_PCIE_TMUX_CTL_OFFSET            (0x00000004)
#define RACC_DCR_SHELL_MASTERARB_MEM_TMUX_CTL_OFFSET             (0x00000005)
#define RACC_DCR_DMA0_TMUX_CTL_HI_OFFSET                         (0x00000006)
#define RACC_DCR_DMA0_TMUX_CTL_LO_OFFSET                         (0x00000007)
#define RACC_DCR_DMA1_TMUX_CTL_HI_OFFSET                         (0x00000008)
#define RACC_DCR_DMA1_TMUX_CTL_LO_OFFSET                         (0x00000009)
/* 0x0A to 0x17 are reserved */
#define RACC_DCR_TMUX_CTL_OFFSET                                 (0x00000018)
/* 0x19 to 0x1A are reserved */
#define RACC_DCR_TMUXSELSYSSMFIFO_OFFSET                         (0x0000001B)
#define RACC_DCR_PLBSLAVE0_TMUX_CTL_LO_OFFSET                    (0x0000001C)
#define RACC_DCR_PLBSLAVE0_TMUX_CTL_HI_OFFSET                    (0x0000001D)
#define RACC_DCR_PLBMASTER0_TMUX_CTL_OFFSET                      (0x0000001E)
#define RACC_DCR_PLBMASTER1_TMUX_CTL_OFFSET                      (0x0000001F)
/* 0x20 to 0x3F are reserved */


typedef struct _RACC_DMA_CDB_HEADER
{
    U8                          Opcode;                     // 0x0
    U8                          CommandOptions;             // 0x1
    U16                         CDBLength;                  // 0x2-0x3
    U8                          ModifyOptions;              // 0x4
    U8                          NumberOfSourceAddresses;    // 0x5
    U16                         CDBFlags;                   // 0x6-0x7
    U32                         DataByteCount;              // 0x8-0xb
    struct _RACC_DMA_CDB_HEADER *NextCDBPointer;           // 0xc-0xf
    U16                         BlockSize;                  // 0x10-0x11
    U8                          SIL;                        // 0x12
    U8                          ComparePolicyGuardCtrl;    // 0x13
    U32                         SourceSkipCount;            // 0x14
    U32                         DestinationSkipCount;       // 0x18
    U32                         Reserved;                   // 0x1C
} RACC_DMA_CDB_HEADER, * PTR_RACC_DMA_CDB_HEADER;

#define RACC_NEXTCDBPTR_END_OF_CHAIN ((U32)(-1UL))
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED0         (0x00000001)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_XOR_ADDR_TYPE     (0x00000002)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_DST_ADDR_TYPE     (0x00000004)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_SRC_ADDR_TYPE     (0x00000008)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_DMA_WAIT_ACK      (0x00000010)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_DMA_NO_IRQ        (0x00000020)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED1         (0x00000040)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_SRC_REF_TAG       (0x00000080)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_ALT_CRC           (0x00000100)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_PCI_RELAX         (0x00000200)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_PCI_NOSNOOP       (0x00000400)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_LSFR_EN_INV       (0x00000800)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_XOR_0_CHK         (0x00001000)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED2         (0x00002000)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_RESERVED3         (0x00004000)
#define RACC_DMA_DCB_HEADER_CDBFLAGS_NXT_CDB_STRICT    (0x00008000)

#define COMPLETION_Q_INT_TIMER_UINIT_500NS  (1)/* value * 500 ns */
#define COMPLETION_Q_INT_TIMER_UINIT_SH     (8)

typedef struct _RACC_DMA_SOURCE
{
    U64                         SourceAddress;                      /* 0x0-0x7 */
    U8                          SourceAttributes;                   /* 0x8 */
    U8                          XOR1Multiplier;                     /* 0x9 */
    U8                          XOR2Multiplier;                     /* 0xA */
    U8                          Reserved1;                          /* 0xB */
    U32                         SourceReferenceTagInitial;          /* 0xC-0xF */
} RACC_DMA_SOURCE, * PTR_RACC_DMA_SOURCE;

typedef struct _RACC_DMA_SOURCE_EEDP
{
    U16                         SourceApplicationTagInitial;        /* 0x0-0x1 */
    U16                         SourceApplicationTagMask;           /* 0x2-0x3 */
    U16                         ApplicationTagTranslationMask;      /* 0x4-0x5 */
    U16                         Reserved1;                          /* 0x6-0x7 */
    U32                         Reserved2;                          /* 0x8-0xB */
    U32                         Reserved3;                          /* 0xC-0xF */
} RACC_DMA_SOURCE_EEDP, * PTR_RACC_DMA_SOURCE_EEDP;

typedef struct _RACC_DMA_SOURCE_CRYPT
{
    U64                         AESKeyPointer;                      /* 0x0-0x7 */
    U64                         TweakKeyPointer;                    /* 0x8-0xF */
    U32                         TweakOrInitialValue[4];             /* 0x10-0x13 */
} RACC_DMA_SOURCE_CRYPT, * PTR_RACC_DMA_SOURCE_CRYPT;

typedef struct _RACC_DMA_SOURCE_HMAC
{
    U64                         HashKeyPointer;                     /* 0x0-0x7 */
} RACC_DMA_SOURCE_HMAC, * PTR_RACC_DMA_SOURCE_HMAC;

typedef struct _RACC_DMA_DEST
{
    U64                         DestinationAddress;                 /* 0x0-0x7 */
    U8                          DestinationAttributes;              /* 0x8 */
    U8                          Reserved;                           /* 0x9 */
    U16                         DestinationApplicationTagInitial;   /* 0xa-0xb */
    U32                         DestinationReferenceTagInitial;     /* 0xc-0xf */
} RACC_DMA_DEST, * PTR_RACC_DMA_DEST;

typedef struct _RACC_DMA_SRC_DST
{
    RACC_DMA_SOURCE            Src;                                /* 0x0-0xF */
    RACC_DMA_DEST              Dst;                                /* 0x10-0x1F */
} RACC_DMA_SRC_DST, * PTR_RACC_DMA_SRC_DST;

typedef struct _RACC_DMA_XOR1
{
    U64                         XOR1Address;                        /* 0x0-0x7 */
    U8                          XOR1BufferAttributes;               /* 0x8 */
    U8                          Reserved;                           /* 0x9 */
    U16                         XOR1ApplicationTagInitial;          /* 0xa-0xb */
    U32                         XOR1ReferenceTagInitial;            /* 0xc-0xf */
} RACC_DMA_XOR1, * PTR_RACC_DMA_XOR1;

typedef struct _RACC_DMA_XOR1_SOURCE_CRYPT
{
    U64                         XOR1AESKeyPointer;                  /* 0x0-0x7 */
    U64                         XOR1TweakKeyPointer;                /* 0x8-0xf */
    U32                         XOR1TweakOrInitialValue[4];         /* 0x10-0x13 */
} RACC_DMA_SOURCE_XOR1_CRYPT, * PTR_RACC_DMA_SOURCE_XOR1_CRYPT;

typedef struct _RACC_DMA_XOR2
{
    U64                         XOR2Address;                        /* 0x0-0x7 */
    U8                          XOR2BufferAttributes;               /* 0x8 */
    U8                          Reserved;                           /* 0x9 */
    U16                         XOR2ApplicationTagInitial;          /* 0xa-0xb */
    U32                         XOR2ReferenceTagInitial;            /* 0xc-0xf */
} RACC_DMA_XOR2, * PTR_RACC_DMA_XOR2;

typedef struct _RACC_DMA_XOR2_SOURCE_CRYPT
{
    U64                         XOR2AESKeyPointer;                  /* 0x0-0x7 */
    U64                         XOR2TweakKeyPointer;                /* 0x8-0xf */
    U32                         XOR2TweakOrInitialValue[4];         /* 0x10-0x13 */
} RACC_DMA_SOURCE_XOR2_CRYPT, * PTR_RACC_DMA_SOURCE_XOR2_CRYPT;

typedef struct _RACC_DMA_PATTERN
{
    U32                         Pattern[4];                         /* 0x0-0xF */
} RACC_DMA_PATTERN, * PTR_RACC_DMA_PATTERN;

typedef struct _RACC_DMA_LFSR
{
    U32                         LFSRSeed[4];                        /* 0x0-0xF */
} RACC_DMA_LFSR, * PTR_RACC_DMA_LFSR;

typedef struct _RACC_DMA_SGL
{
    U64 Address;                                                    /* 0x0-0x7 */
    U32 Length;                                                     /* 0x8-0xB */
    U32 Extension;                                                  /* 0xC-0xF */
} RACC_DMA_SGL, * PTR_RACC_DMA_SGL;


typedef struct _CDB
{
    RACC_DMA_CDB_HEADER Header;                                    /* 0x0-0x13 */
    RACC_DMA_SOURCE Source;                                        /* 0x14-0x23 */
    RACC_DMA_DEST Dest;                                            /* 0x24-0x33 */
    U8              Pad[0xC];                                       /* 0x34-0x3f pad to 0x10 */
} CDB, * PTR_CDB;

//DMA opcodes
#define RACC_DMA_NOOP                                      (0x00)
#define RACC_DMA_SINGLE_SOURCE_MOVE                        (0x01)
#define RACC_DMA_MULTICAST_MOVE                            (0x02)
#define RACC_DMA_MULTI_SOURCE_MOVE                         (0x03)
#define RACC_DMA_XOR1_MOVE                                 (0x04)
#define RACC_DMA_XOR2_MOVE                                 (0x05)
#define RACC_DMA_GENERATE_XOR1                             (0x06)
#define RACC_DMA_GENERATE_XOR2                             (0x08)
#define RACC_DMA_CHECK_PATTERN                             (0x09)
#define RACC_DMA_FILL_PATTERN                              (0x0A)
#define RACC_DMA_CHECK_LFSR                                (0x0B)
#define RACC_DMA_FILL_LFSR                                 (0x0C)
#define RACC_DMA_GENERATE_HASH                             (0x0D)
#define RACC_DMA_GENERATE_HMAC                             (0x0E)
#define RACC_DMA_MOVE_MULTIPLY                             (0x0F)

//Command attributes
#define RACC_DMA_ADD_EEDP                                  (0x10)
#define RACC_DMA_REMOVE_EEDP                               (0x08)
#define RACC_DMA_UPDATE_EEDP                               (0x04)
#define RACC_DMA_ENCRYPT                                   (0x02)
#define RACC_DMA_DECRYPT                                   (0x01)

//EEDP flags - Command Options[7:5]
#define RACC_DMA_CHECK_REF_TAG                             (0x80)
#define RACC_DMA_CHECK_APP_TAG                             (0x40)
#define RACC_DMA_CHECK_CRC                                 (0x20)

//Key size values - Modify Options[7:6]
#define RACC_DMA_KEY_SIZE_128_BIT                          (0x00)
#define RACC_DMA_KEY_SIZE_256_BIT                          (0x80)

//Cipher modes - Modify Options[5:3]
#define RACC_DMA_CIPHER_LRW                                (0x00)
#define RACC_DMA_CIPHER_CBC                                (0x08)
#define RACC_DMA_CIPHER_ECB                                (0x10)
#define RACC_DMA_CIPHER_CTR                                (0x18)
#define RACC_DMA_CIPHER_XTS                                (0x20)

//Hash modes - Modify Options[2:0]
#define RACC_DMA_HASH_SHA_224                              (0x00)
#define RACC_DMA_HASH_SHA_256                              (0x01)
#define RACC_DMA_HASH_SHA_384                              (0x02)
#define RACC_DMA_HASH_SHA_512                              (0x03)

//CDB bits
#define RACC_DMA_NEXT_CDB_STRICT_ORDERING                  (0x8000)
#define RACC_DMA_SINGLE_SOURCE_MOVE_MULTIPLY               (0x4000)
#define RACC_DMA_SECRET_ROOT_KEY_ENABLE                    (0x2000)
#define RACC_DMA_SOURCE_DATA_BIT_BUCKET_XOR_ZERO_CHECK     (0x1000)
#define RACC_DMA_LFSR_INVERT_ENABLE                        (0x0800)
#define RACC_DMA_PCI_NO_SNOOP                              (0x0400)
#define RACC_DMA_PCI_RELAXED_ORDERING                      (0x0200)
#define RACC_DMA_USE_ALT_CRC_SEED                          (0x0100)
#define RACC_DMA_PASSTHROUGH_SRC_REF_TAG                   (0x0080)
#define RACC_DMA_DONT_INTERRUPT                            (0x0020)
#define RACC_DMA_WAIT_FOR_ACK                              (0x0010)
#define RACC_DMA_SOURCE_ADDR_IS_SGL                        (0x0008)
#define RACC_DMA_DEST_ADDR_IS_SGL                          (0x0004)
#define RACC_DMA_XOR_ADDR_IS_SGL                           (0x0002)
#define RACC_DMA_INCREMENTAL_XOR                           (0x0001)

//Application Tag Method
#define RACC_DMA_APP_NO_TAG_CHECK                          (0x00)
#define RACC_DMA_APP_TAG_CHECK_SAME                        (0x04)
#define RACC_DMA_APP_TAG_CHECK_INCREMENT                   (0x08)

//Reference Tag Method
#define RACC_DMA_REF_NO_TAG_CHECK                          (0x00)
#define RACC_DMA_REF_TAG_CHECK_SAME                        (0x01)
#define RACC_DMA_REF_TAG_CHECK_INCREMENT                   (0x02)

//Address Type
#define RACC_DMA_ADDRESS_PCI                               (0x00)
#define RACC_DMA_ADDRESS_MEM                               (0x10)
#define RACC_DMA_ADDRESS_PLB                               (0x20)
#define RACC_DMA_ADDRESS_PLB_NTA                           (0x30)

#define RACC_END_CDB_LIST                              (0xFFFFFFFF)

/************************************************************************************************************
*
* The following structures are for CDBs of a fixed size.  They do not contain fields for Data Protection
* attributes or Encrypt/Decrypt attributes.
*
************************************************************************************************************/

// RACC_DMA_NOOP (opcode 0x0)
// Note: Typical header, RACC_DMA_CDB_HEADER, makes this structure larger than CDB format.  CDB size
// needs to be adjusted manually when used.  Subtract last 4 bytes off.
typedef struct _NOOP_CDB
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x1F */
} NOOP_CDB, * PTR_NOOP_CDB;

// RACC_DMA_SINGLE_SOURCE_MOVE (opcode 0x1)
typedef struct _SSM_CDB
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE        Src;            /* 0x20-0x2F */
    RACC_DMA_DEST          Dst;            /* 0x30-0x3F */
} SSM_CDB, * PTR_SSM_CDB;

typedef struct _SSM_CDB_ALIGNED
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE        Src;            /* 0x20-0x2F */
    RACC_DMA_DEST          Dst;            /* 0x30-0x3F */
} SSM_CDB_ALIGNED, * PTR_SSM_CDB_ALIGNED;

typedef struct _SSM_CRYPT_CDB
{
    RACC_DMA_CDB_HEADER        Header;    /* 0-0x1F    */
    RACC_DMA_SOURCE            Src;       /* 0x20-0x2F */
    RACC_DMA_SOURCE_CRYPT      Aes;       /* 0x30-0x43 */
    RACC_DMA_DEST              Dst;       /* 0x44-0x53 */
    U8                         Pad[12];   /* 0x54-0x5F */
} SSM_CRYPT_CDB, * PTR_SSM_CRYPT_CDB;

typedef struct _SSM_EEDP_CDB
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE        Src;            /* 0x20-0x2F */
    RACC_DMA_SOURCE_EEDP   Eedp;           /* 0x30-0x3F */
    RACC_DMA_DEST          Dst;            /* 0x40-0x4F */
} SSM_EEDP_CDB, * PTR_SSM_EEDP_CDB;

// RACC_DMA_MULTICAST_MOVE (opcode 0x2)
typedef struct _MCM_CDB
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE        Src;            /* 0x20-0x2F */
    RACC_DMA_DEST          Dst[2];         /* 0x30-0x4F */
} MCM_CDB, * PTR_MCM_CDB;

// RACC_DMA_MULTI_SOURCE_MOVE (opcode 0x3)
// Supports maximum amount of source/destination pairs.  This should not be included in the BMID
// CDB union!
typedef struct _MSM_CDB_MAX
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x13 */
    RACC_DMA_SRC_DST       Pair[255];
} MSM_CDB_MAX, * PTR_MSM_CDB_MAX;

// RACC_DMA_GENERATE_XOR1 (opcode 0x6)
typedef struct _XOR1_CDB
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE        Src[2];         /* 0x20-0x3F */
    RACC_DMA_XOR1          Xor1;           /* 0x40-0x4F */
} XOR1_CDB, * PTR_XOR1_CDB;

// RACC_DMA_GENERATE_XOR2 (opcode 0x8)
typedef struct _XOR2_CDB
{
    RACC_DMA_CDB_HEADER    Header;         /* 0-0x1F    */
    RACC_DMA_SOURCE        Src[2];         /* 0x20-0x3F */
    RACC_DMA_XOR1          Xor1;           /* 0x40-0x4F */
    RACC_DMA_XOR2          Xor2;           /* 0x50-0x5F */
} XOR2_CDB, * PTR_XOR2_CDB;

// RACC_DMA_CHECK_PATTERN (opcode 0x9)
typedef struct _CHK_PAT_CDB
{
    RACC_DMA_CDB_HEADER     Header;             /* 0-0x1F    */
    U32                     CheckPattern[4];    /* 0x20-0x2F */
    U64                     SourceAddress;      /* 0x30-0x37 */
    U8                      SourceAttributes;   /* 0x38      */
    U8                      Reserved[7];        /* 0x39-0x3F */
} CHK_PAT_CDB, * PTR_CHK_PAT_CDB;

// RACC_DMA_FILL_PATTERN (opcode 0xA)
typedef struct _FILL_PAT_CDB
{
    RACC_DMA_CDB_HEADER     Header;             /* 0-0x1F */
    U32                     FillPattern[4];     /* 0x20-0x2F */
    RACC_DMA_DEST           Dst;                /* 0x30-0x3F */
} FILL_PAT_CDB, * PTR_FILL_PAT_CDB;

// RACC_DMA_CHECK_LFSR (opcode 0xB)
typedef struct _CHK_LFSR_CDB
{
    RACC_DMA_CDB_HEADER     Header;             /* 0-0x1F    */
    U32                     LfsrSeed[4];        /* 0x20-0x2F */
    U64                     SourceAddress;      /* 0x30-0x37 */
    U8                      SourceAttributes;   /* 0x38      */
    U8                      Reserved[7];        /* 0x39-0x3F */
} CHK_LFSR_CDB, * PTR_CHK_LFSR_CDB;

// RACC_DMA_FILL_LFSR (opcode 0xC)
typedef struct _FILL_LFSR_CDB
{
    RACC_DMA_CDB_HEADER     Header;                 /* 0-0x1F    */
    U32                     LfsrSeed[4];            /* 0x20-0x2F */
    U64                     DestinationAddress;     /* 0x30-0x37 */
    U8                      DestinationAttributes;  /* 0x38      */
    U8                      Reserved[7];            /* 0x39-0x3F */
} FILL_LFSR_CDB, * PTR_FILL_LFSR_CDB;

// RACC_DMA_GENERATE_HASH (opcode 0xD)
typedef struct _GEN_HASH_CDB
{
    RACC_DMA_CDB_HEADER     Header;                 /* 0-0x1F    */
    RACC_DMA_SOURCE         Src;                    /* 0x20-0x2F */
    U64                     DestinationAddress;     /* 0x30-0x37 */
    U8                      DestinationAttributes;  /* 0x38      */
    U8                      DestinationReserved[7]; /* 0x39-0x3F */
} GEN_HASH_CDB, * PTR_GEN_HASH_CDB;

// RACC_DMA_GENERATE_HMAC (opcode 0xE)
typedef struct _GEN_HMAC_CDB
{
    RACC_DMA_CDB_HEADER     Header;                 /* 0-0x1F    */
    RACC_DMA_SOURCE         Src;                    /* 0x20-0x2F */
    U64                     HashKeyPointer;         /* 0x30-0x37 */
    U64                     Reserved1;              /* 0x38-0x3F */
    U64                     DestinationAddress;     /* 0x40-0x47 */
    U8                      DestinationAttributes;  /* 0x48      */
    U8                      DestinationReserved[7]; /* 0x49-0x4f */
} GEN_HMAC_CDB, * PTR_GEN_HMAC_CDB;

// RACC_DMA_MOVE_MULTIPLY (opcode 0xF)
typedef struct _MM_CDB
{
    RACC_DMA_CDB_HEADER     Header;                 /* 0-0x1F */
    RACC_DMA_SOURCE         Src;                    /* 0x20-0x2F */
    RACC_DMA_DEST           Dst;                    /* 0x30-0x3F */
} MM_CDB, * PTR_MM_CDB;

typedef union _RACC_CDB_UNION
{
    NOOP_CDB        Noop;
    SSM_CDB         Move;
    SSM_CDB_ALIGNED MoveAligned;
    SSM_CRYPT_CDB   MoveEncrypt;
    SSM_EEDP_CDB    EEDP;
    MCM_CDB         MultiCastMove;
    //MSM_CDB_MAX     MultiSrcMove; this is 8K!!
    XOR1_CDB        XOR1;
    XOR2_CDB        XOR2;
    CHK_PAT_CDB     CheckPattern;
    FILL_PAT_CDB    FillPattern;
    CHK_LFSR_CDB    CheckLFSR;
    FILL_LFSR_CDB   FillLFSR;
    GEN_HASH_CDB    GenHash;
    GEN_HMAC_CDB    GenHMAC;
    MM_CDB          MoveMultiply;
} RACC_CDB_UNION;

#endif /*   !defined(__LANGUAGE_ASM__) */

#define COMPLETION_Q_BASE_SHIFT     (4)
#define DCR_POM1_REGION_SIZE_1K     (0x00FFFFFF)

#endif
