/***************************************************************************
 *                                                                         *
 *  Copyright 2003 LSI Logic Corporation.  All rights reserved.            *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  IOPISTATUS.H
 *          Title:  SAS Firmware IOP Interface ISTATUS Definitions
 *     Programmer:  Steve Johnson
 *  Creation Date:  February 24, 2003
 *
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface ISTATUS definitions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPISTATUS_H_INCLUDED
#define IOPISTATUS_H_INCLUDED


typedef U32 ISTATUS;


#define ISTATUS_SUCCESS                         (0)


/******************************************************************************
* Generic Error Interface definitions 0 - 0xFFFF
*******************************************************************************/
#define ISTATUS_FAILURE                         (0x00000001)
#define ISTATUS_NOT_HANDLED                     (0x00000002)
#define ISTATUS_REQUEST_PENDING                 (0x00000003)
#define ISTATUS_REQUEST_DENIED                  (0x00000004)
#define ISTATUS_INVALID_PARAMETER               (0x00000005)
#define ISTATUS_INVALID_DEVICE_HANDLE           (0x00000006)
#define ISTATUS_BUSY                            (0x00000007)

#define ISTATUS_CONFIG_INVALID_ACTION           (0x00000020)
#define ISTATUS_CONFIG_INVALID_TYPE             (0x00000021)
#define ISTATUS_CONFIG_INVALID_PAGE             (0x00000022)
#define ISTATUS_CONFIG_INVALID_DATA             (0x00000023)
#define ISTATUS_CONFIG_NO_DEFAULTS              (0x00000024)
#define ISTATUS_CONFIG_CANT_COMMIT              (0x00000025)

/******************************************************************************
* IOP Error definitions 0x10000 - 0x1FFFF
*******************************************************************************/

#define ISTATUS_IOP_BASE_ERROR                          (0x10000)

#define ISTATUS_IOP_FLASH_ERRORS                        (ISTATUS_IOP_BASE_ERROR   + 0x100)
#define ISTATUS_IOP_FLASH_INVALID_REGION_LENGTH         (ISTATUS_IOP_FLASH_ERRORS + 0x04)
#define ISTATUS_IOP_FLASH_INVALID_REGION_TYPE           (ISTATUS_IOP_FLASH_ERRORS + 0x05)
#define ISTATUS_IOP_FLASH_TOGGLE_FAILURE                (ISTATUS_IOP_FLASH_ERRORS + 0x06)
#define ISTATUS_IOP_FLASH_ERASE_BLOCK_ERROR             (ISTATUS_IOP_FLASH_ERRORS + 0x07)
#define ISTATUS_IOP_FLASH_SEGMENT_SIZE_ERROR            (ISTATUS_IOP_FLASH_ERRORS + 0x08)
#define ISTATUS_IOP_FLASH_REGION_BOUNDARY_ERROR         (ISTATUS_IOP_FLASH_ERRORS + 0x09)

#define ISTATUS_IOP_CONFIG_DIR_HDR_BAD                  (ISTATUS_IOP_FLASH_ERRORS + 0x10)
#define ISTATUS_IOP_CONFIG_DIR_ENTRY_BAD                (ISTATUS_IOP_FLASH_ERRORS + 0x11)
#define ISTATUS_IOP_PERSIST_CONFIG_HDR_BAD              (ISTATUS_IOP_FLASH_ERRORS + 0x12)
#define ISTATUS_IOP_PERSIST_CONFIG_CHECKSUM_BAD         (ISTATUS_IOP_FLASH_ERRORS + 0x13)
#define ISTATUS_IOP_PERSIST_CONFIG_POWERLOSS_DETECTED   (ISTATUS_IOP_FLASH_ERRORS + 0x14)
#define ISTATUS_IOP_PERSIST_CONFIG_CORRUPT_DATA         (ISTATUS_IOP_FLASH_ERRORS + 0x15)

#define ISTATUS_IOP_PERSIST_READ_FAILURE                (ISTATUS_IOP_FLASH_ERRORS + 0x50)
#define ISTATUS_IOP_PERSIST_WRITE_FAILURE               (ISTATUS_IOP_FLASH_ERRORS + 0x51)

#define ISTATUS_IOP_IR_MOVED_DATA                       (ISTATUS_IOP_FLASH_ERRORS + 0x60)

#define ISTATUS_IOP_TIMER_ERRORS                        (ISTATUS_IOP_BASE_ERROR   + 0x200)
#define ISTATUS_IOP_TIMER_NOT_PRESENT                   (ISTATUS_IOP_TIMER_ERRORS + 0x01)
#define ISTATUS_IOP_TIMER_ALLOC_FAILED                  (ISTATUS_IOP_TIMER_ERRORS + 0x02)

#define ISTATUS_IOP_RESOURCE_ERRORS                     (ISTATUS_IOP_BASE_ERROR   + 0x300)
#define ISTATUS_IOP_RESOURCE_BELOW_THRESHOLD            (ISTATUS_IOP_RESOURCE_ERRORS + 0x01)
#define ISTATUS_IOP_RESOURCE_UNAVAILABLE                (ISTATUS_IOP_RESOURCE_ERRORS + 0x02)
#define ISTATUS_IOP_RESOURCE_INVALID_BUFFER_SIZE        (ISTATUS_IOP_RESOURCE_ERRORS + 0x03)
#define ISTATUS_IOP_RESOURCE_OUT_OF_MEM                 (ISTATUS_IOP_RESOURCE_ERRORS + 0x04)

#define ISTATUS_IOP_EVENT_ERRORS                        (ISTATUS_IOP_BASE_ERROR   + 0x500)
#define ISTATUS_IOP_EVENT_PERSIST_TABLE_FULL            (ISTATUS_IOP_EVENT_ERRORS + 0x01)
#define ISTATUS_IOP_EVENT_MAP_ALREADY_MAPPED            (ISTATUS_IOP_EVENT_ERRORS + 0x02)
#define ISTATUS_IOP_EVENT_HOST_MAP                      (ISTATUS_IOP_EVENT_ERRORS + 0x03)

#if (RAID)
#define ISTATUS_IOP_IR_ERRORS                           (ISTATUS_IOP_BASE_ERROR   + 0x600)
#define ISTATUS_IOP_IR_NEW_DEVNUM                       (ISTATUS_IOP_IR_ERRORS    + 0x01)
#define ISTATUS_IOP_IR_CHANGED_DEVNUM                   (ISTATUS_IOP_IR_ERRORS    + 0x02)

#define ISTATUS_TM_DEVICE_NOT_HANDLED                   (ISTATUS_IOP_IR_ERRORS    + 0x11)
#define ISTATUS_TM_HIDDEN_DEVICE                        (ISTATUS_IOP_IR_ERRORS    + 0x12)
#define ISTATUS_TM_VIRTUAL_DEVICE                       (ISTATUS_IOP_IR_ERRORS    + 0x13)
#define ISTATUS_TM_MSG_NOT_SUPPORTED                    (ISTATUS_IOP_IR_ERRORS    + 0x14)
#define ISTATUS_IR_DEVICE_OWNED                         (ISTATUS_IOP_IR_ERRORS    + 0x15)
#define ISTATUS_IR_DEVICE_NOT_OWNED                     (ISTATUS_IOP_IR_ERRORS    + 0x16)
#endif


#define ISTATUS_IOP_TARGET_ERRORS                       (ISTATUS_IOP_BASE_ERROR   + 0x700)
#define ISTATUS_IOP_NO_CMD_BUFFS_POSTED                 (ISTATUS_IOP_TARGET_ERRORS  + 0x01)
#define ISTATUS_IOP_NO_CMD_BUFFS_AVAILABLE              (ISTATUS_IOP_TARGET_ERRORS  + 0x02)

#define ISTATUS_DEVICE_NOT_PRESENT                      (ISTATUS_IOP_BASE_ERROR + 0x800)
#define ISTATUS_ISTWI_BUS_NOT_RESERVED                  (ISTATUS_IOP_BASE_ERROR + 0x801)

#if (IOC_ETHERNET_SUPPORT)
#define ISTATUS_ETHERNET_ERRORS                         (ISTATUS_IOP_BASE_ERROR   + 0x900)
#define ISTATUS_ETHERNET_DNS_ERROR                      (ISTATUS_ETHERNET_ERRORS  + 0x001)
#endif

#define ISTATUS_TFTP_ERRORS                             (ISTATUS_IOP_BASE_ERROR   + 0x910)
// These error codes are defined by TFTP
#define ISTATUS_TFTP_ERROR_CODE_FNF                     (ISTATUS_TFTP_ERRORS      + 0x001)
#define ISTATUS_TFTP_ERROR_CODE_ACCESS                  (ISTATUS_TFTP_ERRORS      + 0x002)
#define ISTATUS_TFTP_ERROR_CODE_DISK_FULL               (ISTATUS_TFTP_ERRORS      + 0x003)
#define ISTATUS_TFTP_ERROR_CODE_BAD_OP                  (ISTATUS_TFTP_ERRORS      + 0x004)
#define ISTATUS_TFTP_ERROR_CODE_BAD_TID                 (ISTATUS_TFTP_ERRORS      + 0x005)
#define ISTATUS_TFTP_ERROR_CODE_EXISTS                  (ISTATUS_TFTP_ERRORS      + 0x006)
#define ISTATUS_TFTP_ERROR_CODE_BAD_USER                (ISTATUS_TFTP_ERRORS      + 0x007)
// Additional TFTP Error Codes not defined by TFTP
#define ISTATUS_TFTP_ERROR_CODE_TIMEOUT                 (ISTATUS_TFTP_ERRORS      + 0x008)
#define ISTATUS_TFTP_ERROR_LINK_DOWN                    (ISTATUS_TFTP_ERRORS      + 0x009)

#define ISTATUS_XMODEM_ERRORS                           (ISTATUS_IOP_BASE_ERROR + 0x950)
#define ISTATUS_XMODEM_CANCEL                           (ISTATUS_XMODEM_ERRORS  +  0x01)
#define ISTATUS_XMODEM_CLIENT_FAILURE                   (ISTATUS_XMODEM_ERRORS  +  0x02)
#define ISTATUS_XMODEM_MAX_RETRIES_EXCEEDED             (ISTATUS_XMODEM_ERRORS  +  0x03)
#define ISTATUS_XMODEM_EOT_FAIL                         (ISTATUS_XMODEM_ERRORS  +  0x04)
#define ISTATUS_XMODEM_TIMEOUT                          (ISTATUS_XMODEM_ERRORS  +  0x05)

/******************************************************************************
* PL Error definitions 0x20000 - 0x2FFFF
*******************************************************************************/
#define ISTATUS_DEVICE_NOT_REGISTERED                   (0x00020000)
#define ISTATUS_DEVICE_BLOCKED                          (0x00020001)


/******************************************************************************
IR Error definitions 0x30000 - 0x3FFFF
*******************************************************************************/


#endif
