/***************************************************************************
 *                                                                         *
 *  Copyright 2003 LSI Logic Corporation.  All rights reserved.            *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiMaxes.h
 *          Title:  Maximum values for various MPI values
 *     Programmer:  Roger Clegg
 *  Creation Date:  Jan 8, 2008
 *
 *
 * Description
 * ------------
 * This include file contains various maximums used to define the size
 * if misc. structures including MPI 2.0 config pages.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPIMAXES_H_INCLUDED
#define IOPIMAXES_H_INCLUDED


/******************************************************************************
*
*      I O P I   C O N F I G  M A X   D E F I N I T I O N S
*
******************************************************************************/

#ifndef MPI2_SUPPORTED_DEVICES_IMAGE_NUM_DEVICES
#define MPI2_SUPPORTED_DEVICES_IMAGE_NUM_DEVICES    (3)
#endif

/*
 * The following are the values for NVDATA, they will be different for each
 * chip based upon the expected hardware.  Each of the config pages in
 * both firmware and nvdata are sized to the max possible but the
 * elements such as "NumPhys" are set to these values to indicate
 * how many of the elements are "valid"
 */

/*
 * The following are maximums for all chips (defining page sizes)
 */
/* Max GPIO's supported by any chip (only 32 until page 6 is updated */
/* TO DO: Update for MfgPage 6 */
#define IOPIMAXES_PAGE6_METEOR_GPIO_DEFS              (61)
#define IOPIMAXES_PAGE6_FALCON_GPIO_DEFS              (36)
#define IOPIMAXES_PAGE6_LIBERATOR_GPIO_DEFS           (36)
#define IOPIMAXES_PAGE6_THUNDERBOLT_GPIO_DEFS         (42)
#define IOPIMAXES_PAGE6_GPIO_DEFS                     (IOPIMAXES_PAGE6_METEOR_GPIO_DEFS)

/* Max Number of Phys supported by ANY chip, IOC_NUM_PHYS is used directly for MfgPage3 */
#define IOC_NUM_PHYS                                  (8)
#define IOC_MFG_PAGE6_GPIO_DEFS                       (IOPIMAXES_PAGE6_GPIO_DEFS)
#define IOC_MAN_PAGE8_ISTWI_INFO_ENTRIES              (24)
#define IOC_MAN_PAGE_12_SGPIO_ENTRIES                 (IOC_NUM_PHYS/4)
#define IOC_MAN_PAGE_12_SGPIO_INFO_ENTRIES            (IOC_MAN_PAGE_12_SGPIO_ENTRIES)

#define IOC_MAN_PAGE_13_SGPIO_ENTRIES                 (16)
#define IOC_MAN_PAGE_14_SEP_ENTRIES                   (IOC_MAN_PAGE_13_SGPIO_ENTRIES)

#if COMPILE_OPTION_SAS_PORT_VIRT
#define IOC_MAN_PAGE_18_PFVP_ENTRIES                  (IOC_NUM_PHYS)
#endif /* COMPILE_OPTION_SAS_PORT_VIRT */

/* NOTE: Sensor entries for IO UNIT page 8 & 9 should be same */
#define MPI2_IOUNITPAGE8_SENSOR_ENTRIES               (6)
#define MPI2_IOUNITPAGE9_SENSOR_ENTRIES     (MPI2_IOUNITPAGE8_SENSOR_ENTRIES)

#if THUNDERBOLT
#define MPI2_MAN_PAGE_2_HW_SETTINGS_WORDS   (0x20)    /* This is the size of B0 SBR data */
#else
#define MPI2_MAN_PAGE_2_HW_SETTINGS_WORDS   (0x19)    /* This is the size of SBR data */
#endif
/* The following makes IOC_CONFIG_PAGE_MFG_PAGE_3 and MPI2_CONFIG_PAGE_MANUFACTURING_3 the same size */
#define MPI2_MAN_PAGE_3_INFO_WORDS          ((0x16C + 4*IOC_NUM_PHYS - 8)/4)
#define MPI2_MAN_PAGE_5_PHY_ENTRIES         (IOC_NUM_PHYS)
#define MPI2_MANPAGE7_CONNECTOR_INFO_MAX    (IOC_NUM_PHYS)
#define MPI2_IO_UNIT_PAGE_3_GPIO_VAL_MAX    (IOPIMAXES_PAGE6_GPIO_DEFS)
#define MPI2_BIOS_PAGE_4_PHY_ENTRIES        (IOC_NUM_PHYS)
#define MPI2_SAS_IOUNIT0_PHY_MAX            (IOC_NUM_PHYS)
#define MPI2_SAS_IOUNIT1_PHY_MAX            (IOC_NUM_PHYS)
#define MPI2_SAS_IOUNIT4_PHY_MAX            (IOC_NUM_PHYS)
#define MPI2_SAS_IOUNIT5_PHY_MAX            (IOC_NUM_PHYS)
#if (THUNDERBOLT || INVADER)
#define MPI2_SAS_IOUNIT6_GROUP_MAX          (IOC_NUM_PHYS/4)
#define MPI2_SAS_IOUNIT7_GROUP_MAX          (IOC_NUM_PHYS/4)
#else
#define MPI2_SAS_IOUNIT6_GROUP_MAX          (1)
#define MPI2_SAS_IOUNIT7_GROUP_MAX          (1)
#endif /* (THUNDERBOLT || INVADER) */

#define MPI2_IOUNITPAGE10_FUNCTION_ENTRIES  (17)

#define MPI2_SAS_PHY_PAGE_3_INST_MAX        (IOC_NUM_PHYS)
/* #define MPI2_LOG_0_LOG_DATA_LENGTH          (0x1C)   These are left alone for now
 * #define MPI2_LOG_0_NUM_LOG_ENTRIES          (1) */
#if (RAID)
#define MPI2_RAIDCONFIG0_MAX_ELEMENTS           (34)  /* 16 through phase2, 18 after, double phys disks for dual port */
#define MPI2_RAID_VOL_PAGE_0_PHYSDISK_MAX       (16)  /* 14 through phase 2, 16 after */
#define MPI2_RAID_PHYS_DISK1_PATH_MAX           (2)
#define MPI2_RAID_VOL_CREATION_NUM_PHYSDISKS    (10)
#define MPI2_EVENT_IR_CONFIG_ELEMENT_COUNT      (MPI2_RAIDCONFIG0_MAX_ELEMENTS)
#else
#define MPI2_RAIDCONFIG0_MAX_ELEMENTS           (1)
#define MPI2_RAID_VOL_PAGE_0_PHYSDISK_MAX       (1)
#define MPI2_RAID_PHYS_DISK1_PATH_MAX           (1)
#define MPI2_RAID_VOL_CREATION_NUM_PHYSDISKS    (1)
#endif

#if FW_COMPILE_OPTION_VIRTLY_REAL_FUN
#define IOPI_MAX_VFS                            7
#else
#define IOPI_MAX_VFS                            16
#endif
#define NOVA_MAX_PCI_FUNCTIONS                  (IOPI_MAX_VFS+1)
#define IOPI_MAX_SUPPORTED_IOCS                 (NOVA_MAX_PCI_FUNCTIONS)

#define MPI2_SASPHY2_PHY_EVENT_MAX              4
#define MPI2_SASPHY3_PHY_EVENT_MAX              4

#endif /* end of file */
