/***************************************************************************
 *                                                                         *
 *  Copyright 2003 LSI Logic Corporation.  All rights reserved.            *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  IOPIMSG.H
 *          Title:  SAS Firmware IOP Interface Msg Structures and Definitions
 *     Programmer:  Steve Johnson
 *  Creation Date:  February 20, 2003
 *
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface message structures and
 * definitions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPIMSG_H_INCLUDED
#define IOPIMSG_H_INCLUDED


/* Internal SMP message function number */
#define IOC_FUNCTION_SMP_INTERNAL_REQUEST   (0xF8)


/******************************************************************************
*
*      I O P I   M E S S A G E   S T R U C T S   A N D   D E F I N I T I O N S
*
******************************************************************************/

/******************************************************************************
*  Generic "Raw" IOC Message Frame
*       The message frame size is variable so this structure only defines
*       the first entry.
******************************************************************************/
typedef struct _IOP_MSG_FRAME
{
    union
    {
        U8                   Buf8[IOPI_MIN_FRAME_SIZE];
        U32                  Buf32[IOPI_MIN_FRAME_SIZE/4];
    }u;

} IOP_MESSAGE_FRAME, *PTR_IOP_MESSAGE_FRAME;

/******************************************************************************
*
*  IOC SCSI IO Request and Reply Message structures
*
******************************************************************************/

/******************************************************************************
*  IOC CONFIG  Request Context Structures
******************************************************************************/
typedef struct _IOC_CONFIG_CONTEXT
{
    PTR_PAGE_IOC                PtrInternalPage;
} IOC_CONFIG_CONTEXT, *PTR_IOC_CONFIG_CONTEXT;

/******************************************************************************
*  IOC SCSI IO LUN DEFINITION
******************************************************************************/
typedef union _IOC_LOGICAL_UNIT
{
    U8                          Byte[8];
    U16                         LevelAddressing[4];
    U32                         Dword[2];
    struct
    {
        U8                      Reserved1;
        U8                      Lun;
        U8                      Reserved2[6];
    } SingleLevel;
} IOC_LOGICAL_UNIT, *PTR_IOC_LOGICAL_UNIT;


typedef struct _IOC_TM_CONTEXT_STATUS
{
    U8          Flags;
    U8          IocNum;
    U16         IOCStatus;
} IOC_TM_CONTEXT_STATUS;


typedef union  _IOC_TM_CONTEXT_STATUS_UNION
{
    IOC_TM_CONTEXT_STATUS bits;
    U32                   Dword;
} IOC_TM_CONTEXT_STATUS_UNION;


/******************************************************************************
*  IOC TASK_MGMT Request IOP Context Structure
******************************************************************************/
// This context information is stored in the message frame, not the context
// area.
typedef struct _IOC_TASK_MGMT_IOP_CONTEXT // size = 0x18 (24)
{
    IOC_TM_CONTEXT_STATUS_UNION FlagsIocNumIOCStatus;

    U32                     IOCLogInfo;                          // 0x04
                                                                 //
    U32                     TerminationCount;                    // 0x08

#if !COMPILE_OPTION_ROUTE_HP_TO_REQQ
    /* Old gen1 method doesn't work in gen2 anymore, keep it around for GEMINI */
    U32                     SavedTailIndex;                      // 0x0C
#endif

    U32                     Next;                                // 0x10

    U32                     Event;                               // 0x14

    U32                     SMID;                                // 0x18

} IOC_TASK_MGMT_IOP_CONTEXT, *PTR_IOC_TASK_MGMT_IOP_CONTEXT;

/****************************************************************************/
/*  IOC FW Task Management Request                                          */
/****************************************************************************/

typedef struct _MSG_IOC_TASK_MGMT         // size = 0x34 (52)
{
    MPI2_SCSI_TASK_MANAGE_REQUEST           Request;

    /*
     * Context
     */
    IOC_TASK_MGMT_IOP_CONTEXT   CtxIop;
} MSG_IOC_TASK_MGMT, *PTR_IOC_TASK_MGMT;

/*****************************************************************************
*  IOC Sata Passthrough Request Message
******************************************************************************/
typedef struct _MSG_IOC_SATA_PASSTHROUGH_REQUEST
{
    U16                     DevHandle;
    U8                      ChainOffset;        /* 0x02 */
    U8                      Function;           /* 0x03 */
    U16                     PassthroughFlags;   /* 0x04 */
    U8                      SGLFlags;           /* 0x06 */
    U8                      MsgFlags;           /* 0x07 */
    U8                      VP_ID;              /* 0x08 */
    U8                      VF_ID;              /* 0x09 */
    U16                     Reserved1;          /* 0x0A */
    U32                     Reserved2;          /* 0x0C */
    U32                     Reserved3;          /* 0x10 */
    U32                     Reserved4;          /* 0x14 */
    U32                     DataLength;         /* 0x18 */
    U8                      CommandFIS[20];     /* 0x1C */
    MPI2_SIMPLE_SGE_UNION   SGL;                /* 0x30 */
} MSG_IOC_SATA_PASSTHROUGH_REQUEST, MPI2_POINTER PTR_MSG_IOC_SATA_PASSTHROUGH_REQUEST;

/*****************************************************************************
*  IOC Sata Passthrough Reply Message
******************************************************************************/
typedef struct _MSG_IOC_SATA_PASSTHROUGH_REPLY
{
    U16                     DevHandle;
    U8                      MsgLength;                       /* 02h */
    U8                      Function;                        /* 03h */
    U16                     PassthroughFlags;                /* 04h */
    U8                      Reserved1;                       /* 06h */
    U8                      MsgFlags;                        /* 07h */
    U32                     MsgContext;                      /* 08h */
    U8                      Reserved2;                       /* 0Ch */
    U8                      SASStatus;                       /* 0Dh */
    U16                     IOCStatus;                       /* 0Eh */
    U32                     IOCLogInfo;                      /* 10h */
    U8                      StatusFIS[20];                   /* 14h */
    U32                     StatusControlRegisters;          /* 28h */
    U32                     TransferCount;                   /* 2Ch */
} MSG_IOC_SATA_PASSTHROUGH_REPLY, MPI2_POINTER PTR_MSG_IOC_SATA_PASSTHROUGH_REPLY;

/*****************************************************************************
*  IOC ISTWI Request Message
******************************************************************************/
typedef struct _MSG_IOC_ISTWI_REQUEST
{
    U8      DevIndex;                               /* 00h */
    U8      Action;                                 /* 01h */
    U16     Reserved1;                              /* 02h */
    U16     TxDataLength;                           /* 04h */
    U16     RxDataLength;                           /* 06h */
    U32     Reserved2;                              /* 08h */
    U8      *PtrTxDataBuffer;                       /* 0Ch */
    U8      *PtrRxDataBuffer;                       /* 10h */

} MSG_IOC_ISTWI_REQUEST, * PTR_MSG_IOC_ISTWI_REQUEST;


/* ISTWI Request Action defines */
#define IOPI_ISTWI_REQ_ACTION_READ_DATA             (0x01)
#define IOPI_ISTWI_REQ_ACTION_WRITE_DATA            (0x02)
#define IOPI_ISTWI_REQ_ACTION_SEQUENCE              (0x03) /* Writes data first and then reads, STOP is sent at the end */
#define IOPI_ISTWI_REQ_ACTION_SLAVE_READ            (0x04) /* Reads Istwi controllers Slave buffer */
#define IOPI_ISTWI_REQ_ACTION_SLAVE_WRITE           (0x05) /* Writes to Istwi controllers Slave buffer */

/*****************************************************************************
*  IOC ISTWI Reply Message
******************************************************************************/
typedef struct _MSG_IOC_ISTWI_REPLY
{
    U8      DevIndex;                               /* 00h */
    U8      Action;                                 /* 01h */
    U8      Status;                                 /* 02h */
    U8      Reserved1;                              /* 03h */
    U32     Reserved2;                              /* 04h */
    U16     TxTransferCount;                        /* 08h */
    U16     RxTransferCount;                        /* 0Ah */
    U8      *PtrTxDataBuffer;                       /* 0Ch */
    U8      *PtrRxDataBuffer;                       /* 10h */

} MSG_IOC_ISTWI_REPLY, * PTR_MSG_IOC_ISTWI_REPLY;


/* ISTWI Reply status defines */
#define IOPI_ISTWI_STATUS_SUCCESS                   (0x00)
#define IOPI_ISTWI_STATUS_TRANSFER_FAILED           (0x01)
#define IOPI_ISTWI_STATUS_STOP_RECEIVED             (0x02)
#define IOPI_ISTWI_STATUS_ADDRESS_NOT_ACKED         (0x03)
#define IOPI_ISTWI_STATUS_TRANSFER_ABORTED          (0x04)
#define IOPI_ISTWI_STATUS_BUFFER_OVERRUN            (0x05)


/****************************************************************************
*  Union of all MPI and IOC messages
*****************************************************************************/

typedef union _MSG_IOC
{
    /* mpi2_init.h */
    MPI2_SCSI_IO_REQUEST                        ScsiIo;
    MPI2_SCSI_IO_REPLY                          ScsiIoReply;
    MPI2_SCSI_TASK_MANAGE_REQUEST               TaskMgmt;
    MPI2_SCSI_TASK_MANAGE_REPLY                 TaskMgmtReply;
    MPI2_SEP_REQUEST                            Sep;
    MPI2_SEP_REPLY                              SepReply;
    MPI2_RAID_ACCELERATOR_REQUEST               RaidAccelerator;
    MPI2_RAID_ACCELERATOR_REPLY                 RaidAcceleratorReply;
    MPI2_HBD_ACTION_REQUEST                     HBDAction;
    MPI2_HBD_ACTION_REPLY                       HBDActionReply;

    /* mpi2_cnfg.h */
    MPI2_CONFIG_REQUEST                         Config;
    MPI2_CONFIG_REPLY                           ConfigReply;

    /* mpi2_ioc.h */
    MPI2_IOC_INIT_REQUEST                       IocInit;
    MPI2_IOC_INIT_REPLY                         IocInitReply;
    MPI2_IOC_FACTS_REQUEST                      IocFacts;
    MPI2_IOC_FACTS_REPLY                        IocFactsReply;
#if CBB_COMPILE_OPTION_SRIOV_EN
    MPI2_SEND_HOST_MESSAGE_REQUEST              SendHostMessage;
    MPI2_SEND_HOST_MESSAGE_REPLY                SendHostMessageReply;
#endif

    /* mpi2_targ.h */
    MPI2_TARGET_CMD_BUF_POST_BASE_REQUEST       TargetCmdBufferPostBase;
    MPI2_TARGET_CMD_BUF_POST_LIST_REQUEST       TargetCmdBufferList;
    MPI2_TARGET_BUF_POST_BASE_LIST_REPLY        TargetCmdBuffPostBaseListReply;
    MPI2_TARGET_ASSIST_REQUEST                  TargetAssist;
    MPI2_TARGET_STANDARD_REPLY                  TargetErrorReply;
    MPI2_TARGET_STATUS_SEND_REQUEST             TargetStatusSend;
    MPI2_TARGET_MODE_ABORT                      TargetAbort;
    MPI2_TARGET_MODE_ABORT_REPLY                TargetAbortReply;

    /* mpi2_sas.h */
    MPI2_SMP_PASSTHROUGH_REQUEST                SMPPassthrough;
    MPI2_SMP_PASSTHROUGH_REPLY                  SMPPassthroughReply;
    MPI2_SAS_IOUNIT_CONTROL_REQUEST             SasIoUnitControl;
    MPI2_SAS_IOUNIT_CONTROL_REPLY               SasIoUnitControlReply;
    MPI2_SATA_PASSTHROUGH_REQUEST               SataPassthrough;
    MPI2_SATA_PASSTHROUGH_REPLY                 SataPassthroughReply;
    MPI2_FW_DOWNLOAD_REQUEST                    IocFwDownload;
    MPI2_FW_UPLOAD_REQUEST                      IocFwUpload;
    MPI2_FW_DOWNLOAD_REQUEST                    FwDownload;
    MPI2_FW_UPLOAD_REQUEST                      FwUpload;
#if COMPILE_OPTION_SAS_PORT_VIRT
    MPI2_SAS_VP_CONTROL_REQUEST                 SasVirtualPortControl;
    MPI2_SAS_VP_CONTROL_REPLY                   SasVirtualPortControlReply;
    MPI2_VP_SAS_ADDRESS_REQUEST                 VirtualPortSasAddress;
    MPI2_VP_SAS_ADDRESS_REPLY                   VirtualPortSasAddressReply;
#endif

    /*
     * This has both MPI 1.x and 2.x definitions in it already
     * This can only be eliminated if we eliminate the context storage
     */
    MSG_IOC_TASK_MGMT                       IocTaskMgmt;

    MPI2_SEP_REQUEST                        IocSep;
    MPI2_SEP_REPLY                          IocSepReply;

    MSG_IOC_ISTWI_REQUEST                   Istwi;
    MSG_IOC_ISTWI_REPLY                     IstwiReply;

    IOP_MESSAGE_FRAME                       RawFrame;

    /* mpi2.h */
    MPI2_REQUEST_HEADER                     RequestHeader;
    MPI2_DEFAULT_REPLY                      DefaultReply;

    MPI2_PORT_FACTS_REQUEST                 PortFacts;
    MPI2_PORT_FACTS_REPLY                   PortFactsReply;
    MPI2_PORT_ENABLE_REQUEST                PortEnable;
    MPI2_PORT_ENABLE_REPLY                  PortEnableReply;
    MPI2_EVENT_NOTIFICATION_REQUEST         EventNotify;
    MPI2_EVENT_NOTIFICATION_REPLY           EventNotifyReply;
    MPI2_EVENT_ACK_REQUEST                  EventAck;
    MPI2_EVENT_ACK_REPLY                    EventAckReply;
    MPI2_FW_DOWNLOAD_REPLY                  FwDownloadReply;
    MPI2_FW_UPLOAD_REPLY                    FwUploadReply;
    MPI2_PWR_MGMT_CONTROL_REQUEST           PwrMgmtControl;
    MPI2_PWR_MGMT_CONTROL_REPLY             PwrMgmtControlReply;

    /* mpi2_raid.h */
    MPI2_RAID_ACTION_REQUEST                RaidAction;
    MPI2_RAID_ACTION_REPLY                  RaidActionReply;
    MPI2_SCSI_IO_REQUEST                    RaidPtIo;
    MPI2_SCSI_IO_REPLY                      RaidPtIoReply;

    /* mpi2_tool.h */
    MPI2_TOOLBOX_CLEAN_REQUEST              ToolboxClean;
    MPI2_TOOLBOX_MEM_MOVE_REQUEST           ToolboxMemMove;
    MPI2_TOOLBOX_ISTWI_READ_WRITE_REQUEST   ToolboxIstwiReadWrite;
    MPI2_TOOLBOX_DIAGNOSTIC_CLI_REQUEST     ToolBoxDiagCli;

    MPI2_TOOLBOX_DIAGNOSTIC_CLI_REPLY       ToolBoxDiagCliReply;
    MPI2_TOOLBOX_ISTWI_REPLY                ToolboxIstwiReply;
    MPI2_TOOLBOX_REPLY                      ToolboxReply;

    MPI2_DIAG_BUFFER_POST_REQUEST           DiagBufferPostReq;
    MPI2_DIAG_BUFFER_POST_REPLY             DiagBufferPostRep;
    MPI2_DIAG_RELEASE_REQUEST               DiagBufferReleaseReq;
    MPI2_DIAG_RELEASE_REPLY                 DiagBufferReleaseRep;

} MSG_IOC, *PTR_MSG_IOC;

typedef union _IOC_REPLY_FRAMES {
    /* mpi2_init.h */
    MPI2_SCSI_IO_REPLY                          ScsiIoReply;
    MPI2_SCSI_TASK_MANAGE_REPLY                 TaskMgmtReply;
    MPI2_SEP_REPLY                              SepReply;
    MPI2_RAID_ACCELERATOR_REPLY                 RaidAcceleratorReply;

    /* mpi2_cnfg.h */
    MPI2_CONFIG_REPLY                           ConfigReply;

    /* mpi2_ioc.h */
    MPI2_IOC_INIT_REPLY                         IocInitReply;
    MPI2_IOC_FACTS_REPLY                        IocFactsReply;
    MPI2_PORT_FACTS_REPLY                       PortFactsReply;
    MPI2_PORT_ENABLE_REPLY                      PortEnableReply;
    MPI2_EVENT_NOTIFICATION_REPLY               EventNotifyReply;
    MPI2_EVENT_ACK_REPLY                        EventAckReply;
    MPI2_FW_DOWNLOAD_REPLY                      FwDownloadReply;
    MPI2_FW_UPLOAD_REPLY                        FwUploadReply;

    /* mpi2_targ.h */
    MPI2_TARGET_BUF_POST_BASE_LIST_REPLY        TargetCmdBuffPostBaseListReply;
    MPI2_TARGET_STANDARD_REPLY                  TargetErrorReply;
    MPI2_TARGET_MODE_ABORT_REPLY                TargetAbortReply;

    /* mpi2_sas.h */
    MPI2_SMP_PASSTHROUGH_REPLY                  SMPPassthroughReply;
    MPI2_SAS_IOUNIT_CONTROL_REPLY               SasIoUnitControlReply;
    MPI2_SATA_PASSTHROUGH_REPLY                 SataPassthroughReply;

    MSG_IOC_ISTWI_REQUEST                       Istwi;
    MSG_IOC_ISTWI_REPLY                         IstwiReply;

    /* mpi2.h */
    MPI2_DEFAULT_REPLY                          DefaultReply;

    /* mpi_tool.h */
    MPI2_TOOLBOX_REPLY                          ToolboxReply;
    MPI2_TOOLBOX_ISTWI_REPLY                    ToolboxIstwiReply;
    MPI2_TOOLBOX_DIAGNOSTIC_CLI_REPLY           ToolBoxDiagCliReply;
    MPI2_DIAG_BUFFER_POST_REPLY                 DiagBufferPostRep;
    MPI2_DIAG_RELEASE_REPLY                     DiagBufferReleaseRep;
} IOC_REPLY_FRAMES;

#endif /* end of file */
