/***************************************************************************
 *                                                                         *
 *  Copyright 2003-2012 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pliDefs.H
 *          Title:  Protocal Layer Interface (PLI) Definitions
 *     Programmer:  Steve Johnson
 *  Creation Date:  February 5, 2003
 *
 *
 *  Description
 *  ------------
 *  This include file contains SAS firmware Protocol Layer (PL) exports used
 *  by other FW modules to call PL functions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef PLI_DEFS_H_INCLUDED
#define PLI_DEFS_H_INCLUDED

//  These macros define the size of structures for which the IOP must allocate
//  memory.  These macros define the size of an individual element within
//  an array of these structures.  So the IOP must perform some math to
//  determine the amount of memory to be allocated.


// Definitions for PL Devhandles
typedef U16 PLI_DEV_HANDLE;
#define PLI_DEV_INVALID_HANDLE  (0)

typedef struct _PLI_RUNTIME_CONFIG
{
    U16     NumberOfMsgFrames;
    U16     NumberOfCmndBuffs;

    U16     NumberOfTargets;
    U16     NumberOfInitiators;

    U16     NumberOfVP_IDs;
    U16     NumberOfIocs;

    U16     NumberOfEnclosurePhys;
    U16     NumberOfEnclosures;

    U16     NumberOfExpanders;
    U16     NumberOfRouteTable;

    U16     Reserved1;
    U16     Flags;

    U16     Pad[6];

    U16     NumberOfVPhysPerVE; //Not used for production FW yet, set to 0
    U16     NumberOfVEs;        //Not used for production FW yet, set to 0

} PLI_RUNTIME_CONFIG, *PTR_PLI_RUNTIME_CONFIG;

#define PL_RTCFG_FLAG_NUM_ADDL_SGLS_MASK    (0x03)
#define PL_RTCFG_FLAG_ENABLE_BIDIR_CONTEXT  (0x04)


/* PL Parameter Structure */

/* PL Get Memory requirements defines for Multiplier field */
#define PLI_MEMORY_MULTIPLIER_RESERVED_0                                 (0x00)
#define PLI_MEMORY_MULTIPLIER_NUM_DEVICES                                (0x01)
#define PLI_MEMORY_MULTIPLIER_NUM_DEVICES_PLUS                           (0x02)
/* #define PLI_MEMORY_MULTIPLIER_NUM_DEVICES_PLUS_EXTRA_CONTROL_Q           (0x03) */ /* no longer needed */
#define PLI_MEMORY_MULTIPLIER_NUM_INITIATORS_PLUS_NUM_NUM_VP_IDS         (0x04)
#define PLI_MEMORY_MULTIPLIER_NUM_VP_IDS                                 (0x05)
#define PLI_MEMORY_MULTIPLIER_NUM_MIDS                                   (0x06)
#define PLI_MEMORY_MULTIPLIER_1BIT_PER_NUM_MID                           (0x07)
#define PLI_MEMORY_MULTIPLIER_NUM_COMMAND_BUFS                           (0x08)
#define PLI_MEMORY_MULTIPLIER_NUM_COMMAND_BUFS_AND_VPIDS                 (0x09)
#define PLI_MEMORY_MULTIPLIER_NUM_COMMAND_BUFS_AND_VPIDS_PLUS_FREE_PTRS  (0x0A)
#define PLI_MEMORY_MULTIPLIER_NUM_FUNCTIONS                              (0x0B)
#define PLI_MEMORY_MULTIPLIER_NUM_ENCLOSURES                             (0x0C)
#define PLI_MEMORY_MULTIPLIER_NUM_TOPOLOGY_PHYS                          (0x0D)
#define PLI_MEMORY_MULTIPLIER_NUM_TOPOLOGY_PHYS_DIV_4                    (0x0E)
#define PLI_MEMORY_MULTIPLIER_NUM_EXPANDERS                              (0x0F)
#define PLI_MEMORY_MULTIPLIER_NUM_ROUTE_TABLE                            (0x10)
#define PLI_MEMORY_MULTIPLIER_NUM_HW_PHYS                                (0x11)
#define PLI_MEMORY_MULTIPLIER_NUM_HW_PHYS_PLUS                           (0x12)
#define PLI_MEMORY_MULTIPLIER_1BIT_PER_NUM_DEVICES_PLUS                  (0x13)
#define PLI_MEMORY_MULTIPLIER_NUM_INITIATORS                             (0x14)
#define PLI_MEMORY_MULTIPLIER_NUM_RX_DMAS                                (0x15)
#define PLI_MEMORY_MULTIPLIER_NUM_LINKS                                  (0x16)
#define PLI_MEMORY_MULTIPLIER_ONE_BYTE                                   (0x17)
#define PLI_MEMORY_MULTIPLIER_MAX_PORT_WIDTH_MOD_CONFIGURATION           (0x18) /* Maximum # of port width mod groups * WindowLength==255 */
//Not used for production FW yet
#define PLI_MEMORY_MULTIPLIER_NUM_VPHYS                                  (0x19)
#define PLI_MEMORY_MULTIPLIER_NUM_VES                                    (0x1A)

#define PLI_MEMORY_MULTIPLIER_NUM_MIDS_BY_NUM_VFS                        (0x1B)
#if ENABLE_FPE_DEV_TEST_SUPPORT
#define PLI_MEMORY_MULTIPLIER_NUM_SM_ENTRIES                             (0x1C) /* Used for DevTest */
#endif /* ENABLE_FPE_DEV_TEST_SUPPORT */

/* PL Get Memory requirements defines for Attribute - Speed field */
#define PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_RESERVED_0                 (0x00)
#define PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_REQUIRED_FAST              (0x01)
#define PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_PREFERRED_FAST             (0x02)
#define PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_DONT_CARE                  (0x03)
#define PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_SLOW                       (0x04)
#define PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_MASK                       (0x07)

/* PL Get Memory requirements defines for Attribute - DMAable field */
#define PLI_MEMORY_ATTRIBUTE_NON_DMA_ABLE                            (0x00)
#define PLI_MEMORY_ATTRIBUTE_DMA_ABLE                                (0x10)
#define PLI_MEMORY_ATTRIBUTE_DMA_ABLE_MASK                           (0x10)


#define PLI_MEMORY_MIN_INITIATOR_TABLE_ENTRIES                        (128)

// used as index into MemoryRequired, MemoryDescription and MemoryBaseAddress arrays
#if INVADER

enum _PLI_MEMORY
{
/* 0*/    PLI_MEMORY_HW_DEVICE_TABLE,
/* 1*/    PLI_MEMORY_HW_INIT_TABLE,
/* 2*/    PLI_MEMORY_HW_VIRTUAL_ID_TABLE,
/* 3*/    PLI_MEMORY_HW_SATA_TABLE,
/* 4*/    PLI_MEMORY_FW_DEVICE_TABLE,
/* 5*/    PLI_MEMORY_FW_TARGET_CONTEXT,
/* 6*/    PLI_MEMORY_FW_TARGET_ARRAY,
/* 7*/    PLI_MEMORY_FW_INIT_CONTEXT,
/* 8*/    PLI_MEMORY_HW_CONTROL_Q,
/* 9*/    PLI_MEMORY_HW_DMA_TRANS_Q,
/*10*/    PLI_MEMORY_HW_COMPLETION_Q,
/*11*/    PLI_MEMORY_HW_COMMAND_RCVD_Q,
/*12*/    PLI_MEMORY_HW_COMMAND_FREE_Q,
/*13*/    PLI_MEMORY_HW_CONTEXT_MEMORY,
/*14*/    PLI_MEMORY_FW_DEV_DETAILS,
/*15*/    PLI_MEMORY_FW_PARAMETERS,
/*16*/    PLI_MEMORY_FW_ENCLOSURES,
/*17*/    PLI_MEMORY_FW_PHYTABLE_PHY,
/*18*/    PLI_MEMORY_FW_DISCOVERY_NOTUPDELAY,
/*19*/    PLI_MEMORY_FW_DISCOVERY_RESETDRIVE,
/*20*/    PLI_MEMORY_FW_EXP_DETAILS,
/*21*/    PLI_MEMORY_FW_EXP_FIFO,
/*22*/    PLI_MEMORY_FW_ROUTE_TABLE_DEV,
/*23*/    PLI_MEMORY_FW_GPLPHY,
/*24*/    PLI_MEMORY_FW_LOCAL_SGE,
/*25*/    PLI_MEMORY_FW_DEVH_USED,
/*26*/    PLI_MEMORY_FW_DEV_MISSING_TABLE,
/*27*/    PLI_MEMORY_NUM_LINKS,
/*28*/    PLI_MEMORY_NON_AUTO_FRAME,
/*29*/    PLI_MEMORY_STATIC_DMA_STRUCTS,
/*30*/    PLI_MEMORY_PORT_WIDTH_MOD_HISTORY_BUFFER,
/*31*/    PLI_MEMORY_PORT_WIDTH_MOD_SCRATCH_BUFFER,
/*32*/    PLI_MEMORY_FW_VE_TABLE,                   //Not used for production FW yet, set to 0
/*33*/    PLI_MEMORY_FW_VPORT_TABLE,                //Not used for production FW yet, set to 0
/*34*/    PLI_MEMORY_FW_VPHY_TABLE,                 //Not used for production FW yet, set to 0
/*35*/    PLI_MEMORY_FW_VEID_PPORT_TABLE,           //Not used for production FW yet, set to 0
/*36*/    PLI_MEMORY_FW_ACTIVE_ZONE_PHY_INFO_TABLE, //Not used for production FW yet, set to 0
/*37*/    PLI_MEMORY_FW_SHADOW_ZONE_PHY_INFO_TABLE, //Not used for production FW yet, set to 0
/*38*/    PLI_MEMORY_FPE_EXCEPTION_QUEUE,
/*39*/    PLI_MEMORY_FPE_SAS_COMPLETION_Q,
/*40*/    PLI_MEMORY_FPE_IOP_COMPLETION_Q,
/*41*/    PLI_MEMORY_FPE_MID_STATE_TABLE,
/*42*/    PLI_MEMORY_FPE_DEV_STATE_TABLE,
#if ENABLE_FPE_DEV_TEST_SUPPORT
/*44*/    PLI_MEMORY_FPE_DEV_SM_STATE_INFO_TABLE,
/*45*/    PLI_MEMORY_NUM_ENTRIES  // must be last entry
#else
/*44*/    PLI_MEMORY_NUM_ENTRIES  // must be last entry
#endif /* #if ENABLE_FPE_DEV_TEST_SUPPORT */
};

#else
enum _PLI_MEMORY
{
/* 0*/    PLI_MEMORY_HW_DEVICE_TABLE,
/* 1*/    PLI_MEMORY_HW_INIT_TABLE,
/* 2*/    PLI_MEMORY_HW_VIRTUAL_ID_TABLE,
/* 3*/    PLI_MEMORY_HW_SATA_TABLE,
/* 4*/    PLI_MEMORY_FW_DEVICE_TABLE,
/* 5*/    PLI_MEMORY_FW_TARGET_CONTEXT,
/* 6*/    PLI_MEMORY_FW_TARGET_ARRAY,
/* 7*/    PLI_MEMORY_FW_INIT_CONTEXT,
/* 8*/    PLI_MEMORY_HW_CONTROL_Q,
/* 9*/    PLI_MEMORY_HW_DMA_TRANS_Q,
/*10*/    PLI_MEMORY_HW_COMPLETION_Q,
/*11*/    PLI_MEMORY_HW_COMMAND_RCVD_Q,
/*12*/    PLI_MEMORY_HW_COMMAND_FREE_Q,
/*13*/    PLI_MEMORY_HW_CONTEXT_MEMORY,
/*14*/    PLI_MEMORY_FW_DEV_DETAILS,
/*15*/    PLI_MEMORY_FW_PARAMETERS,
/*16*/    PLI_MEMORY_FW_ENCLOSURES,
/*17*/    PLI_MEMORY_FW_PHYTABLE_PHY,
/*18*/    PLI_MEMORY_FW_DISCOVERY_NOTUPDELAY,
/*19*/    PLI_MEMORY_FW_DISCOVERY_RESETDRIVE,
/*20*/    PLI_MEMORY_FW_EXP_DETAILS,
/*21*/    PLI_MEMORY_FW_EXP_FIFO,
/*22*/    PLI_MEMORY_FW_ROUTE_TABLE_DEV,
/*23*/    PLI_MEMORY_FW_GPLPHY,
/*24*/    PLI_MEMORY_FW_LOCAL_SGE,
/*25*/    PLI_MEMORY_FW_DEVH_USED,
/*26*/    PLI_MEMORY_FW_DEV_MISSING_TABLE,
/*27*/    PLI_MEMORY_NUM_LINKS,
/*28*/    PLI_MEMORY_NON_AUTO_FRAME,
/*29*/    PLI_MEMORY_STATIC_DMA_STRUCTS,
/*30*/    PLI_MEMORY_PORT_WIDTH_MOD_HISTORY_BUFFER,
/*31*/    PLI_MEMORY_PORT_WIDTH_MOD_SCRATCH_BUFFER,
/*32*/    PLI_MEMORY_FW_VE_TABLE,                   //Not used for production FW yet, set to 0
/*33*/    PLI_MEMORY_FW_VPORT_TABLE,                //Not used for production FW yet, set to 0
/*34*/    PLI_MEMORY_FW_VPHY_TABLE,                 //Not used for production FW yet, set to 0
/*35*/    PLI_MEMORY_FW_VEID_PPORT_TABLE,           //Not used for production FW yet, set to 0
/*36*/    PLI_MEMORY_FW_ACTIVE_ZONE_PHY_INFO_TABLE, //Not used for production FW yet, set to 0
/*37*/    PLI_MEMORY_FW_SHADOW_ZONE_PHY_INFO_TABLE, //Not used for production FW yet, set to 0
/*38*/    PLI_MEMORY_NUM_ENTRIES  // must be last entry
};
#endif

typedef struct _PLI_GET_MEMORY_REQUIRED_ENTRY
{
    union
    {
        struct
        {
            U32   Size            :24;  /* Size in bytes of memory block */
            U32   Attributes      : 8;  /* Speed and type of memory necessary */
        } bits;
        U32 Dword;
    } u;
} PLI_GET_MEMORY_REQUIRED_ENTRY, *PTR_PLI_GET_MEMORY_REQUIRED_ENTRY;

#define PLI2_MEM_REQUIRED_SIZE_MASK                     (0x00FFFFFF)
#define PLI2_MEM_REQUIRED_SIZE_SHIFT                    (0)
#define PLI2_MEM_REQUIRED_ATTRIBUTES_MASK               (0xFF000000)
#define PLI2_MEM_REQUIRED_ATTRIBUTES_SHIFT              (24)

typedef struct _PLI_GET_MEMORY_REQUIRED
{
    U32                          NumEntries;
    PLI_GET_MEMORY_REQUIRED_ENTRY MemoryRequired[PLI_MEMORY_NUM_ENTRIES];

} PLI_GET_MEMORY_REQUIRED, *PTR_PLI_GET_MEMORY_REQUIRED;

typedef struct _PLI_SET_PARAMS
{
    U32 SenseBufferHighAddress;
    U32 ReplyFrameSize;
    U8  SenseBufferVF_ID;
    U8  Reserved8;
    U16 Reserved16;
} PLI_SET_PARAMS, *PTR_PLI_SET_PARAMS;

/* VF Queue parameters for FPE configuration */
typedef struct _PLI_FPE_VF_PARAMS
{
    U32     FPEVFRequestQueueBaseAddrLow;

    U32     Reserved0;
    U32     Reserved1;

    U16     VFReqQueueEntries;        /* number of entries available in VF Request Queue*/

    U16     Reserved2;

} PLI_FPE_VF_PARAMS, *PTR_PLI_FPE_VF_PARAMS;

typedef struct _PLI_SET_SOD_PARAMS
{
    /* IO Message Base */
    U32     HwMessageBaseAddress0Low;
    U32     HwMessageBaseAddress1Low;
    U32     HwMessageFirstSecMid;       /* First Mid in second message frame pool */
    U32     HwMessageSize;

    U32     FWDebugAreaBaseAddress;                     /* IOP defined debug area */

    U32     MemoryBaseAddress[PLI_MEMORY_NUM_ENTRIES]; /* base addresses for memory allocated by IOP for PL */

    U32     FPEMessageUnitCoreOffsets1;
    U32     FPEMessageUnitCoreOffsets2;
    PTR_PLI_FPE_VF_PARAMS   PtrFPEVFParams;                             /* array of VF FPE info */
    U32                     NumFPEVFParamEntries;                       /* the number of VF Paramss in the array */
} PLI_SET_SOD_PARAMS, *PTR_PLI_SET_SOD_PARAMS;


/* Definitions for the various critical frames that are needed */
enum
{
    PLI_CRIT_FRAME_NON_CRITICAL,
    PLI_CRIT_FRAME_CONFIG,
    PLI_CRIT_FRAME_SENSE,
    PLI_CRIT_FRAME_TASK_MGMT_DISCOVERY,
    PLI_CRIT_FRAME_TASK_MGMT_1,
    PLI_CRIT_FRAME_TASK_MGMT_2,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_0,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_1,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_2,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_3,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_4,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_5,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_6,
    PLI_CRIT_FRAME_TASK_MGMT_TXFM_7,
    PLI_CRIT_FRAME_SATA,
    PLI_CRIT_FRAME_SMP_0,
    PLI_CRIT_FRAME_SMP_1,
    PLI_CRIT_FRAME_SMP_2,
    PLI_CRIT_FRAME_SMP_3,
    PLI_CRIT_FRAME_SMP_4,
    PLI_CRIT_FRAME_SMP_5,
    PLI_CRIT_FRAME_SMP_6,
    PLI_CRIT_FRAME_SMP_7,
    PLI_CRIT_FRAME_SMP_8,
    PLI_CRIT_FRAME_SMP_9,
    PLI_CRIT_FRAME_SMP_10,
    PLI_CRIT_FRAME_SMP_11,
    PLI_CRIT_FRAME_SMP_12,
    PLI_CRIT_FRAME_SMP_13,
    PLI_CRIT_FRAME_SMP_14,
    PLI_CRIT_FRAME_SMP_15,
    PLI_CRIT_FRAME_TGT_REPLY,
    PLI_NUM_CRIT_FRAMES
};

#ifndef SAS_ADDRESS_STRUCT_DEFINED
#define SAS_ADDRESS_STRUCT_DEFINED

/*
 * SAS Address (full 8 byte WWID)
 * This structure is in little endian, so a SAS address of 0x50(MSB) 08 05 EF FF FF 80 00(LSB)
 * would show up here as High = 0xEF050850, Low = 0x0080FFFF
 */
#define SAS_WWID_LENGTH                                         (8)
#define SAS_DEVICE_NAME_LENGTH                                  (8)
typedef union _SAS_ADDRESS
{
    U8                      WWID[SAS_WWID_LENGTH];              /* 0x00 */
    struct
    {
        U32                 High;
        U32                 Low;
    }Word;

} SAS_ADDRESS, *PTR_SAS_ADDRESS;
#endif  /* SAS_ADDRESS_STRUCT_DEFINED */

#if SAS_PL_ONLY
extern const U32  gPlLibraryVersion;
#endif


#define PLI_SCSI_IO_FLAGS_NONE                          0x00000000

#define PLI_FPE_SCSI_IO_REPLY_DEST_HOST_MU_REPLY_FIFO           0
#define PLI_FPE_SCSI_IO_REPLY_DEST_IOP_COMPLETION_QUEUE         1

#if ENABLE_TRACE_BUFFER_MANAGER
/* TBM */
/* Buffer Size */
#define PLI_TBM_BUFFER_SIZE_SHIFT         (0x8)

/* Buffer Flags - DLAS */
#define PLI_TBM_BUFFER_ADDR_LOC_PLB       (0x0)
#define PLI_TBM_BUFFER_ADDR_LOC_PLB_NTA   (0x1)
#define PLI_TBM_BUFFER_ADDR_LOC_MASK      (0x3)

/* The buffer posted by host has to be 16KB aligned (i.e. last 14 bits must be zero) */
#define PLI_TBM_HOST_BUFFER_ADDR_ALIGNMENT_MASK (0x3FFF)
#define PLI_TBM_HOST_BUFFER_ADDR_ALIGNMENT      (0x4000)

/* Default Segment Size in bytes (16K bytes) */
#define PLI_TBM_DEFAULT_SEGMENT_RAM_SIZE  (0x4000)
/* Trace Buffer Size in bytes (32K bytes) */
#define PLI_TBM_TRACE_BUFFER_RAM_SIZE     (0x8000)
/* Number of TRACE_BUFFER sized entries */
#define PLI_TBM_TRACE_BUFFER_SIZE         (0x2000)
/* Maximum number of arguments supported by TBM */
#define PLI_TBM_TRACE_BUFFER_MAX_ARGS     (4)
#endif /* ENABLE_TRACE_BUFFER_MANAGER */

/* Control Request*/
/* subsystems */
#define PLI_CTRL_REQ_SUBSYSTEM_SHIFT        (16)
#define PLI_CTRL_REQ_SUBSYS_FPE             (0x0000)
#define PLI_CTRL_REQ_SUBSYS_TBM             (0x0001)
#define PLI_CTRL_REQ_SUBSYS_SASCORE         (0x0002)
#if PL_FEATURE_PL_LIGHT
#define PLI_LIGHT_CTRL_REQ_SUBSYS_FPE       (0x0010)
#endif

#define PLI_CTRL_REQ_SUBSYS_VS              (0xFF00)

/* operations */
#define PLI_CTRL_REQ_OPERATION_MASK                     (0x0000FFFF)
/* FPE ops */
#define PLI_CTRL_REQ_OPERATION_FPE_PAUSE_REQUESTOR      ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000001)
#define PLI_CTRL_REQ_OPERATION_FPE_UNPAUSE_REQUESTOR    ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000002)
#define PLI_CTRL_REQ_OPERATION_FPE_PAUSE_COMPLETOR      ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000003)
#define PLI_CTRL_REQ_OPERATION_FPE_UNPAUSE_COMPLETOR    ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000004)
#define PLI_CTRL_REQ_OPERATION_FPE_WRITE_FPE_VF_ENABLED ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000007)
#define PLI_CTRL_REQ_OPERATION_FPE_READ_FPE_VF_ENABLED  ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000008)
#define PLI_CTRL_REQ_OPERATION_FPE_SET_REPLYDEST_DEFAULT ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000009)
#define PLI_CTRL_REQ_OPERATION_FPE_FAULT                ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x0000000A)

#define PLI_CTRL_REQ_OPERATION_FPE_SET_LD_TABLE_BASE    ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x0000000B)
#define PLI_CTRL_REQ_OPERATION_FPE_PAUSE_HOST_REQUESTS  ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x0000000C)
#define PLI_CTRL_REQ_OPERATION_FPE_RESUME_HOST_REQUESTS ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x0000000D)

#define PLI_CTRL_REQ_OPERATION_FPE_DISABLE_FPE_DEVICE   ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000010)
#define PLI_CTRL_REQ_OPERATION_FPE_ENABLE_FPE_DEVICE    ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000011)
#define PLI_CTRL_REQ_OPERATION_FPE_DISABLE_FPE_ALL      ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000012)
#define PLI_CTRL_REQ_OPERATION_FPE_ENABLE_FPE_ALL       ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000013)

#define PLI_CTRL_REQ_OPERATION_FPE_ROUTE_TO_EXCEPT_Q    ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000014)
#define PLI_CTRL_REQ_OPERATION_FPE_LD_TABLE_ENABLE      ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000015)

#define PLI_CTRL_REQ_OPERATION_FPE_SET_POWER_SAVINGS_MODE      ((PLI_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000016)

/* TBM ops */
#define PLI_CTRL_REQ_OPERATION_TBM_INIT                 ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000001)
#define PLI_CTRL_REQ_OPERATION_TBM_SET_TIMESTAMP        ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000002)
#define PLI_CTRL_REQ_OPERATION_TBM_GET_TRACE_BUFFER     ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000003)
#define PLI_CTRL_REQ_OPERATION_TBM_ENABLE_EXT_MODE      ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000004)
#define PLI_CTRL_REQ_OPERATION_TBM_RELEASE_EXT_BUFFER   ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000005)
#define PLI_CTRL_REQ_OPERATION_TBM_PAUSE                ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000006)
#define PLI_CTRL_REQ_OPERATION_TBM_UNPAUSE              ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000007)
#define PLI_CTRL_REQ_OPERATION_TBM_GET_TRACE_DWORD      ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000008)
#define PLI_CTRL_REQ_OPERATION_TBM_SET_TRACETYPE_DEVICE ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000009)
#define PLI_CTRL_REQ_OPERATION_TBM_SET_TRACETYPE_ALL    ((PLI_CTRL_REQ_SUBSYS_TBM << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x0000000A)

/* SAS Core ops */
#define PLI_CTRL_REQ_OPERATION_SASCORE_GET_REV                      ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000001)
#define PLI_CTRL_REQ_OPERATION_SASCORE_SET_NONGOOD_RESPONSE         ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000002)
#define PLI_CTRL_REQ_SET_CONTROL_PLB_SNOOP_BASE_ADDR_AND_SIZE       ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000003)
#define PLI_CTRL_REQ_GET_CONTROL_PLB_SNOOP_BASE_ADDR_AND_SIZE       ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000004)
#define PLI_CTRL_REQ_SET_CONTROL_PLB_DIVIDER_POWER_AND_CONFIG_PORTS ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000005)
#define PLI_CTRL_REQ_GET_CONTROL_PLB_DIVIDER_POWER_AND_CONFIG_PORTS ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000006)
#define PLI_CTRL_REQ_SET_DMA_PLB_SNOOP_BASE_ADDR_AND_SIZE           ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000007)
#define PLI_CTRL_REQ_GET_DMA_PLB_SNOOP_BASE_ADDR_AND_SIZE           ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000008)
#define PLI_CTRL_REQ_SET_DMA_PLB_DIVIDER_POWER_AND_CONFIG_PORTS     ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000009)
#define PLI_CTRL_REQ_GET_DMA_PLB_DIVIDER_POWER_AND_CONFIG_PORTS     ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x0000000A)
#define PLI_CTRL_REQ_SET_SINGLE_DUAL_CONTEXT_TXDMA                  ((PLI_CTRL_REQ_SUBSYS_SASCORE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x0000000B)

/* Vendor specific */
#define PLI_CTRL_REQ_OPERATION_VENDOR_SPECIFIC_1        ((PLI_CTRL_REQ_SUBSYS_VS << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000001)
#define PLI_CTRL_REQ_OPERATION_VENDOR_SPECIFIC_2        ((PLI_CTRL_REQ_SUBSYS_VS << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000002)
#define PLI_CTRL_REQ_OPERATION_VENDOR_SPECIFIC_3        ((PLI_CTRL_REQ_SUBSYS_VS << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000003)
#define PLI_CTRL_REQ_OPERATION_VENDOR_SPECIFIC_4        ((PLI_CTRL_REQ_SUBSYS_VS << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000004)

/* PL Light ops */
#if PL_FEATURE_PL_LIGHT
#define PLI_LIGHT_CTRL_REQ_OPERATION_FPE_SET_REPLYDEST_DEFAULT ((PLI_LIGHT_CTRL_REQ_SUBSYS_FPE << PLI_CTRL_REQ_SUBSYSTEM_SHIFT) | 0x00000001)
#endif

typedef struct _PLI_FPE_MID_INFO
{
    U8      Timerbucket;
    U8      MSIIndex;
    U8      VF_ID;
    U8      FPEStatus;

    U16     FPEFlags;
    U16     DeviceHandle;

    U16     FPEFWFlags;
    U16     PendNextMID;

    union
    {
        struct
        {
            U16 SMID;
            U16 Reserved0;

        }Mpi;
        U32 I2oHostContext;
    }HostRequest;

} PLI_FPE_MID_INFO,*PTR_PLI_FPE_MID_INFO;


#define PLI_FPE_MID_INFO_FPEFLAGS_FPE_OWNED     (0x0001)


/*  FPE status of a SCSI_IO as follows: */
#define PLI_FPESTATUS_NONE                       0x00
#define PLI_FPESTATUS_DEVICE_FP_DISABLED         0x01
#define PLI_FPESTATUS_INVALID_DESCRIPTOR_TYPE    0x02
#define PLI_FPESTATUS_DEVHANDLE_OUT_OF_RANGE     0x03
#define PLI_FPESTATUS_FPE_MU_REQUEST             0x04
#define PLI_FPESTATUS_QDEPTH_LIMIT               0x05
#define PLI_FPESTATUS_VF_MISMATCH                0x06
#define PLI_FPESTATUS_LD_FP_DISABLED             0x07
#define PLI_FPESTATUS_LD_VF_MISMATCH             0x08
#define PLI_FPESTATUS_LD_OUT_OF_RANGE            0x09
#define PLI_FPESTATUS_LD_CFG_MISMATCH            0x0A



/* FPE Power Management Modes */
typedef enum _PLI_FPE_POWER_SAVING_MODE
{
    PLI_FPE_PSM_NONE        = 0,
    PLI_FPE_PSM_LIGHT_SLEEP = 1,
    PLI_FPE_PSM_DEEP_SLEEP  = 2,
    PLI_FPE_PSM_SHUTDOWN    = 3

} PLI_FPE_POWER_SAVING_MODE;

#endif /* End PLI_H_INCLUDED */
