/***************************************************************************
 *                                                                         *
 *  Copyright 2003-2012 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pliFault.h
 *          Title:  SAS Firmware IOP Interface IFAULT code Definitions
 *     Programmer:
 *  Creation Date:  November 30, 2007
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface IFAULT codes
 *
 *-------------------------------------------------------------------------
 */

#ifndef PLI_FAULT_H_INCLUDED
#define PLI_FAULT_H_INCLUDED


/******************************************************************************
* Generic Interface IFAULT code definitions 0 - 0xFFFF
*******************************************************************************/

/* See iopiFault.h */

/******************************************************************************
* IOP IFAULT code definitions 0x0000 - 0x3FFF
*******************************************************************************/

/* See iopiFault.h */

/******************************************************************************
* PL IFAULT code definitions 0x4000 - 0x7FFF
*******************************************************************************/
//#define IFAULT_PL_INVALID_DEVICE_TYPE                         (0x4000)
//#define IFAULT_PL_UNHANDLED_INTERRUPT                         (0x4100)
#define IFAULT_PL_UNHANDLED_INTERRUPT_LINK_1                    (0x4101) /* Encountered an unhandled link-layer interrupt.  */
//#define IFAULT_PL_UNHANDLED_INTERRUPT_LINK_2                  (0x4102)
//#define IFAULT_PL_UNHANDLED_INTERRUPT_LINK_3                  (0x4103)
//#define IFAULT_PL_UNHANDLED_INTERRUPT_PORT                    (0x4102)
//#define IFAULT_PL_UNHANDLED_INTERRUPT_QUAD_ERROR              (0x4103)
//#define IFAULT_PL_UNHANDLED_INTERRUPT_QUAD_INT                (0x4104)
//#define IFAULT_PL_UNHANDLED_INTERRUPT_INT_TRANS               (0x4105)
//#define IFAULT_PL_UNHANDLED_INTERRUPT_ERR_TRANS               (0x4106)
#define IFAULT_PL_INVALID_CALLBACK_STATE                        (0x4200) /* Tried to use an undefined callback. */
#define IFAULT_PL_CALLBACK_STACK_UNDERFLOW_2                    (0x4202) /* Callback stack head == 0. */
#define IFAULT_PL_CALLBACK_STACK_OVERFLOW                       (0x4203) /* Callback stack head == PL_MAX_CALLBACKS. */
#define IFAULT_PL_CALLBACK_STACK_INVALID_DEVH                   (0x4204) /* Found a device handle of zero in the DMA transmission queue. */
#define IFAULT_PL_INVALID_CALLBACK_1                            (0x4205) /* An invalid SMP response callback occurred on a port that was not currently under discovery. */
#define IFAULT_PL_INVALID_CALLBACK_2                            (0x4206) /* An invalid SMP response callback occurred while discovery was inactive. */
#define IFAULT_PL_INVALID_CALLBACK_3                            (0x4207) /* Encountered an invalid callback state in the SMP cleanup. */
#define IFAULT_PL_CALLBACK_STACK_UNDERFLOW                      (0x4300) /* Callback stack head == 0. */

//#define IFAULT_PL_INVALID_PHY_NUM                             (0x4300)
//#define IFAULT_PL_FRM_ALLOC_FAIL_DISC_EVT                     (0x4320)
//#define IFAULT_PL_FRM_ALLOC_FAIL_ENC_DEV_CNG                  (0x4321)
//#define IFAULT_PL_FRM_ALLOC_FAIL_TOP_CNG_LST                  (0x4322)
//#define IFAULT_PL_DISCOVERY_WRONG_SMP_FUNC                    (0x4400)
#define IFAULT_PL_DISCOVERY_ROUTE_TBL_INVALID_ACCESS            (0x4410) /* Erroneously tried to release a route table lock. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_FULL                      (0x4420) /* Failed to create a phy table entry for the controller phys. */
#define IFAULT_PL_DISCOVERY_SMP_OUTSTANDING                     (0x4430) /* Discovery SMPs were outstanding when expander discovery began. */
#define IFAULT_PL_DISCOVERY_INVALID_OPERATION_1                 (0x4441) /* Encountered an invalid SATA Reset state. */
#define IFAULT_PL_DISCOVERY_INVALID_OPERATION_2                 (0x4442) /* Encountered an invalid Discovery state. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_1                     (0x4451) /* Failed to locate the phy table index for an expander phy. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_2                     (0x4452) /* Failed to reserve the phy table entry.  */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_3                     (0x4453) /* Failed to find the phy table entry SMP_FUNCTION_DISCOVER. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_4                     (0x4454) /* Failed to find the phy table entry SMP_FUNCTION_DISCOVER_LIST. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_5                     (0x4455) /* Failed to find the phy table entry SMP_FUNCTION_REPORT_PHY_SATA. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_6                     (0x4456) /* Failed to find the phy table entry. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_7                     (0x4457) /* Could not find the phy table entry (SMP_FUNCTION_PHY_CONTROL). */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_8                     (0x4458) /* Could not find the phy table entry (SMP_FUNCTION_REPORT_EDFB_SETTINGS). */
#define IFAULT_PL_DISCOVERY_INVALID_STATE                       (0x4460) /* Encountered an invalid Discovery state. */
#define IFAULT_PL_DISCOVERY_PHY_TABLE_ERR_9                     (0x4461) /* Failed to find the phy table entry.  */
//#define IFAULT_PL_DEVICE_TABLE_FULL                           (0x4500)
//#define IFAULT_PL_FREED_UNUSED_HWDEVINDEX                     (0x4800)
#define IFAULT_PL_DEVMGR_INVALID_SSP_MAXQDEPTH                  (0x4850) /* Invalid MaxQdepth specified in NVDATA SASNarrowMaxQDepth field. */
#define IFAULT_PL_INVALID_SSPT_DEV_QDEPTH_INCR                  (0x4851) /* While incrementing Qdepth, the current QDepth for SAS device is found to be less than the minimum Qdepth allowed. */
#define IFAULT_PL_INVALID_SSPT_DEV_QDEPTH_DECR                  (0x4852) /* While decrementing Qdepth, the current QDepth for SAS device is found to be less than the minimum Qdepth allowed.  */
#define IFAULT_PL_INVALID_MID                                   (0x4900) /* Invalid MID used. */
#define IFAULT_PL_LINK_LAYER_PARITY_ERROR                       (0x4A00) /* Link Layer Parity error detected. */
#define IFAULT_PL_LINK_LAYER_PARITY_ERROR_TXFM                  (0x4A01) /* Link Layer Parity error detected. Possibly due to TxFM cleanup */
//#define IFAULT_PL_UNUSED                                      (0x4B00)
#define IFAULT_PL_SATA_ENGINE_RAM_PARITY_ERROR                  (0x4C00) /* Encountered a fatal parity error in the SATA engine. */
//#define IFAULT_PL_SATA_PR_OUT_INVALID_PHYTABLE                (0x4C1F)
#define IFAULT_PL_SATA_UNEXPECTED_MID                           (0x4C20) /* Failed to find the current MID in the expected location. */
#define IFAULT_PL_SATA_INVALID_COMMAND_WRV                      (0x4C30) /* Decode of the Write Verify command failed. */
#define IFAULT_PL_SATA_INVALID_COMMAND_VR                       (0x4C31) /* Decode of the Verify command failed. */
#define IFAULT_PL_SATA_UNEXPECTED_SECTOR_SIZE                   (0x4C32) /* Recieved unexpected or unsupported sector size. */
#define IFAULT_PL_ATAPI_UNEXPECTED_FIS_1                        (0x4D01) /* The controller received an unexpected FIS (Frame Information Structure). */
#define IFAULT_PL_ATAPI_UNEXPECTED_FIS_2                        (0x4D02) /* The controller received an unexpected FIS (Frame Information Structure). */
#define IFAULT_PL_ATAPI_UNEXPECTED_FIS_3                        (0x4D03) /* The controller received an unexpected FIS (Frame Information Structure). */
#define IFAULT_PL_ATAPI_UNEXPECTED_FIS_4                        (0x4D04) /* The controller received an unexpected FIS (Frame Information Structure). */
#define IFAULT_PL_ATAPI_INVALID_SENSE_COUNT                     (0x4D05) /* Incorrect reply sense count. */
#define IFAULT_PL_ATAPI_UNEXPECTED_FPE_STATE                    (0x4D06) /* Unexpected FPE state. */
#define IFAULT_PL_ATAPI_UNEXPECTED_ADDRESS_SPACE                (0x4D07) /* Unexpected address space. */
#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_EVENT_1                (0x4E01) /* Failed to initialize the IOP message frame. */
#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_EVENT_2                (0x4E02) /* An unhandled case occurred in the SGL extraction. */
//#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_CONFIG               (0x4E02)
#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_SENSE_1                (0x4E03) /* Failed to allocate a critical frame for the sense data. */
#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_SENSE_2                (0x4E04) /* Failed to allocate a critical frame for the sense data. */
#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_SENSE_3                (0x4E05) /* Failed to allocate a critical frame for the sense data. */
//#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_TASK_MGMT            (0x4E04)
//#define IFAULT_PL_NO_IOP_RESOURCE_FRAMES_SATA_INIT            (0x4E05) /* Failed to allocate non-critical frame during SATA init. */
#define IFAULT_PL_LINK_TIMER_FAILED_1                           (0x4F01) /* Failed to initialize the timer in the Link Reset state machine. */
#define IFAULT_PL_LINK_TIMER_FAILED_2                           (0x4F02) /* Failed to initialize the timer in the Link Reset state machine setup. */
#define IFAULT_PL_PRSNC_DTCT_TIMER_CREATION_FAILED              (0x4F03) /* Failed to allocate presence detect timer. */
#define IFAULT_PL_LINK_RESET_INITIALFIS_RCVD_INVALID            (0x5001) /* Unexpected state on receiving initial FIS (Frame Information Structure). */
#define IFAULT_PL_LINK_RESET_IDENTIFY_RCVD_INVALID              (0x5002) /* Unexpected state on receiving identify. */
#define IFAULT_PL_LINK_RESET_SERDES_CONFIG_PWR_ON_PHY           (0x5003) /* Reminder if firmware needs to configure serdes registers to power on the phy. */
#define IFAULT_PL_LINK_RESET_SERDES_CONFIG_PWR_OFF_PHY          (0x5004) /* Reminder if firmware needs to configure serdes registers to power off the phy. */
#define IFAULT_PL_LINK_RESET_CONFIG_LINK_RATE_FAILED            (0x5030) /* Link rate configuration failed. */
#define IFAULT_PL_LINK_RESET_SEQUENCE_FAILURE                   (0x5040) /* The link reset sequence failed (0x504x, where x = next state). */
#define IFAULT_PL_QUEUE_MGR_INIT_FAILED                         (0x5100) /* Failed to acquire manufacturing page 15 for queue manager initialization. */
//#define IFAULT_PL_COMPQ_PARITY                                (0x5101)
//#define IFAULT_PL_QUEUE_DMAQ_PARITY                           (0x5102)
//#define IFAULT_PL_QUEUE_DMAQ_OVERFLOWIOP                      (0x5103)
//#define IFAULT_PL_QUEUE_DMAQ_OVERFLOWRXFM                     (0x5104)
//#define IFAULT_PL_QUEUE_IQRAM_PARITY                          (0x5105)
//#define IFAULT_PL_QUEUE_MGR_IQRAM_PARITY                      (0x5106)
//#define IFAULT_PL_QUEUE_RXFMSCHED_PARITY                      (0x5107)
#define IFAULT_PL_QUEUE_MGR_DMA_DISABLE_TIMEOUT                 (0x5108) /* QM - a DMA engine did not go disabled or to wait state. */
#define IFAULT_PL_CONTROL_QUEUE_DIS_TIMEOUT_1                   (0x5109) /* Control queue disable has timed out. */
#define IFAULT_PL_CONTROL_QUEUE_DIS_TIMEOUT_2                   (0x510A) /* Control queue disable has timed out. */
#define IFAULT_PL_CONTROL_QUEUE_LIST_CORRUPTION_1               (0x510B) /* Control queue's list corruption detected. */
#define IFAULT_PL_CONTROL_QUEUE_LIST_CORRUPTION_2               (0x510C) /* Control queue's list corruption detected. */
#define IFAULT_PL_CONTROL_QUEUE_LIST_CORRUPTION_3               (0x510D) /* Control queue's list corruption detected. */
#define IFAULT_PL_CONTROL_QUEUE_EXPANSION                       (0x510E) /* Control queue is expanding beyond the number of devices we have. */

//#define IFAULT_PL_RXDMA_CLEANUP_IN_PROG_MISMATCH              (0x5111) /* Tried to clear NonAuto for an IO that was not marked as CleanupInProg. */
#define IFAULT_PL_RX_CLN_ABORT_LIST_CORRUPTED                   (0x5112) /* Number of I/Os on abort list has decreased; implies abort list corruption. */
#define IFAULT_PL_QUEUE_MGR_BAD_TXDMA_QUEUE_ENTRY               (0x5113) /* Trying to build a bad SGE to add an entry to the TxDMA Queue. */
#define IFAULT_PL_QUEUE_MGR_BAD_COMPL_QUEUE_REQUEST             (0x5114) /* Trying to add an entry to an invalid completion queue. */
#define IFAULT_PL_QUEUE_MGR_TXDMA_QUEUE_WONT_STOP               (0x5115) /* Trying to stop DMAQ dequeing to add entry to the TxDMA Queue. */
#define IFAULT_PL_QUEUE_MGR_TXDMA_QUEUE_WONT_ENQ                (0x5116) /* Trying to add an entry to the TxDMA Queue but it is not EnQing. */
#define IFAULT_PL_QUEUE_MGR_DMA_GROUP_DUP                       (0x5117) /* Found more devices than we have DMA Groups. */

#define IFAULT_PL_DMAQ_LENGTH_ZERO                              (0x5120) /* Zero Length is passed to DMAQ. */
#define IFAULT_PL_DMAQ_NO_MID                                   (0x5121) /* No mid for DMAQ entry */
//#define IFAULT_PL_DMA_ENGINE_RAM_PARITY_ERROR                 (0x5200)
#define IFAULT_PL_UNEXPECTED_TX_FM_ERROR                        (0x5300) /* Encountered an unexpected Tx frame manager error. */
//#define IFAULT_PL_CONTROLQ_CLNUP_TX_FM_NOT_FOUND              (0x5301)
#define IFAULT_PL_TX_FM_CLEANUP_NOT_STOPPING                    (0x5302) /* Tx Frame Manager state machine did not reache IDLE state as expected during Tx frame manager clean up. */
#define IFAULT_PL_TX_FM_NULL_PTRPTRCURBUF                       (0x5303) /* Encountered NULL address of pointer to current buffer. */
#define IFAULT_PL_TX_FM_BAD_ERROR_MID_1                         (0x5304) /* Error mid in TxFM cleanup is bad. */
#define IFAULT_PL_TX_FM_BAD_ERROR_MID_2                         (0x5305) /* Error mid in TxFM cleanup is bad. */
#define IFAULT_PL_TX_FM_START_WHILE_RUNING                      (0x5306) /* Tried to start a TxFM which is already running. */
#define IFAULT_PL_PHYLINK_TABLE_PARITY_ERROR                    (0x5400) /* Phy link table contained a parity error. */
//#define IFAULT_PL_HW_CLEANUP_SE_TASK_FILE_BITS                (0x5500)
//#define IFAULT_PL_ENTRY_NOT_FOUND_GENERIC                     (0x5600)
//#define IFAULT_PL_PASS_THRU_SEND_REENTRY                      (0x5700)
#define IFAULT_PL_UNHANDLED_QUEUE_MANAGER_INT_1                 (0x5801) /* Tried to enqueue the DMA request when no entry was available. */
#define IFAULT_PL_UNHANDLED_QUEUE_MANAGER_INT_2                 (0x5802) /* A clear affiliation request occurred to an ID with outstanding I/Os, or a clear affiliation request was sent to a non-STP device, or an attempt occurred to enqueue a full control queue group.  */
#define IFAULT_PL_UNHANDLED_QUEUE_MANAGER_INT_3                 (0x5803) /* The master's completion interface received a nonzero completion status, a parity error was detected at the master's XferInbound interface, or a tag greater than 6 was received at the master's completion or inbound interface. */
#define IFAULT_PL_UNHANDLED_QUEUE_MANAGER_INT_4                 (0x5804) /* A parity error was detected at the slave's write interface or a reserved address was detected at the slave's read or write interface. */
#define IFAULT_PL_UNHANDLED_QUEUE_MANAGER_INT_5                 (0x5805) /* Encountered an invalid MID or ID during a firmware-controlled enqueue operation. */
#define IFAULT_PL_UNHANDLED_QUEUE_MANAGER_INT_6                 (0x5806) /* Encountered an invalid MID or ID during a firmware-controlled DMA enqueue operation. */
#define IFAULT_PL_UNHANDLED_TX_FRAME_MANAGER_INT_1              (0x5811) /* A fault occurred for several unhandled Tx frame manager interrupts.*/
#define IFAULT_PL_UNHANDLED_TX_FRAME_MANAGER_INT_2              (0x5812) /* An unhandled SATA length mapping error occurred. */
#define IFAULT_PL_TX_FRAME_MGR_INTERRUPT_NOT_CLEARED            (0x5813) /* TxFrmMgr interrupt not cleared after doing control queue cleanup. */
#define IFAULT_PL_TX_FRAME_MGR_INTERRUPT_MID_IN_TM_LIST         (0x5814) /* TxFrmMgr interrupted on a mid which is already on a TM's list. */
#define IFAULT_PL_UNHANDLED_RX_FRAME_MANAGER_INT_1              (0x5821) /* An unhandled error in the Rx frame manager interrupt handler occurred. */
#define IFAULT_PL_UNHANDLED_RX_FRAME_MANAGER_INT_2              (0x5822) /* An unhandled error in the Rx frame manager interrupt handler occurred. */
#define IFAULT_PL_UNHANDLED_RX_FRAME_MANAGER_INT_3              (0x5823) /* An unhandled error in the Rx frame manager interrupt handler occurred. */
#define IFAULT_PL_UNHANDLED_RX_FRAME_MANAGER_INT_4              (0x5824) /* An unhandled error in the Rx frame manager interrupt handler occurred. */
#define IFAULT_PL_UNHANDLED_RX_FRAME_MANAGER_INT_5              (0x5825) /* An important SATA frame was dropped.  */

#define IFAULT_PL_UNHANDLED_RX_FRAME_MANAGER_INT_FSM_TO         (0x5827) /* TxFrmMgr FSM (or Send FSM) state machine is stuck!. */
#define IFAULT_PL_RX_FRAME_MANAGER_FAILED_STOP                  (0x5828) /* RxFrameMgr failed to stop. */

#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_INT_1                (0x5831) /* Unhandled errors occurred in the Tx DMA interrupt handler. */
#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_INT_2                (0x5832) /* An unhandled DMA transmission queue type occurred. */
#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_INT_3                (0x5833) /* Unhandled interrupt which did not get detected by plTxCtxMgrFaultOnInterrupt. */
#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_INT_4                (0x5834) /* An unhandled DMA type occurred. */
#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_INT_5                (0x5835) /* An unhandled error in Tx DMA interrupt handler occurred. */
#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_INT_6                (0x5836) /* An unhandled error in Tx DMA interrupt handler occurred. */
#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_INT_7                (0x5837) /* An unhandled error in Tx DMA interrupt handler occurred. */
#define IFAULT_PL_TX_CTX_MANAGER_INT_FATAL_ERROR                (0x5838) /* Received CntxError (bit 2 of status) in txctxmgr and it is not of the non-auto type. */
//#define IFAULT_PL_TX_CTX_MANAGER_BAD_NON_AUTO_INT_A             (0x5839) /* Received CntxError (bit 2 of status) of non-auto type, but the cleanup fails. */
//#define IFAULT_PL_TX_CTX_MANAGER_BAD_NON_AUTO_INT_B             (0x583A) /* Received CntxError (bit 2 of status) of non-auto type, but the cleanup fails. */
#define IFAULT_PL_TX_CTX_MANAGER_IMPOSSIBLE_NON_AUTO            (0x583B) /* This interrupt should never happen when the txctxhang workaround is on. */
#define IFAULT_PL_UNHANDLED_TX_CTX_MANAGER_CTX_ERR_INT          (0x583C) /* Unhandled TxCtxMgr Context interrupt. */
#define IFAULT_PL_TX_CTX_MANAGER_INV_CTX                        (0x583D) /* Trying to get a pointer to an invalid context. */
#define IFAULT_PL_TX_CTX_MANAGER_NO_DATO_INT                    (0x583E) /* The TxCtxMgr DATO interrupt did not fire and data length not less than 1K. */
#define IFAULT_PL_TX_CTX_MANAGER_BAD_MID                        (0x583F) /* Bad Mid in the TxCtxMgr registers. */
#define IFAULT_PL_TX_CTX_BAD_HALFBUSY_MODE                      (0x5840) /* TxCtxMgr in BusyWhenHalfBusy mode when it should not be. */

#define IFAULT_PL_UNHANDLED_RX_CTX_MANAGER_INT_1                (0x5841) /* Unhandled errors occurred in the Rx context manager interrupt handler. */
#define IFAULT_PL_UNHANDLED_RX_CTX_MANAGER_INT_2                (0x5842) /* Unhandled error occurred  in the Rx context manager interrupt handler. */
#define IFAULT_PL_UNHANDLED_RX_CTX_MANAGER_INT_3                (0x5843) /* Unhandled error occurred in the Rx context manager interrupt handler. */
#define IFAULT_PL_UNHANDLED_RX_CTX_MANAGER_ADDR_CONT            (0x5844) /* Encountered an Rx context manager address contention interrupt. */
#define IFAULT_PL_RX_CTX_MANAGER_INT_NOT_VALID                  (0x5845) /* ContextValid was never asserted. */
#define IFAULT_PL_RX_CTX_MANAGER_BAD_NON_AUTO_INT               (0x5846) /* Received a frame for an I/O with the non-auto bit set, but the MID is not in the scope of a TM. */
#define IFAULT_PL_RX_CTX_MANAGER_DMA_OFFSET_ERROR               (0x5847) /* DMA Engine start address not equal to DMA Completion Address. */
#define IFAULT_PL_RX_CTX_MANAGER_DATA_OVERRUN_ERROR             (0x5848) /* Attempting to send more data than requested. */
#define IFAULT_PL_RX_CTX_MANAGER_PARITY_ERROR                   (0x5849) /* Rx Context Manager internal parity error. */
#define IFAULT_PL_RX_CTX_MANAGER_INT_NOT_VALID_2                (0x584A) /* ContextValid never asserted. */
#define IFAULT_PL_RX_CTX_MANAGER_UNHANDLED_DMAOVERFLOW_INT      (0x584B) /* Rx Context Manager triggered unhandled RxDMA Overflow Interrupt. */
#define IFAULT_PL_RX_CTX_MANAGER_WA_INVALID_LEN                 (0x584C) /* Unable to clear a single frame of invalid size from the RxDma buffer. */

#define IFAULT_PL_UNHANDLED_TX_DMA_ENGINE_INT_1                 (0x5851) /* Unhandled errors occurred in the Tx DMA engine interrupt handler. */
#define IFAULT_PL_UNHANDLED_TX_DMA_ENGINE_INT_2                 (0x5852) /* An unhandled DMA type occurred. */
#define IFAULT_PL_UNHANDLED_TX_DMA_ENGINE_INT_3                 (0x5853) /* Unhandled DMA type - new interrupts for Gen-3 or previously unused. */
#define IFAULT_PL_UNHANDLED_TX_DMA_ENGINE_INT_4                 (0x5854) /* Received a TxDMA SysDMA error, which is an unhandled interrupt from the SAS Core. */
#define IFAULT_PL_UNHANDLED_RX_DMA_ENGINE_INT_1                 (0x5861) /* Unhandled errors occurred in the RxDMA engine interrupt handler. */
#define IFAULT_PL_UNHANDLED_RX_DMA_ENGINE_INT_2                 (0x5862) /* Unhandled DMA type - new interrupts for Gen-3 or previously unused. */
#define IFAULT_PL_UNHANDLED_RX_DMA_ENGINE_INT_3                 (0x5863) /* Received an RxDMA SysDMA error, which is an unhandled interrupt from the SAS Core. */
#define IFAULT_PL_UNHANDLED_RX_DMA_ENGINE_WA_INVALID_CONDITION  (0x5864) /* PL_HW_WORKAROUND_RX_FRM_MGR_DISABLE_TIMER failed. */

#define IFAULT_PL_UNHANDLED_PORT_LAYER_INT_1                    (0x5871) /* Unhandled errors occurred in the port layer Tx interrupt handler. */
#define IFAULT_PL_UNHANDLED_PORT_LAYER_INT_2                    (0x5872) /* Encountered an invalid device handle in the port PLLC interrupt handler. */
#define IFAULT_PL_UNHANDLED_RX_EEDP_INT_1                       (0x5881) /* Unhandled errors occurred in the Rx EEDP interrupt handler. */
#define IFAULT_PL_UNHANDLED_RX_EEDP_INT_2                       (0x5882) /* An invalid device handle occurred in the Rx EEDP context register. */

#define IFAULT_PL_RX_EEDP_STALLNEWFRAMES_STUCK                  (0x5885) /* The Rx EEDP hardware did not suspend for the BT1783 workaround. */
#define IFAULT_PL_RX_EEDP_NOT_SUSPENDED_BT1786                  (0x5886) /* The Rx EEDP hardware did not suspend for the BT1786 workaround. */

#define IFAULT_PL_TX_DMA_CTX_INVALID_1                          (0x5890) /* The TxDma Context (A/B) is invalid. */
#define IFAULT_PL_TX_DMA_CTX_INVALID_2                          (0x5891) /* The TxDma Context (A/B) is invalid. */
#define IFAULT_PL_TX_DMA_CTX_INVALID_3                          (0x5892) /* The TxDma Context (A/B) is invalid. */
#define IFAULT_PL_TX_DMA_CTX_NOT_CAPTURED                       (0x5893) /* The TxDma Context (A/B) was not set by hardware. */
#define IFAULT_PL_TX_DMA_CTX_BAD_USAGE                          (0x5894) /* Incorrect attempt to get Tx context (A or B) after an interrupt.*/
#define IFAULT_PL_TX_DMA_PORT_PARITY_ERR                        (0x5895) /* TxDma Transport Cleanup detected a fatal port parity error. */
#define IFAULT_PL_TX_DMA_CTX_INVALID_INTERUPT_CTX               (0x5896) /* There is a TxCtxMgr interrupt, but we cannot determine which context it was on. */

//#define IFAULT_PL_UNHANDLED_LINK_LAYER_INT                    (0x5890)
#define IFAULT_PL_UNHANDLED_PHLYNK_TBL_XFACE_INT                (0x58A0) /* An unhandled error occurred in the phlynk (phy/link) interrupt handler. */
// Phy Layer fault code                                                  /* Note: PHLYNK/phlynk is used in the Sentinal Specification. */
#define IFAULT_PL_UNHANDLED_PHY_LAYER_INT_1                     (0x58B1) /* The phy layer detected a parity error on a register write. */
#define IFAULT_PL_UNEXPECTED_PHY_POWER_MODE                     (0x58B2) /* Incorrectly tried to set an unexpected Phy power mode. */
#define IFAULT_PL_UNEXPECTED_LINK_RATE                          (0x58B3) /* Tried to set an incorrect or invalid link rate for a link. */
#define IFAULT_PL_PLL_READY_TIMEOUT                             (0x58B4) /* PLL TxReady timed out. */
#define IFAULT_PL_OPTICAL_MODE_PROGRAMMING_ERROR                (0x58B5) /* Optical mode programming error. */

#define IFAULT_PL_INVALID_MID_TRACKER_SET                       (0x5900) /* Tried to set a bit on an invalid MID tracker. */
#define IFAULT_PL_INVALID_MID_TRACKER_CLEAR                     (0x5910) /* Tried to clear a bit on an invalid MID tracker. */
#define IFAULT_PL_INVALID_MID_TRACKER_ASSIGN                    (0x5920) /* Tried to assign a bit on an invalid MID tracker. */
#define IFAULT_PL_TXFM_NULL_PTRCURBUF                           (0x5A00) /* Encountered a null pointer to the current buffer. */

//#define IFAULT_PL_TASK_MGMT_HW_CLEANUP_FAILURE                (0x6000) // 0x6000 ~ 0x61ff are TM specific fault codes.
#define IFAULT_PL_TASK_MGMT_OUTSTD_IOS_ON_DEV_RST               (0x6001) /* Tried to complete a device reset with outstanding I/Os.  */
#define IFAULT_PL_TASK_MGMT_TX_FRAME_NOT_CLEAN                  (0x6002) /* Failed to find an aborted I/O on the DMA transmission queue. */
#define IFAULT_PL_TASK_MGMT_ABORT_TAG_MISSED_IO                 (0x6003) /* Abort task failed to find the I/O, but the MidUsed tag indicated that the I/O was in use. */
#define IFAULT_PL_TASK_MGMT_IO_COUNT_CORRUPTION                 (0x6004) /* Corrupt I/O count. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_TM_START                (0x6005) /* Encountered an invalid MID during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_ABT_MATCH               (0x6006) /* Encountered an invalid MID during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_ABT_TRANSQ              (0x6007) /* Encountered an invalid MID during task management.*/
#define IFAULT_PL_TASK_MGMT_INVALID_MID_ABT_MID                 (0x6008) /* Encountered an invalid MID during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_ABT_TXTRANS             (0x6009) /* Encountered an invalid MID during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_CLEANUP                 (0x600A) /* Encountered an invalid MID during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_TM_TRAP                 (0x600B) /* Encountered an invalid MID during task management.*/
#define IFAULT_PL_TASK_MGMT_INVALID_MID_CMP_ABT_IO              (0x600C) /* Encountered an invalid MID during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_ABT_TM                  (0x600D) /* Encountered an invalid MID during task management.*/
#define IFAULT_PL_TASK_MGMT_INVALID_DEVH_TM_START               (0x600E) /* Encountered an invalid device handle during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_HWDI                        (0x600F) /* Tried to perform task management on a nonexistent device. */
#define IFAULT_PL_TASK_MGMT_INVALID_DEVH                        (0x6010) /* Encountered an invalid device handle during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_ABT_RXTRANS             (0x6011) /* Encountered an invalid MID during task management. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_1                     (0x6012) /* The Tx transport cleanup stalled. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_2                     (0x6013) /* The Tx transport cleanup stalled. */

#define IFAULT_PL_TASK_MGMT_INVALID_MID_RXEEDP_NON_AUTO         (0x6014) /* Invalid MID encountered during NON_AUTO RxEEDP interrupt */

#define IFAULT_PL_TASK_MGMT_INTERNAL_INVALID_DEVH               (0x6015) /* Tried internal task management on an invalid device handle. */
#define IFAULT_PL_TASK_MGMT_ABORT_LIST_CORRUPTION_1             (0x6016) /* Corrupt task management abort list.*/
#define IFAULT_PL_TASK_MGMT_ABORT_LIST_CORRUPTION_2             (0x6017) /* Corrupt task management abort list.*/
#define IFAULT_PL_TASK_MGMT_ABORT_LIST_CORRUPTION_3             (0x6018) /* Corrupt task management abort list. */
#define IFAULT_PL_TASK_MGMT_ABORT_LIST_CORRUPTION_4             (0x6019) /* Corrupt task management abort list. */
#define IFAULT_PL_TASK_MGMT_IQ_DISABLE_HANG_1                   (0x601A) /* Control queue stalled. */
#define IFAULT_PL_TASK_MGMT_IQ_DISABLE_HANG_2                   (0x601B) /* Control queue stalled. */
#define IFAULT_PL_TASK_MGMT_TXFM_NOT_INT_STATE                  (0x601C) /* TxFM didn't go to interrupt state as expected. */
#define IFAULT_PL_TASK_MGMT_INVALID_DEVH_TM_ST_CLEANUP          (0x601D) /* Encountered an invalid device handle during task management. */
#define IFAULT_PL_TASK_MGMT_INVALID_MID_ABT_UNCONF_RESP         (0x601E) /* The Tx frame manager did not go to the interrupt state as expected. */

#define IFAULT_PL_TASK_MGMT_TM_TAIL_NOT_NULL                    (0x6020) /* The TM head is NULL, but the TM tail is not NULL. Invalid state. */

#define IFAULT_PL_TASK_MGMT_SMP_SEND_BAD_STATE                  (0x6040) /* The control queue stalled.  */
#define IFAULT_PL_TASK_MGMT_CRIT_FRM_1_NOT_FREED                (0x6050) /* The task management SSP critical frame was not freed. */
#define IFAULT_PL_TASK_MGMT_CRIT_FRM_2_NOT_FREED                (0x6051) /* The task management SMP critical frame was not freed. */

#define IFAULT_PL_TASK_MGMT_UNEXPECTED_ABORT                    (0x6100) /* Tried to abort an unexpected function type (0x61xx, where xx == function type). */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_3                     (0x6101) /* The Tx transport cleanup stalled.*/
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_4                     (0x6102) /* The Tx transport cleanup stalled.*/
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_5                     (0x6103) /* The Tx transport cleanup stalled.*/
#define IFAULT_PL_TASK_MGMT_TX_HANG_LOST_MID                    (0x6104) /* Cannot find hung mid in txctxmgrs. */
#define IFAULT_PL_TASK_MGMT_TX_HANG_UNKNOWN_MID                 (0x6105) /* Cannot determine the mid that may have caused the hang. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_6                     (0x6106) /* The Tx transport cleanup stalled. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_7                     (0x6107) /* The Tx transport cleanup stalled. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_8                     (0x6108) /* The Tx transport cleanup stalled. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_TXDMAQ_RUNNING             (0x6109) /* The Tx transport cleanup called with TxDMAQ running. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_9                     (0x610A) /* The Tx transport cleanup stalled. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_PORTXFER_NOT_IDLE          (0x610B) /* PortXferState did not go to idle and we could not find a link to break. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_INVALID_MID                (0x610C) /* Trying to abort an invalid mid. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_10                    (0x610D) /* The Tx transport cleanup stalled. */
#define IFAULT_PL_TASK_MGMT_TX_CLEAN_HANG_11                    (0x610E) /* The Tx transport cleanup stalled. */
#define IFAULT_PL_TASK_MGMT_INVALID_DEVH_TM_REM_ABORTEDIO       (0x6110) /* Encountered an invalid DevHandle while removing an I/O from TM abort list. */

#define IFAULT_PL_TASK_MGMT_START_FAIL_BAD_FAST_PATH_HW         (0x61A0) /* Tried to start a TM, but the Fast Path Hardware was not in the expected state. */
#define IFAULT_PL_TASK_MGMT_FAILED_TO_REMOVE_FPETIME            (0x61A2) /* There was a fatal error removing the FPE timer from an I/O aborted by a TM. */
#define IFAULT_PL_TASK_MGMT_COMPLETION_IOCOUNT_NOT_ZERO         (0x61A3) /* IOcount for device handle 0 was not zero when trying to complete a TM. */
#define IFAULT_PL_TASK_MGMT_TM_LIST_CORRUPTION                  (0x61A4) /* List of linked TMs for a device is corrupted. */


#define IFAULT_PL_UNEXPECTED_COMPLETION                         (0x6201) /* An entry occurred on the completion queue for a non-I/O request. */
//#define IFAULT_PL_UNKNOWN_IO_BEING_FINISHED                   (0x6202) /* PL is trying to complete/finish an IO that it doesn't know about */
#define IFAULT_PL_SCSI_IO_INVALID_MID_3                         (0x6208) /* Invalid MID encountered. */
#define IFAULT_PL_SCSI_INVALID_SENSE_BUFFER                     (0x6209) /* Internal I/O did not provide an address for the sense data. */
#define IFAULT_PL_RESTART_SCSI_IO_INVALID_MID                   (0x620A) /* Encountered an invalid MID. */
#define IFAULT_PL_SCSI_IO_INVALID_MID_1                         (0x620B) /* Encountered an invalid MID. */
#define IFAULT_PL_SCSI_IO_INVALID_MID_2                         (0x620C) /* Encountered an invalid MID. */
//#define IFAULT_PL_TX_FM_CLEANUP_INVALID_MID                   (0x620C)
#define IFAULT_PL_RESTART_SCSI_IO_BAD_IOCOUNT_DEC               (0x620D) /* Tried to restart an I/O when no I/O is outstanding. */
#define IFAULT_PL_PEND_SCSI_IO_BAD_IOCOUNT_DEC                  (0x620E) /* Tried to pend an I/O when no I/O is outstanding. */
#define IFAULT_PL_SCSI_IO_CMPLT_BAD_IOCOUNT_DEC                 (0x620F) /* Tried to complete an I/O when no I/O is outstanding.*/
//#define IFAULT_PL_SCSI_IO_INVALID_DEVHANDLE                   (0x6301)
//#define IFAULT_PL_PORT_ERR_INVALID_DEVHANDLE                  (0x6303)
#define IFAULT_PL_INVALID_HWDI_LINK_LAYER                       (0x6400) /* Encountered an invalid device handle. */
#define IFAULT_PL_INVALID_HWDI_SATA_ENG_ABRT_TAG                (0x6402) /* Encountered an invalid device handle. */
#define IFAULT_PL_INVALID_HWDI_NON_AUTO_DMA_COMP                (0x6403) /* Encountered an invalid device handle. */
#define IFAULT_PL_INVALID_HWDI_NON_AUTO_FRAME                   (0x6404) /* Encountered an invalid device handle. */
#define IFAULT_PL_INVALID_NON_AUTO_FRAME_HANDLER                (0x6405) /* Trying to handle an invalid type of non auto frame. */
//#define IFAULT_PL_INVALID_HWDI_ADD_TO_DMA_TRANS_Q             (0x6405)
//#define IFAULT_PL_INVALID_HWDI_FIS_W_ERR_BIT                  (0x6406)
//#define IFAULT_PL_INVALID_HWDI_RX_DMA_ERR                     (0x6409)
//#define IFAULT_PL_INVALID_HWDI_RX_FM_ERR                      (0x640A)
//#define IFAULT_PL_INVALID_HWDI_TX_CTX_ERR                     (0x640B)
//#define IFAULT_PL_INVALID_HWDI_TX_DMA_ERR                     (0x640C)
//#define IFAULT_PL_INVALID_HWDI_RX_CTX_ERR                     (0x640D)
//#define IFAULT_PL_INVALID_HWDI_TX_FM_ERR                      (0x640E)
#define IFAULT_PL_INVALID_HWDI_ABORT_TX_TRANS                   (0x640F) /* Encountered an invalid device handle. */
//#define IFAULT_PL_INVALID_HWDI_NON_PT_WKWD_START              (0x6410)
//#define IFAULT_PL_INVALID_HWDI_PORT_ERR                       (0x6411)
//#define IFAULT_PL_INVALID_HWDI_TX_FM_CLEANUP_HW               (0x6412)
#define IFAULT_PL_INVALID_HWDI_PORT_TXDMA_ERR                   (0x6413) /* Reserved. */
#define IFAULT_PL_INVALID_DEV_HANDLE_HUNG_LINK                  (0x6414) /* Invalid device handle encountered while handling a stuck link. */
//#define IFAULT_PL_PORT_ERR_INVALID_TXIDENTIFIER               (0x6500)
#define IFAULT_PL_INVALID_HWDI_TX_CTX_CLEANUP                   (0x6600) /* Encountered an invalid device handle.  */
#define IFAULT_PL_CLEARED_RX_CTX_W_O_INTERRUPT                  (0x6700) /* The Rx context cleared without an interrupt. */
#define IFAULT_PL_CLEAR_RX_CTX_INVALID_TRANS_NUM                (0x6701) /* Tried to clear a transfer number greater than the number of RxDMAs. */
#define IFAULT_PL_CLEAR_RX_CTX_CTX_INVALID                      (0x6702) /* Tried to clear context with no valid current context. */
#define IFAULT_PL_CLEAR_RX_CTX_XFER_ACTIVE                      (0x6703) /* Tried to clear context while the DMA engine is active. */
#define IFAULT_PL_RX_CTX_STOP_DE_FAIL                           (0x6704) /* Failed to stop the DMA engine for overrun cleanup. */
#define IFAULT_PL_CLEAR_RX_CTX_UPDATES_NOT_DONE                 (0x6707) /* Memory updates were not done when clearing the Rx context. */
#define IFAULT_PL_CLEAR_RX_CTX_NO_INTERRUPT                     (0x6708) /* Trying to clear Rx context with no interrupt active. */
#define IFAULT_PL_RX_CTX_DMA_ACTIVE_TIMEOUT                     (0x6709) /* Trying to clear Rx context with no interrupt active. */
#define IFAULT_PL_RX_CTX_FAILED_STOP                            (0x670A) /* RxCtxMgr failed to STOP. */

#define IFAULT_PL_RXEEDP_HW_CNTXT_UPD_FAILURE                   (0x670B) /* RXEEDP failed to update the HWContext. */
#define IFAULT_PL_RXEEDP_CTX_CMPL_NOT_SET                       (0x670C) /* RXEEDP NonAuto interrupt - ContextComplete not set yet. */

#define IFAULT_PL_RXEEDP_SET_SUSPEND_TIMEOUT                    (0x670D) /* After setting suspend bit in General Control, timeout while waiting for confirmation. */
#define IFAULT_PL_RXEEDP_SET_PM_STATE_TIMEOUT                   (0x670E) /* After writing power management setting in General Control, timeout while waiting for confirmation. */

#define IFAULT_PL_UNEXPECTED_PHY_LAYER_INT_LOW                  (0x6800) /* Encountered an unexpected phy layer interrupt. */
//#define IFAULT_PL_UNEXPECTED_PHY_LAYER_INT_HIGH               (0x6801)
//#define IFAULT_PL_UNEXPECTED_PHY_LAYER_INT_CRIT               (0x6802)
//#define IFAULT_PL_UNEXPECTED_LINK_LAYER_INT_HIGH              (0x6804)
#define IFAULT_PL_INVALID_SAS_ADDRESS_IN_IDENTIFY_FRAME         (0x6805) /* Reserved. Encountered an invalid SAS address in the identify frame. */
#define IFAULT_PL_UNEXPECTED_PHLYNKTBL_INT_LOW                  (0x6806) /* Encountered an unexpected phlynk (phy/link) table interrupt. */
//                                                                       /* Note: PHLYNK/phlynk is used in the Sentinal Specification. */
//#define IFAULT_PL_UNEXPECTED_PHLYNKTBL_INT_HIGH               (0x6807)
//#define IFAULT_PL_UNEXPECTED_PHLYNKTBL_INT_CRIT               (0x6808)
#define IFAULT_PL_UNEXPECTED_RXEEDP_INT_LOW                     (0x6809) /* Encountered an unexpected Rx EEDP interrupt. */
//#define IFAULT_PL_UNEXPECTED_RXEEDP_INT_HIGH                  (0x680A)
//#define IFAULT_PL_UNEXPECTED_RXEEDP_INT_CRIT                  (0x680B)
#define IFAULT_PL_UNEXPECTED_PORT_LAYER_INT_LOW_1               (0x680C) /* Encountered an unexpected port layer interrupt. */
#define IFAULT_PL_UNEXPECTED_PORT_LAYER_INT_LOW_2               (0x680D) /* Encountered an unexpected port layer interrupt. */
//#define IFAULT_PL_UNEXPECTED_PORT_LAYER_INT_HIGH              (0x680D)
//#define IFAULT_PL_UNEXPECTED_PORT_LAYER_INT_CRIT              (0x680E)
#define IFAULT_PL_UNEXPECTED_RXDMA_INT_LOW                      (0x680F) /* Encountered an unexpected RxDMA engine interrupt. */
//#define IFAULT_PL_UNEXPECTED_RXDMA_INT_HIGH                   (0x6810)
//#define IFAULT_PL_UNEXPECTED_RXDMA_INT_CRIT                   (0x6811)
#define IFAULT_PL_UNEXPECTED_TXDMA_INT_LOW                      (0x6812) /* Encountered an unexpected TxDMA engine interrupt.*/
//#define IFAULT_PL_UNEXPECTED_TXDMA_INT_HIGH                   (0x6813)
//#define IFAULT_PL_UNEXPECTED_TXDMA_INT_CRIT                   (0x6814)
#define IFAULT_PL_UNEXPECTED_RXFM_INT_LOW                       (0x6815) /* Encountered an unexpected Rx frame manager interrupt. */
//#define IFAULT_PL_UNEXPECTED_RXFM_INT_HIGH                    (0x6816)
//#define IFAULT_PL_UNEXPECTED_RXFM_INT_CRIT                    (0x6817)
//#define IFAULT_PL_UNEXPECTED_TXFM_INT_LOW                     (0x6818)
//#define IFAULT_PL_UNEXPECTED_TXFM_INT_HIGH                    (0x6819)
//#define IFAULT_PL_UNEXPECTED_TXFM_INT_CRIT                    (0x681A)
#define IFAULT_PL_UNEXPECTED_QM_INT_LOW                         (0x681B) /* Encountered an unexpected queue manager interrupt. */
//#define IFAULT_PL_UNEXPECTED_QM_INT_HIGH                      (0x681C)
//#define IFAULT_PL_UNEXPECTED_QM_INT_CRIT                      (0x681D)
#define IFAULT_PL_UNEXPECTED_PORT_LAYER_TXTYPE                  (0x681E) /* Encountered an unexpected port layer type. */
#define IFAULT_PL_UNEXPECTED_PORT_LAYER_TXTYPE_ON_CLEAR         (0x681F) /* Encountered an unexpected port layer type. */
#define IFAULT_PL_PORT_LAYER_CLEAN_NO_MID_FOUND                 (0x6820) /* Failed to find the MID. */
#define IFAULT_PL_PORT_LAYER_STP_BSY_NO_MID_ATAPI               (0x6821) /* Failed to find the MID. */
#define IFAULT_PL_PORT_LAYER_STP_BSY_NO_MID                     (0x6822) /* Failed to find the MID. */
#define IFAULT_PL_PORT_LAYER_STP_BSY_UNEXPECTED                 (0x6823) /* An unexpected OpenFailCode = STP_RESOURCES_BUSY occurred. */
//#define IFAULT_PL_SDMW_INVALID_DFE_GAIN                       (0x6824)
#define IFAULT_PL_INVALID_COMPLETION_INTERRUPT                  (0x6825) /* Invalid completion interrupt occurred. */
#define IFAULT_PL_INVALID_SAS_COMPLETION_MID_OWNERSHIP          (0x6826) /* Invalid ownership of a I/O completion received from the SAS Completion Queue. */
#define IFAULT_PL_PORT_LAYER_OPENFAILED_NOT_FOUND               (0x6827) /* Open failed interrupt occurred, but not found in hardware. */
#define IFAULT_PL_PORT_LAYER_UNEXPECTED_TX_TYPE                 (0x6828) /* Unexpected TxType when finding MID */
#define IFAULT_PL_PORT_LAYER_NO_DEST_THRES_NOT_EXCEEDED         (0x6829) /* The no destination counter threshold exceeded interrupt did not hit prior to checking for expired timer. */
#define IFAULT_PL_PORT_LAYER_BAD_TXTYPE_IN_OPEN_FAILED_STATUS   (0x682A) /* The TxType in the OpenFailedStatus is one we cannot clear in the TxFrmMgr's OpenFailed interrupt. */
#define IFAULT_PL_PORT_LAYER_CHECK_FOR_EXPIRED_TIMER_FAILED     (0x682B) /* Port layer check for expired timer failed. */

#define IFAULT_PL_INT_TRACKER_RECORD_INVALID                    (0x6901) /* Encountered an invalid tracker record value. */
#define IFAULT_PL_INT_TRACKER_REPORT_INVALID                    (0x6902) /* Encountered an invalid tracker report value. */
#define IFAULT_PL_INT_TRACKER_SHOW_INVALID                      (0x6903) /* Encountered an invalid tracker structure. */


#define IFAULT_PL_DIRECT_ATTACHED_SATA_PHYNUM_OUT_OF_RANGE      (0x6B00) /* The gplDevDetails.FirstPhyIdentifier value is not a direct-attached phy. */
#define IFAULT_PL_PWR_MGMT_BAD_DATA_IN_SASIOUNIT_5              (0x6B01) /* The NumPhys field value within SAS I/O Unit Page 5 is zero. */
#define IFAULT_PL_PWR_MGMT_POLICY_QUEUE_EMPTY_1                 (0x6B02) /* Tried to dequeue a policy change request from an empty queue. */
#define IFAULT_PL_PWR_MGMT_POLICY_QUEUE_EMPTY_2                 (0x6B03) /* Tried to dequeue a policy change request from an empty queue. */
#define IFAULT_PL_PWR_MGMT_POLICY_QUEUE_EMPTY_3                 (0x6B04) /* Tried to dequeue an empty policy request queue in host-controlled PWM code .*/
#define IFAULT_PL_PWR_MGMT_UNKNOWN_RESPONSE_MESSAGE             (0x6B05) /* Tried to reply to a message type that was not SAS I/O Unit Control. */
#define IFAULT_PL_PWR_MGMT_POLICY_QUEUE_EMPTY_4                 (0x6B06) /* Tried to dequeue an empty policy request while changing global power mode. */

#define IFAULT_PL_UNIMPLEMENTED_FEATURE                         (0x6BFF) /* Reserved. */

#define IFAULT_PL_PWR_MGMT_INVALID_GROUP_MEMBERSHIP             (0x6BE0) /* Invalid link->group membership in plPwrMgmtGatherStatistics(). */

#define IFAULT_PL_PWR_MGMT_HARDWARE_NOT_INCREMENTING            (0x6B0A) /* Hardware issue. LCSampCnt did not increment. */
#define IFAULT_PL_PWR_MGMT_BAD_PERCENTAGE_CALCULATION           (0x6B0B) /* Calculated greater than 100% port-bandwidth use. */
#define IFAULT_PL_PWR_MGMT_PORT_WIDTH_MOD_CHANGE_FAILURE_1      (0x6B0C) /* Incorrectly calculated PHYs to turn on and turn off. */
#define IFAULT_PL_PWR_MGMT_PORT_WIDTH_MOD_CHANGE_FAILURE_2      (0x6B0D) /* Incorrectly calculated PHYs to turn on and turn off. */
#define IFAULT_PL_PWR_MGMT_UNEXPECTED_LINK_ALIAS                (0x6B0E) /* The LinkAlias value in the port layer did not match the expected value. */
#define IFAULT_PL_PWR_MGMT_PHY_POWER_STATE_IDLE_TIMEOUT         (0x6B10) /* Timeout occurred while waiting for link to settle down. (lower 8 bits = linknum). */
#define IFAULT_PL_PWR_MGMT_HARDWARE_COUNTERS_NONSENSICAL        (0x6B1A) /* LcSampOn is greater than  LcSampCnt. */
#define IFAULT_PL_PWR_MGMT_FIRMWARE_COUNTERS_NONSENSICAL        (0x6B1B) /* (normalized) LcSampOn is greater than  LcSampCnt. */
#define IFAULT_PL_PWR_MGMT_GENERATED_NULL_SAMPLE                (0x6B1C) /* Power management tried to generate a port use sample for a group with no active links. */
#define IFAULT_PL_PWR_MGMT_LINKALIAS_MISMATCH_ON_SAVE           (0x6B1D) /* A link alias mismatch between the current link alias and the old link alias. */
#define IFAULT_PL_PWR_MGMT_LINKALIAS_MISMATCH_ON_RESTORE        (0x6B1E) /* A link alias mismatch between the current link alias and the old link alias. */
#define IFAULT_PL_PWR_MGMT_PWM_REQUESTED_BAD_REGION_CHG         (0x6B1F) /* The PWM control algorithm requested an invalid region change. */

#define IFAULT_PL_PWM_INVALID_ABORT_REQUEST                     (0x6B20) /* Tried to abort port width modulation for a group that is not modulated. */
#define IFAULT_PL_PWM_PHY_SM_POWEROFF_BAD_LINK_ALIAS            (0x6B21) /* Tried to bring back a phy with the wrong LinkAlias. */
#define IFAULT_PL_PWM_PHY_SM_POWERON_RCVD_GROW_EVENT            (0x6B22) /* Tried to power on a phy that is already on. */
#define IFAULT_PL_PWM_PHY_SM_SLUMBER_STATE_MISMATCH             (0x6B23) /* Port width modulation tried to wake up a phy in Slumber mode, but the phy was not in Slumber mode. */
#define IFAULT_PL_PWM_PHY_SM_UNKNOWN_TRANSITION                 (0x6B24) /* State machine did not transition. Check the ring buffer for the phy number, event, and state-machine state. */

#define IFAULT_PL_PWM_PHY_SM_SLUMBER_STATE_UNEXPECTED_EVENT     (0x6B30) /* Received an unexpected event while in the SLUMBER state. The lower four bits encode the event number. */
#define IFAULT_PL_PWM_PHY_SM_POWEROFF_STATE_UNEXPECTED_EVENT    (0x6B40) /* Received an unexpected event while in the POWER_OFF state. The lower four bits encode the event number. */
#define IFAULT_PL_PWM_PHY_SM_WAIT_LR_STATE_UNEXPECTED_EVENT     (0x6B50) /* Received an unexpected event while in the WAIT_LR_COMPLETE state. The lower four bits encode the event number. */
#define IFAULT_PL_PWM_PHY_SM_DEV_CHECK_STATE_UNEXPECTED_EVENT   (0x6B60) /* Received an unexpected event while in the DEVICE_CHECK state. The lower four bits encode the event number. */
#define IFAULT_PL_PWM_PHY_SM_ERROR_STATE_UNEXPECTED_EVENT       (0x6B70) /* Received an event that was not an abort event while in ERROR state. The lower four bits encode the event number. */

#define IFAULT_PL_PWR_MGMT_RXEEDP_RAM_ACCESS_SET                (0x6B71) /* Firmware has set the access bit for RxEEDP memory. */
#define IFAULT_PL_PWR_MGMT_NOT_PM_IDLE                          (0x6B72) /* The Tx DMA or RxDMA PM logic is not in PM Idle state.  */

#if (PL_FEATURE_PWR_MGMT_COMM_INTENT)
#define IFAULT_PL_PWR_MGMT_SMP_RESP_CI_STATE_UNEXPTD1           (0x6B80) /* Received a good SMP Response but the PwrMgmt CommIntent State was unexpected. */
#define IFAULT_PL_PWR_MGMT_SMP_RESP_CI_STATE_UNEXPTD2           (0x6B81) /* Received a SMP Resp error or a non-good Func Result, the CommIntent State was unexpected. */
#endif  /* PL_FEATURE_PWR_MGMT_COMM_INTENT */

#define IFAULT_PL_MATH_DIVIDE_BY_ZERO                           (0x6C00) /* Attempted to divide by 0. */

//#define IFAULT_PL_UNEXPECTED_SMP_ABORT_BUG                    (0x7000)
//#define IFAULT_PL_UNEXPECTED_INIT_FRAME_BUG                   (0x7001) /* Encountered unsupported SSP frame type. */
//#define IFAULT_PL_UNEXPECTED_TARG_FRAME_BUG                   (0x7002)
//#define IFAULT_PL_RX_CTX_WORKAROUND_BUG                       (0x7004)
//#define IFAULT_PL_NON_DWORD_ALIGNED_QUAD_MEMCPY               (0x7005)
//#define IFAULT_PL_INVALID_HWDI_TXFM_CLEANUP                   (0x7006)
#define IFAULT_PL_STARTING_IO_WHEN_IQ_DISABLED                  (0x7008) /* Tried to enqueue an I/O to a disabled device. */
//#define IFAULT_PL_HOST_BUF_DMA_UNEXPECTED_LOCAL               (0x7009)
//#define IFAULT_PL_HOST_BUF_DMA_UNEXPECTED_MID                 (0x700A)
//#define IFAULT_PL_HOST_BUF_DMA_UNEXPECTED_FLAGS               (0x700B)
//#define IFAULT_PL_HOST_BUF_DMA_UNEXPECTED_ORDER               (0x700C)
#define IFAULT_PL_STARTING_IO_WHEN_CLEANUP_ACTIVE               (0x700D) /* Tried to enqueue an I/O to a disabled device. */
//#define IFAULT_PL_TARGET_CONTEXT_INFO_SIZE_ERR                (0x7101)
#define IFAULT_PL_SEP_BUFFER_SIZE_ERR                           (0x7200) /* Tried to build an SES message larger than the buffer. */
#define IFAULT_PL_SEP_CMD_QUEUE_ERR                             (0x7201) /* Tried to remove an enclosure management message that was not on the queue. */
#define IFAULT_PL_SEP_CONTROL_BITS_ERROR                        (0x7202) /* Received a SCSI message without the data direction bits set. */
#define IFAULT_PL_SEP_ISTWI_BUS_RESERVE_ERROR                   (0x7203) /* Failed to reserve the ISTWI bus. */
#define IFAULT_PL_ENCL_MGMT_SLOT_NUM_ERROR                      (0x720A) /* Encountered an SEP message with slot number 0 when the starting slot number is 1. */
#define IFAULT_PL_ENCL_MGMT_DEVHANDLE_CONVERT_ERROR             (0x720B) /* Encountered an SEP message is not in the correct enclosure handle/slot form. */
//#define IFAULT_PL_PORT_LAYER_ERR                              (0x7300)
#define IFAULT_PL_START_SMP_INVALID_DEVHANDLE                   (0x7400) /* An SMP request was made to an invalid device handle. */
#define IFAULT_PL_INTERNAL_SMP_TOO_BIG                          (0x7401) /* Internal SMP request is too large to fit inside one message frame. */
#define IFAULT_PL_SMP_TIMED_OUT                                 (0x7410) /* The SMP timed out three times. Not compiled into production builds. */
//#define IFAULT_PL_SLD_OUT_OF_RESOURCES                        (0x7501) // 0x75xx are SLD specific fault codes
//#define IFAULT_PL_SLD_INVALID_DEV_HANDLE                      (0x7502)
//#define IFAULT_PL_SLD_INVALID_ENTRY                           (0x7503)
//#define IFAULT_PL_SLD_CLEAR_AFFILIATION_ERROR                 (0x7504)
#define IFAULT_PL_INVALID_PHY_TO_PORTNUM                        (0x7600) /* Failed to find a free port or a port that is associated with the phy. */
#define IFAULT_PL_SOD_SIZE_NOT_16_BYTE_ALIGNED                  (0x7700) /* Found a start-of-day size parameter that was not 16-byte aligned. */
#define IFAULT_PL_INITIALIZATION_SEQUENCE_ERROR_1               (0x7801) /* Tried the start-of-day function before the parameters initialized.*/
#define IFAULT_PL_INITIALIZATION_SEQUENCE_ERROR_2               (0x7802) /* Tried PL initialization before the start-of-day function completed. */
#define IFAULT_PL_SATA_REASSIGN_BLOCKS_1                        (0x7821) /* An unexpected operation occurred in reassigning blocks. */
#define IFAULT_PL_SATA_REASSIGN_BLOCKS_2                        (0x7822) /* An unexpected status occurred in reassigning blocks. */
#define IFAULT_PL_SOD_MEMORY_NOT_16_BYTE_ALIGNED                (0x7900) /* Found a start-of-day parameter that was not 16-byte aligned. */
#define IFAULT_PL_UNSUPPORTED_SENTINEL_VERSION                  (0x7A00) /* Found an unsupported version of the SAS core during start-of-day function operation. */
#define IFAULT_PL_UNSUPPORTED_COLT_VERSION                      (0x7A01) /* Found an unsupported version of Colt during Start-Of-Day. */
#define IFAULT_PL_MISSING_BITS_HW_WKRD_MATRIX                   (0x7B00) /* Invalid number of bits defined in the run-time workaround matrix. */
#define IFAULT_PL_INVALID_QD_HW_SACTIVE_WKRD                    (0x7C00) /* Tried to use invalid internal queue depth with a workaround. */
#define IFAULT_PL_SATA_INQ_BAD_LENGTH                           (0x7C10) /* An unexpected data length was found in the SATA inquiry command. */
#define IFAULT_PL_SATA_INIT_START_BAD_DEV                       (0x7C20) /* Failed to find a phy table entry for the attached device. */
//#define IFAULT_PL_SATA_WRV_INVALID_CDB                          (0x7C21) /* Reserved. */
#define IFAULT_PL_SATA_SCT_INVALID_CDB                          (0x7C22) /* Encountered an unexpected CDB. */
//#define IFAULT_PL_SATA_SCT_INVALID_CDB2                         (0x7C23) /* Reserved. */
#define IFAULT_PL_SATA_CLRAFF_BAD_DEVH                          (0x7C30) /* Tried to clear the affiliation with an invalid device handle. */
#define IFAULT_PL_SATA_CLRAFF_BAD_DEVTYPE                       (0x7C31) /* Tried to clear the affiliation of an SMP target. */
#define IFAULT_PL_MSG_FRAME_SIZE_TOO_BIG                        (0x7C32) /* Start-of-day parameter HwMessageSize is larger than 1008 bytes. */
#define IFAULT_PL_SATA_CLRAFF_BAD_IOCOUNT                       (0x7C33) /* Tried to clear the affiliation with outstanding I/Os. */
#define IFAULT_PL_SATA_SACTIVE_LEFT_ON                          (0x7C34) /* SActive bits remain set after a translated command. */
#define IFAULT_PL_SATA_MIDVALID_ON                              (0x7C35) /* Hardware MidValid bit remains set after a translated command. */
#define IFAULT_PL_SATA_BREAKUP_CDB_TOO_BIG                      (0x7C36) /* Tried to perform breakup on a CDB that is too large. */
#define IFAULT_PL_SATA_INVALID_BREAKUP                          (0x7C37) /* Tried to perform breakup on read/write 6. */
#define IFAULT_PL_SATA_INVALID_BREAKUP_UNSUP                    (0x7C38) /* Tried to perform breakup on an unsupported read/write size. */
#define IFAULT_PL_SIZE_OF_FW_USE_OF_HW_CTX_TOO_BIG              (0x7C40) /* The hardware context structure size that firmware uses was larger than the minimum hardware context size. */
#define IFAULT_PL_SATA_SCSI_SEC_PROTO_OUT_SIZE_TOO_BIG          (0x7C39) /* The SCSI Security Protocol Out size (data + zero + param) is greater than the message frame size. */
#define IFAULT_PL_INVALID_MSG_FRAME                             (0x7C41) /* The message frame address is not in the allocated pool. */
#define IFAULT_PL_WRITE_SAME_TRIM_BAD_RANGE_LIST                (0x7C42) /* Failed to add ATA RANGE ENTRY to RANGE LIST. */
#define IFAULT_PL_ERROR_CTX_NULL_POINTER                        (0x7C43) /* The caller passed in a NULL pointer for the error ctx. */
#define IFAULT_PL_UNEXPECTED_PORTINFO                           (0x7C44) /* Unsupported PortInfo value coming from SASCore. */
#define IFAULT_PL_NO_STP_PTHRU_HANDLER_FOUND                    (0x7C45) /* Could not find a handler for the requested STP Pass Through command. */
#define IFAULT_PL_MID_FOR_DMA_Q_ENTRY                           (0x7C46) /* Could not get a MID for DMA Q entry at SOD  */


//#define IFAULT_PL_PORT_LAYER_HUNG_SMP                         (0x7C50)
#define IFAULT_PL_PORT_LAYER_HUNG_CLR_AFFL                      (0x7C51) /* The port layer workaround stalled. */
#define IFAULT_PL_PORT_LAYER_HUNG_UNKNOWN                       (0x7C52) /* The port layer workaround stalled. */
#define IFAULT_PL_SAS_CONFIG_REG_INT                            (0x7C53) /* A configuration register low-priority interrupt. */
#define IFAULT_PL_SAS_SLAVE_INT                                 (0x7C54) /* SAS Slave low-priority interrupt. */

#define IFAULT_PL_QUIESCE_REDUCE_FUNC_IN_USE                    (0x7C60) /* Cannot quiesce because Reduced Functionality is already in use. */
#define IFAULT_PL_QUIESCE_SMP_RESTART_PORT_CHANGED              (0x7C61) /* Cannot quiesce because a device with SMP moved ports on us. */

#if COMPILE_OPTION_SAS_PORT_VIRT
/* SOD Activities Error Codes */
//#define IFAULT_PL_SMP_TARGET_NO_OPEN_RCVD_WRKARND_FAIL        (0x7D00) /* HW_NO_OPEN_RCVD_INT workaround failed. This can be taken out later. */
#define IFAULT_PL_VEM_SAS_ADDRESS_ALLOC_FAILURE                 (0x7D01) /* Failed to allocate a SAS address for the virtual port. */
#define IFAULT_PL_VEM_NULL_POINTER_ADD                          (0x7D02) /* Tried to add a virtual port with a NULL address. */
#define IFAULT_PL_VEM_NULL_POINTER_REMOVE                       (0x7D03) /* Tried to remove a virtual port with a NULL address. */
#define IFAULT_PL_VEM_WRONG_TBL_ENTRY_ADD                       (0x7D04) /* Tried to add the wrong table entry. */
#define IFAULT_PL_VEM_WRONG_TBL_ENTRY_REMOVE_1                  (0x7D05) /* Failed to remove a virtual port ID entry. */
#define IFAULT_PL_VEM_WRONG_TBL_ENTRY_REMOVE_2                  (0x7D06) /* Failed to remove a virtual port ID entry. */
#define IFAULT_PL_VEM_UNEXPECTED_VPID_1                         (0x7D07) /* Failed to add an entry to the Virtual ID Originator table. */
#define IFAULT_PL_VEM_UNEXPECTED_VPID_2                         (0x7D08) /* Failed to add an entry to the Hardware Initiator table. */
#define IFAULT_PL_VEM_UNEXPECTED_VPID_3                         (0x7D09) /* Tried to allocate a virtual port ID that already exists. */

#define IFAULT_PL_VEM_INVALID_PHYNUM                            (0x7D0A) /* The identify frame contained an invalid phy number. */
#define IFAULT_PL_VEM_UNEXPECTED_PPORT_1                        (0x7D0B) /* An invalid physical port ID. */
#define IFAULT_PL_VEM_UNEXPECTED_PPORT_2                        (0x7D0C) /* An invalid physical port ID. */
//#define IFAULT_PL_VEM_UNEXPECTED_PPORT_3                      (0x7D0D) /* An invalid physical port ID. */
#define IFAULT_PL_VEM_UNEXPECTED_PPORT_4                        (0x7D0E) /* An invalid physical port ID. */
#define IFAULT_PL_VEM_UNEXPECTED_PPORT_5                        (0x7D0F) /* An invalid physical port ID. */
#define IFAULT_PL_VEM_UNEXPECTED_PPORT_6                        (0x7D10) /* An invalid physical port ID. */

#define IFAULT_PL_VEM_DM_REPORT_DEVICES_FAILED                  (0x7D1D) /* Attached-device reporting failed. */
#define IFAULT_PL_VEM_INVALID_PHYBITS                           (0x7D1E) /* An invalid port number or no phys are associated with the port. */

#define IFAULT_PL_VEM_DATA_MISMATCH_1                           (0x7D20) /* DMA data corrupted. */
#define IFAULT_PL_VEM_DATA_MISMATCH_2                           (0x7D21) /* DMA data corrupted. */
#define IFAULT_PL_VEM_DATA_MISMATCH_3                           (0x7D22) /* DMA data corrupted. */
#define IFAULT_PL_VEM_SMP_RESPONSE_SEND_FAILURE_1               (0x7D23) /* Failed to process the Discover List response. */
#define IFAULT_PL_VEM_SMP_RESPONSE_SEND_FAILURE_2               (0x7D24) /* Failed to process the Discover List response. */
#define IFAULT_PL_VEM_SMP_RESPONSE_SEND_FAILURE_3               (0x7D25) /* Failed to process the Discover List response.*/
#define IFAULT_PL_VEM_SMP_RESPONSE_SEND_FAILURE_4               (0x7D26) /* Failed to process the Discover List response. */
#define IFAULT_PL_VEM_SMP_RESPONSE_SEND_FAILURE_5               (0x7D27) /* Failed the SMP Report Zone Permission table response. */
#define IFAULT_PL_VEM_SMP_RESPONSE_SEND_FAILURE_6               (0x7D28) /* Failed the SMP Report Zone Permission table response. */

#define IFAULT_PL_VEM_INVALID_VPHY_ZONING                       (0x7D30) /* Virtual Expander Manager. All the virtual phys in the same virtual port are not in the same zone group. */

//#define IFAULT_PL_VEM_INVALID_DEVHANDLE_1                     (0x7D40) /* The SAS address was not found. */
#define IFAULT_PL_VEM_INVALID_DEVHANDLE_2                       (0x7D41) /* Invalid device handle. */
#define IFAULT_PL_VEM_INVALID_DEVHANDLE_3                       (0x7D42) /* The SAS address was not found. */
#define IFAULT_PL_VEM_INVALID_DEVHANDLE_4                       (0x7D43) /* Invalid device handle. */
#define IFAULT_PL_VEM_INVALID_DEVHANDLE_5                       (0x7D44) /* The SAS address was not found. */
#define IFAULT_PL_VEM_INVALID_DEVHANDLE_6                       (0x7D45) /* Invalid device handle. */
#define IFAULT_PL_VEM_INVALID_DEVHANDLE_7                       (0x7D46) /* Invalid device handle. */

#define IFAULT_PL_VEM_HOTPLUG_UNKNOWN_CHANGETYPE                (0x7D50) /* The Virtual Expander Manager detected an unrecognized change on a physical port. */

#define IFAULT_PL_VEM_SMP_PASS_NO_RSRC_FRAME_1                  (0x7D6B) /* The Virtual Expander Manager failed to obtain an IOP resource frame for processing an SMP pass-through command. */
#define IFAULT_PL_VEM_SMP_PASS_NO_RSRC_FRAME_2                  (0x7D6C) /* The Virtual Expander Manager failed to obtain an IOP resource frame for processing an SMP pass-through command. */

#define IFAULT_PL_VEM_EVENTS_NO_REPLICATED_ENCL_HNDL            (0x7D70) /* An enclosure handle was not found or was replicated for a virtual port. */
#define IFAULT_PL_VEM_HWINIT_TABLE_ENTRY_ERROR_1                (0x7D71) /* The Virtual Expander Manager failed to add the Static Root Virtual Port into the hardware Initiator/Virtual ID Response table. */
#define IFAULT_PL_VEM_VID_TABLE_ENTRY_ERROR_1                   (0x7D72) /* The Virtual Expander Manager failed to add the Static Root Virtual Port into the hardware Virtual ID Originator table. */
#define IFAULT_PL_VEM_HWINIT_TABLE_ENTRY_ERROR_2                (0x7D73) /* The Virtual Expander Manager failed to add the Physical Function Virtual Port into the hardware Initiator/Virtual ID Response table. */
#define IFAULT_PL_VEM_VID_TABLE_ENTRY_ERROR_2                   (0x7D74) /* The Virtual Expander Manager failed to add the Physical Function Virtual Port into the hardware Virtual ID Originator table. */

#define IFAULT_PL_VEM_FAILED_REPLICATION                        (0x7D80) /* The Virtual Expander Manager failed to replicate the physical device. */
#define IFAULT_PL_VEM_DEV_REPLICATION_INVALID_DEVH              (0x7D81) /* The Virtual Expander Manager failed to replicate a physical device with an invalid device handle. */
#define IFAULT_PL_VEM_FAILED_VPHY_ALLOC_SRVP_1                  (0x7D82) /* The Virtual Expander Manager failed to allocate virtual phys during Static Root Virtual Port creation. */
#define IFAULT_PL_VEM_FAILED_VPHY_FREE_SRVP_1                   (0x7D83) /* The Virtual Expander Manager failed to release virtual phys during Static Root Virtual Port creation.  */
#define IFAULT_PL_VEM_FAILED_CREATE_SRVP                        (0x7D84) /* The Virtual Expander Manager failed to create the Static Root Virtual Port during Virtual Expander creation. */
#define IFAULT_PL_VEM_FAILED_CREATE_VE                          (0x7D85) /* The Virtual Expander Manager failed to create the Virtual Expander around a physical port. */
#define IFAULT_PL_VEM_FAILED_CONF_SRVP                          (0x7D86) /* The Virtual Expander Manager failed to configure the Static Root Virtual Port. */
#define IFAULT_PL_VEM_FAILED_CREATE_PFVP                        (0x7D87) /* The Virtual Expander Manager failed to create the Physical Function Virtual Port during Virtual Expander creation. */

#define IFAULT_PL_VEM_ZONING_ILLEGAL_PHY_TABLE_ACCESS           (0x7D90) /* The Virtual Expander Manager received an SMP to configure a phy that does not exist. */
#define IFAULT_PL_VEM_CORRUPT_VP_TABLE                          (0x7D91) /* The Virtual Expander Manager Virtual Port table is corrupt. */
#define IFAULT_PL_VEM_INVALID_PPORT_VE_MAPPING                  (0x7D92) /* The Virtual Expander Manager physical port does not match with the corresponding Virtual Expander. */
#define IFAULT_PL_VEM_INVALID_PHYSICAL_DEVICE                   (0x7D93) /* Virtual Expander Manager. Invalid SAS address. */
#define IFAULT_PL_VEM_VP_DELETE_FAILED_VPHY_DELETE              (0x7D94) /* The Virtual Expander Manager failed to release virtual phys during Virtual Port Delete. */

#endif /* COMPILE_OPTION_SAS_PORT_VIRT */

/* RESERVED
 * The 0x7E00 to 0x7E14 block is reserved for BT1680 usage in Gen 2 controllers.
 * Do not use this block.
 */

//#define IFAULT_PL_NO_SAS_PLL_LOCK                       (0x7F00) // SAS1078 only

/*******************************************************************************
* IR IFAULT code definitions 0x8000 - 0xBFFF
*******************************************************************************/

/* See iopiFault.h */


/******************************************************************************
* UFF IFAULT code definitions 0xC000 - 0xFFFF
*******************************************************************************/

//#define IFAULT_TARG_UNDEFINED_FAULT                           (0xC000)
#define IFAULT_TARG_NO_MSG_FRAMES_ERR_RSP                       (0xC001) /* Frame requests exceed available transports. */
#define IFAULT_TARG_CMD_RCVD_BAD_STATUS                         (0xC002) /* Hardware reused a command buffer that firmware thought was already in use .*/
//#define IFAULT_TARG_ERR_MID_NOT_OUTSTANDING                   (0xC003) /* Tried to clean up a mid that is not outstanding. */
#define IFAULT_TARG_NON_AUTO_CMPLT_ERR_LCL_MID                  (0xC004) /* Unexpected MID on DMA completion of an internal request. */
#define IFAULT_TARG_MISMATCH_MID_IOINDEX_ERROR                  (0xC005) /* MID and I/O index in message does not match firmware state information. */
//#define IFAULT_TARG_BAD_MID_ON_COMPLETION_ERROR               (0xC006)
//#define IFAULT_TARG_UNUSED1                                   (0xC007)
//#define IFAULT_TARG_NO_MSG_FRAMES_STATUS_SEND                 (0xC008)
#define IFAULT_TARG_NON_AUTO_CMPLT_ERR_SYS_MID                  (0xC009) /* Unexpected MID on DMA completion of a host request. */
//#define IFAULT_TARG_NON_AUTO_CMPLT_ERROR                      (0xC00A)
//#define IFAULT_TARG_MISSING_CMD_BUFFS                         (0xC00B)
#define IFAULT_TARG_COMPLETE_UNKNOWN_STATUS                     (0xC00C) /* Hardware reported a completion of an IO that firmware did not think was outstanding. */
#define IFAULT_TARG_NO_MSG_FRAMES_CMD_BUFF_POST_ERR_RPLY        (0xC00D) /* Could not allocate message frame to send aborted command buffer reply. */
//#define IFAULT_TARG_UNPOSTED_INFO_MISMATCH                    (0xC00E)
//#define IFAULT_TARG_NO_FRAMES_FOR_RESET_EVENT                 (0xC00F)
//#define IFAULT_TARG_INVALID_DEV_HANDLE                        (0xC100)
//#define IFAULT_TARG_INVALID_DEV_HANDLE_TARGET_ASSIST          (0xC101)
//#define IFAULT_TARG_INVALID_DEV_HANDLE_TARGET_STATUS_SEND     (0xC102)
//#define IFAULT_TARG_UNEXPECTED_STATUS_SEND                    (0xC103)

#define TARG_MID_NA_ON_FAULT 0xffff0000  // used for data when mid does not apply

/********************************************************************************
*       FPE MANAGER SPECIFIC FAULT CODES - 0xD000 - 0xDFFF
********************************************************************************/
#define IFAULT_PL_FPE_MGR_TEMP_GENERIC_FAULT                    (0xD000)   /* Generic Fast Path Engine Fault. */

#define IFAULT_PL_FPE_MGR_FAILED_INCR_IOCOUNT                   (0xD011)   /* Failed to increment device ioCount. */
#define IFAULT_PL_FPE_MGR_FAILED_DECR_IOCOUNT                   (0xD012)   /* Failed to decrement device ioCount. */
#define IFAULT_PL_FPE_MGR_CLEARED_FAST_PATH_CAPABLE             (0xD013)   /* Firmware should not be doing clearing fastPathCapable once it is set. */
#define IFAULT_PL_FPE_MGR_FAILED_BLK_IOCOUNT_WA                 (0xD014)   /* Failed to modify device ioCount in blocked device workaround .*/
#define IFAULT_PL_FPE_FW_IOCOUNT_INCR_DECR_ERROR                (0xD015)   /* Error in incrementing or decrementing ioCount. */

#define IFAULT_PL_FPE_MGR_INVALID_STATE_PROPERTIES              (0xD020)   /* Could not find state in "Properties Table". */
#define IFAULT_PL_FPE_MGR_INVALID_STATE_PROPERTIES_2            (0xD021)   /* Could not find state in "Properties Table". */
#define IFAULT_PL_FPE_MGR_INFINITE_LOOP_DETECTED                (0xD022)   /* FPE manager state machine entered an infinite loop. */
#define IFAULT_PL_FPE_MGR_NO_TRANSITION_PICKED                  (0xD023)   /* FPE manager state machine failed to pick a transition. */
#define IFAULT_PL_FPE_MGR_REQ_PAUSE_FAILED                      (0xD024)   /* FPE Requestor hardware failed to pause. */
#define IFAULT_PL_FPE_MGR_COM_PAUSE_FAILED                      (0xD025)   /* FPE Completor hardware failed to pause. */
#define IFAULT_PL_FPE_MGR_INVALID_COMPLETOR_STATE               (0xD026)   /* FPE Completor incorrectly stuck in the wrong state. */
#define IFAULT_PL_FPE_MGR_OVERWRITING_SAVED_DEVHANDLE           (0xD027)   /* FPE Fpc Pending: Overwriting saved Fpc pending device handle. */

#define IFAULT_PL_FPE_INVALID_MID_SCSI_IO_REQUEST               (0xD030)   /* Received invalid message identifier in a SCSI I/O request to FPE from PL. */
#define IFAULT_PL_FPE_INVALID_DEVHANDLE_SCSI_IO_REQUEST         (0xD031)   /* Received invalid device handle in a SCSI I/O request to FPE from PL. */
#define IFAULT_PL_FPE_INVALID_DEVHANDLE_START_PENDING_IO        (0xD032)   /* Invalid device handle when attempting to start pending SCSI I/O. */

#define IFAULT_PL_FPE_UNHANDLED_GENERAL_ERROR_INTERRUPT         (0xD033)   /* Received an unhandled general error interrupt from FPE. */

#define IFAULT_PL_FPE_TIMER_ERROR_INT_INVALID_ERROR_1           (0xD035)   /* Received an invalid timer error interrupt. */
#define IFAULT_PL_FPE_TIMER_ERROR_INT_INVALID_ERROR_2           (0xD036)   /* Received an invalid timer error interrupt. */
#define IFAULT_PL_FPE_TIMER_ERROR_INVALID_REQ_MID               (0xD037)   /* Recevied an invalid Requestor message identifier during FPE timer overflow interrupt handling. */
#define IFAULT_PL_FPE_TIMER_ERROR_INVALID_REQ_DEVHANDLE         (0xD038)   /* Recevied an invalid Requestor device handle during FPE timer overflow interrupt handling. */
#define IFAULT_PL_FPE_TIMER_ERROR_INVALID_CPL_MID               (0xD039)   /* Recevied an invalid Completor message identifier during FPE timer overflow interrupt handling. */
#define IFAULT_PL_FPE_TIMER_ERROR_INVALID_CPL_DEVHANDLE         (0xD03A)   /* Recevied an invalid Completor device handle during FPE timer overflow interrupt handling.  */
#define IFAULT_PL_FPE_TIMER_ERROR_INVALID_OVERFLOW_SOURCE       (0xD03B)   /* Received a timer overflow interrupt from an invalid timer overflow source. */
#define IFAULT_PL_FPE_TIMER_ERROR_SRC_FW_NOT_SUPPORTED          (0xD03C)   /* Firmware is not a supported timer error overflow source. */

#define IFAULT_PL_FPE_TIMER_ERROR_INT_PARITY_ERROR              (0xD040)   /* Received a timer wheel RAM parity error. ( Bits [0:3] - Timer Error Src ). */

#define IFAULT_PL_FPE_INT_UNSUPPORTED_ASYNC_EVENT_RCVD          (0xD050)   /* Received an unsupported aysnchronous event (Bits [0:3] - EventStatus). */
#define IFAULT_PL_FPE_INT_COMPLETION_ASYNC_EVENT_RCVD           (0xD051)   /* Received an aysnchronous completion event from h/w, which is not supported. */
#define IFAULT_PL_FPE_INT_AUTO_PEND_ASYNC_EVENT_RCVD            (0xD052)   /* Received an aysnchronous auto pend event from h/w, which is not supported. */

#define IFAULT_PL_FPE_EXCP_RCVD_FPESTATUS_NONE                  (0xD060)   /* Received an exception message identifier(Mid) with an invalid or no FPE status. */
#define IFAULT_PL_FPE_EXCP_RCVD_INVALID_FPESTATUS               (0xD061)   /* Received an invalid or unrecognized FPE Exception. */
#define IFAULT_PL_FPE_ERR_LESS_THAN_ZERO_IOCOUNT                (0xD062)   /* FPE h/w attempted to decrement IoCount below Zero. */
#define IFAULT_PL_FPE_PEND_NEXT_NOT_NULL                        (0xD063)   /* NextPend MID not NULL when removing last entry from FPE Pending List. */
#define IFAULT_PL_FPE_INVALID_REQR_ERROR_INT                    (0xD064)   /* Received an invalid Requestor error interrupt. */
#define IFAULT_PL_FPE_INVALID_CMPL_ERROR_INT                    (0xD065)   /* Received an invalid Completor error interrupt. */
#define IFAULT_PL_FPE_CMPL_ERR_UNEXPCTD_SAS_COMPL_Q_ENTRY       (0xD066)   /* Received a Completor error interrupt with an unexpected SAS completion queue entry. */
#define IFAULT_PL_FPE_CMPL_ERR_UNEXPCTD_SAS_COMPL_Q_FLAG        (0xD067)   /* Received a Completor error interrupt with an unexpected SAS completion queue flag. */
#define IFAULT_PL_FPE_CMPL_ERR_UNCONFIRMED_RESPONSE_ERROR       (0xD068)   /* An unconfirmed response frame does not match with the corresponding entry in the Completor Status register. */
#define IFAULT_PL_FPE_CMPL_ERR_UNEXPECTED_SAS_CQ_FLAG_ERROR     (0xD069)   /* An unexpected SAS completion queue flag message identifier(Mid) entry does not match with entry in the Completor Status register. */
#define IFAULT_PL_FPE_INVALID_MID_CHANGE_MIDSTATE_FPE_FLAG      (0xD070)   /* Received an invalid message identifier while clearing Mid State Table Fpe Flag. */
#define IFAULT_PL_FPE_INVALID_MID_CHECK_MIDSTATE_FPE_FLAG       (0xD071)   /* Received an invalid message identifier while testing Mid State Table Fpe Flag. */
#define IFAULT_PL_FPE_MIDSTATE_DEVHANDLE_MISMATCH               (0xD072)   /* Device handle does not match the one in the Mid State Table for the FPE started message identifier. */
#define IFAULT_PL_FPE_TIMEOUT_MISSED_IOS                        (0xD073)   /* Timeout handler coud not find as many mids with the same bucket number as the bucket count. */
#define IFAULT_PL_FPE_START_PENDING_IO_HW_TIMEOUT               (0xD074)   /* Could not start FPE's pending I/Os because the 'Active' bit never went low. */
#define IFAULT_PL_FPE_REQUESTOR_NOT_RUNNING                     (0xD075)   /* Tried to start FPE's pending I/Os, but the requestor is not running. */
#define IFAULT_PL_FPE_INVALID_EXCEPTION_Q_ENTRY                 (0xD076)   /* Exception queue interrupt handler found an invalid entry in the queue. */
#define IFAULT_PL_FPE_ABORTED_IO_TIMEOUT                        (0xD077)   /* An FPE I/O which is in a TM's abort list has timed out. */
#define IFAULT_PL_FPE_CMPL_ERR_TBM_ARG_COUNT                    (0xD078)   /* Trace Buffer Manager detected Completor requesting > 4 arguments. */
#define IFAULT_PL_FPE_REQR_ERR_TBM_ARG_COUNT                    (0xD079)   /* Trace Buffer Manager detected Requestor requesting > 4 arguments. */
#define IFAULT_PL_FPE_IOP_COMP_Q_TM_MID_NOT_IN_USE              (0xD07A)   /* Invalid Mid Used during TM cleanup of the completion queue. */

#define IFAULT_PL_FPE_RESET_HUNG                                (0xD080)   /* FPE never came out of reset. */
#define IFAULT_PL_FPE_GETMIDINFO_INVALID_MID                    (0xD081)   /* Recevied a PLI request with an invalid message identifier. */
#define IFAULT_PL_FPE_GETDEVIOCOUNT_INVALID_DEVH                (0xD082)   /* Recevied a PLI request with an invalid device handle. */
#define IFAULT_PL_FPE_CP_CONDITION_NOT_IN_SPEC                  (0xD083)   /* Received a checkpoint condition which is out of current hardware specifications. */
#define IFAULT_PL_FPE_CP_RCVD_BUT_NOT_ENABLED                   (0xD084)   /* Received a checkpoint interrupt that was not enabled. */
#define IFAULT_PL_FPE_INVALID_MID_SET_REPLY_DEST_REQ            (0xD085)   /* Received an invalid message identifier value on change reply destination request. */
#define IFAULT_PL_FPE_INVALID_SET_REPLY_DEST_REQ                (0xD086)   /* Received an invalid reply destination value on change reply destination request. */

#define IFAULT_PL_FPE_IOP_START_QUEUE_FULL                      (0xD090)   /* Firmware could not start sending I/O to FPE since IOP Start Queue is full. */
#define IFAULT_PL_FPE_IOP_COMPL_MID_NOT_IN_USE                  (0xD091)   /* MidUsed for a message identifier set by FPE via IOP Completion queue is not set. */
#define IFAULT_PL_FPE_START_PENDING_IOS_WHILE_ACTIVE            (0xD092)   /* Firmware tried to start FPE pended I/O while a previous request was still outstanding. */
#define IFAULT_PL_FPE_INVALID_MID_CHECK_ADD_IO_TIMER            (0xD093)   /* Received an invalid message identifier while testing Mid State Table Fpe flag for "Add I/O Timer". */
#define IFAULT_PL_FPE_INVALID_EXCP_HANDLER                      (0xD094)   /* IOP handler for an invalid descriptor type exception returned a failure. */
#define IFAULT_PL_FPE_REQ_RESUME_IOCNT_WA_FAIL                  (0xD095)   /* Requestor failed to go to IDLE state in resume requestor workaround. */
#define IFAULT_PL_FPE_EXCP_FPESTATUS_DEVICE_FP_DISABLED         (0xD096)   /* Recevied an FPE Exception with FPE Status DEVICE_FP_DISABLED even after IOP enabling FP on the device */

#define IFAULT_PL_FPE_MGR_AUTOPEND_TO_EMPTY_PEND_Q              (0xD0A0)   /* Tried to set AUTO_PEND on an empty Pend Q in Fpe Manager. */

#define IFAULT_PL_FPE_MGR_SM_EXIT_CRITERIA_INVALID_STATE        (0xD0B0)   /* Fault on Invalid State while exiting FPE Manager State. */

#define IFAULT_PL_FPE_TM_INVALID_MID_IN_FPE_EXCP_Q              (0xD0ED)    /* TM found a Mid with MessageValid Bit set in Exception Queue. */

#define IFAULT_PL_FPE_INVALID_FRAME_OWNERSHIP                   (0xD100)   /* Frame ownership is invalid.*/
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_SCSI_IO_ERROR_REPLY     (0xD101)   /* Failed to change frame ownership on SCSI I/O error reply. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_TM_COMPL_ABORTED_IO         (0xD102)   /* Failed to change frame ownership on TM complete aborted I/Os. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_TIMER_OVERFLOW_ERROR        (0xD103)   /* Failed to change frame ownership on TimerOverflow error. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_FPE_HW_PENDQ_CLEANUP        (0xD104)   /* Failed to change frame ownership on FPE HW Pend Queue cleanup. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_TXFMGR_ERROR                (0xD105)   /* Failed to change frame ownership on TxFrameManager SATA error. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_RXCTXMGR_CLEANUP            (0xD106)   /* Failed to change frame ownership on RxContextManager cleanup. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_FPE_EXCP_FP_DISABLED        (0xD107)   /* Failed to change frame ownership on FPE Exception FPESTATUS_DEVICE_FP_DISABLED. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_FPE_EXCP_DEVH_OOR           (0xD108)   /* Failed to change frame ownership on FPE Exception FPESTATUS_DEVHANDLE_OUT_OF_RANGE. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_FPE_EXCP_QDEPTH_LIMIT       (0xD109)   /* Failed to change frame ownership on FPE Exception FPESTATUS_QDEPTH_LIMIT. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_FPE_EXCP_VF_MISMATCH        (0xD10A)   /* Failed to change frame ownership on FPE Exception FPESTATUS_VF_MISMATCH. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_IOP_COMP_Q_1            (0xD10B)   /* Failed to change frame ownership on an I/O Completion while resolving IOP completion queue full. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_IOP_COMP_Q_2            (0xD10C)   /* Failed to change frame ownership on an I/O Completion while handling a completor error interrupt. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_IOP_COMP_Q_3            (0xD10D)   /* Failed to change frame ownership on an FPE I/O Completion. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_IOP_COMP_Q_4            (0xD10E)   /* Failed to change frame ownership on a normal I/O Completion. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_FPE_TIMEOUT_INT         (0xD10F)   /* Failed to change frame ownership during I/O timeout handling. */
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_RXDMA_OVERFLOW_INT      (0xD110)   /* Failed to change frame ownership while handling Rx DMA Overflow Int.*/
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_RXEEDP_RESET            (0xD111)   /* Failed to change frame ownership while handling RxEEDP Reset.*/
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_RXCTX_RETRANSMIT        (0xD112)   /* Failed to change frame ownership while handling RxCtxMgr retransmit Int.*/
#define IFAULT_PL_FPE_FRM_OWNERSHIP_ERR_IOP_COMP_Q_CLEANUP      (0xD113)   /* Failed to change frame ownership during TM cleanup of the completion queue. */

#define IFAULT_PL_FPE_PENDQ_CLEANUP_INVALID_REQUEST             (0xD130)   /* TM tried to cleanup FPE hardware "Pend Queue" without disabling FPE on that device. */
#define IFAULT_PL_FPE_PENDQ_CLEANUP_DEVHANDLE_MISMATCH          (0xD131)   /* Device handle mismatch while cleaning up FPE hardware "Pend Queue". */
#define IFAULT_PL_FPE_GET_HW_PENDCOUNT_FAILED                   (0xD132)   /* Failed to get FPE hardware Pend Count since Fast Path is not disabled for this device */
#define IFAULT_PL_FPE_PENDQ_MID_NOT_PENDING                     (0xD133)   /* Mid in FPE Hw Pend Q with Pending Flag not set. */
#define IFAULT_PL_FPE_TIMER_REMOVAL_FAILURE                     (0xD134)   /* Failed to remove timer for an I/O. */
#define IFAULT_PL_FPE_TIMER_REMOVAL_FAILURE_1                   (0xD135)   /* Failed to remove timer for an I/O while handling exception. */

#if ENABLE_TRACE_BUFFER_MANAGER
/* Trace Buffer Manager */
#define IFAULT_PL_TBM_RESET_HUNG                                (0xD201)   /* TBM never came out of reset. */
#define IFAULT_PL_TBM_FLUSH_HUNG                                (0xD202)   /* TBM did not complete flushing the residual trace entries to extended buffer. */
#define IFAULT_PL_TBM_INVALID_MODE_DIAG_REQUEST                 (0xD203)   /* TBM not in Extended Mode when processing Diagnostic requests. */
#define IFAULT_PL_TBM_UNABLE_TO_ADD_FW_TRACE_ENTRY              (0xD204)   /* Unable to add a firmware trace entry .*/
#define IFAULT_PL_TBM_DATA_DMAD_NOT_MULTIPLE_SEGMENT_SIZE       (0xD205)   /* Data DMAd is not multiples of TBM_DEFAULT_SEGMENT_RAM_SIZE. */
#define IFAULT_PL_TBM_UNSUPPORTED_BUFFER_TYPE_LOC_COMBO         (0xD206)   /* Unsupported combination of Buffer Type and Buffer Location to Copy/DMA data. */
#define IFAULT_PL_TBM_INVALID_TRACE_ERROR                       (0xD207)   /* Invalid trace error. */
#define IFAULT_PL_TBM_FW_ILLEGAL_REQ                            (0xD208)   /* Firmware entered illegal trace entry. */
#define IFAULT_PL_TBM_REQUESTOR_ILLEGAL_REQ                     (0xD209)   /* Requestor entered illegal trace entry. */
#define IFAULT_PL_TBM_COMPLETOR_ILLEGAL_REQ                     (0xD20A)   /* Completor entered illegal trace entry. */
#define IFAULT_PL_TBM_TRACE_PARITY_ERROR                        (0xD20B)   /* TBM detected parity error in Trace Buffer. */
#define IFAULT_PL_TBM_DMA_PARITY_ERROR                          (0xD20C)   /* TBM DMA Engine detected parity error in Trace Buffer. */
#define IFAULT_PL_TBM_DMA_CMPL_ERROR                            (0xD20D)   /* TBM DMA Completion Error. */
#define IFAULT_PL_TBM_PAUSE_HUNG                                (0xD20E)   /* TBM did not go to IDLE state after setting PauseTracer. */
#define IFAULT_PL_TBM_INVALID_NUM_ARGS                          (0xD20F)   /* Found a trace entry with an illegal number of arguments. */
#define IFAULT_PL_TBM_RESET_IDLE_TIMEOUT                        (0xD210)   /* Timed out waiting for idle state on reset. */
#endif

#endif /* end of file */
