/***************************************************************************
 *                                                                         *
 *  Copyright 2003-2011 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pliProtos.h
 *          Title:  Protocal Layer Interface (PLI) Prototypes
 *     Programmer:  Steve Johnson
 *  Creation Date:  February 5, 2003
 *
 *
 *  Description
 *  ------------
 *  This include file contains SAS firmware Protocol Layer (PL) exports used
 *  by other FW modules to call PL functions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef PLI_PROTOS_H_INCLUDED
#define PLI_PROTOS_H_INCLUDED


/******************************************************************************
*
*      P L I    E X P O R T    F U N C T I O N S
*
*******************************************************************************/

// Please keep this list alphabetized. Add TODO - DOC if make any changes w/o changing PLI Specification

ISTATUS pliConfigPage(PLI_DEV_HANDLE DevHandle, U32 Action, U32 PageAddress, U32 PageType, U32 PageNum, U32 Length, PTR_PAGE_IOC PtrIocConfig, U32 Context);
ISTATUS pliExtendedBufferPosted( U8 ExtBufferType, PU8 PtrBufferInfo, U32 BufferInfoSize );
ISTATUS pliExtendedBufferReleased( U8 ExtBufferType );
ISTATUS pliGetMemoryRequirements(PTR_PLI_GET_MEMORY_REQUIRED PtrPlMemReq, PTR_PLI_RUNTIME_CONFIG Config);
U16     pliGetMaxDevHandle(void);
U16     pliGetNumDevHandles(PLI_RUNTIME_CONFIG *PtrRtConfig);
void    pliIstwiIORequestComplete (PTR_MSG_IOC PtrMsg);
ISTATUS pliMsg(PTR_MSG_IOC PtrMsg, U32 Mid, PLI_DEV_HANDLE DevHandle);
ISTATUS pliPortEnable( PTR_MSG_IOC PtrMsg );
ISTATUS pliRegister(PLI_DEV_HANDLE DevHandle, U32 Register);
ISTATUS pliBlockDevice(PLI_DEV_HANDLE DevHandle, U32 BlockDevice);
void    pliResourceAvailable(IOPI_RESOURCE_TYPE ResType);
ISTATUS pliSasCoreCompleteInt( U32 MaxCompletionsToProcess );
ISTATUS pliSasCoreCriticalInterrupt( void );
ISTATUS pliSasCoreHighInterrupt( void );
ISTATUS pliSasCoreLowInterrupt( void );
void    pliSasCoreCommandReceived(U32 MaxCmdsToProcess);
ISTATUS pliTargetAssist(PTR_MSG_IOC PtrMsg, U32 Mid, U16 IoIndex );
ISTATUS pliSetFpeMidInfo(U32 Mid, U32 ReplyDest, PTR_PLI_FPE_MID_INFO PtrMidInfo);
ISTATUS pliSetParams(U32 IocNum, PTR_PLI_SET_PARAMS ptrPlParams);
ISTATUS pliSetSODParams(PTR_PLI_SET_SOD_PARAMS PlSetSODParams, PTR_PLI_RUNTIME_CONFIG Config);
ISTATUS pliStartOfDay(U32 PortEnable);
ISTATUS pliStartScsiIo(PTR_MSG_IOC PtrMsg, U32 Mid, PLI_DEV_HANDLE DevHandle, MSG_IOC const *PtrMsgCached, U32 IoFlags);
ISTATUS pliTaskMgmt(PTR_MSG_IOC PtrMsg, U32 Mid, PLI_DEV_HANDLE DevHandle, IOPI_SUBSYSTEM SubSystemId);
ISTATUS pliTaskMgmtComplete(PTR_MSG_IOC PtrMsg);

#if INVADER && PL_FEATURE_SIM_PL_FW
////HACK: Detroit FPGA Test Hack
void pliRunFakeFPEIO();
void pliSimFakeDADrives(BOOL FastPathCapable);
////HACK:------
#endif

#if CBB_COMPILE_OPTION_SRIOV_EN
ISTATUS pliCreateAddRemoveEvent( PLI_DEV_HANDLE DevHandle, PTR_MSG_IOC PtrMsg, U32 Reason );
#endif

#if (COMPILE_OPTION_SAS_PORT_VIRT)
ISTATUS pliVemCreateVP(PTR_MSG_IOC PtrMsg);
ISTATUS pliVemDeleteVP(PTR_MSG_IOC PtrMsg);
#endif  /*  COMPILE_OPTION_SAS_PORT_VIRT  */

void    pliSendEpow( void );

#if INVADER
ISTATUS pliFpeGeneralErrorInterrupt( void );
ISTATUS pliFpeCheckpointOrTraceInterrupt( void );
ISTATUS pliFpeAsyncEventInterrupt( U32 MaxReadCount );
ISTATUS pliFpeExceptionInterrupt( U32 MaxReadCount );
ISTATUS pliFpeCompletionInterrupt( U32 MaxReadCount );

ISTATUS pliGetFpeMidInfo(PTR_PLI_FPE_MID_INFO PtrMidInfo, U32 Mid);
ISTATUS pliGetDeviceIOCount(PLI_DEV_HANDLE DeviceHandle, PU32 PtrActiveIOCount, PU32 PtrFWPendCount, PU32 PtrFPEPendHead);

#if ENABLE_TRACE_BUFFER_MANAGER
/* Trace Buffer Manager */
ISTATUS pliTBMAddEntry(U32 Index, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4, U8 NumArgs);
#endif

ISTATUS pliControlRequest(U32 Operation, void * Param0, void * Param1, void * Param2, U32 RequestContext);

#if PL_FEATURE_PL_LIGHT
ISTATUS pliLightSetSODParams(PTR_PLI_SET_SOD_PARAMS PlSetSODParams, PTR_PLI_RUNTIME_CONFIG Config);
ISTATUS pliLightEnable(void);
ISTATUS pliLightStartScsiIo(PTR_MSG_IOC PtrMsg, U32 Mid, PLI_DEV_HANDLE DevHandle, MSG_IOC const *PtrMsgCached, U32 IoFlags);
ISTATUS pliLightGetFpeMidInfo(PTR_PLI_FPE_MID_INFO PtrMidInfo, U32 Mid);
ISTATUS pliLightSetFpeMidInfo(U32 Mid, U32 ReplyDest, PTR_PLI_FPE_MID_INFO PtrMidInfo);
ISTATUS pliLightControlRequest(U32 Operation, void * Param0, void * Param1, void * Param2, U32 RequestContext);
#endif
#endif

#endif /* End PLI_H_INCLUDED */

