/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iocEngineDrv.h
 *          Title:  IOC Engine Driver Header File
 *     Programmer:  Nilesh Wilanker
 *  Creation Date:  Augest 17, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  08/17/11  NW   Initial OSS Compliance version.
 *
 *
 * Description
 * ------------
 * This file contains the definitions used for IOC Engine driver as
 * an interface.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOC_ENGINE_DRV_H
#define IOC_ENGINE_DRV_H

#include <linux/module.h>
#include "basetype.h"        /* Base types */

typedef enum _IOC_STATUS
{
    IOC_SUCCESS = 0,
    IOC_FAILURE
    
} IOC_STATUS;

typedef struct _IOC_ENGINE_DRIVER
{
    U32  (*osalMoveFromDcr)            (U32);
    void (*osalMoveToDcr)              (U32, U32);
    void (*osalAcquireSpinLockIrqSave) (void *, Ul32);
    void (*osalReleaseLockIrqSave)     (void *, Ul32);
    void (*osalSpinLockInit)           (void *);
    void *(*osalKmalloc)               (U32);
    void *(*osalIoremap)               (unsigned long long, U32);
    void (*osalIounmap)                (void*);
    void (*osalKfree)                  (void *);
    void (*osalMemset)                 (void*, U8, U32);
    void (*osalWaitForWrite)           (void);
    void (*osalUDelay)                 (U32);
    S32  (*osalRegisterIrq)            (U32, void *, Ul32, const char *,
                                        void *);
    void (*osalFreeIrq)                (U32 , void *);
    void (*osalMemcpy)                 (void*, void*, U32);  
    void (*iopiTracePrint0)            (long long Pattern, U32 Index);
    void (*iopiTracePrint1)            (long long Pattern, U32 Index, U32 Arg1);
    void (*iopiTracePrint2)            (long long Pattern, U32 Index, U32 Arg1, U32 Arg2);
    void (*iopiTracePrint3)            (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3);
    void (*iopiTracePrint4)            (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);
    void (*iopiConsolePrint0)          (char * FmtStr);
    void (*iopiConsolePrint1)          (char * FmtStr, U32 Arg1);
    void (*iopiConsolePrint2)          (char * FmtStr, U32 Arg1, U32 Arg2);
    void (*iopiConsolePrint3)          (char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3);
    void (*iopiConsolePrint4)          (char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);
} IOC_ENGINE_DRIVER, *PTR_IOC_ENGINE_DRIVER;

extern IOC_ENGINE_DRIVER gIocEngineDriver;

/* Exported Functions - IOC Engine Driver Functions */
extern IOC_STATUS iocEngineRegisterDriver(ENGINE_DRIVER EngineDriver);
extern IOC_STATUS iocEngineUnregisterDriver(U32 Major);

#endif /* IOC_ENGINE_DRV_H */
