/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  basetype.h
 *          Title:  Header File for base types
 *     Programmer:  Azhar Shaikh
 *  Creation Date:  November 19, 2009
 *
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  19/11/09  AS   Creation.
 *
 * Description
 * ------------
 *  This include file contains the typedefs for the data types to be used.
 *
 *-------------------------------------------------------------------------
 */

#ifndef BASETYPE_H_INCLUDED
#define BASETYPE_H_INCLUDED

/*******************************************************************************
 * Define MPI2_POINTER if it hasn't already been defined. By default
 * MPI2_POINTER is defined to be a near pointer. MPI2_POINTER can be defined as
 * a far pointer by defining MPI2_POINTER as "far *" before this header file is
 * included.
 */
#ifndef MPI2_POINTER
#define MPI2_POINTER     *
#endif

/* Use this to prevent compiler warnings for unused parameters. The compiler
 * should be smart enough that this doesn't create any extra code.
 */
#define UNUSED_PARAMETER(Param) (Param=Param)

/* Define Basic Types */

typedef signed   char    S8;
typedef unsigned char    U8;
typedef signed   short   S16;
typedef unsigned short   U16;
typedef signed   int     S32;
typedef unsigned int     U32;
typedef unsigned long    Ul32;

typedef struct _S64
{
    U32 Low;
    S32 High;
} S64;


typedef struct _U64
{
    U32 Low;
    U32 High;
} U64;


typedef struct _U96
{
    U32 Word0;
    U32 Word1;
    U32 Word2;
} U96;

typedef struct _U128
{
    U32 Word0;
    U32 Word1;
    U32 Word2;
    U32 Word3;
} U128;


/* Define Pointers to Basics Types  */

typedef S8   *PS8;
typedef U8   *PU8;
typedef S16  *PS16;
typedef U16  *PU16;
typedef S32  *PS32;
typedef U32  *PU32;
typedef S64  *PS64;
typedef U64  *PU64;
typedef U96  *PU96;
typedef U128 *PU128;

typedef U32   BOOL;

/* Define some constant types */
#ifndef NULL
#define NULL    (0)
#endif

#define TRUE    (1)
#define FALSE   (0)

#define SUCCESS (0)
#define FAILURE (-1)

#define INLINE inline

INLINE static U64 make64(U32 Low, U32 High) {
    U64 Scratch;
    Scratch.Low = Low;
    Scratch.High = High;
    return Scratch;
}

#define  MAKE64(HIGH,LOW) make64((U32)(LOW),(U32)(HIGH))
#define GET64_LOW(x)   (x.Low)
#define GET64_HIGH(x)  (x.High)

#define SWAB2(Value)  ((((Value) & 0x00ff) << 8) \
                       | (((Value) & 0xff00) >> 8) \
                      )

INLINE static U32 SWAB4(U32 Value)
{
    U32 tempVar;

    tempVar = Value ^ ((Value <<16) | (Value >> 16));
    tempVar &= ~0x00ff0000;
    Value = (Value << 24) | (Value >> 8);
    return Value ^ (tempVar >> 8);
}

#endif

