/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopi.h
 *          Title:  IOPI Header
 *     Programmer:  Aseem Sharma
 *  Creation Date:  January 29, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  01/04/10  AS    Initial Version
 *  04/28/10  AS    After Self-Review
 *  05/13/10  AAS   After Peer-Review
 *  06/25/10  AAS   Including macro for manufacturing page 15
 *  07/10/10  AAS   Fixes done while testing
 *  08/13/10  AAS   Adding SASIO Unit pages 5 and 7 needed for PL-7.5
 *  09/03/10  AAS   Implementing Review COmments
 *
 * Description
 * ------------
 * This file contains the structure declarations and function prototypes for
 * implementation of IOPI functions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPI_H_HEADER
#define IOPI_H_HEADER

/*Defines used by other header files */
#define RAID                                        (0)
#define IOP_DBG_MASK                                (0xFFFFFFFFFFFFFFFF)
#define PL_HW_WORKAROUND_SATA_POWER_MANAGEMENT_COMPILED_IN (0)
#define CBB_COMPILE_OPTIONS_FORCE_MPT               (0)
#define PCIE_ZIP_CHIP_WORK_AROUND                   (1)
#define COMPILE_OPTION_WORKAROUND_OPTICAL_SUPPORT   (0)
#define COMPILE_OPTION_SAS_PORT_VIRT                (0)
#define COMPILE_OPTION_POWER_MANAGEMENT             (0)
#define CHIP_TYPE_21XX                              (1)
#define IOC_ETHERNET_SUPPORT                        (0)
#define SAS_PL_ONLY                                 (1) //From PLI Spec
#define IOC_MAN_PAGE_13_SGPIO_ENTRIES               (16)
#define FW_COMPILE_OPTION_VIRTLY_REAL_FUN            0
#define IOP_DRV_WRITE_MAX_ARGS                      (8)

#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/spinlock.h>
#include <linux/delay.h>
#include <linux/of.h>
#include <linux/gpio.h>
#include <linux/of_gpio.h>
#include <linux/timer.h>
#include <asm/param.h>
#include <linux/jiffies.h>
#include <asm/dcr-native.h>
#include <linux/mutex.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/stat.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/io.h>
#include <linux/kthread.h>
#include <linux/err.h>


#include "basetype.h"
#include "mpi2.h"
#include "mpi2_ioc.h"
#include "mpi2_init.h"
#include "iopiMaxes.h"
#include "mpi2_cnfg.h"
#include "mpi2_targ.h"
#include "mpi2_raid.h"
#include "mpi2_sas.h"
#include "mpi2_tool.h"
#include "iop_mpic.h"
#include "iopiIStatus.h"
#include "iopiDefs.h"
#include "iopiIFault.h"
#include "iopiIocLogInfo.h"
#include "iopiConfig.h"
#include "mpi2_ra.h"
#include "mpi2_hbd.h"
#include "iopiMsg.h"
#include "pliDefs.h"
#include "pliProtos.h"
#include "pliIocLogInfo.h"
#include "iopPrint.h"        /* Print Module Header File */

#define DEVICE_NODE                                 "/dev/lsi/iop"
#define IOP_SAS_CORE_BASE_ADDR                      (0xD0000000)
#define IOP_SAS_CORE_HIGH_ADDR                      (0x050)
#define IOP_CLEAR_INT_MASK                          (0xFFFFFFFF)
#define IOP_TIMER_SLEEP_INTERVAL                    (1000/18)
#define IOP_MAX_CRIT_FRAMES                         (5)
#define IOP_MAX_FRAMES_COUNT                        (50)
#define IOP_START_FRAME_NUMBER                      (2)
#define IOP_MAX_HANDLES_SUPPORTED                   (30)
#define IOP_INVALID_HANDLE                          (0xFF)
/* IOP Initailization Failure */
#define IOPFAULT_INITIALIZATION_FAILED              (0x1001)
#define IOPI_SUBSYSTEM_RESOURCE                     (-1)

/* Definitions for PL Devhandles */
#define PLI_DEV_INVALID_HANDLE                      (0)

/* ISTWI related Macros */
#define IOP_ISTWI_BUS_QUEUE_SIZE                    (10)
#define IOP_ISTWI_MAX_BUS_NUM                       (2)
#define IOP_ISTWI_BUS_NO_SYSTEM                     (0xFF)
#define IOP_ISTWI_QUEUE_EMPTY                       (0xFF)

/* Default SAS Addresss */
#define IOP_DEFAULT_SAS_ADDRESS_HIGH                (0x500605b0)
#define IOP_DEFAULT_SAS_ADDRESS_LOW                 (0x11223300)

#define SASHW_DCR_CCR                               (0x0000E000)  // Clock, configuration, and reset module
#define SASHW_DCR_HW_ACCELERATOR                    (0x0000D000)  // Hardware IO accelerators
#define SASHW_DCR_CCR_CUDAKEY_ADDR                  (SASHW_DCR_CCR + 0x28)  // Cuda Feature Key
#define SASHW_DCR_CCR_MRKEY_ADDR                    (SASHW_DCR_CCR + 0x29)  // MR Feature Key
#define SASHW_DCR_PLB_DETROIT_CONTROL_ADDR          (SASHW_DCR_HW_ACCELERATOR + 0x0F) // Detroit Control
#define SASHW_DCR_PLB_SLAVE_REGION_CONTROL_ADDR     (SASHW_DCR_HW_ACCELERATOR + 0x13) // PLB slave region control

/* Definitions for COnfiguration Pages */
#define MAN_PAGE_0                                  (0)
#define MAN_PAGE_3                                  (3)
#define MAN_PAGE_5                                  (5)
#define MAN_PAGE_6                                  (6)
#define MAN_PAGE_7                                  (7)
#define MAN_PAGE_8                                  (8)
#define MAN_PAGE_11                                 (11)
#define MAN_PAGE_12                                 (12)
#define MAN_PAGE_13                                 (13)
#define MAN_PAGE_14                                 (14)
#define MAN_PAGE_15                                 (15)
#define MAN_PAGE_17                                 (17)
#define MAN_PAGE_30                                 (30)
#define MAN_PAGE_31                                 (31)
#define EXT_MAN_PAGE_40                             (40)
#define EXT_MAN_PAGE_41                             (41)
#define IOC_PAGE_1                                  (1)
#define IOC_PAGE_8                                  (8)
#define IO_UNIT_PAGE_0                              (0)
#define IO_UNIT_PAGE_1                              (1)
#define SAS_IO_UNIT_PAGE_0                          (0)
#define SAS_IO_UNIT_PAGE_1                          (1)
#define SAS_IO_UNIT_PAGE_4                          (4)
#define SAS_IO_UNIT_PAGE_5                          (5)
#define SAS_IO_UNIT_PAGE_7                          (7)
#define SAS_IO_UNIT_PAGE_8                          (8)
#define SAS_PHY_PAGE_0                              (0)
#define SAS_PHY_PAGE_1                              (1)
#define SAS_PHY_PAGE_3                              (3)
#define BIOS_PAGE_2                                 (2)
#define BIOS_PAGE_3                                 (3)

/*
 * Defines for Descriptor Queues Indexes
 */
#define IOP_REQ_DESCQ_DEFAULT                   (0)
#define IOP_REQ_DESCQ_SCSI_IO                   (IOP_REQ_DESCQ_DEFAULT)
#define IOP_REQ_DESCQ_SCSI_TARG                 (IOP_REQ_DESCQ_DEFAULT)
#define IOP_REQ_DESCQ_HI_PRIO                   (1) //(IOP_REQ_DESCQ_DEFAULT)
#define IOP_REQ_DESCQ_FPE                       (2)
#define IOP_REQ_DESCQ_RAIDA                     (IOP_REQ_DESCQ_DEFAULT) //(1)
#define IOP_REQ_DESCQ_REQ_FREE                  (3)

#define IOP_REPLYQ_FPE                          (2)


/*
 * Possible values for the ReplyDestination Mid State FW flags
 * 0 : Post completion into appropriate Host Message Unit Reply FIFIO
 *   : NOTE: This is what we clear the value to finish off a mid when completing/aborting
 *
 * 1 : Post completion on the IOP completion Queue.
 */
/* added CLEARED define to avoid confusion in the IO completion cases */
#define FPE_MID_REPLY_DEST_CLEARED                  0
#define FPE_MID_REPLY_DEST_HOST_MU_REPLY_FIFO       0
#define FPE_MID_REPLY_IOP_COMPLETION_QUEUE          1

#define PCIE_FMU_DESCQ4_CNSMR_IDX_OFFSET    (0x00EC)
#define PCIE_FMU_DESCQ0_CNSMR_IDX_OFFSET    (0x012C)
#define PCIE_FMU_DESCQ1_CNSMR_IDX_OFFSET    (0x013C)
#define PCIE_FMU_DESCQ2_CNSMR_IDX_OFFSET    (0x014C)
#define PCIE_FMU_DESCQ3_CNSMR_IDX_OFFSET    (0x015C)

#define PCIE_FMU_REPLYQ_PORT0_OFFSET        (0x01D8)
#define PCIE_FMU_REPLYQ_PORT1_OFFSET        (0x01E8)
#define PCIE_FMU_REPLYQ_PORT2_OFFSET        (0x01F8)
#define PCIE_FMU_MSG_BIDQ_PORT_OFFSET       (0x0220)


/* Calculate the offset to the Descriptor Queue Consumer Index for
 * a given Descriptor Queue Number.  Number should be between 0
 * and 4.  Undefined results otherwise */
#define PCIE_FMU_DESCQ_CNSMR_IDX_OFFSET(Num)(Num==0 ? (PCIE_FMU_DESCQ0_CNSMR_IDX_OFFSET) : (Num==1 ? (PCIE_FMU_DESCQ1_CNSMR_IDX_OFFSET) : (Num==2 ? (PCIE_FMU_DESCQ2_CNSMR_IDX_OFFSET) : (Num==3 ? (PCIE_FMU_DESCQ3_CNSMR_IDX_OFFSET) : (Num==4 ? (PCIE_FMU_DESCQ4_CNSMR_IDX_OFFSET) : (0xFFFF) ) ) ) ) )
/* Calculate the offset to the Reply Queue Port given a Reply Queue
 * port number.  The port number should be between 0 and 2. Otherwise
 * the offset to Reply Queue Port 2 will be provided */
#define PCIE_FMU_REPLYQ_PORT_OFFSET(Num)    (Num==0 ? (PCIE_FMU_REPLYQ_PORT0_OFFSET) : (Num==1 ? (PCIE_FMU_REPLYQ_PORT1_OFFSET) : (PCIE_FMU_REPLYQ_PORT2_OFFSET)) )



#define IOP_SAS_INTS (  SAS2HW_UIC0_SASCOMPQNOTEMPTY  | \
                        SAS2HW_UIC0_SASCMDRX          | \
                        SAS2HW_UIC0_ALL_INT_CRIT      | \
                        SAS2HW_UIC0_ALL_INT_HIGH      | \
                        SAS2HW_UIC0_ALL_INT_LOW         \
                      )

/* Invalid Frame Request */
#define TM_FLAGS_IOP_DONE                           (0x0001)

#define ROUND_UP_U32(Bytes, Size)       ((((U32)(Bytes)+(U32)(Size)-1)/(U32)(Size))*(U32)(Size))
#define MAX(a, b)                       (((a) > (b)) ? (a) : (b))

#define IOP_DEVH_VALID(DevHandle) ( ((DevHandle)==0 || (DevHandle) > pliGetMaxDevHandle()) ? ISTATUS_FAILURE : ISTATUS_SUCCESS)

/* Macro for acquiring the lock */
#define IOP_SET_LOCK(structLock, lockFlag) spin_lock_irqsave (&(structLock)->SetLock, lockFlag)

/* Macro for releasing the lock */
#define IOP_REL_LOCK(structLock, lockFlag) spin_unlock_irqrestore (&(structLock)->SetLock, lockFlag)

#define IOPDRV              "iopiDrv"

/* ISTWI related variables */
typedef struct _IOP_ISTWI_RESERVE_QUEUE
{
    U8                      SubSys;
    U8                      BusNum;
    U8                      Next;
    void                    (*Fnct)(void);
    U32                     IocNum;
}IOP_ISTWI_RESERVE_QUEUE, *PTR_IOP_ISTWI_RESERVE_QUEUE;

extern IOP_ISTWI_RESERVE_QUEUE
/* This variable contain i2c reserve request */
        gIopIstwiBusQueue[IOP_ISTWI_MAX_BUS_NUM][IOP_ISTWI_BUS_QUEUE_SIZE];

typedef struct _IOP_ISTWI_CUR_BUS_SUBSYS
{
    U8                      SubSys;
    U8                      BusNum;
}IOP_ISTWI_CUR_BUS_SUBSYS, *PTR_IOP_ISTWI_CUR_BUS_SUBSYS;

extern IOP_ISTWI_CUR_BUS_SUBSYS
        gIopIstwiCurrentBusSubSys[IOP_ISTWI_MAX_BUS_NUM];

typedef struct _RUNTIME_CONFIG
{
    U32                     HostFrames;
    U32                     ResourceFrames;
    U32                     SysIfFIFODepth;
    U32                     FreeFIFODepth;
    U32                     FreeLMIDs;
    U32                     HwMessageSize;
    U32                     RingBufferBytes;
    U32                     RingBufferSize;
    U32                     HiPriorityDepth;
    U32                     DataDescSoftFifoSize;
    U32                     NumberOfIocs; /* Number of Iocs */
    U32                     NumDevHandles;
    PLI_RUNTIME_CONFIG      PliRtCfg; /* Number of Iocs */
    PLI_GET_MEMORY_REQUIRED PliMemReq;
} RUNTIME_CONFIG, * PTR_RUNTIME_CONFIG;

/* This variable is used to store PL runtime memory and
configuration parameters */
extern RUNTIME_CONFIG gRuntimeConfig;

/* Structures and variables to implement timers */
typedef enum
{
    IOP_TIMER_STATUS_FREE = 0,  // Force the first entry to be 0
    IOP_TIMER_STATUS_CREATED,
    IOP_TIMER_STATUS_RUNNING,
    IOP_TIMER_STATUS_MAX        // This has to be last
} IOP_TIMER_STATUS;

typedef struct _IOP_TIMER_PARAMS
{
    IOPI_SUBSYSTEM          SubSystemId;
    IOP_TIMER_HANDLE        Handle;
    void                    (*Fnct)(void *);
    void                    *Arg;
    U32                     Sixteenths;
} IOP_TIMER_PARAMS, *PTR_IOP_TIMER_PARAMS;

typedef struct _IOP_TIMER
{
    IOP_TIMER_STATUS        Status;
    unsigned long           StartTime;    // Value of jiffies at timer start
    IOP_TIMER_PARAMS        Params;
} IOP_TIMER, *PTR_IOP_TIMER;

extern IOP_TIMER gIopTimer[IOP_MAX_HANDLES_SUPPORTED];

typedef struct _IOP_FRAME_CONTEXT
{
    S8                      Owner ;
    S8                      Source;
} IOP_FRAME_CONTEXT, *PTR_IOP_FRAME_CONTEXT;

typedef struct _IOP_RUNTIME_CONTEXT
{
    PTR_MSG_IOC             PtrMsgCtx;
    PTR_IOP_FRAME_CONTEXT   PtrFrameCtx;
} IOP_RUNTIME_CONTEXT, *PTR_IOP_RUNTIME_CONTEXT;

extern IOP_RUNTIME_CONTEXT  gIopRuntimeContext;

typedef struct _IOP_DRIVER_REG
{
    U32     MajorNumber;
    PU8 Name;
    void (*ScsiIOComplete)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE, U32);
    void (*ScsiIOCompleteError)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE, U32);
    void (*TargetIOComplete)(PTR_MSG_IOC, U32);
    void (*TargetIOCompleteError)(PTR_MSG_IOC, U32);
    void (*SmpPassthruComplete)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE);
    void (*ScsiEnclMgmtComplete)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE);
    void (*SataPassthruComplete)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE);
    void (*TargetModeAbortComplete)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE);
    void (*CmdBufferPostBaseComplete)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE);
    void (*CmdBufferPostListComplete)(PTR_MSG_IOC, U32, PLI_DEV_HANDLE);
    void (*EventDetected)(PTR_MPI2_EVENT_NOTIFICATION_REPLY, U32, U32);
    void (*TargetSendCommandBufferReply)
        (MPI2_TARGET_COMMAND_BUFFER_REPLY_DESCRIPTOR);
    void (*DiscoveryComplete)(void);
    void (*TaskMgmtComplete)(PTR_MSG_IOC, U32);
    void (*ConfigComplete)(PTR_MSG_IOC, ISTATUS, U32, U32, void *, U32);
    void (*SysDmaXfer)(U32, U32, U32, U32, U32);
    ISTATUS (*ChangeFrameOwner)(IOPI_SUBSYSTEM, U32);
    void (*FreeMid)(U16);
} IOP_DRIVER_REG, *PTR_IOP_DRIVER_REG;

extern IOP_DRIVER_REG    gIopDriverReg;

/* Implementation of spinlock for IOP System Request Message Resource */
typedef struct {
    spinlock_t              SetLock ;
} IOP_SRM_RESOURCE_LOCK, *PTR_IOP_SRM_RESOURCE_LOCK ;

/* System Request Message Resource Declaration */
typedef struct _IOP_SRM_QUEUE
{
    U32                     FrameCount; /* Free Frame count */
    U32                     Head; /* Refer 5.5.5 in MPI specs */
    U32                     Tail; /* Refer 5.5.5 in MPI specs */
    U16                     *Mid; /* Message ID */
} IOP_SRM_QUEUE, *PTR_IOP_SRM_QUEUE;

typedef struct _IOP_SRM_RESOURCE
{
    IOP_SRM_QUEUE           SrmQueue;
    U32                     RsrcAllocated; /* Already allocated frames count */
    IOP_SRM_RESOURCE_LOCK   SrmResourceLock ;
} IOP_SRM_RESOURCE, *PTR_IOP_SRM_RESOURCE;

extern IOP_SRM_RESOURCE            gIopResourceQueue;

/* Global Vraiables declaration */
extern DIAG_COMMAND_TABLE   gIopDiagCmdTbl[MAX_TABLE_DEPTH];

extern PTR_MSG_IOC gTmQueueHead;

/* --------------Function Prototypes-------------- */

/* Driver Initialization */
S32 iopDrvInit(void);

/* Driver Entry points */
S32 iopDrvOpen(struct inode *PtrInode, struct file *Ptrfilp);
S32 iopDrvClose(struct inode *PtrInode, struct file *Ptrfilp);
ssize_t iopDrvRead(struct file *Ptrfilp, char *PtrBuf, size_t Count,
                                        loff_t *PtrPos);
ssize_t iopDrvWrite(struct file *Ptrfilp, const char *PtrBuf, size_t Count,
                                            loff_t *PtrPos);
long iopDrvIoctl (struct file *PtrFilep, U32 IoctlCode, unsigned long Arg);

/* Driver clean up and exit */
void iopDrvExit(void);
void iopInit(void);
void iopFinish(void);
void iopResQueueFinish(void);
void iopSetSODFinish(void);
void iopIstwiFinish(void);

void iopIstwiInit(void);
ISTATUS iopResourceQueueInit(void);
U32 iopiGetMsgPoolBaseAddress(void);
/* Configuration Pages */
ISTATUS iopGetMfgPage0(PTR_MPI2_CONFIG_PAGE_MAN_0 PtrMfgPage0);
ISTATUS iopGetMfgPage5(PTR_MPI2_CONFIG_PAGE_MAN_5 PtrMfgPage5);
ISTATUS iopGetMfgPage6(PTR_IOC_CONFIG_PAGE_MFG_PAGE_6 PtrMfgPage6);
ISTATUS iopGetMfgPage7(PTR_MPI2_CONFIG_PAGE_MAN_7 PtrMfgPage7);
ISTATUS iopGetMfgPage8(PTR_IOC_CONFIG_PAGE_MFG_PAGE_8 PtrMfgPage8);
ISTATUS iopGetMfgPage9(PTR_IOC_CONFIG_PAGE_MFG_PAGE_9 PtrIopMfgPage9);
ISTATUS iopGetMfgPage11(PTR_IOC_CONFIG_PAGE_MFG_PAGE_11 PtrMfgPage11);
ISTATUS iopGetMfgPage12(PTR_IOC_CONFIG_PAGE_MFG_PAGE_12 PtrMfgPage12);
ISTATUS iopGetMfgPage13(PTR_IOC_CONFIG_PAGE_MFG_PAGE_13 PtrMfgPage13);
ISTATUS iopGetMfgPage14(PTR_IOC_CONFIG_PAGE_MFG_PAGE_14 PtrMfgPage14);
ISTATUS iopGetMfgPage17(PTR_IOC_CONFIG_PAGE_MFG_PAGE_17 PtrIopMfgPage17);
ISTATUS iopGetMfgPage30(PTR_IOC_CONFIG_PAGE_MFG_PAGE_30 PtrIopMfgPage30);
ISTATUS iopGetMfgPage31(PTR_IOC_CONFIG_PAGE_MFG_PAGE_31 PtrIopMfgPage31);
ISTATUS iopGetExtMfgPage40(PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_40 PtrIopExtMfgPage40);
ISTATUS iopGetExtMfgPage41(PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_41 PtrIopExtMfgPage41);
ISTATUS iopGetIOUnitPage0(PTR_MPI2_CONFIG_PAGE_IO_UNIT_0 PtrIopIOUnitPage0);
ISTATUS iopGetIOUnitPage1(PTR_MPI2_CONFIG_PAGE_IO_UNIT_1 PtrIopIOUnitPage1);
ISTATUS iopGetIOCPage1(PTR_MPI2_CONFIG_PAGE_IOC_1 PtrIopIOCPage1);
ISTATUS iopGetIOCPage8(PTR_MPI2_CONFIG_PAGE_IOC_8 PtrIopIOCPage8);
ISTATUS iopGetSASIOUnitPage1(PTR_MPI2_CONFIG_PAGE_SASIOUNIT_1
                                                        PtrIopSASIOUnitPage1);
ISTATUS iopGetSASIOUnitPage4(PTR_MPI2_CONFIG_PAGE_SASIOUNIT_4
                                                        PtrIopSASIOUnitPage4);
ISTATUS iopGetSASIOUnitPage5(PTR_MPI2_CONFIG_PAGE_SASIOUNIT_5
                                                        PtrIopSASIOUnitPage5);
ISTATUS iopGetSASIOUnitPage7(PTR_MPI2_CONFIG_PAGE_SASIOUNIT_7
                                                        PtrIopSASIOUnitPage7);
ISTATUS iopGetSASIOUnitPage8(PTR_MPI2_CONFIG_PAGE_SASIOUNIT_8
                                                        PtrIopSASIOUnitPage8);

ISTATUS iopiGetSASPHY_3(PTR_MPI2_CONFIG_PAGE_SAS_PHY_3 PtrIopSASPhy3);
ISTATUS iopGetBIOSPage2(PTR_MPI2_CONFIG_PAGE_BIOS_2 PtrIopBIOSPage2);
ISTATUS iopGetBIOSPage3(PTR_MPI2_CONFIG_PAGE_BIOS_3 PtrIopBIOSPage3);

ISTATUS iopTaskMgmtResetTarget(U32 DevHandle);
ISTATUS iopSASTopologyChangeListEvent(PTR_MPI2_EVENT_NOTIFICATION_REPLY PtrMsg);
/* Exported Functions */
extern ISTATUS iopiInit(void);
extern void iopInitFastPathAccess(void);
extern ISTATUS iopiRegisterDriver(IOP_DRIVER_REG IopDriverReg);
extern ISTATUS iopiUnregisterDriver(U32 Major);
extern ISTATUS iopiConfig(U32 IocNum, U32 Action, U32 PageType, U32 PageNum,
    U32 PageAddress, PU32 Length, PTR_PAGE_IOC *PtrPtrIocConfig);
extern ISTATUS iopiConfigComplete(ISTATUS Istatus, U32 LogInfo, U32 Context,
    void *  AltConfigPageBuffer, U32 CompletePendingRequest);
extern void iopiDelayUSec(U32 USec);
extern ISTATUS iopiDiscoveryComplete(IOPI_SUBSYSTEM SubSystemId);
extern ISTATUS iopiEvent(PTR_MPI2_EVENT_NOTIFICATION_REPLY PtrMsg, U32 Mid,
    IOPI_SUBSYSTEM SubSystemId, U32 SendEventToHost);
extern void iopiFault(U32 FaultCode);
extern ISTATUS iopiFrameOwned(IOPI_SUBSYSTEM SubSys, U32 Mid);
extern ISTATUS iopiFPEHandleException(PTR_MSG_IOC PtrMsg, U32 Mid, U32 ExceptionStatus);
extern ISTATUS iopiTimedOutIOFound(U32 Mid, PLI_DEV_HANDLE DeviceHandle);
extern ISTATUS iopiTimeoutNotification(U32 TimerBucket, U32 BucketCount);
extern ISTATUS iopiRequestChangeFrameOwner(const IOPI_SUBSYSTEM SubSystem, const U32 Mid);
extern ISTATUS iopiFreeFrame(PTR_MSG_IOC PtrMsg, U32 Mid);
extern ISTATUS iopiGetFrame(IOPI_SUBSYSTEM SubSystem, PTR_MSG_IOC *PtrPtrMsg,
    PU32 PtrMid);
extern ISTATUS iopiGetFrameCritical(IOPI_SUBSYSTEM SubSystem,
    PTR_MSG_IOC *PtrPtrMsg, U32 * PtrMid, U32 Frame, U32 FaultCode);
extern ISTATUS iopiGetTimeSinceStartOfDayUsec(PU64 PtrReturnCurrentTime);
extern ISTATUS iopiMemoryMove(U32 IocNum, PTR_MPI2_SGE_IO_UNION PtrSge,
    void * PtrEffectiveAddr, U32 Length, U32 SkipCount,
    PU32 PtrDataTransferred, U32 Flags);
extern U32 iopPtrMsg2Mid(PTR_MSG_IOC PtrMsg);
extern ISTATUS iopiMsgComplete(PTR_MSG_IOC PtrMsg, U32 Mid,
    PLI_DEV_HANDLE DevHandle, U32 Length, IOPI_SUBSYSTEM SubSystemId);
extern ISTATUS iopiOutOfResource( IOPI_SUBSYSTEM Subsys,
    IOPI_RESOURCE_TYPE ResType);

extern void iopiDiagConsoleRegister(char * CmdName,
    PTR_DIAG_COMMAND_TABLE SubTable, char * HelpStr);
extern void iopiDiagCmdReadMem(U8 *Addr, U32 NumBytes);
extern PTR_MSG_IOC iopMid2PtrMsg(U32 Mid);
extern ISTATUS iopiSCSIIOComplete(PTR_MSG_IOC PtrMsg, U32 Mid,
    PLI_DEV_HANDLE DevHandle, U32 TaskTag);
extern ISTATUS iopiSCSIIOCompleteError(PTR_MSG_IOC PtrMsg, U32 Mid,
    PLI_DEV_HANDLE DevHandle, U8 SCSIStatus, U8 TaskTag, U8 SCSIState,
    U16 IOCStatus, U32 IOCLogInfo, U32 TransferCount, U32 SenseCount,
    U32 ResponseInfo, U32 BiDirTransferCount,  U32 EEDPDataOffset );
extern ISTATUS iopiTargetIoComplete(PTR_MSG_IOC PtrMsg, U32 Mid);
extern ISTATUS iopiTargetIoCompleteError(PTR_MSG_IOC PtrMsg, U32 Mid,
    U16 IOCStatus, U32 IOCLogInfo, U16 IoIndex, U32 TransferCount,
    U32 BiDirTransferCount);
extern void iopiTargetProcessCompletions(void);
extern ISTATUS iopiTargetSendCommandBufferReply(
    MPI2_TARGET_COMMAND_BUFFER_REPLY_DESCRIPTOR BufferReply);
extern void iopMsgHandlerCmdBuffBase(U32 IocNum, U32 Mid, PTR_MSG_IOC PtrMsg);
extern void iopMsgHandlerCmdBuffList(U32 IocNum, U32 Mid, PTR_MSG_IOC PtrMsg);
extern void iopMsgHandlerTargetAssist(U32 IocNum, U32 Mid, PTR_MSG_IOC PtrMsg);
extern void iopMsgHandlerTargetStatusSend(U32 IocNum, U32 Mid, PTR_MSG_IOC PtrMsg);
extern void iopMsgHandlerTargetModeAbort(U32 IocNum, U32 Mid, PTR_MSG_IOC PtrMsg);
extern ISTATUS iopiTaskMgmt(PTR_MSG_IOC PtrMsg, PLI_DEV_HANDLE DevHandle,
        IOPI_SUBSYSTEM SubSystemId);
extern void iopiTaskMgmtComplete(PTR_MSG_IOC PtrMsg,  U32 Mid, U16 IOCStatus,
    U32 IOCLogInfo, U32 TerminationCount, U32 ResponseCode);
extern ISTATUS iopiSataPassthrough(U32 Mid, PTR_MSG_IOC PtrMsg);
extern ISTATUS iopiSMPPassthrough(U32 Mid, PTR_MSG_IOC PtrMsg);
/* Timer functions */
extern ISTATUS iopiTimerCreate(IOPI_SUBSYSTEM SubSystemId,
    IOP_TIMER_HANDLE * Handle, void (*Fnct)(void *), void *Arg, U32 Sixtenths);
extern ISTATUS iopiTimerFree(IOP_TIMER_HANDLE Handle);
extern ISTATUS iopiTimerSet(IOP_TIMER_HANDLE Handle, U32 Sixteenths);
extern ISTATUS iopiTimerChk(IOP_TIMER_HANDLE Handle, U32 *Val);
extern ISTATUS iopiTimerCfgFnct (IOP_TIMER_HANDLE Handle, void (*Fnct)(void *),
    void *Arg);
extern ISTATUS iopiTimerStop(IOP_TIMER_HANDLE Handle);
extern void iopTimerFreeAll(void);
extern void iopTimerInit(void);

extern ISTATUS iopMemSetup(void);
extern void    iopMemExit(void);
extern void    *iopMemAlloc(U32 Len);
extern ISTATUS iopMemFree(void *Ptr);
extern ISTATUS iopiMemCmp(const void * Ptr1, const void * Ptr2, U32 Len);
extern void    *iopiMemCpy(const void *PtrDest, const void *PtrSrc, U32 Len);
extern void    *iopiMemSet(const void *Ptr1, U32 Val, U32 Len);
extern void    *iopiIORemap(U32 High, U32 Low, U32 Size);
extern void    iopiIOUnmap(void *Addr);

extern ISTATUS iopiGPIORead (U8 GpioNum, PU8 PtrGpioValue);
extern ISTATUS iopiGPIOWrite (U8 GpioNum, U8 GpioValue);

extern ISTATUS iopiIstwiIORequest(PTR_MSG_IOC PtrMsg, U32 IocNum,
    IOPI_SUBSYSTEM SubSys);
extern ISTATUS iopiIstwiReleaseBus(IOPI_SUBSYSTEM SubSys, U8 BusNum);
extern ISTATUS iopiIstwiReserveBus(IOPI_SUBSYSTEM SubSys, void (*Fnct)(void),
    U8 BusNum, U32 IocNum);
extern ISTATUS iopiIstwiReset(IOPI_SUBSYSTEM SubSys, U32 DevIndex);

extern void iopiDaSepReset(U32 DevIndex);
extern ISTATUS iopiExtendedBufferDataDump(U8 ExtBufferType, PU8 PtrData, U32 Length);
extern ISTATUS iopiExtendedBufferRelease(U8 ExtBufferType);
extern U16 iopiGetMaxDevHandle(void);
extern ISTATUS iopiSetParams( U8 IocNum, PTR_PLI_SET_PARAMS PlSetParams );
extern ISTATUS iopiPortEnable( PTR_MSG_IOC PtrMsg );
extern void iopiSendEpow(void);
extern ISTATUS iopiStartScsiIO(PTR_MSG_IOC PtrMsg, U32 Mid, PLI_DEV_HANDLE DevHandle,
                               MSG_IOC const *PtrMsgCached);
extern ISTATUS iopiGetRtConfig(PTR_RUNTIME_CONFIG RtConfig);
extern void flush_dcache_range(unsigned long start, unsigned long stop);
extern void invalidate_dcache_range(unsigned long start, unsigned long stop);
void iopIrqHandlerPli( U8 IrqVector );

#endif /* IOPI_H_HEADER */
