/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  onfiEngineDrv.h
 *          Title:  ONFI Engine Driver header file
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  31-Jan-2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  01/31/12  NB    Initial version.
 *
 * Description
 * ------------
 * This file contains the definitions for the ONFI Engine Driver.
 *
 * -------------------------------------------------------------------------
 */

#ifndef ONFI_ENGINE_DRV_H
#define ONFI_ENGINE_DRV_H

#include <linux/module.h>

/* Include header file that defines the interface between the interface and
 * engine driver
 */
#include "commonInterface.h"

/* Component engine driver interfaces */
typedef struct _COMPONENT_ENGINE_DRIVER
{
    void  (*iopiFault)             (U32 FaultCode);
    void  (*iopiDelayUSec)         (U32 USec);
    void* (*iopiMemSet)            (void * Ptr, U32 Val, U32 Len);
    void* (*iopiDynMalloc)         (U32 ReqSize, U32 MemFlag);
    void  (*iopiFree)              (void *Ptr);
    int   (*mmuVirtualToPhysical)  (U32 VirtualAddress, PU64 PtrPhysicalAddress);
    void  (*iopiTracePrint0)       (long long Pattern, U32 Index);
    void  (*iopiTracePrint1)       (long long Pattern, U32 Index, U32 Arg1);
    void  (*iopiTracePrint2)       (long long Pattern, U32 Index, U32 Arg1, U32 Arg2);
    void  (*iopiTracePrint3)       (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3);
    void  (*iopiTracePrint4)       (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);
    void  (*iopiDiagConsolePrint0) ( char * FmtStr);
    void  (*iopiDiagConsolePrint1) ( char * FmtStr, U32 Arg1);
    void  (*iopiDiagConsolePrint2) ( char * FmtStr, U32 Arg1, U32 Arg2);
    void  (*iopiDiagConsolePrint3) ( char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3);
    void  (*iopiDiagConsolePrint4) ( char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);

} COMPONENT_ENGINE_DRIVER, *PTR_COMPONENT_ENGINE_DRIVER;

extern COMPONENT_ENGINE_DRIVER gComponentEngineDriver;

/* Externs */
extern int onfiHalRegisterInterfaceDriver(ENGINE_DRIVER IfaceData);
extern int onfiHalUnregisterInterfaceDriver(U32 MajorNumber);

#endif /* ONFI_ENGINE_DRV_H */
