/***************************************************************************
 *                                                                         *
 *  Copyright 2010-2011 LSI Corporation.  All rights reserved.             *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ethHali.h
 *          Title:  Ethernet HAL Interface Header.
 *     Programmer:  Mike Kopps
 *  Creation Date:  February 22, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  02/22/11  MK    Initial version.
 *  05/03/12  NB    Alignment changes.
 *
 * Description
 * ------------
 * This file contains header information (#defines and #includes) needed
 * by the Ethernet HAL
 *
 *-------------------------------------------------------------------------
 */

#ifndef ETHHALI_H
#define ETHHALI_H

#define ETHI_HW_ADDR_LEN                (6) /* 6 Bytes in an Ethernet MAC address */

#define ETHI_SPEED_DUPLEX_MASK          (0x80)
#define ETHI_SPEED_DUPLEX_HALF          (0x00)
#define ETHI_SPEED_DUPLEX_FULL          (0x80)

#define ETHI_SPEED_DATA_RATE_MASK       (0x07)
#define ETHI_SPEED_DATA_RATE_AUTO       (0x00)
#define ETHI_SPEED_DATA_RATE_10MBIT     (0x01)
#define ETHI_SPEED_DATA_RATE_100MBIT    (0x02)
#define ETHI_SPEED_DATA_RATE_1GBIT      (0x03)

#ifndef ISTATUS_SUCCESS
typedef U32 ISTATUS;
#define ISTATUS_SUCCESS                 (0)
#endif
#ifndef ISTATUS_FAILURE
#define ISTATUS_FAILURE                 (0x00000001)
#endif
#ifndef ISTATUS_INVALID_PARAMETER
#define ISTATUS_INVALID_PARAMETER       (0x00000005)
#endif
#ifndef ISTATUS_BUSY
#define ISTATUS_BUSY                    (0x00000007)
#endif

#ifndef MII_REG_MAX
#define MII_REG_MAX                     (31)
#endif
#ifndef MII_PHY_ADDR_RESERVED
#define MII_PHY_ADDR_RESERVED           (0)
#endif

typedef enum
{
    ETHI_STAT_TR64 = 0,
    ETHI_STAT_TR127,
    ETHI_STAT_TR255,
    ETHI_STAT_TR511,
    ETHI_STAT_TR1K,
    ETHI_STAT_TRMAX,
    ETHI_STAT_TRMGV,

    ETHI_STAT_RX_BYTE,
    ETHI_STAT_RX_PKT,
    ETHI_STAT_RX_FCSERR,
    ETHI_STAT_RX_MCAST,
    ETHI_STAT_RX_BCAST,
    ETHI_STAT_RX_CTRLFRM,
    ETHI_STAT_RX_PAUSE,
    ETHI_STAT_RX_UNKN,
    ETHI_STAT_RX_ALIGNERR,
    ETHI_STAT_RX_FRMLENERR,
    ETHI_STAT_RX_CODEERR,
    ETHI_STAT_RX_CARRIERSENSEERR,
    ETHI_STAT_RX_UNDERSIZE,
    ETHI_STAT_RX_OVERSIZE,
    ETHI_STAT_RX_FRAGMENTS,
    ETHI_STAT_RX_JABBER,
    ETHI_STAT_RX_DROP,

    ETHI_STAT_TX_BYTE,
    ETHI_STAT_TX_PKT,
    ETHI_STAT_TX_MCAST,
    ETHI_STAT_TX_BCAST,
    ETHI_STAT_TX_PAUSE,
    ETHI_STAT_TX_DEFERRALS,
    ETHI_STAT_TX_EXDEFERRALS,
    ETHI_STAT_TX_SNGLCOL,
    ETHI_STAT_TX_MULTCOL,
    ETHI_STAT_TX_LATECOL,
    ETHI_STAT_TX_EXCOL,
    ETHI_STAT_TX_TOTALCOL,
    ETHI_STAT_TX_PAUSEHONORED,
    ETHI_STAT_TX_DROP,
    ETHI_STAT_TX_JABBER,
    ETHI_STAT_TX_FCSERROR,
    ETHI_STAT_TX_CTRL,
    ETHI_STAT_TX_OVERSIZE,
    ETHI_STAT_TX_UNDERSIZE,
    ETHI_STAT_TX_FRAGMENTS,

    ETHI_STAT_NUM
} ETHI_STATS;

#define ETHI_STATVAL_INVALID    (0xFFFFFFFF)

typedef ISTATUS (*ETHI_GET_FRAME)  ( U32 Length, void** PtrFrame, void** FrameCtx );
typedef void    (*ETHI_SENT_FRAME) ( void* Context );
typedef void    (*ETHI_RECV_FRAME) ( void );
typedef void    (*ETHI_VIRT2PHYS)  ( U32 Virtual, PU64 Physical );
typedef void    (*ETHI_DELAYUSEC)  ( U32 USecs );


/* Initialization Flags */
#define ETHI_FLAGS_STATSEN      (0x0001) /* Statistics Enable */
#define ETHI_FLAGS_STATSAC      (0x0002) /* Statistics Auto Clear after read */
#define ETHI_FLAGS_BYTEMODE     (0x0004) /* Use Byte mode on MII, otherwise use Nibble mode */

/*****************************************************************************/
/* Interface Function Prototypes                                             */
/*****************************************************************************/
#ifdef __cplusplus
extern "C"
{
#endif
    void    ethiInterruptHandler ( void );
    ISTATUS ethiSendFrame        ( void* PtrFrame, void* FrameCtx, const U32 Length );
    ISTATUS ethiReceiveFrame     ( void** PtrPtrFrame, PU32 PtrLength );
    ISTATUS ethiConfigSpeed      ( const U8 Speed );
    U32     ethiGetMemReq        ( const U8 NumRxDescriptors, const U8 NumTxDescriptors );
    ISTATUS ethiShutdown         ( void );
    U32     ethiGetStat          ( ETHI_STATS Stat );
    U32     ethiSetStat          ( ETHI_STATS Stat, U32 Value );

    ISTATUS ethiMdioScan  ( U8 PhyAddr, U8 Register, BOOL Scan );
    ISTATUS ethiMdioRead  ( U8 PhyAddr, U8 Register, PU16 Value );
    ISTATUS ethiMdioWrite ( U8 PhyAddr, U8 Register, U16 Value );

    ISTATUS ethiInit( const void*     PtrEthHw,
                      void*           PtrMem,
                      U8              NumRxDescriptors,
                      U8              NumTxDescriptors,
                      U16             Flags,
                      PU8             MacAddr,
                      ETHI_GET_FRAME  GetFrame,
                      ETHI_SENT_FRAME SentFrame,
                      ETHI_RECV_FRAME RecvFrame,
                      ETHI_VIRT2PHYS  MemV2P,
                      ETHI_DELAYUSEC  DelayUSec );
    

#ifdef __cplusplus
}
#endif

#endif /* ETHHALI_H */
