/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  flashDefs.h
 *          Title:  Private Header File for Library
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  April 20, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  04/20/12  NB    Initial version.
 *
 * Description
 * ------------
 * This file contains the definitions and the function prototypes of the
 * functions implemented in the FLASH library.
 *
 *--------------------------------------------------------------------------
 */

#ifndef FLASH_DEFS_H
#define FLASH_DEFS_H

#include "basetype.h" /* Basic types */

#define FLASH_DRIVER_NAME "flash"

typedef struct _CFI_QRY_INFO
{
    U8   QRY[3];            /* 0x10 : Q , 0x11 : R, 0x12 : Y               */
    U16  CmdSetType;        /* 0x13 : CFI Command Set Type                 */
    U16  FlashSize;         /* 0x27 : FLASH Size                           */
    U16  CFIVersion;        /* 0x33, 0x34 : CFI Version                    */
    Ul32 SecEraseTimeTyp;   /* 0x21 : Sector Erase Timeout Typical         */
    Ul32 SecEraseTimeMax;   /* 0x25 : Sector Erase Timeout Maximum         */
    Ul32 ChipEraseTimeTyp;  /* 0x22 : Chip Erase Timeout Typical           */
    Ul32 ChipEraseTimeMax;  /* 0x26 : Chip Erase Timeout Maximum           */
    U8   BusWidth;          /* Bit 8th of FLASH Status & Control register  */
} CFI_QRY_INFO, *PTR_CFI_QRY_INFO;

/* For Read and Write from/to Flash Address, a: Address, o: Offset */
#define EA_ADDR(addr, off)                (*(( (volatile PU16)(addr) ) + (off)))

/* For FLASH Poll operation - polling location verification */
#define BUS_WIDTH_EXPECTED_DATA           (0xFFFFFFFF)
#define BUS_WIDTH_16BIT_READ_MASK         (0x0000FFFF)
#define BUS_WIDTH_8BIT_READ_MASK          (0x000000FF)

/* CFI Command Set Type */
#define CFI_CMDSET_AMD_STANDARD           (0x0002)
#define CFI_CMDSET_AMD_EXTENDED           (0x0004)
#define CFI_CMDSET_INTEL_STANDARD         (0x0003)
#define CFI_CMDSET_INTEL_EXTENDED         (0x0001)

/* Flash Write */
#define FLASH_WRITE_ENABLED               (0x1)
#define FLASH_WRITE_DISABLED              (0x0)
#define FLASH_PROTECT_SUCCESS             (0x1)
#define FLASH_UNPROTECT_SUCCESS           (0x0)

/* Common CFI Commands */
#define FLASH_CMD_CFI_QUERY               (0x0098)    /* CFI Query Command */

/* Unlock Bypass Address for 16-Bit Bus Width */
#define FLASH_CMD_UNLOCK_ADDR1_16BIT      (0x555)
#define FLASH_CMD_UNLOCK_ADDR2_16BIT      (0x2AA)

/* Unlock Bypass Address for 8-Bit Bus Width */
#define FLASH_CMD_UNLOCK_ADDR1_8BIT       (0xAAA)
#define FLASH_CMD_UNLOCK_ADDR2_8BIT       (0x555)

/* Unlocl Bypass Data for 8-bit/16-Bit Flash Bus Width */
#define FLASH_CMD_UNLOCK_DATA1            (0x00AA)    /* Unlock Cycle 1 Data */
#define FLASH_CMD_UNLOCK_DATA2            (0x0055)    /* Unlock Cycle 2 Data */

/*-----------------------------------------------------------------------------
            AMD / FUJITSU Flash CFI Commands
-----------------------------------------------------------------------------*/
/* CFI Commands' data values */
#define FLASH_AMD_CMD_LOCK_SETUP          (0x0080)    /* Lock Setup Command     */
#define FLASH_AMD_CMD_SECTOR_LOCK         (0x0040)    /* Sector Lock Command    */
#define FLASH_AMD_CMD_ERASE_SETUP         (0x0080)    /* Write Setup Command    */
#define FLASH_AMD_CMD_CHIP_ERASE          (0x0010)    /* Chip Erase Command     */
#define FLASH_AMD_CMD_SECTOR_ERASE        (0x0030)    /* Sector Erase Command   */
#define FLASH_AMD_CMD_PROGRAM             (0x00A0)    /* Write Program Command  */
#define FLASH_AMD_CMD_WR_BUFFER_LOAD      (0x0025)    /* "Write Buffer" Command */
#define FLASH_AMD_CMD_WR_BUFFER_CONFIRM   (0x0029)    /* Write Confirm Command  */
#define FLASH_AMD_CMD_RESET               (0x00F0)    /* Write Reset Command    */

#define FLASH_AMD_CMD_AUTOSELECT_CMD      (0x0090)    /* CFI Autoselect Command */

/*-----------------------------------------------------------------------------
            INTEL / SHARP Flash CFI Commands
-----------------------------------------------------------------------------*/

#define FLASH_INTEL_CMD_SECTOR_LOCK       (0x0040)    /* Sector Lock Command         */
#define FLASH_INTEL_CMD_UNLOCK_DATA1      (0x00AA)    /* Sector Unlock command       */
#define FLASH_INTEL_CMD_CLEAR_STATUS      (0x0050)    /* Clear Status Before Protect */
#define FLASH_INTEL_CMD_PROTECT           (0x0060)    /* Protect Command             */
#define FLASH_INTEL_CMD_ERASE_SETUP       (0x0020)    /* Write Setup Command         */
#define FLASH_INTEL_CMD_SECTOR_ERASE      (0x00D0)    /* Sector Erase Command        */
#define FLASH_INTEL_CMD_PROGRAM           (0x0040)    /* Write Program Setup Command */
#define FLASH_INTEL_CMD_WR_BUFFER_LOAD    (0x00E8)    /* "Write Buffer" Command      */
#define FLASH_INTEL_CMD_WR_BUFFER_CONFIRM (0x00D0)    /* Write Confirm Command       */
#define FLASH_INTEL_CMD_RESET             (0x00FF)    /* Write Reset Command         */

#define FLASH_INTEL_CMD_AUTOSELECT_CMD    (0x0090)    /* CFI Autoselect Command      */

#define FLASH_INTEL_OFFSET_CFI            (0x55)      /* For CFI Query               */

/*---------------------------------------------------------------------------*/

/* For Flash Device ID - Autoselect Addresses */
#define FLASH_AUTOSELECT_BASE0            (0x0000)
#define FLASH_AUTOSELECT_BASE1            (0x0001)
#define FLASH_AUTOSELECT_DIV_MASK         (0x000000FF)

/* CFI Query Identification String */
/* Flash - CFI Compliance */
#define FLASH_CFI_QUERY_ADDRESS_10        (0x10)
#define FLASH_CFI_QUERY_ADDRESS_11        (0x11)
#define FLASH_CFI_QUERY_ADDRESS_12        (0x12)
#define FLASH_CFI_QUERY_ADDRESS_13        (0x13)

/* System Interface Information */
#define FLASH_CFI_QUERY_ADDRESS_21        (0x21)
#define FLASH_CFI_QUERY_ADDRESS_22        (0x22)
#define FLASH_CFI_QUERY_ADDRESS_25        (0x25)
#define FLASH_CFI_QUERY_ADDRESS_26        (0x26)

/* Flash Geometry Informa */
#define FLASH_CFI_QUERY_ADDRESS_27        (0x27)
#define FLASH_CFI_QUERY_ADDRESS_43        (0x43)
#define FLASH_CFI_QUERY_ADDRESS_44        (0x44)

/* Flash - Poll & Status */
#define FLASH_BIT_DQ2_MASK                (0x04)      /* DQ2 mask */
#define FLASH_BIT_DQ6_MASK                (0x40)      /* DQ6 mask */
#define FLASH_BIT_DQ6_TGL_DQ5_MASK(dq)    (dq >> 1)   /* Mask for DQ5 when device DQ6 toggling */

#define FLASH_POLL_COUNT                  (0xFFFFFFFF)

#define FLASH_BUS_WIDTH_16                (1)
#define FLASH_BUS_WIDTH_8                 (0)

/* Return value (Status) from APIs */
typedef enum _FLASH_STATUS
{
    FLASH_STATUS_SUCCESS = 0,
    FLASH_STATUS_FAIL,
    FLASH_STATUS_DEV_BUSY,
    FLASH_STATUS_DEV_NOT_BUSY,
    FLASH_STATUS_DEV_VERIFY_ERROR,
    FLASH_STATUS_DEV_EXCEEDED_TIME_LIMITS,
    FLASH_STATUS_DEV_SUSPEND

}FLASH_STATUS;


/* FLASH APIs required by FLASH library
 * Flash Support : 1. AMD/Fujitsu
 *                 2. Intel/Sharp
 */
FLASH_STATUS flashiInit         ( void *PtrFlashBank );
FLASH_STATUS flashiCFIInfo      ( void *PtrFlashBank, PTR_CFI_QRY_INFO PtrCFIInfo );
FLASH_STATUS flashiGetID        ( void *PtrFlashBank, U32 Offset, PU16 PtrDID, PU16 PtrMID );
FLASH_STATUS flashiWrite        ( void *PtrFlashBank, U32 Offset, U16 Data );
FLASH_STATUS flashiWriteBuffer  ( void *PtrFlashBank, U32 Offset, U16 WordCount, PU8 PtrBuffer );
FLASH_STATUS flashiRead         ( void *PtrFlashBank, U32 Offset, PU16 PtrData );
FLASH_STATUS flashiSectorErase  ( void *PtrFlashBank, U32 Offset );
FLASH_STATUS flashiChipErase    ( void *PtrFlashBank );
FLASH_STATUS flashiProtect      ( void *PtrFlashBank, U32 Offset, U8 PFlag );
FLASH_STATUS flashiExit         ( void );

#endif /* FLASH_DEFS_H */
