/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  gpioDefs.h
 *          Title:  API Header File for Library
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  April 06, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  04/06/12  NB   Initial version.
 *
 *  Description
 *  ------------
 *  This file contains the definitions and the function prototypes of the
 *  functions implemented in the GPIO library.
 *
 *--------------------------------------------------------------------------
 */

#ifndef GPIO_DEFS_H_SOURCE
#define GPIO_DEFS_H_SOURCE

#define GPIO_MAX                    (40)

/* GPIO alternate select modes */
#define GPIO_ALT_SEL_MODE_GPIO      (0x0)
#define GPIO_ALT_SEL_MODE_1         (0x1)
#define GPIO_ALT_SEL_MODE_2         (0x2)
#define GPIO_ALT_SEL_MODE_3         (0x3)

/* GPIO input modes */
#define GPIO_IN_MODE_DISABLED       (0x0)
#define GPIO_IN_MODE_ENABLED        (0x1)

/* GPIO output modes */
#define GPIO_OUT_MODE_DISABLED      (0x0)  /* Pad may be used as an input */
#define GPIO_OUT_MODE_OPEN_DRAIN    (0x1)  /* 1: High impedance, 0: Enabled,
                                             pull up required*/
#define GPIO_OUT_MODE_OPEN_SOURCE   (0x2)  /* 1: Enabled 0: High Impedance,
                                              pull down required */
#define GPIO_OUT_MODE_PUSH_PULL     (0x3)  /* Always driving */

/* Hearbeat LED */
#define GPIO_HB_DISABLED            (0x0)  /* Disable heartbeat */
#define GPIO_HB_ENABLED             (0x1)  /* Enable hearbeat  */

/* External interrupts */
#define GPIO_EXTINTSEL_COUNT        (8)    /* 8 GPIO signals can be sent */

#define GPIO_EXTINTSEL_DISABLE      (0)    /* Disable the external interrupt */
#define GPIO_EXTINTSEL_ENABLE       (1)    /* Enable the external interrupt */

/* External Interrupt Enable Byte */
/* This is used by gpioiExternalIntSetup() */
#define GPIO_EXTINT_ENABLE          (0x80)

/* GPIO states */
/* These are used by gpioiGetValue(), gpioiSetValue() */
#define GPIO_ASSERT                 (0)    /* Switch on the GPIO */
#define GPIO_DEASSERT               (1)    /* Switch off the GPIO */

/* These are used by gpio interrupts */
#define GPIO_INTR_ASSERT            (GPIO_DEASSERT)  /* Send interrupt */
#define GPIO_INTR_DEASSERT          (GPIO_ASSERT)    /* Clear interrupt */

typedef enum _GPIO_STATUS
{
    GPIO_STATUS_SUCCESS = 0,
    GPIO_STATUS_FAIL,
/* External Interrupt Register For GPIO Available */
/* This is used by  gpioiExternalIntSelect() */
    GPIO_EXTINT_PREV_AVAIL,
} GPIO_STATUS;

/* Shared structures */
typedef struct _GPIO_DATA
{
    int i;
}GPIO_DATA;

/* APIs exported by GPIO library */
GPIO_STATUS gpioiInit             (void);
GPIO_STATUS gpioiFinish           (void);
GPIO_STATUS gpioiGetValue         (U8 GpioNum, PU8 PtrGpioValue);
GPIO_STATUS gpioiSetValue         (U8 GpioNum, U8  GpioValue);
GPIO_STATUS gpioiGetInputMode     (U8 GpioNum, PU8 PtrGpioInputMode);
GPIO_STATUS gpioiSetInputMode     (U8 GpioNum, U8  GpioInputMode);
GPIO_STATUS gpioiGetOutputMode    (U8 GpioNum, PU8 PtrGpioOutputMode);
GPIO_STATUS gpioiSetOutputMode    (U8 GpioNum, U8  GpioOutputMode);
GPIO_STATUS gpioiSetHeartbeat     (U8 GpioHBSetting);
GPIO_STATUS gpioiToggleHeartbeat  (void);
GPIO_STATUS gpioiExternalIntSetup (U32 ExtIntNum, U32 GpioNum, U8 ExtIntEn);
GPIO_STATUS gpioiAltSelect        (U32 GpioNum,   U32 GpioAltSel);
GPIO_STATUS gpioiExternalIntSelect(U32 GpioNum,   PU8 ExtIntNum);

/* APIs required by GPIO library */
void* osalIoremap     (unsigned long long Address, U32 SizeBytes);
void  osalIounmap     (void * Address);
U32   osalMoveFromDcr (S32 RegAddress);
void  osalMoveToDcr   (U32 RegAddress, U32 Value);

#endif /* GPIO_DEFS_H_SOURCE */
