/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  gpioEngineDrv.h
 *          Title:  GPIO Engine Driver Header File
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  April 06, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  04/06/12  NB   Initial version.
 *
 *
 *  Description
 *  ------------
 *  This file contains the definitions used for GPIO driver as an interface.
 *
 *-------------------------------------------------------------------------
 */

#ifndef GPIOENGDRV_H
#define GPIOENGDRV_H

#include "basetype.h"
#include <linux/module.h>

typedef enum _GPIO_ENGINE_DRV_STATUS
{
    GPIO_ENGINE_DRV_SUCCESS = 0,
    GPIO_ENGINE_DRV_FAILURE
} GPIO_ENGINE_DRV_STATUS;

/* Common Interface */
typedef struct _GPIO_ENGINE_DRIVER
{
    void (*osalMoveToDcr)     ( U32, U32 );
    U32  (*osalMoveFromDcr)   ( S32 );
    void *(*osalIoremap)      ( unsigned long long, U32 );
    void (*osalIounmap)       ( void* );
    void (*iopiTracePrint0)   ( long long Pattern, U32 Index );
    void (*iopiTracePrint1)   ( long long Pattern, U32 Index, U32 Arg1 );
    void (*iopiTracePrint2)   ( long long Pattern, U32 Index, U32 Arg1, U32 Arg2 );
    void (*iopiTracePrint3)   ( long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3 );
    void (*iopiTracePrint4)   ( long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4 );
    void (*iopiConsolePrint0) ( char * FmtStr );
    void (*iopiConsolePrint1) ( char * FmtStr, U32 Arg1 );
    void (*iopiConsolePrint2) ( char * FmtStr, U32 Arg1, U32 Arg2 );
    void (*iopiConsolePrint3) ( char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3 );
    void (*iopiConsolePrint4) ( char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4 );

} GPIO_ENGINE_DRIVER, *PTR_GPIO_ENGINE_DRIVER;
extern GPIO_ENGINE_DRIVER gGpioEngineDriver;

/* Exported Functions - GPIO Engine Driver Functions */
extern GPIO_ENGINE_DRV_STATUS gpioEngineRegisterDriver   (ENGINE_DRIVER EngineDriver);
extern GPIO_ENGINE_DRV_STATUS gpioEngineUnregisterDriver (U32 Major);

#endif /* GPIOENGDRV_H */
