/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation. The   *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  uartDefs.h
 *          Title:  Header file for UART API
 *     Programmer:  Jignesh Moradiya
 *  Creation Date:  April 06, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  04/06/12  JM   Initial version.
 *  
 * Description
 * ------------
 *  This file contains the structures and APIs exported by the UART API.
 *
 *--------------------------------------------------------------------------
 */

#ifndef __UART_DEFS_H_SOURCE
#define __UART_DEFS_H_SOURCE

#include <basetype.h>

/* Valid UART numbers */
#define UART_0      (0)
#define UART_1      (1)
#define UART_CHECKNUM(UartNum)                            \
{                                                         \
    if( (UART_0 != UartNum) && (UART_1 != UartNum) )      \
        return UART_STATUS_INVALID_UART_NUM;              \
}                                                         \

#define UART_COUNT  (2)

/* Number of possible UART Baudrates */
#define UART_BAUDRATES (13)

/* Default baud rate */
#define UART_DEFAULT_BAUDRATE (115200)

/* Values for UART timeouts */
#define UART_GET_TIMEOUT (0)
#define UART_PUT_TIMEOUT (1)

/* TimeOut in Micro Seconds */
#define UART_GET_TIMEOUT_DEFAULT (0)
#define UART_PUT_TIMEOUT_DEFAULT (1000000)


/* Values for the UART registers */
/* Interrupt enable register bits */
#define UART_IER_RCV_DATA           (0x01)  // Enable receive data available interrupt (ERBFI)
#define UART_IER_TX_EMPTY           (0x02)  // Transmitter holding register empty interrupt (ETBEI)
#define UART_IER_RCV_LINE_STATUS    (0x04)  // Receiver line status interrupt (ELSI)
#define UART_IER_MDM_STATUS         (0x08)  // Modem status interrupt (EDSSI)

/* Interrupt identification register bits */
#define UART_IIR_NO_INTR            (0x01)  // Interrupt not pending
#define UART_IIR_INTR_ID_MASK       (0x0E)  // Interrupt ID mask
#define UART_IIR_INTR_ID_MDM_STATUS (0x00)  // Modem status (priority 4)
#define UART_IIR_INTR_ID_THR_EMPTY  (0x02)  // Transmitter holding register empty (priority 3)
#define UART_IIR_INTR_ID_RCV_DATA   (0x04)  // Receive data available (priority 2)
#define UART_IIR_INTR_ID_RCV_LINE   (0x06)  // Receiver line status (priority 1)
#define UART_IIR_INTR_ID_CHAR_TO    (0x0C)  // Character timeout (priority 2)
#define UART_IIR_FIFOS_ENABLED      (0xC0)  // Transmit and receive FIFOs enabled

/* FIFO control register bits */
#define UART_FCR_FIFO_ENABLE        (0x01)  // Enable transmit and receive FIFOs
#define UART_FCR_RCV_RESET          (0x02)  // Reset receiver FIFO
#define UART_FCR_TX_RESET           (0x04)  // Reset transmit FIFO
#define UART_FCR_DMA_SELECT         (0x08)  // DMA mode 1 (multiple transfer DMA)
#define UART_FCR_RCV_MASK           (0xC0)  // Receiver interrupt trigger threshold mask
                                            // 16-byte FIFO  64-byte FIFO
#define UART_FCR_RCV_TRIG_1_1       (0x00)  //      1              1
#define UART_FCR_RCV_TRIG_4_16      (0x40)  //      4             16
#define UART_FCR_RCV_TRIG_8_32      (0x80)  //      8             32
#define UART_FCR_RCV_TRIG_14_56     (0xC0)  //     14             56

/* Line control registers bits */
#define UART_LCR_WORD_LEN_MASK      (0x03)  // Word length mask
#define UART_LCR_WORD_LEN_5         (0x00)  // Word length = 5 bits
#define UART_LCR_WORD_LEN_6         (0x01)  // Word length = 6 bits
#define UART_LCR_WORD_LEN_7         (0x02)  // Word length = 7 bits
#define UART_LCR_WORD_LEN_8         (0x03)  // Word length = 8 bits
#define UART_LCR_NUM_STOP_BITS_2    (0x04)  // Number of stop bits (1.5 bits or 2 bits)
#define UART_LCR_NUM_STOP_BITS_1    (0x00)
#define UART_LCR_PARITY_ENABLE      (0x08)  // Enable parity generation and checking
#define UART_LCR_PARITY_DISABLE     (0x00)
#define UART_LCR_PARITY_EVEN        (0x10)  // Even parity
#define UART_LCR_PARITY_STICK       (0x20)  // Sticky parity enable
#define UART_LCR_SET_BREAK          (0x40)  // Set control break
#define UART_LCR_DLAB               (0x80)  // Clock divisor latch access bit

/* Modem control register bits */
#define UART_MCR_DTR                (0x01)  // Activate DTR
#define UART_MCR_RTS                (0x02)  // Activate RTS
#define UART_MCR_OUT1               (0x04)  // Activate user designated output 1
#define UART_MCR_OUT2               (0x08)  // Activate user designated output 2
#define UART_MCR_LOOP               (0x10)  // Activate loop-back mode
#define UART_MCR_AFC                (0x20)  // Enable flow control

/* Line status register bits */
#define UART_LSR_DATA_READY         (0x01)  // Received character available
#define UART_LSR_OVERRUN_ERROR      (0x02)  // Received character lost
#define UART_LSR_PARITY_ERROR       (0x04)  // Received character has bad parity
#define UART_LSR_FRAMING_ERROR      (0x08)  // Valid stop bit not received
#define UART_LSR_BREAK_INT          (0x10)  // Break interrupt
#define UART_LSR_THR_EMPTY          (0x20)  // Transmitter holding register empty
#define UART_LSR_XMTR_EMPTY         (0x40)  // FIFO and transmitter shift register empty
#define UART_LSR_RCV_FIFO_ERROR     (0x80)  // Receiver FIFO error (parity, framing, etc.)

/* Modem status register bits */
#define UART_MSR_DCTS               (0x01)  // CTS changed since this register last read
#define UART_MSR_DDSR               (0x02)  // DSR changed since this register last read
#define UART_MSR_TERI               (0x04)  // Ring indicator changed from 0 to 1
#define UART_MSR_DDCD               (0x08)  // DCD changed since this register last read
#define UART_MSR_CTS                (0x10)  // CTS is active
#define UART_MSR_DSR                (0x20)  // DSR is active
#define UART_MSR_RI                 (0x40)  // RI is active
#define UART_MSR_DCD                (0x80)  // DCD is active

/* List of UART registers */
typedef enum _UART_REGISTERS
{
    UART_REG_RBR = 0,
    UART_REG_THR,
    UART_REG_DLL,
    UART_REG_IER,
    UART_REG_DLM,
    UART_REG_FCR,
    UART_REG_IIR,
    UART_REG_LCR,
    UART_REG_MCR,
    UART_REG_LSR,
    UART_REG_MSR,
    UART_REG_SCR,
    UART_REG_SVL,
    UART_REG_SVM,
    UART_REG_SVH,
    UART_REG_SS,
    UART_REG_SSC
} UART_REGISTERS;

/* UART return status */
typedef enum _UART_STATUS
{
    UART_STATUS_SUCCESS = 0,
    UART_STATUS_FAIL,
    UART_STATUS_INVALID_UART_NUM,
    UART_STATUS_UART_ADDRESS_ERROR,
    UART_STATUS_TIMEOUT,
    UART_STATUS_WRITEONLY_REGISTER,
    UART_STATUS_READONLY_REGISTER
} UART_STATUS;





/* APIs exported by UART library */
UART_STATUS uartiInit       (U8 UartNum, U32 BaudRate);
UART_STATUS uartiPutChar    (U8 UartNum, U8 Char);
UART_STATUS uartiPutStr     (U8 UartNum, const U8 *PtrStr);
UART_STATUS uartiGetChar    (U8 UartNum, U8 *PtrChar);
UART_STATUS uartiSpeedSense (U8 UartNum);
UART_STATUS uartiSetRegValue(U32 UartNum, UART_REGISTERS UartReg, U8 UartValue);
UART_STATUS uartiGetRegValue(U32 UartNum, UART_REGISTERS UartReg, PU8 UartValue);
UART_STATUS uartiSetTimeout (U8 UartNum, U32 Type, U32 Timeout);
UART_STATUS uartiGetTimeout (U8 UartNum, U32 Type, U32 *Timeout);
UART_STATUS uartiFinish     (U8 UartNum);


/* APIs required by UART library */
void * osalConvertPhysicalToVirtual (unsigned long long Address);
void   osalMemset                   (void * Address, U8 Value, U32 Count);
void   osalPrintk                   (const U8* format, ... );
void * osalIoremap                  (unsigned long long Address, U32 SizeBytes);
void   osalIounmap                  (void * Address);
void   osalUdelay                   (U32 USec);

#endif /* __UART_DEFS_H_SOURCE */

