/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2010 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pciei.h
 *          Title:  PCI Express (PCIe) Prototypes
 *     Programmer:  Patni
 *  Creation Date:  May 25, 2010
 *
 *
 *  Description
 *  ------------
 *  This include file contains PCIe (PCIe) exports to be used
 *  by other modules to interface with PCIe core.
 *
 *-------------------------------------------------------------------------
 */

#ifndef PCIEI_H_INCLUDED
#define PCIEI_H_INCLUDED

#include "pcieiDefs.h"

/******************************************************************************
*
*      PCIe    E X P O R T    F U N C T I O N S
*
******************************************************************************/

/*
 * PCIe Init and Cleanup APIs
 */
PCIEI_STATUS pcieiInitCoreRegs(void);
PCIEI_STATUS pcieiCleanUpCoreRegs(void);
void         pcieiInitHwAccelInterruptControl(void);

PCIEI_STATUS pcieiInitFmuRegs(void);
PCIEI_STATUS pcieiCleanUpFmuRegs(void);


void pcieiDumpFMURegs(void);
void pcieiHandleMsgPullError(void);
void pcieiSysIfPostReqFree(U16 Mid);
/*
 * PCIe Core Registers Interface APIs
 */

/* PIM/POM Initialization APIs */
PCIEI_STATUS pcieiConfigPIM(PCIEI_CONFIG_PIM_PARAMS PIMParams);
PCIEI_STATUS pcieiConfigPOM(PCIEI_CONFIG_POM_PARAMS POMParams);
/* PLB control related APIs */
PCIEI_STATUS pcieiGetPLBControl(U32 BitMask, PTR_PCIEI_BIT_STATUS PtrBitStatus);
PCIEI_STATUS pcieiSetClearPLBControl(U32 BitMask, BOOL SetFlag);
/* DMA related APIs */
PCIEI_STATUS pcieiSetSystemDma(PCIEI_SYSDMA_CMD_PARAMS DmaParams);
PCIEI_STATUS pcieiGetSystemDma(PTR_PCIEI_SYSDMA_CMD_PARAMS PtrDmaParams);
PCIEI_STATUS pcieiGetDmaStatus(U32 BitMask, PTR_PCIEI_BIT_STATUS PtrBitStatus);
PCIEI_STATUS pcieiClearDmaStatus(U32 BitMask);
/* Setup of fusion chain limit */
PCIEI_STATUS pcieiSetFusionChainLimit(U32 ChainLimit);
/* Descriptor Soft Fifo related APIs */
PCIEI_STATUS pcieiInitDescSoftFifo(PCIEI_DESC_GENERIC_FIFO_PARAMS SoftFifoParams);
PCIEI_STATUS pcieiInitGenericHsFifo(PCIEI_DESC_GENERIC_FIFO_PARAMS SoftFifoParams);
void         pcieiDumpCoreRegs(void);
PCIEI_STATUS pcieiGetDataDescSize(PU32 PtrSize);
/*
 * Fusion Registers Interface APIs
 */

/* System Doorbell Register Interface APIs */
PCIEI_STATUS pcieiGetDoorbellValue(PU32 PtrValue);
PCIEI_STATUS pcieiSetDoorbellValue(U32 Value);

/* Write Sequence Register Interface APIs */
PCIEI_STATUS pcieiWriteSequence(U32 Value);

/* Host Interrupt Status Interface APIs */
PCIEI_STATUS pcieiGetHostInterruptStatus(U32 StatusBitMask,
                                PTR_PCIEI_BIT_STATUS PtrBitValue);

/* Reply Free Host Index APIs */
PCIEI_STATUS pcieiGetReplyFreeHostIndex(PU32 PtrValue);
PCIEI_STATUS pcieiSetReplyFreeHostIndex(U32 Value);

/*Reply Post Host Index APIs*/
PCIEI_STATUS pcieiGetReplyPostHostIndex(PU32 PtrValue);
PCIEI_STATUS pcieiSetReplyPostHostIndex(U32 Value);

PCIEI_STATUS pcieiGetReplyPostListProducerIndex(PU32 PtrValue);
PCIEI_STATUS pcieiSetReplyPostListProducerIndex(U32 Value);

PCIEI_STATUS pcieiGetReplyPostListConsumerIndex(PU32 PtrValue);
PCIEI_STATUS pcieiSetReplyPostListConsumerIndex(U32 Value);

/* Producer/Consumer Index manipulation for local queues */
PCIEI_STATUS pcieiGetProducerIndex(U32 QueueId, PU32 PtrValue);
PCIEI_STATUS pcieiSetProducerIndex(U32 QueueId, U32 Value);
PCIEI_STATUS pcieiGetConsumerIndex(U32 QueueId, PU32 PtrValue);
PCIEI_STATUS pcieiSetConsumerIndex(U32 QueueId, U32 Value);

/* Reply Post FIFO Interface APIs */
PCIEI_STATUS pcieiGetReplyPostFifo(PU64 PtrValue);
PCIEI_STATUS pcieiSetReplyPostFifo(U64 Value);

/* DMA Chain Overflow Error Interface APIs */
//PCIEI_STATUS pcieiGetMsgPullDmaChain(PU32 PtrValue);
PCIEI_STATUS pcieiGetMsgPullDmaChain(PU64 PtrValue);

/* Messaging Unit Error Interface APIs */
PCIEI_STATUS pcieiGetMessagingUnitError(U32 BitMask,
                                PTR_PCIEI_BIT_STATUS PtrBitValue);
PCIEI_STATUS pcieiSetMessagingUnitError(U32 BitMask);
PCIEI_STATUS pcieiGetMessagingUnitErrorTest(U32 BitMask,
                                PTR_PCIEI_BIT_STATUS PtrBitValue);
PCIEI_STATUS pcieiSetClearMessagingUnitErrorTest(U32 BitMask, BOOL SetFlag);

/* IOP Interrupt APIs*/
PCIEI_STATUS pcieiGetIopInterruptStatus(U32 StatusBitMask,
                                PTR_PCIEI_BIT_STATUS PtrBitValue);
PCIEI_STATUS pcieiSetClearIopInterruptMask(U32 BitMask, BOOL SetFlag);

/*Queue Status Register Interface*/
PCIEI_STATUS pcieiGetQueueStatus(U32 BitMask, PTR_PCIEI_BIT_STATUS PtrBitValue);

/*Sys2IOP Doorbell Clear Register Interface*/
PCIEI_STATUS pcieiClearSys2IopDb(void);
PCIEI_STATUS pcieiClearDoorbellTakenStatus(void);

/* Local Queues Interface APIs */
//PCIEI_STATUS pcieiInitLocalQueue(U32 QueueId, U32 BaseAddress, U32 QueueSize);
PCIEI_STATUS pcieiInitLocalQueue(U32 QueueId, U64 BaseAddress, U32 QueueSize);
PCIEI_STATUS pcieiEnableLocalQueue(U32 QueueId);
PCIEI_STATUS pcieiDisableLocalQueue(U32 QueueId);

/* Message Pool Control Interface */
PCIEI_STATUS pcieiGetMsgPullControlStatus(
                    PTR_PCIEI_CONFIG_MSG_PULL_CONTROL PtrMsgPullControl);
PCIEI_STATUS pcieiSetMsgPullControlStatus(
                    PTR_PCIEI_CONFIG_MSG_PULL_CONTROL PtrMsgPullControl);

/*Message Queue Control Register Interface*/
PCIEI_STATUS pcieiSetMsgQueueControl(U32 Value);

/* Initialization of HostBasedReplyQHigh queue */
PCIEI_STATUS pcieiInitHostBasedReplyQueueHigh(U32 BaseAddressHigh);

/* Initialization of System Message Pool Interface */
PCIEI_STATUS pcieiInitSystemMsgPoolAddress(U32 PoolAddressLow, U32 PoolAddressHigh, U32 Size);

/* Initialization of Local Message Pool Interface */
PCIEI_STATUS pcieiInitLocalMsgPoolAddress(U32 PoolAddressLow, U32 PoolAddressHigh, U32 Size);

PCIEI_STATUS pceiSetClearResetAdapter( BOOL Value);

#endif /* End PCIEI_H_INCLUDED */
