/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  memport.h
 *          Title:  MemPort Definitions
 *
 *
 * Description
 * ------------
 *  This file contains definitions used in setting up and configuring the
 *  Memports in the Invader memory system.
 *
 */

#ifndef MEMPORT_H_INCLUDED
#define MEMPORT_H_INCLUDED

/*Base (DCR) address of memory subsytem */
#define MSS_REG_ADDR 0x20000

/*****************************************************************************
  Memport config registers
*****************************************************************************/

/*DCR addresses of registers*/
#define MSS_MP_TRANS_CFG_0 (MSS_REG_ADDR+0x0053) /* Memport 0, SAS SHELL */
#define MSS_MP_TRANS_CFG_1 (MSS_REG_ADDR+0x0063) /* Memport 1, PCE SHELL */
#define MSS_MP_TRANS_CFG_2 (MSS_REG_ADDR+0x0073) /* Memport 2, DMA SHELL */
#define MSS_MP_TRANS_CFG_3 (MSS_REG_ADDR+0x0083) /* Memport 3, DMA SHELL */

/*Bit definitions for Memport config registers*/

/*
 * MSS_MP_TRANS_CFG_0_RD_FIFO_BLK_FSET
 * This register will hold the number of entries in the RD
 * Data RateMatch FIFO that will be reserved to ensure that
 * the RD Data Fifo will not become full.
 */
#define MSS_MP_TRANS_CFG_RD_FIFO_BLK_FSET_MASK                          (0x00000003)
#define MSS_MP_TRANS_CFG_RD_FIFO_BLK_FSET_SHIFT                         ( 0)
#define MSS_MP_TRANS_CFG_RESERVED4_MASK                                 (0x000000FC)
#define MSS_MP_TRANS_CFG_RESERVED4_SHIFT                                ( 2)
/*
 * MSS_MP_TRANS_CFG_WR_FIFO_BLK_FSET
 * This register will hold the number of 32-byte words that
 * must be written to the WR Data Fifo for each maximumtransfer-
 * size block before the WCSM is allowed to issue
 * an AXI WR command for that block.
 */
#define MSS_MP_TRANS_CFG_WR_FIFO_BLK_FSET_MASK                          (0x00000F00)
#define MSS_MP_TRANS_CFG_WR_FIFO_BLK_FSET_SHIFT                         ( 8)
#define MSS_MP_TRANS_CFG_RESERVED3_MASK                                 (0x0000F000)
#define MSS_MP_TRANS_CFG_RESERVED3_SHIFT                                (12)
/*
 * MSS_MP_TRANS_CFG_BLK_SIZE
 * This register determines the maximum transfer size to
 * the DC. Bit-0 indicates 128-byte transfers, Bit-1 indicates
 * 256-byte transfers, and Bit-2 indicates 512-byte transfers.
 * These bits are mutually exclusive.
 * 000: Not Supported
 * 001: 128-byte transfers
 * 010: 256-byte transfers
 * 011: Not Supported
 * 100: 512-byte transfers
 * 101: Not Supported
 * 110: Not Supported
 * 111: Not Supported
 */
#define MSS_MP_TRANS_CFG_BLK_SIZE_MASK                                  (0x00070000)
#define MSS_MP_TRANS_CFG_BLK_SIZE_SHIFT                                 (16)
#define MSS_MP_TRANS_CFG_RESERVED1_MASK                                 (0x00180000)
#define MSS_MP_TRANS_CFG_RESERVED1_SHIFT                                (19)
/*
 * MSS_MP_TRANS_CFG_SINGLE_THRD_DISABLE
 * If this bit is set, then MP will enqueue as many SHELL
 * transactions as there are empty slots and process them
 * as fast as possible. If this bit is clear, then MP will accept
 * ONLY one SHELL transaction at a time and process it until
 * completion before accepting the next new SHELL
 * transaction.
 * 0: Single-threaded mode is enabled: enqueue one SHELL
 * transaction until completion.
 * 1: Single-threaded mode is disabled: enqueue as many
 * SHELL transactions as allowed.
 */
#define MSS_MP_TRANS_CFG_SINGLE_THRD_DISABLE_MASK                       (0x00200000)
#define MSS_MP_TRANS_CFG_SINGLE_THRD_DISABLE_SHIFT                      (21)
/*
 * MSS_MP_TRANS_CFG_TRANS_PROC_DISABLE
 * If this bit is set, then the MP will no longer issue any
 * more requests to the DC. Any outstanding requests to
 * the DC will finish appropriately.
 * 0: Process enqueued SHELL requests
 * 1: Do not process any more enqueued SHELL requests:
 * stop sending DC requests
 */
#define MSS_MP_TRANS_CFG_TRANS_PROC_DISABLE_MASK                        (0x00400000)
#define MSS_MP_TRANS_CFG_TRANS_PROC_DISABLE_SHIFT                       (22)
/*
 * MSS_MP_TRANS_CFG_TRANS_ALLOC_DISABLE
 * If this bit is set, then the MP will no longer accept SHELL
 * transactions. SHELL transactions enqueued before
 * setting this bit will finish to completion if allowed.
 * 0: Accept new SHELL transactions
 * 1: Do not accept new SHELL transactions
 */
#define MSS_MP_TRANS_CFG_TRANS_ALLOC_DISABLE_MASK                       (0x00800000)
#define MSS_MP_TRANS_CFG_TRANS_ALLOC_DISABLE_SHIFT                      (23)
/*
 * MSS_MP_TRANS_CFG_ALIGN_64N32
 * If this bit is set, then the MemPort will align all
 * transactions to the AXI target on a 64-byte boundary. If
 * this bit is clear, then the MemPort will align all
 * transactions to the AXI target on a 32-byte boundary.
 */
#define MSS_MP_TRANS_CFG_ALIGN_64N32_MASK                               (0x01000000)
#define MSS_MP_TRANS_CFG_ALIGN_64N32_SHIFT                              (24)
#define MSS_MP_TRANS_CFG_RESERVED0_MASK                                 (0x3E000000)
#define MSS_MP_TRANS_CFG_RESERVED0_SHIFT                                (25)
/*
 * MSS_MP_TRANS_CFG_MP_HALT_REQ
 * Assert this bit to request halting AXI traffic between the
 * MemPort and the DRAM Controller. When traffic is
 * halted the MP_HALTED bit is set in the MP Status
 * register. Refer to the Memory Port EDS for specifics.
 */
#define MSS_MP_TRANS_CFG_MP_HALT_REQ_MASK                               (0x40000000)
#define MSS_MP_TRANS_CFG_MP_HALT_REQ_SHIFT                              (30)
/*
 * MSS_MP_TRANS_CFG_BLOCK_ENABLE
 * If this bit is set, then the MemPort will accept
 * transactions. If this bit is clear, then the MemPort will
 * not accept transactions by returning a FAULT completion.
 */
#define MSS_MP_TRANS_CFG_BLOCK_ENABLE_MASK                              (0x80000000)
#define MSS_MP_TRANS_CFG_BLOCK_ENABLE_SHIFT                             (31)

/*****************************************************************************
  Memport XOR config registers
*****************************************************************************/

/*DCR addresses of registers*/
#define MSS_MP_WXOR_CFG_0 (MSS_REG_ADDR+0x0050)
#define MSS_MP_WXOR_CFG_1 (MSS_REG_ADDR+0x0060)
#define MSS_MP_WXOR_CFG_2 (MSS_REG_ADDR+0x0070)
#define MSS_MP_WXOR_CFG_3 (MSS_REG_ADDR+0x0080)

/*Bit definitions for Memport XOR config registers*/

/*
 * MSS_MP_WXOR_CFG_0_POLY_11D_NOT_14D
 * Determines which polynomial to use for the multiply.
 * If this feature is set, then the 11D value will be
 * used as the polynomial. If this feature is clear,
 * then the 14D value will be used.
 */
#define MSS_MP_WXOR_CFG_POLY_11D_NOT_14D_MASK                           (0x00000001)
#define MSS_MP_WXOR_CFG_POLY_11D_NOT_14D_SHIFT                          ( 0)
#define MSS_MP_WXOR_CFG_RESERVED1_MASK                                  (0x00000FFE)
#define MSS_MP_WXOR_CFG_RESERVED1_SHIFT                                 ( 1)
/*
 * MSS_MP_WXOR_CFG_0_XOR_BASE_ADDR
 * Provides the base address for the XOR window. MP will
 * use the XOR base address as an XOR decode against the
 * top 4-bits of the system address.
 */
#define MSS_MP_WXOR_CFG_XOR_BASE_ADDR_MASK                              (0x0000F000)
#define MSS_MP_WXOR_CFG_XOR_BASE_ADDR_SHIFT                             (12)
#define MSS_MP_WXOR_CFG_RESERVED0_MASK                                  (0xFFFF0000)
#define MSS_MP_WXOR_CFG_RESERVED0_SHIFT                                 (16)


/*****************************************************************************
  Memory Subsystem interrupt and status registers
*****************************************************************************/

/* DCR addresses of registers*/

/* Int status -- read to check for interrupt */
#define MSS_ERR_INT_STAT (MSS_REG_ADDR+0x0002)

/* Int mask -- set to mask interrupt*/
#define MSS_ERR_INT_MASK (MSS_REG_ADDR+0x0003)

/* Bit definitions.  */

/* Indicates when MemPort X (0,1,2, or 3) has detected an error. The core interrupt
 * will be set if the corresponding Interrupt Mask is not set. The cause of
 * the interrupt is captured in the MP_X Error Log register. Clearing all
 * errors in the MP_X Error Log register will clear this bit and will also reset
 * the core interrupt if it was asserted, assuming no other interrupt
 * assertion conditions exist.
 */
#define MSS_ERR_INT_MP0_ERR_INT_MASK                               (0x00000001)
#define MSS_ERR_INT_MP0_ERR_INT_SHIFT                              ( 0)
#define MSS_ERR_INT_MP1_ERR_INT_MASK                               (0x00000002)
#define MSS_ERR_INT_MP1_ERR_INT_SHIFT                              ( 1)
#define MSS_ERR_INT_MP2_ERR_INT_MASK                               (0x00000004)
#define MSS_ERR_INT_MP2_ERR_INT_SHIFT                              ( 2)
#define MSS_ERR_INT_MP3_ERR_INT_MASK                               (0x00000008)
#define MSS_ERR_INT_MP3_ERR_INT_SHIFT                              ( 3)

/* The following definitions are included for completeness, but are not memport
 * interrupts.
 */
#define MSS_ERR_INT_DCR_ERR_LOG_VALID_MASK                         (0x00000010)
#define MSS_ERR_INT_DCR_ERR_LOG_VALID_SHIFT                        ( 4)
#define MSS_ERR_INT_RESERVED3_MASK                                 (0x000000E0)
#define MSS_ERR_INT_RESERVED3_SHIFT                                ( 5)
#define MSS_ERR_INT_RDIMM_PAR_ERR_MASK                             (0x00000100)
#define MSS_ERR_INT_RDIMM_PAR_ERR_SHIFT                            ( 8)
#define MSS_ERR_INT_PHY_EVENT_INT_MASK                             (0x00000200)
#define MSS_ERR_INT_PHY_EVENT_INT_SHIFT                            ( 9)
#define MSS_ERR_INT_RESERVED2_MASK                                 (0x00000C00)
#define MSS_ERR_INT_RESERVED2_SHIFT                                (10)
#define MSS_ERR_INT_MO_INT_STAT_MASK                               (0x00001000)
#define MSS_ERR_INT_MO_INT_STAT_SHIFT                              (12)
#define MSS_ERR_INT_RESERVED1_MASK                                 (0x0000E000)
#define MSS_ERR_INT_RESERVED1_SHIFT                                (13)
#define MSS_ERR_INT_FC0_DP_INT_MASK                                (0x00010000)
#define MSS_ERR_INT_FC0_DP_INT_SHIFT                               (16)
#define MSS_ERR_INT_FC1_DP_INT_MASK                                (0x00020000)
#define MSS_ERR_INT_FC1_DP_INT_SHIFT                               (17)
#define MSS_ERR_INT_RESERVED0_MASK                                 (0xFFFC0000)
#define MSS_ERR_INT_RESERVED0_SHIFT                                (18)


/*****************************************************************************
  Mem Port error log registers
*****************************************************************************/

/* DCR addresses of registers*/
#define MSS_MP_ERR_LOG_0 (MSS_REG_ADDR+0x0052)
#define MSS_MP_ERR_LOG_1 (MSS_REG_ADDR+0x0062)
#define MSS_MP_ERR_LOG_2 (MSS_REG_ADDR+0x0072)
#define MSS_MP_ERR_LOG_3 (MSS_REG_ADDR+0x0082)

/* Bit definitions (applies to all 4 of the above registers) */

/* Latched parity error detection when pulling data out of the RD data
 * FIFO. This is the data that will be sent on XferInboundData from
 * SDRAM. Register will remain latched until cleared.
 */
#define MSS_MP_ERR_LOG_X_RD_FIFO_PAR_ERR_MASK                         (0x00000001)
#define MSS_MP_ERR_LOG_X_RD_FIFO_PAR_ERR_SHIFT                        ( 0)
#define MSS_MP_ERR_LOG_X_RESERVED3_MASK                               (0x0000000E)
#define MSS_MP_ERR_LOG_X_RESERVED3_SHIFT                              ( 1)

/* Latched parity error detection when pulling data out of the WR data
 * FIFO. This is the data that will be sent to SDRAM received from
 * XferOutboundData. Register will remain latched until cleared.
 */
#define MSS_MP_ERR_LOG_X_WR_FIFO_PAR_ERR_MASK                         (0x00000010)
#define MSS_MP_ERR_LOG_X_WR_FIFO_PAR_ERR_SHIFT                        ( 4)
#define MSS_MP_ERR_LOG_X_RESERVED2_MASK                               (0x000000E0)
#define MSS_MP_ERR_LOG_X_RESERVED2_SHIFT                              ( 5)

/* Latched parity error detection when pulling data out of the XOR buffer
 * for a WXOR operation. Register will remain latched until cleared.
 */
#define MSS_MP_ERR_LOG_X_XOR_BUF_PAR_ERR_MASK                         (0x00000100)
#define MSS_MP_ERR_LOG_X_XOR_BUF_PAR_ERR_SHIFT                        ( 8)
#define MSS_MP_ERR_LOG_X_RESERVED1_MASK                               (0x0000FE00)
#define MSS_MP_ERR_LOG_X_RESERVED1_SHIFT                              ( 9)
#define MSS_MP_ERR_LOG_X_RESERVED0_MASK                               (0xFFFF0000)
#define MSS_MP_ERR_LOG_X_RESERVED0_SHIFT                              (16)


#endif
