/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibDevMsc.h
 *          Title:  USB library mass storage class header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains header definitions for the USB mass storage class
 * interfacing functionality.
 *
 * Note:
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIBDEVMSC_H_
#define _USBLIBDEVMSC_H_

#if (IOC_USB_SUPPORT)

/* ************************************************************************** */
/* USB MSC Common Constant Definitions                                        */
/* ************************************************************************** */
#define USB_REQ_TYPE_MSC_RST                     0x21
#define USB_MSC_RST_REQ                          0xFF
#define USB_REQ_TYPE_MSC_GET_MAX_LUN             0xA1

#define USB_MSC_GET_MAX_LUN_REQ                  0xFE

#define USB_CBW_SIGNATURE                        0x43425355
#define USB_CSW_SIGNATURE                        0x53425355

#define USB_CSW_STATUS_PASS                      0x00
#define USB_CSW_STATUS_CMD_FAIL                  0x01
#define USB_CSW_STATUS_PHASE_ERR                 0x02

#define USB_NUM_CBW                              1
#define USB_NUM_CSW                              1
#define USB_NUM_CMDS_PER_DEV                     1

#define USB_DATA_DIRECTION_IN                    1
#define USB_DATA_DIRECTION_OUT                   0

#define USB_IO_CMD_TIMEOUT                       5000  // milliseconds - first request cab be really slow
#define USB_IO_CMD_TIMEOUT_READ_WRITE            2000

#define USB_INQUIRY_DATA_LENGTH                  0x24
#define USB_REQUEST_SENSE_DATA_LENGTH            0x12

/* defines to be used in call to usbMscReset() */
#define USB_MSC_RESET_TYPE_RESET_ONLY            0
#define USB_MSC_RESET_TYPE_SEND_CLEAR_FEATURE    1

/* Sense Data offsets */
#define USB_REQUEST_SENSE_SENSE_KEY_OFFSET       0x2
#define USB_REQUEST_SENSE_ASC_OFFSET             0xC
#define USB_REQUEST_SENSE_ASCQ_OFFSET            0xD


/* ************************************************************************** */
/* USB Command Block Wrapper definition                                       */
/* ************************************************************************** */
#pragma pack(push)
#pragma pack(1)
typedef struct _USB_CBW
{
    U32                 dCBWSignature;
    U32                 dCBWTag;
    U32                 dCBWDataTransferLength;
    U8                  bmCBWFlags;
    U8                  bCBWLUN;
    U8                  bCBWCBLength;
    U8                  CBWCB[16];

} USB_CBW, *PTR_USB_CBW;

/* ************************************************************************** */
/* USB Status Block Wrapper definition                                        */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CSW
{
    U32                 dCBWSignature;
    U32                 dCBWTag;
    U32                 dCBWDataResidue;
    U8                  bCSWStatus;

} USB_CSW, *PTR_USB_CSW;

/* ************************************************************************** */
/* USB Command - Inquiry                                                      */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CMD_SCSI_INQUIRY
{
    U8                  OpCode;
    union
    {
        U8              Byte;

        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U8          Res      :7;
            U8          CmdDT    :1;
            U8          EVDT     :1;
#else
            U8          EVDT     :1;
            U8          CmdDT    :1;
            U8          Res      :7;
#endif
        }
        fields;

    } flags;

    U8                  PageOpCode;
    U16                 AllocLength;
    U8                  Control;

} USB_CMD_SCSI_INQUIRY, *PTR_USB_CMD_SCSI_INQUIRY;


/* ************************************************************************** */
/* USB Command - Test Unit Ready                                              */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CMD_SCSI_TEST_UNIT_READY
{
    U8                  OpCode;
    U8                  Reserved[4];
    U8                  Control;

} USB_CMD_SCSI_TEST_UNIT_READY, *PTR_USB_CMD_SCSI_TEST_UNIT_READY;


/* ************************************************************************** */
/* USB Command - Request Sense                                                */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CMD_SCSI_REQUEST_SENSE
{
    U8                  OpCode;
    U8                  Reserved[3];
    U8                  AllocLength;
    U8                  Control;

} USB_CMD_SCSI_REQUEST_SENSE, *PTR_USB_CMD_SCSI_REQUEST_SENSE;


/* ************************************************************************** */
/* USB Command - Read Capacity                                                */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CMD_SCSI_READ_CAPACITY
{
    U8                  OpCode;
    union
    {
        U8              Byte;
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U8          Res      :7;
            U8          LONGLBA  :1;
            U8          RELADR   :1;
#else
            U8          RELADR   :1;
            U8          LONGLBA  :1;
            U8          Res      :7;
#endif
        }
        fields;

    }
    flags1;

    U32                 LBA;
    U8                  Reserved[2];

    union
    {
        U8              Byte;

        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U8          Res      :7;
            U8          PMI      :1;
#else
            U8          PMI      :1;
            U8          Res      :7;
#endif
        }
        fields;
    }
    flags2;

    U8                  Control;

} USB_CMD_SCSI_READ_CAPACITY, *PTR_USB_CMD_SCSI_READ_CAPACITY;


/* ************************************************************************** */
/* USB Command - Mode Sense                                                   */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CMD_SCSI_MODE_SENSE
{
    U8                  OpCode;

    union
    {
        U8              Byte;

        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U8          Res2     :4;
            U8          DBD      :1;
            U8          Res1     :3;
#else
            U8          Res1     :3;
            U8          DBD      :1;
            U8          Res2     :4;
#endif
        }
        fields;

    }
    flags;

    union
    {
        U8              Byte;

        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U8          PC       :2;
            U8          PageCode :6;
#else
            U8          PageCode :6;
            U8          PC       :2;
#endif
        }
        fields;
    }
    page;

    U8                  SubPageCode;
    U8                  AllocLength;
    U8                  Control;

} USB_CMD_SCSI_MODE_SENSE, *PTR_USB_CMD_SCSI_MODE_SENSE;


/* ************************************************************************** */
/* USB Command - Read (10)                                                    */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CMD_SCSI_READ_10
{
    U8                  OpCode;
    union
    {
        U8              Byte;

        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U8          Res      :3;
            U8          DPO      :1;
            U8          FUA      :1;
            U8          Res1     :2;
            U8          RELADR   :1;
#else
            U8          RELADR   :1;
            U8          Res1     :2;
            U8          FUA      :1;
            U8          DPO      :1;
            U8          Res      :3;
#endif
        }fields;

    }flags;

    U8                  LBA[4];
    U8                  Reserved;
    U8                  TransLength[2];
    U8                  Control;

} USB_CMD_SCSI_READ_10, *PTR_USB_CMD_SCSI_READ_10;


/* ************************************************************************** */
/* USB Command - Write (10)                                                   */
/* ************************************************************************** */
//#pragma pack(1)
typedef struct _USB_CMD_SCSI_WRITE_10
{
    U8                  OpCode;

    union
    {
        U8              Byte;

        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U8          Res      :3;
            U8          DPO      :1;
            U8          FUA      :1;
            U8          Res1     :2;
            U8          RELADR   :1;
#else
            U8          RELADR   :1;
            U8          Res1     :2;
            U8          FUA      :1;
            U8          DPO      :1;
            U8          Res      :3;
#endif
        } fields;
    } flags;

    U8                  LBA[4];
    U8                  Reserved;
    U8                  TransLength[2];
    U8                  Control;

} USB_CMD_SCSI_WRITE_10, *PTR_USB_CMD_SCSI_WRITE_10;


/* ************************************************************************** */
/* Function Prototypes - USB stack access only                                */
/* ************************************************************************** */
//#pragma pack
#pragma pack(pop)

USB_STATUS usbMscInit( U32 UsbPort );

USB_STATUS usbMscReset( U32 UsbPort, U32 type );

USB_STATUS usbMscSendScsiCmd(
    U32               UsbPort,
    U32               TransLength,
    BOOL              Direction,
    U8                Lun,
    U8                CmdLen,
    void*             cmdPtr,
    U32               BuffAddr,
    U32               IoTimeoutMs
    );

/* Accessible from host firmware */
USB_STATUS usbiMscSendInquiry( U32 UsbPort, U32 CmdHandle, U32 BuffAddr );
USB_STATUS usbiMscSendReqSense( U32 UsbPort, U32 CmdHandle, U32 BuffPtr );
USB_STATUS usbiMscSendTUR( U32 UsbPort, U32 CmdHandle );
USB_STATUS usbiMscSendReadCap( U32 UsbPort, U32 CmdHandle, U32 BuffPtr );
USB_STATUS usbiMscScsiRead10( U32 UsbPort, U32 CmdHandle, U32 SrcLBA, U32 DataBuffAddr, U32 TransferLength );
USB_STATUS usbiMscScsiWrite10( U32 UsbPort, U32 CmdHandle, U32 DstLBA, U32 DataBuffAddr, U32 TransferLength );
USB_STATUS usbiMscSendCDB(U32 UsbPort, U32 CmdHandle, U32 BufferAddr, U8 Direction, U32 TransferLength, void * PtrCdb, U8 CdbLength);


/* ************************************************************************** */
/* Externs                                                                    */
/* ************************************************************************** */
extern USB_STACK_GLOBAL_STATE            gUsbStackGlobalState[USB_MAX_PORTS];

/* ************************************************************************** */
/* USB - SCSI Command structures                                              */
/* ************************************************************************** */
extern USB_CMD_SCSI_INQUIRY              gUsbCmdScsiInquiry;
extern USB_CMD_SCSI_TEST_UNIT_READY      gUsbCmdScsiTUR;
extern USB_CMD_SCSI_REQUEST_SENSE        gUsbCmdScsiReqSense;
extern USB_CMD_SCSI_READ_CAPACITY        gUsbCmdScsiReadCapacity;
extern USB_CMD_SCSI_READ_10              gUsbCmdScsiRead10;
extern USB_CMD_SCSI_WRITE_10             gUsbCmdScsiWrite10;

/* ************************************************************************** */
/* USB - SCSI Command structures                                              */
/* ************************************************************************** */
extern PTR_USB_CBW                       gPtrUsbCBW[USB_MAX_PORTS];
extern PTR_USB_CSW                       gPtrUsbCSW[USB_MAX_PORTS];

//extern PU8                               gPtrUsbScsiGenericCmdStatus[USB_MAX_PORTS];
extern U32                               gTag;

extern PTR_USB_PKT_REQUEST               gPtrUsbRequest[USB_MAX_PORTS];

#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIBDEVMSC_H_ */

