/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibHw.h
 *          Title:  USB Library core hardware header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains header definitions for the USB core hardware registers.
 *
 * Note:
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIBHW_H_
#define _USBLIBHW_H_

#if (IOC_USB_SUPPORT)

#ifndef SASHW_PBAM_CORES_EA_BASE 
#define SASHW_PBAM_CORES_EA_BASE               (0xC2300000)
#endif

#ifndef SASHW_USB_ETH_CFG_EA_BASE
#define SASHW_USB_ETH_CFG_EA_BASE              (SASHW_PBAM_CORES_EA_BASE + 0x3800)
#endif

#ifndef PBAM_USB0_OFFSET
#define PBAM_USB0_OFFSET                    (0x00000000)
#endif

#ifndef PBAM_USB1_OFFSET
#define PBAM_USB1_OFFSET                    (0x00000200)
#endif

#ifndef PBAM_TRNG_OFFSET
#define PBAM_TRNG_OFFSET                    (0x00000400)
#endif

#ifndef PBAM_ETHERNET_OFFSET
#define PBAM_ETHERNET_OFFSET                (0x00000600)
#endif

#ifndef PBAM_USB0_REGS_VADDR
#define PBAM_USB0_REGS_VADDR                (SASHW_USB_ETH_CFG_EA_BASE+PBAM_USB0_OFFSET)
#endif

#ifndef PBAM_USB1_REGS_VADDR
#define PBAM_USB1_REGS_VADDR                (SASHW_USB_ETH_CFG_EA_BASE+PBAM_USB1_OFFSET)
#endif

#ifndef PBAM_TRNG_REGS_VADDR
#define PBAM_TRNG_REGS_VADDR                (SASHW_USB_ETH_CFG_EA_BASE+PBAM_TRNG_OFFSET)
#endif

#ifndef PBAM_ETHERNET_REGS_VADDR
#define PBAM_ETHERNET_REGS_VADDR            (SASHW_USB_ETH_CFG_EA_BASE+PBAM_ETHERNET_OFFSET)
#endif


/* ************************************************************************** */
/* Definitions / Enumerations / Constants / Limits                            */
/* ************************************************************************** */
typedef enum _PBAM_USB_CORE_INDEX
{
    PBAM_USB_CORE_INDEX_0,
    PBAM_USB_CORE_INDEX_1,

    PBAM_USB_CORE_INDEX_INVALID

} PBAM_USB_CORE_INDEX;


typedef enum _PBAM_USB_GP_TIMER_NUMBER
{
    PBAM_USB_GP_TIMER_NUMBER_0,         /* used for delays */
    PBAM_USB_GP_TIMER_NUMBER_1          /* used for timing I/Os */

} PBAM_USB_GP_TIMER_NUMBER;


/* Definition of values Reg: USBCMD; Field: Interrupt Threshold Control (ITC) */
#define PBAM_USB_ITC_VAL_IMMEDIATE                      (0x00)
#define PBAM_USB_ITC_VAL_MICRO_FRAME_1                  (0x01)
#define PBAM_USB_ITC_VAL_MICRO_FRAME_2                  (0x02)
#define PBAM_USB_ITC_VAL_MICRO_FRAME_4                  (0x04)
#define PBAM_USB_ITC_VAL_MICRO_FRAME_8                  (0x08)
#define PBAM_USB_ITC_VAL_MICRO_FRAME_16                 (0x10)
#define PBAM_USB_ITC_VAL_MICRO_FRAME_32                 (0x20)
#define PBAM_USB_ITC_VAL_MICRO_FRAME_64                 (0x40)


/* Definition of values Reg: USBCMD; Field: Frame List Size (FS) */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_1024          (0x00)  /* Default value, 4096 bytes */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_512           (0x01)  /* 2048 bytes */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_256           (0x02)  /* 1024 bytes */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_128           (0x03)  /* 512 bytes */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_64            (0x04)  /* 256 bytes */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_32            (0x05)  /* 128 bytes */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_16            (0x06)  /* 64 bytes */
#define PBAM_USB_FRAME_LIST_SIZE_ELEMENTS_8             (0x07)  /* 32 bytes */


/* Definition of values Reg: USBMODE; Field: Endian Select (ES) */
#define PBAM_USB_ENDIAN_SELECT_LITTLE_ENDIAN            (0x00)
#define PBAM_USB_ENDIAN_SELECT_BIG_ENDIAN               (0x01)

/* Definition of values Reg: USBMODE; Field: Controller Mode (CM) */
#define PBAM_USB_CTRLR_MODE_IDLE                        (0x00)
#define PBAM_USB_CTRLR_MODE_RESERVED                    (0x01)
#define PBAM_USB_CTRLR_MODE_DEVICE_ONLY                 (0x02)
#define PBAM_USB_CTRLR_MODE_HOST_ONLY                   (0x03)


/* ************************************************************************** */
/*                                                                            */
/*              I D E N T I F I C A T I O N    R E G I S T E R S              */
/*                                                                            */
/* ************************************************************************** */

/* ************************************************************************** */
/* Register - USB ID; Offset: Base + 0000h                                    */
/* ************************************************************************** */

typedef struct _USB_ID
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved    :8;
            U32     Revision    :8;
            U32     Nid         :8;
            U32     Id          :8;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     Id          :8;
            U32     Nid         :8;
            U32     Revision    :8;
            U32     Reserved    :8;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;

    } u;

} USB_ID, *PTR_USB_ID;


/* ************************************************************************** */
/* Register - HWGENERAL; Offset: Base + 0004h                                 */
/* ************************************************************************** */

typedef struct _USB_HW_GENERAL
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved    :22;
            U32     PhySerial   : 1;
            U32     PhyType     : 3;
            U32     PhyWidth    : 2;
            U32     BWT         : 1;
            U32     ClockConfig : 2;
            U32     ResetType   : 1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     ResetType   : 1;
            U32     ClockConfig : 2;
            U32     BWT         : 1;
            U32     PhyWidth    : 2;
            U32     PhyType     : 3;
            U32     PhySerial   : 1;
            U32     Reserved    :22;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_HW_GENERAL, *PTR_USB_HW_GENERAL;


/* ************************************************************************** */
/* Register - HWHOST; Offset: Base + 0008h                                    */
/* ************************************************************************** */

typedef struct _USB_HW_HOST
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     TTPeriodic  : 8;
            U32     TTAsync     : 8;
            U32     Reserved    :12;
            U32     HC          : 1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     HC          : 1;
            U32     Reserved    :12;
            U32     TTAsync     : 8;
            U32     TTPeriodic  : 8;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_HW_HOST, *PTR_USB_HW_HOST;


/* ************************************************************************** */
/* Register - HWDEVICE; Offset: Base + 000Ch                                  */
/* ************************************************************************** */

typedef struct _USB_HW_DEVICE
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved    :26;
            U32     DevEp       : 5;
            U32     DevCapable  : 1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     DevCapable  : 1;
            U32     DevEp       : 5;
            U32     Reserved    :26;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_HW_DEVICE, *PTR_USB_HW_DEVICE;


/* ************************************************************************** */
/* Register - HWTXBUF; Offset: Base + 0010h                                   */
/* ************************************************************************** */

typedef struct _USB_HW_TX_BUFFER
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     TxLcr               :1;
            U32     Reserved            :7;
            U32     TxChannelAddress    :8;
            U32     TxAdd               :8;
            U32     TxBurst             :8;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     TxBurst             :8;
            U32     TxAdd               :8;
            U32     TxChannelAddress    :8;
            U32     Reserved            :7;
            U32     TxLcr               :1;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_HW_TX_BUFFER, *PTR_USB_HW_TX_BUFFER;


/* ************************************************************************** */
/* Register - HWRXBUF; Offset: Base + 0014h                                   */
/* ************************************************************************** */

typedef struct _USB_HW_RX_BUFFER
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved    :16;
            U32     RxAdd       : 8;
            U32     RxBurst     : 8;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     RxBurst     : 8;
            U32     RxAdd       : 8;
            U32     Reserved    :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_HW_RX_BUFFER, *PTR_USB_HW_RX_BUFFER;


/* ************************************************************************** */
/*                                                                            */
/*     D E V I C E    A N D    H O S T    C A P A B I L I T Y    R E G S.     */
/*                                                                            */
/* ************************************************************************** */

/* ************************************************************************** */
/* Register - HWRXBUF; Offset: Base + 0014h                                   */
/* ************************************************************************** */

typedef struct _USB_EHCI_COMPLIANCE
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     HCIVersion          :16;
            U32     Reserved            : 8;
            U32     CapLength           : 8;

            U32     Reserved60to63      : 4;
            U32     NumTT               : 4;
            U32     NumPortsPerTT       : 4;
            U32     Reserved49to51      : 3;
            U32     PortIndicator       : 1;
            U32     NumCompanionCtrlr   : 4;
            U32     NumPortsPerCC       : 4;
            U32     Reserved37to39      : 3;
            U32     PortPowerControl    : 1;
            U32     NumDownPorts        : 4;

            U32     Reserved80to95      :16;
            U32     EHCIExtendCapability: 8;
            U32     IsoSchedThresh      : 4;
            U32     Reserved67          : 1;
            U32     AsyncSchedParkCap   : 1;
            U32     ProgFrameListFlag   : 1;
            U32     AddressingCap       : 1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     CapLength           : 8;
            U32     Reserved8to15       : 8;
            U32     HCIVersion          :16;

            U32     NumDownPorts        : 4;
            U32     PortPowerControl    : 1;
            U32     Reserved37to39      : 3;
            U32     NumPortsPerCC       : 4;
            U32     NumCompanionCtrlr   : 4;
            U32     PortIndicator       : 1;
            U32     Reserved49to51      : 3;
            U32     NumPortsPerTT       : 4;
            U32     NumTT               : 4;
            U32     Reserved60to63      : 4;

            U32     AddressingCap       : 1;
            U32     ProgFrameListFlag   : 1;
            U32     AsyncSchedParkCap   : 1;
            U32     Reserved67          : 1;
            U32     IsoSchedThresh      : 4;
            U32     EHCIExtendCapability: 8;
            U32     Reserved80to95      :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_EHCI_COMPLIANCE, *PTR_USB_EHCI_COMPLIANCE;


/* ************************************************************************** */
/* Register - ; Offset: Base + 0000h                                   */
/* ************************************************************************** */

typedef struct _USB_DCI_VERSION
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved    :16;
            U32     DciVersion  :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     DciVersion  :16;
            U32     Reserved    :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_DCI_VERSION, *PTR_USB_DCI_VERSION;


/* ************************************************************************** */
/* Register - ; Offset: Base + 0000h                                   */
/* ************************************************************************** */

typedef struct _USB_DCC_PARAMS
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved        :24;
            U32     HostCapable     : 1;
            U32     DeviceCapable   : 1;
            U32     Reserved2       : 2;
            U32     DevEndPointNum  : 4;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     DevEndPointNum  : 4;
            U32     Reserved2       : 2;
            U32     DeviceCapable   : 1;
            U32     HostCapable     : 1;
            U32     Reserved        :24;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_DCC_PARAMS, *PTR_USB_DCC_PARAMS;


/* ************************************************************************** */
/*                                                                            */
/*          D E V I C E    A N D    H O S T    T I M E R    R E G S.          */
/*                                                                            */
/* ************************************************************************** */

/**
 * Note: Non-EHCI timer registers are not defined. ChipIdea
 * Manual Section 3.5 Device/Host Timer Registers (Non-EHCI) */

typedef struct _USB_GP_TIMER_LD
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved       :  8;
            U32     GPTimerLoadVal : 24;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     GPTimerLoadVal : 24;
            U32     Reserved       :  8;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_GP_TIMER_LD, *PTR_USB_GP_TIMER_LD;


typedef struct _USB_GP_TIMER_CTRL
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     GPTimerRun      :  1;
            U32     GPTimerReset    :  1;
            U32     Reserved25to29  :  5;
            U32     GPTimerMode     :  1;
            U32     GPTimerCounter  : 24;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     GPTimerCounter  : 24;
            U32     GPTimerMode     :  1;
            U32     Reserved25to29  :  5;
            U32     GPTimerReset    :  1;
            U32     GPTimerRun      :  1;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_GP_TIMER_CTRL, *PTR_USB_GP_TIMER_CTRL;


/* ************************************************************************** */
/*                                                                            */
/*    D E V I C E    A N D    H O S T    O P E R A T I O N A L    R E G S.    */
/*                                                                            */
/* ************************************************************************** */

/* ************************************************************************** */
/* Register - USBCMD; Offset: Base + 0140h                                    */
/* ************************************************************************** */

typedef struct _USB_CMD
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved                :8;
            U32     IntThreshControl        :8;
            U32     FrameListSize2          :1;
            U32     AddTdTripWire           :1;
            U32     SetupTripWire           :1;
            U32     Reserved12              :1;
            U32     AsyncSchedParkModeEnable:1;
            U32     Reserved10              :1;
            U32     AsyncSchedParkModeCount :2;
            U32     LightHostReset          :1;
            U32     IntAsyncAdvDoorbell     :1;
            U32     AsyncSchedEnable        :1;
            U32     PeriodicSchedEnable     :1; /* See HW bugtrack TB 780 if this is to be used. */
            U32     FrmaeListSize0to1       :2;
            U32     ControllerReset         :1;
            U32     RunStop                 :1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     RunStop                 :1;
            U32     ControllerReset         :1;
            U32     FrmaeListSize0to1       :2;
            U32     PeriodicSchedEnable     :1; /* See HW bugtrack TB 780 if this is to be used. */
            U32     AsyncSchedEnable        :1;
            U32     IntAsyncAdvDoorbell     :1;
            U32     LightHostReset          :1;
            U32     AsyncSchedParkModeCount :2;
            U32     Reserved10              :1;
            U32     AsyncSchedParkModeEnable:1;
            U32     Reserved12              :1;
            U32     SetupTripWire           :1;
            U32     AddTdTripWire           :1;
            U32     FrameListSize2          :1;
            U32     IntThreshControl        :8;
            U32     Reserved                :8;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_CMD, *PTR_USB_CMD;


/* ************************************************************************** */
/* Register - USBSTS; Offset: Base + 0144h                                    */
/* ************************************************************************** */

typedef struct _USB_STATUS_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved26to31          :6;
            U32     TimerInt1               :1;
            U32     TimerInt0               :1;
            U32     Reserved20to23          :4;
            U32     USBHostPeriodicInt      :1;
            U32     USBHostAsyncInt         :1;
            U32     Reserved17              :1;
            U32     NAKInt                  :1;
            U32     AsyncSchedStatus        :1;
            U32     PeriodicSchedStatus     :1;
            U32     Reclamation             :1;
            U32     HCHalted                :1;
            U32     Reserved11              :1;
            U32     ULPIInt                 :1;
            U32     Reserved9               :1;
            U32     DevContSuspend          :1;
            U32     SOFReceived             :1;
            U32     USBResetReceived        :1;
            U32     IntOnAsyncAdvance       :1;
            U32     SystemError             :1;
            U32     FrameListRollover       :1;
            U32     PortChangeDetect        :1;
            U32     USBErrorInt             :1;
            U32     USBInt                  :1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     USBInt                  :1;
            U32     USBErrorInt             :1;
            U32     PortChangeDetect        :1;
            U32     FrameListRollover       :1;
            U32     SystemError             :1;
            U32     IntOnAsyncAdvance       :1;
            U32     USBResetReceived        :1;
            U32     SOFReceived             :1;
            U32     DevContSuspend          :1;
            U32     Reserved9               :1;
            U32     ULPIInt                 :1;
            U32     Reserved11              :1;
            U32     HCHalted                :1;
            U32     Reclamation             :1;
            U32     PeriodicSchedStatus     :1;
            U32     AsyncSchedStatus        :1;
            U32     NAKInt                  :1;
            U32     Reserved17              :1;
            U32     USBHostAsyncInt         :1;
            U32     USBHostPeriodicInt      :1;
            U32     Reserved20to23          :4;
            U32     TimerInt0               :1;
            U32     TimerInt1               :1;
            U32     Reserved26to31          :6;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_STATUS_REG, *PTR_USB_STATUS_REG;


/* ************************************************************************** */
/* Register - USBINTR; Offset: Base + 0148h                                   */
/* ************************************************************************** */

typedef struct _USB_INTERRUPT_REG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved26to31          :6;
            U32     TimerInt1               :1;
            U32     TimerInt2               :1;
            U32     Reserved20to23          :4;
            U32     USBHostPeriodicInt      :1;
            U32     USBHostAsyncInt         :1;
            U32     Reserved17              :1;
            U32     NAKInt                  :1;
            U32     Reserved11to15          :5;
            U32     ULPIInt                 :1;
            U32     Reserved9               :1;
            U32     DevContSuspend          :1;
            U32     SOFReceived             :1;
            U32     USBResetReceived        :1;
            U32     IntOnAsyncAdvance       :1;
            U32     SystemError             :1;
            U32     FrameListRollover       :1;
            U32     PortChangeDetect        :1;
            U32     USBErrorInt             :1;
            U32     USBInt                  :1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     USBInt                  :1;
            U32     USBErrorInt             :1;
            U32     PortChangeDetect        :1;
            U32     FrameListRollover       :1;
            U32     SystemError             :1;
            U32     IntOnAsyncAdvance       :1;
            U32     USBResetReceived        :1;
            U32     SOFReceived             :1;
            U32     DevContSuspend          :1;
            U32     Reserved9               :1;
            U32     ULPIInt                 :1;
            U32     Reserved11to15          :5;
            U32     NAKInt                  :1;
            U32     Reserved17              :1;
            U32     USBHostAsyncInt         :1;
            U32     USBHostPeriodicInt      :1;
            U32     Reserved20to23          :4;
            U32     TimerInt2               :1;
            U32     TimerInt1               :1;
            U32     Reserved26to31          :6;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_INTERRUPT_REG, *PTR_USB_INTERRUPT_REG;


/* ************************************************************************** */
/* Register - FRINDEX; Offset: Base + 014Ch                                   */
/* ************************************************************************** */

typedef struct _USB_FRINDEX
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved        :18;
            U32     FRIndex         :14;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     FRIndex         :14;
            U32     Reserved        :18;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_FRINDEX, *PTR_USB_FRINDEX;


/* ************************************************************************** */
/* Register - PERIODICLISTBASE; DEVICEADDR; Offset: Base + 0154h              */
/* ************************************************************************** */

/* ************************************************************************** */
/* Register - Host Controller (PERIODICLISTBASE); Offset: Base + 0154h        */
/* ************************************************************************** */
typedef struct _USB_HOST_CONTROLLER_PERIODIC_LIST_BASE
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     BaseAddress     :20;
            U32     Reserved        :12;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     Reserved        :12;
            U32     BaseAddress     :20;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_HOST_CONTROLLER_PERIODIC_LIST_BASE, *PTR_USB_HOST_CONTROLLER_PERIODIC_LIST_BASE;


/* ************************************************************************** */
/* Register - Device Controller (USB DEVICEADDR); Offset: Base + 0154h        */
/* ************************************************************************** */

typedef struct _USB_DEVICE_ADDRESS
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     USBDeviceAddress        : 7;
            U32     DeviceAddressAdvance    : 1;
            U32     Reserved                :24;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     Reserved                :24;
            U32     DeviceAddressAdvance    : 1;
            U32     USBDeviceAddress        : 7;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_DEVICE_CONTROLLER, *PTR_USB_DEVICE_CONTROLLER;


/* ************************************************************************** */
/* Register - ASYNCLISTADDR; ENDPOINTLISTADDR; Offset: Base + 0158h           */
/* ************************************************************************** */

/* ************************************************************************** */
/* Register - Host Controller (ASYNCLISTADDR); Offset: Base + 0158h           */
/* ************************************************************************** */
typedef struct _USB_ASYNC_LIST_ADDR
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     AsyncListBaseAddr   :27;
            U32     Reserved            :5;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     Reserved            :5;
            U32     AsyncListBaseAddr   :27;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_ASYNC_LIST_ADDR, *PTR_USB_ASYNC_LIST_ADDR;


/* ************************************************************************** */
/* Register - Device Controller (ENDPOINTLISTADDR); Offset: Base + 0158h      */
/* ************************************************************************** */
typedef struct _USB_DEVICE_END_POINT_LIST_ADDR
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     EndPointBaseAddr    :21;
            U32     Reserved            :11;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     Reserved            :11;
            U32     EndPointBaseAddr    :21;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_DEVICE_END_POINT_LIST_ADDR, *PYR_USB_DEVICE_END_POINT_LIST_ADDR;


/* ************************************************************************** */
/* Register - TTCTRL; Offset: Base + 015Ch                                    */
/* ************************************************************************** */

typedef struct _USB_TT_CONTROL
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved31      : 1;
            U32     TTHubAddress    : 7;
            U32     Reserved0to23   :24;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     Reserved0to23   :24;
            U32     TTHubAddress    : 7;
            U32     Reserved31      : 1;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_TT_CONTROL, *PTR_USB_TT_CONTROL;


/* ************************************************************************** */
/* Register - BURSTSIZE; Offset: Base + 0160h                                 */
/* ************************************************************************** */

typedef struct _USB_BURST_SIZE
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved        :16;
            U32     TxBurstLength   : 8;
            U32     RxBurstLength   : 8;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     RxBurstLength   : 8;
            U32     TxBurstLength   : 8;
            U32     Reserved        :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_BURST_SIZE, *PTR_USB_BURST_SIZE;


/* ************************************************************************** */
/* Register - TXFILLTUNING; Offset: Base + 0164h                              */
/* ************************************************************************** */

typedef struct _USB_TX_FILL_TUNING
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved22to31      :10;
            U32     TxFifoBurstThresh   : 6;
            U32     Reserved13to15      : 3;
            U32     TxSchedHealthCount  : 5;
            U32     TxSchedOverhead     : 8;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     TxSchedOverhead     : 8;
            U32     TxSchedHealthCount  : 5;
            U32     Reserved13to15      : 3;
            U32     TxFifoBurstThresh   : 6;
            U32     Reserved22to31      :10;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_TX_FILL_TUNING, *PTR_USB_TX_FILL_TUNING;


/* ************************************************************************** */
/* Register - ULPI VIEWPORT; Offset: Base + 0170h                             */
/* ************************************************************************** */

typedef struct _USB_ULPI_VIEWPORT
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
             U32    WakeUp          :1;
             U32    Run             :1;
             U32    ReadWrite       :1;
             U32    Reserved        :1;
             U32    SyncState       :1;
             U32    PortNumber      :3;
             U32    DataAddress     :8;
             U32    DataRead        :8;
             U32    DataWrite       :8;
#else // if defined( __LITTLE_ENDIAN__ )
             U32    DataWrite       :8;
             U32    DataRead        :8;
             U32    DataAddress     :8;
             U32    PortNumber      :3;
             U32    SyncState       :1;
             U32    Reserved        :1;
             U32    ReadWrite       :1;
             U32    Run             :1;
             U32    WakeUp          :1;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_ULPI_VIEWPORT, *PTR_USB_ULPI_VIEWPORT;


/* ************************************************************************** */
/* Register - ENDPTNAK; Offset: Base + 0178h                                  */
/* ************************************************************************** */

typedef struct _USB_END_POINT_NAK
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     TxEndPointNak   :16;
            U32     RxEndPointNak   :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     RxEndPointNak   :16;
            U32     TxEndPointNak   :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_NAK, *PTR_USB_END_POINT_NAK;


/* ************************************************************************** */
/* Register - ENDPTNAKEN; Offset: Base + 017Ch                                */
/* ************************************************************************** */

typedef struct _USB_END_POINT_NAK_ENABLE
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     TxEndPointNakEnable :16;
            U32     RxEndPointNakEnable :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     RxEndPointNakEnable :16;
            U32     TxEndPointNakEnable :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_NAK_ENABLE, *PTR_USB_END_POINT_NAK_ENABLE;


/* ************************************************************************** */
/* Register - CONFIGFLAG; Offset: Base + 0180h                                */
/* ************************************************************************** */

typedef struct _USB_CONFIG_FLAG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved    :31;
            U32     ConfigFlag  : 1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     ConfigFlag  : 1;
            U32     Reserved    :31;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_CONFIG_FLAG, *PTR_USB_CONFIG_FLAG;


/* ************************************************************************** */
/* Register - ; Offset: Base + 0000h                                   */
/* ************************************************************************** */

typedef struct _USB_PORT_REGISTER
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     ParallelTransSelect     :2;
            U32     SerialTransSelect       :1;
            U32     ParallelTransWidth      :1;
            U32     PortSpeed               :2;
            U32     Reserved25              :1;
            U32     PortForceFullSpeed      :1;
            U32     PhyLowPowerSuspend      :1;
            U32     WakeOnOverCurrent       :1;
            U32     WakeOnDisconnect        :1;
            U32     WakeOnConnect           :1;
            U32     PortTestControl         :4;
            U32     PortIndicatorControl    :2;
            U32     PortOwner               :1;
            U32     PortPower               :1;
            U32     LineStatus              :2;
            U32     HighSpeedPort           :1;
            U32     PortReset               :1;
            U32     Suspend                 :1;
            U32     ForcePortResume         :1;
            U32     OverCurrentChange       :1;
            U32     OverCurrentActive       :1;
            U32     PortEnableChange        :1;
            U32     PortEnable              :1;
            U32     ConnectStatusChange     :1;
            U32     CurrentConnectStatus    :1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     CurrentConnectStatus    :1;
            U32     ConnectStatusChange     :1;
            U32     PortEnable              :1;
            U32     PortEnableChange        :1;
            U32     OverCurrentActive       :1;
            U32     OverCurrentChange       :1;
            U32     ForcePortResume         :1;
            U32     Suspend                 :1;
            U32     PortReset               :1;
            U32     HighSpeedPort           :1;
            U32     LineStatus              :2;
            U32     PortPower               :1;
            U32     PortOwner               :1;
            U32     PortIndicatorControl    :2;
            U32     PortTestControl         :4;
            U32     WakeOnConnect           :1;
            U32     WakeOnDisconnect        :1;
            U32     WakeOnOverCurrent       :1;
            U32     PhyLowPowerSuspend      :1;
            U32     PortForceFullSpeed      :1;
            U32     Reserved25              :1;
            U32     PortSpeed               :2;
            U32     ParallelTransWidth      :1;
            U32     SerialTransSelect       :1;
            U32     ParallelTransSelect     :2;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_PORT_REGISTER, *PTR_USB_PORT_REGISTER;


/* ************************************************************************** */
/* Register - ; Offset: Base + 0000h                                   */
/* ************************************************************************** */

typedef struct _USB_OTG
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved31              :1;
            U32     DataPulseIntEn          :1;
            U32     OneMsTimerIntEn         :1;
            U32     BSessionEndIntEn        :1;
            U32     BSessionValidIntEn      :1;
            U32     ASessionValidIntEn      :1;
            U32     AVBusValidIntEn         :1;
            U32     USBIdIntEn              :1;
            U32     Reserved23              :1;
            U32     DataPulseIntStatus      :1;
            U32     OneMsTimerIntStatus     :1;
            U32     BSessionEndIntStatus    :1;
            U32     BSessionValidIntStatus  :1;
            U32     ASessionValidIntStatus  :1;
            U32     AVBusValidIntStatus     :1;
            U32     USBIdIntStatus          :1;
            U32     Reserved15              :1;
            U32     DataPulseInt            :1;
            U32     OneMsTimerInt           :1;
            U32     BSessionEndInt          :1;
            U32     BSessionValidInt        :1;
            U32     ASessionValidInt        :1;
            U32     AVBusValidInt           :1;
            U32     USBIdInt                :1;
            U32     HWAssistBDisconnect     :1;
            U32     HWAssistDataPulse       :1;
            U32     IDPullUp                :1;
            U32     DataPulsing             :1;
            U32     OTGTermination          :1;
            U32     HWAssistAutoReset       :1;
            U32     VBUSCharge              :1;
            U32     VBUSDischarge           :1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     VBUSDischarge           :1;
            U32     VBUSCharge              :1;
            U32     HWAssistAutoReset       :1;
            U32     OTGTermination          :1;
            U32     DataPulsing             :1;
            U32     IDPullUp                :1;
            U32     HWAssistDataPulse       :1;
            U32     HWAssistBDisconnect     :1;
            U32     USBIdInt                :1;
            U32     AVBusValidInt           :1;
            U32     ASessionValidInt        :1;
            U32     BSessionValidInt        :1;
            U32     BSessionEndInt          :1;
            U32     OneMsTimerInt           :1;
            U32     DataPulseInt            :1;
            U32     Reserved15              :1;
            U32     USBIdIntStatus          :1;
            U32     AVBusValidIntStatus     :1;
            U32     ASessionValidIntStatus  :1;
            U32     BSessionValidIntStatus  :1;
            U32     BSessionEndIntStatus    :1;
            U32     OneMsTimerIntStatus     :1;
            U32     DataPulseIntStatus      :1;
            U32     Reserved23              :1;
            U32     USBIdIntEn              :1;
            U32     AVBusValidIntEn         :1;
            U32     ASessionValidIntEn      :1;
            U32     BSessionValidIntEn      :1;
            U32     BSessionEndIntEn        :1;
            U32     OneMsTimerIntEn         :1;
            U32     DataPulseIntEn          :1;
            U32     Reserved31              :1;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_OTG, *PTR_USB_OTG;


/* ************************************************************************** */
/* Register - USBMODE; Offset: Base + 01A8h                                   */
/* ************************************************************************** */

typedef struct _USB_MODE_SELECT
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved            :26;
            U32     VBusPowerSelect     :1;
            U32     StreamDisableMode   :1;
            U32     SetupLockOutMode    :1;
            U32     EndianSelect        :1;
            U32     ControllerMode      :2;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     ControllerMode      :2;
            U32     EndianSelect        :1;
            U32     SetupLockOutMode    :1;
            U32     StreamDisableMode   :1;
            U32     VBusPowerSelect     :1;
            U32     Reserved            :26;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_MODE_SELECT, *PTR_USB_MODE_SELECT;


/* ************************************************************************** */
/* Register - ENDPTSETUPSTAT; Offset: Base + 01ACh                            */
/* ************************************************************************** */

typedef struct _USB_END_POINT_SETUP_STAT
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved            :16;
            U32     EndPointSetupStatus :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     EndPointSetupStatus :16;
            U32     Reserved            :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_SETUP_STAT, *PTR_USB_END_POINT_SETUP_STAT;


/* ************************************************************************** */
/* Register - ENDPTPRIME; Offset: Base + 01B0h                                */
/* ************************************************************************** */

typedef struct _USB_END_POINT_PRIME
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     PrimeEndPointTxBuf  :16;
            U32     PrimeEndPointRxBuf  :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     PrimeEndPointRxBuf  :16;
            U32     PrimeEndPointTxBuf  :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_PRIME, *PTR_USB_END_POINT_PRIME;


/* ************************************************************************** */
/* Register - ENDPTFLUSH; Offset: Base + 01B4h                                */
/* ************************************************************************** */

typedef struct _USB_END_POINT_FLUSH
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     FlushEndPointTxBuf  :16;
            U32     FlushEndPointRxBuf  :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     FlushEndPointRxBuf  :16;
            U32     FlushEndPointTxBuf  :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_FLUSH, *PTR_USB_END_POINT_FLUSH;


/* ************************************************************************** */
/* Register - ENDPTSTAT; Offset: Base + 01B8h                                 */
/* ************************************************************************** */

typedef struct _USB_END_POINT_STATUS
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     EndPointTxBufReady  :16;
            U32     EndPointRxBufReady  :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     EndPointRxBufReady  :16;
            U32     EndPointTxBufReady  :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_STATUS, *PTR_USB_END_POINT_STATUS;


/* ************************************************************************** */
/* Register - ENDPTCOMPLETE; Offset: Base + 01BCh                             */
/* ************************************************************************** */

typedef struct _USB_END_POINT_COMPLETE
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     EndPointTxBufComplete  :16;
            U32     EndPointRxBufComplete  :16;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     EndPointRxBufComplete  :16;
            U32     EndPointTxBufComplete  :16;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_COMPLETE, *PTR_USB_END_POINT_COMPLETE;


/* ************************************************************************** */
/* Register - ENDPTCTRL0; Offset: Base + 01C0h                                */
/* ************************************************************************** */

typedef struct _USB_END_POINT_CONTROL_0
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved24to31      :8;
            U32     TxEndPointEnable    :1;
            U32     Reserved20to22      :3;
            U32     TxEndPointType      :2;
            U32     Reserved17          :1;
            U32     TxEndPointStall     :1;

            U32     Reserved8to15       :8;
            U32     RxEndPointEnable    :1;
            U32     Reserved04to06      :3;
            U32     RxEndPointType      :2;
            U32     Reserved01          :1;
            U32     RxEndPointStall     :1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     RxEndPointStall     :1;
            U32     Reserved01          :1;
            U32     RxEndPointType      :2;
            U32     Reserved04to06      :3;
            U32     RxEndPointEnable    :1;
            U32     Reserved8to15       :8;

            U32     TxEndPointStall     :1;
            U32     Reserved17          :1;
            U32     TxEndPointType      :2;
            U32     Reserved20to22      :3;
            U32     TxEndPointEnable    :1;
            U32     Reserved24to31      :8;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_CONTROL_0, *PTR_USB_END_POINT_CONTROL_0;


/* ************************************************************************** */
/* Register - ENDPTCTRL1-15; Offset: Base + 01C0h                                */
/* ************************************************************************** */

typedef struct _USB_END_POINT_CONTROL_GENERAL
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved24to31      :8;
            U32     TxEndPointEnable    :1;
            U32     TxDataToggleReset   :1;
            U32     TxDataToggleInhibit :1;
            U32     Reserved20          :1;
            U32     TxEndPointType      :2;
            U32     TxEndPointDataSource:1;
            U32     TxEndPointStall     :1;

            U32     Reserved8to15       :8;
            U32     RxEndPointEnable    :1;
            U32     RxDataToggleReset   :1;
            U32     RxDataToggleInhibit :1;
            U32     Reserved4           :1;
            U32     RxEndPointType      :2;
            U32     RxEndPointDataSource:1;
            U32     RxEndPointStall     :1;
#else // if defined( __LITTLE_ENDIAN__ )
            U32     RxEndPointStall     :1;
            U32     RxEndPointDataSource:1;
            U32     RxEndPointType      :2;
            U32     Reserved4           :1;
            U32     RxDataToggleInhibit :1;
            U32     RxDataToggleReset   :1;
            U32     RxEndPointEnable    :1;
            U32     Reserved8to15       :8;

            U32     TxEndPointStall     :1;
            U32     TxEndPointDataSource:1;
            U32     TxEndPointType      :2;
            U32     Reserved20          :1;
            U32     TxDataToggleInhibit :1;
            U32     TxDataToggleReset   :1;
            U32     TxEndPointEnable    :1;
            U32     Reserved24to31      :8;
#endif //if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
        } bits;

        U32         DWord;
    } u;

} USB_END_POINT_CONTROL_GENERAL, *PTR_USB_END_POINT_CONTROL_GENERAL;


/* ************************************************************************** */
/* Register - ; Offset: Base + 0000h                                   */
/* ************************************************************************** */

typedef volatile struct _PBAM_USB_CTRLR_REGISTERS
{
    USB_ID                                  Id;                             /* 0x000 */
    USB_HW_GENERAL                          HwGeneral;                      /* 0x004 */
    USB_HW_HOST                             HostHwParams;                   /* 0x008*/
    USB_HW_DEVICE                           DeviceHwParams;                 /* 0x00c */
    USB_HW_TX_BUFFER                        TxBuffHwParams;                 /* 0x010 */
    USB_HW_RX_BUFFER                        RxBuffHwParams;                 /* 0x014 */

    //HWTTTXBUF                                                     ----- /* 0x018 */
    //HWTTRXBUF                                                     ----- /* 0x01C */
    //Reserved                                                      ----- /* 0x020 - 0x07C */
    U32                                     Reserved0[26];                  /* 0x018 - 0x07C */
    //U32                                     Reserved0[57];       /* 0x18 - 0xFC */

    USB_GP_TIMER_LD                         GPTmrLoadReg0;                  /* 0x80 */
    USB_GP_TIMER_CTRL                       GPTmrCtrlReg0;                  /* 0x84 */
    USB_GP_TIMER_LD                         GPTmrLoadReg1;                  /* 0x88 */
    USB_GP_TIMER_CTRL                       GPTmrCtrlReg1;                  /* 0x8C */

    U32                                     Reserved1[28];                  /* 0x90 - 0xFF */

    USB_EHCI_COMPLIANCE                     EHCICompliance;                 /* 0x100 - 0x108*/

    U32                                     Reserved2[5];                   /* 0x10c - 0x11F */

    USB_DCI_VERSION                         DCIVersion;                     /* 0x120 */
    USB_DCC_PARAMS                          DCCParams;                      /* 0x124 */

    U32                                     Reserved3[6];                   /* 0x128 - 0x13F */

    USB_CMD                                 Command;                        /* 0x140 */
    USB_STATUS_REG                          Status;                         /* 0x144 */
    USB_INTERRUPT_REG                       InterruptReg;                   /* 0x148 */
    USB_FRINDEX                             FrameIndex;                     /* 0x14c */

    U32                                     Reserved4;                      /* 0x150 */

    USB_HOST_CONTROLLER_PERIODIC_LIST_BASE  PeriodicListBase;               /* 0x154 */ /* See HW bugtrack TB 780 if this is to be used. */
    USB_ASYNC_LIST_ADDR                     AsyncListAddress;               /* 0x158 */
    USB_TT_CONTROL                          TTStatusControl;                /* 0x15c */
    USB_BURST_SIZE                          ProgrammableBurstSize;          /* 0x160 */
    USB_TX_FILL_TUNING                      TxFillTuning;                   /* 0x164 */

    U32                                     Reserved5[2];                   /* 0x168 - 0x16F */

    USB_ULPI_VIEWPORT                       ULPIViewPort;                   /* 0x170 */

    U32                                     Reserved6;                      /* 0x174 */

    USB_END_POINT_NAK                       EndPointNak;                    /* 0x178 */
    USB_END_POINT_NAK_ENABLE                EndPointNakEnable;              /* 0x17C */
    USB_CONFIG_FLAG                         ConfigFlag;                     /* 0x180 */
    USB_PORT_REGISTER                       PortStatusControl[8];           /* 0x184 */
    USB_OTG                                 OtgStatusControl;               /* 0x1A4 */
    USB_MODE_SELECT                         UsbDeviceModeSelect;            /* 0x1A8 */
    USB_END_POINT_SETUP_STAT                EndPointSetupStat;              /* 0x1AC */
    USB_END_POINT_PRIME                     EndPointInit;                   /* 0x1B0 */
    USB_END_POINT_FLUSH                     EndPointFlush;                  /* 0x1B4 */
    USB_END_POINT_STATUS                    EndPointStatus;                 /* 0x1B8 */
    USB_END_POINT_COMPLETE                  EndPointComplete;               /* 0x1BC */
    USB_END_POINT_CONTROL_0                 EndPointControl0;               /* 0x1C0 */
    USB_END_POINT_CONTROL_GENERAL           EndPointControlGeneral[15];     /* 0x1C4 - 0x1FC */

} PBAM_USB_CTRLR_REGISTERS, *PTR_PBAM_USB_CTRLR_REGISTERS;


#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIBHW_H_ */
