/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibProto.h
 *          Title:  USB Library protocol header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains definitions to be used by the USB HAL source code.
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIBPROTO_H_
#define _USBLIBPROTO_H_

#if (IOC_USB_SUPPORT)

/* The following defines are used to indicate the non dword aligned errata for
   the A0 chipset and whether or not to invoke the workaround for the same.
   The A0 revision is not able to handle non-dword aligned data. A MoveToDCR
   call is used to workaround the problem. A fix is expected in the later chip
   revisions.

   When a hardware fix is available then these should be set to '0'.
 */
#define USB_NON_DWORD_ERRATA                            0
#define USB_NON_DWORD_ERRATA_WORKAROUND                 0

/* ************************************************************************** */
/* Defines - Sizes/Limits/Numbers/etc.                                        */
/* ************************************************************************** */

#define USB_MAX_PORTS                                   2

#define USB_PORT_NUM_MIN                                0
#define USB_PORT_NUM_MAX                                1

#define USB_MAX_DEV                                     2 //(1 + 1)
#define USB_MAX_CONFIG                                  1
#define USB_MAX_EP                                      2

/* Transaction related parameters */
#define USB_MAX_QUEUE_HEAD                              3

#define USB_MAX_CTRL_EP_TRANS_ELEMENT                   16
#define USB_MAX_TRANS_ELEMENT                           64 //1026

#define USB_MAX_SIZE_PER_PAGE                           4096

#define USB_MAX_PAGES_PER_ELEM                          4

#define USB_MAX_CHUNK_SIZE_PER_TRANS_ELEMENT            (16 * 1024) /* 16K */

#define USB_GENERIC_BUFF_LENGTH                         512
#define USB_GENERIC_DATA_LENGTH                         (4 * 1024 * 1024) /* 4 MB */

/* PID Codes */
#define USB_PID_TOKEN_OUT                               0x01
#define USB_PID_TOKEN_IN                                0x09
#define USB_PID_TOKEN_SOF                               0x05
#define USB_PID_TOKEN_SETUP                             0x0d

#define USB_PID_DATA_DAT0                               0x03
#define USB_PID_DATA_DAT1                               0x0b
#define USB_PID_DATA_DAT2                               0x07
#define USB_PID_DATA_MDAT                               0x0f

#define USB_PID_HANDSHAKE_ACK                           0x02
#define USB_PID_HANDSHAKE_NACK                          0x0a
#define USB_PID_HANDSHAKE_STALL                         0x0e
#define USB_PID_HANDSHAKE_NYET                          0x06

#define USB_TOKEN_SPECIAL_PRE                           0x0c
#define USB_TOKEN_SPECIAL_ERR                           0x0c
#define USB_TOKEN_SPECIAL_SPLIT                         0x08
#define USB_TOKEN_SPECIAL_PING                          0x04
#define USB_TOKEN_SPECIAL_RESERVED                      0x00


/* IO directions */
#define USB_IO_SETUP_READ                               0
#define USB_IO_SETUP_WRITE                              1

/* ************************************************************************** */
/* Defines - Protocol data structures                                         */
/* ************************************************************************** */

/* ************************************************************************** */
/* Defines - User defined data structures                                     */
/* ************************************************************************** */

/* used only by the software this is not a standard, but is based on the
   standard values and the controller details */
typedef struct _HOST_QUEUE_HEAD_CFG_PARAMS
{
    U32         Type;
    BOOL        HeadOfHeads;
    BOOL        DataToggleCtrl;
    U32         EndPointNum;
    U32         DevAddr;
    U32         CtrlEP;
    U32         EpSpeed;
    U32         MaxPacketSize;

} HOST_QUEUE_HEAD_CFG_PARAMS, *PTR_HOST_QUEUE_HEAD_CFG_PARAMS;

/* Transaction element - used only by software */
typedef struct _TRANSACTION_ELEMENT
{
    U32         Address;
    U32         Size;
    U32         Pid;
    U32         Dt;
    U32         Ping;

} TRANSACTION_ELEMENT, *PTR_TRANSACTION_ELEMENT;

/* Transaction status */
typedef struct _TRANSACTION_STATUS
{
    U32         Reclaim;            //1- reclaim the buffer, 0 - buffer cannot be reclaimed
    U32         HCErrState;         //1- SYS Error observed, 0- no system error observed
    U32         NumberOfTransComp;  //Number of times transfer complete observed

} TRANSACTION_STATUS, *PTR_TRANSACTION_STATUS;

/* Endpoint feature */
typedef struct _USB_ENDPOINT_FEATURE
{
    U8          Addr;
    U16         MaxPacketSize;

} USB_ENDPOINT_FEATURE, *PTR_USB_ENDPOINT_FEATURE;

/* ************************************************************************** */
/*Externs                                                        */
/* ************************************************************************** */
extern USB_SCSI_STATUS                              gScsiSenseNone;
extern USB_STACK_GLOBAL_STATE                       gUsbStackGlobalState[USB_MAX_PORTS];

extern PTR_PBAM_USB_CTRLR_REGISTERS                 gPtrUsbReg;

extern PTR_USB_DESC_QUEUE_TRANSFER_ELEMENT          gPtrUsbQueueElement[USB_MAX_PORTS][USB_MAX_QUEUE_HEAD];
extern PTR_USB_DESC_QUEUE_HEAD                      gPtrUsbQueueHead[USB_MAX_PORTS];

extern PTR_TRANSACTION_ELEMENT                      gPtrUsbSetupTransElementsBuff;
extern HOST_QUEUE_HEAD_CFG_PARAMS                   gPtrUsbSetupQHElementParamsBuff[];

extern PTR_USB_PKT_REQUEST                          gPtrUsbRequest[USB_MAX_PORTS];

extern PTR_USB_DEV_QUALIFIER                        gPtrUsbDevQualifier[USB_MAX_PORTS];

extern PTR_USB_DESC_DEVICE                          gPtrUsbDeviceDesc[USB_MAX_PORTS];
extern PTR_USB_DESC_CONFIG                          gPtrUsbConfigDesc[USB_MAX_PORTS];
extern PTR_USB_DESC_OTH_SPEED_CONFIG                gPtrUsbOthSpeedConfigDesc[USB_MAX_PORTS];

extern USB_DESC_INTERFACE                           gUsbInterfaceDesc[USB_MAX_PORTS][USB_MAX_CONFIG];
extern USB_DESC_ENDPOINT                            gUsbEndPointDesc[USB_MAX_PORTS][USB_MAX_CONFIG][USB_MAX_EP+1];

extern USB_ENDPOINT_FEATURE                         gUsbEndPointFeature[USB_MAX_PORTS][USB_MAX_EP+1];

extern U8                                           gUsbDeviceAddr[USB_MAX_PORTS];
extern PU8                                          gPtrUsbCompleteConfigDesc[USB_MAX_DEV];
extern PU32                                         gPtrStatus;

/* ************************************************************************** */
/* Function prototypes                                                        */
/* ************************************************************************** */

USB_STATUS usbInitStack( U32 UsbPort );
USB_STATUS usbProtocolInit( U32 UsbPort );
USB_STATUS usbProtoInitiateEnumeration( U32 UsbPort );
USB_STATUS usbProtoEnumerate( U32 UsbPort );

USB_STATUS usbProtoSendScsiCmd(
    U32                             UsbPort,
    U32                             TransLength,
    BOOL                            Direction,
    U32                             BuffAddr,
    U32                             CmdAddr,
    U32                             StatAddr,
    U32                             IoTimeoutMs
    );

USB_STATUS usbProtoAbortScsiCmd(
    U32                             UsbPort,
    BOOL                            Direction
    );

USB_STATUS usbProtoFillQueueHead(
    U32                             UsbPort,
    U32                             QHeadIdx,
    PTR_HOST_QUEUE_HEAD_CFG_PARAMS  PtrQhCp
    );

USB_STATUS usbProtoFillTransactionElement(
    U32                             UsbPort,
    U32                             QueueHead,
    U32                             NumOfElements,
    PTR_TRANSACTION_ELEMENT         PtrTransElementArray
    );

USB_STATUS usbAllocGlobals( U32 UsbPort );

USB_STATUS usbInitGlobals( U32 UsbPort );

USB_STATUS usbProtoClearTransferDesc( U32 UsbPort, U8 QHeadNum );
USB_STATUS usbProtoSetupTransaction(
    U32                             UsbPort,
    U32                             EndPoint,
    U32                             ReqPacketAddr,
    BOOL                            DataTransPresent,
    U32                             DataBufferAddr,
    U32                             Size
    );
USB_STATUS usbProtoCompleteTransaction( U32 UsbPort, BOOL DataTransPresent );
USB_STATUS usbProtoAnalyzeConfigDescriptor(
    U32                             UsbPort,
    U8                              UsbConfig,
    PU8                             PtrUsbCompleteConfigDesc
    );

void usbProtoCheckTransactionStatus(U32 UsbPort);
void usbLibBuildRequest(U32 UsbPort, U8 RequestType, U8 Request, U16 Value, U16 Index, U16 Length);

#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIBPROTO_H_ */
