/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibStackGlobals.h
 *          Title:  USB library stack globals header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains definitions to be used by the USB HAL source code.
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIBSTACKGLOBALS_H_
#define _USBLIBSTACKGLOBALS_H_

#if (IOC_USB_SUPPORT)

/* ************************************************************************** */
/* USB stack states                                                           */
/* ************************************************************************** */

#define USB_STACK_STATE_IDLE                                    (0x00)
#define USB_STACK_STATE_BUSY                                    (0x01)
#define USB_STACK_STATE_ERROR                                   (0x02)
#define USB_STACK_STATE_ENUMERATION                             (0x03)
#define USB_STACK_STATE_IO                                      (0x04)

/* Sub states */
#define USB_STACK_SUB_STATE_IDLE                                (0x00)
/* Enumeration sub states */
#define USB_STACK_SUB_STATE_GET_DESCRIPTORS_1                   (0x01)
#define USB_STACK_SUB_STATE_SETUP_ADDRESS_1                     (0x02)
#define USB_STACK_SUB_STATE_GET_DESCRIPTORS_2                   (0x03)
#define USB_STACK_SUB_STATE_GET_CONFIG_DESCRIPTORS_1            (0x04)
#define USB_STACK_SUB_STATE_GET_CONFIG_DESCRIPTORS_2            (0x05)
#define USB_STACK_SUB_STATE_SET_CONFIGURATION                   (0x06)
#define USB_STACK_SUB_STATE_ANALYZE_DESCRIPTORS                 (0x07)
#define USB_STACK_SUB_STATE_DISCOVERY_COMPLETED                 (0x08)
#define USB_STACK_SUB_STATE_ERROR                               (0x09)
/* I/O sub states */
#define USB_STACK_SUB_STATE_IO_CMD                              (0x0A)
#define USB_STACK_SUB_STATE_IO_DATA                             (0x0B)
#define USB_STACK_SUB_STATE_IO_STATUS                           (0x0C)

/* setup transaction sub states */
#define USB_STACK_SETUP_STATE_COMPLETE                          (0x00)
#define USB_STACK_SETUP_STATE_START_DATA                        (0x01)
#define USB_STACK_SETUP_STATE_START_NO_DATA                     (0x02)

#define USB_PROTO_RETRY_COUNT_THRESHOLD         2

/* ************************************************************************** */
/* USB Stack Global State definition                                          */
/* ************************************************************************** */
typedef struct _USB_STACK_GLOBAL_STATE
{
    USB_STATUS       StackLastStatus;

    U8               StackState;
    U8               StackSubState;
    U8               SetupState;
    U8               DevState;

    U8               Direction;
    U8               ActiveQueueHead;
    U8               CmdHandle;
    U8               IoCmd;
    U8               Retries;

    U32              TransLength;
    U32              BuffAddr;
    U32              StatAddr;

    struct
    {
        U32          ControllerInitialized       :  1;
        U32          CallbackRegistered          :  1;
        U32          EnumerationComplete         :  1;
        U32          MscInitDone                 :  1;
        U32          AutoReqSense                :  1;

        U32          Reserved                    : 27;
    }
    Flags;

    CompletionHandler RequestCompleteCallback;

} USB_STACK_GLOBAL_STATE, *PTR_USB_STACK_GLOBAL_STATE;

#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIBSTACKGLOBALS_H_ */
